@@ -28,7 +28,7 @@ THE SOFTWARE.
 #include "thread.h"
 #include "privs.h"
 #include "sigevent.h"
-#include "version.h"
+#include "lib/version.h"
 #include "command.h"
 #include "vty.h"
 #include "memory.h"
@@ -50,7 +50,6 @@ THE SOFTWARE.
 
 static void babel_fail(void);
 static void babel_init_random(void);
-static void babel_replace_by_null(int fd);
 static void babel_exit_properly(void);
 static void babel_save_state_file(void);
 
@@ -115,7 +114,7 @@ babel_sigusr1 (void)
     zlog_rotate ();
 }
 
-static struct quagga_signal_t babel_signals[] =
+static struct frr_signal_t babel_signals[] =
   {
     {
       .signal = SIGUSR1,
@@ -137,6 +136,7 @@ struct option longopts[] =
   };
 
 static const struct frr_yang_module_info *const babeld_yang_modules[] = {
+	&frr_filter_info,
 	&frr_interface_info,
 	&frr_vrf_info,
 };
@@ -152,7 +152,7 @@ FRR_DAEMON_INFO(babeld, BABELD,
 
 		.yang_modules = babeld_yang_modules,
 		.n_yang_modules = array_size(babeld_yang_modules),
-		)
+);
 
 int
 main(int argc, char **argv)
@@ -182,8 +182,7 @@ main(int argc, char **argv)
 	  case 0:
 	    break;
 	  default:
-	    frr_help_exit (1);
-	    break;
+	    frr_help_exit(1);
 	  }
     }
 
@@ -199,8 +198,6 @@ main(int argc, char **argv)
     resend_delay = BABEL_DEFAULT_RESEND_DELAY;
     change_smoothing_half_life(BABEL_DEFAULT_SMOOTHING_HALF_LIFE);
 
-    babel_replace_by_null(STDIN_FILENO);
-
     /* init some quagga's dependencies, and babeld's commands */
     if_zapi_callbacks(babel_ifp_create, babel_ifp_up,
 		      babel_ifp_down, babel_ifp_destroy);
@@ -247,32 +244,6 @@ babel_init_random(void)
     srandom(seed);
 }
 
-/*
- close fd, and replace it by "/dev/null"
- exit if error
- */
-static void
-babel_replace_by_null(int fd)
-{
-    int fd_null;
-    int rc;
-
-    fd_null = open("/dev/null", O_RDONLY);
-    if(fd_null < 0) {
-        flog_err_sys(EC_LIB_SYSTEM_CALL, "open(null): %s", safe_strerror(errno));
-        exit(1);
-    }
-
-    rc = dup2(fd_null, fd);
-    if(rc < 0) {
-        flog_err_sys(EC_LIB_SYSTEM_CALL, "dup2(null, 0): %s",
-		  safe_strerror(errno));
-        exit(1);
-    }
-
-    close(fd_null);
-}
-
 /*
  Load the state file: check last babeld's running state, usefull in case of
  "/etc/init.d/babeld restart"
