@@ -35,6 +35,7 @@ THE SOFTWARE.
 #include "message.h"
 #include "kernel.h"
 #include "babel_main.h"
+#include "babel_errors.h"
 
 static unsigned char packet_header[4] = {42, 2};
 
@@ -126,9 +127,8 @@ network_prefix(int ae, int plen, unsigned int omitted,
     return ret;
 }
 
-static void
-parse_update_subtlv(const unsigned char *a, int alen,
-                    unsigned char *channels)
+static bool parse_update_subtlv(const unsigned char *a, int alen,
+				unsigned char *channels)
 {
     int type, len, i = 0;
 
@@ -139,38 +139,53 @@ parse_update_subtlv(const unsigned char *a, int alen,
             continue;
         }
 
-        if(i + 1 > alen) {
-            zlog_err("Received truncated attributes.");
-            return;
-        }
+        if(i + 1 >= alen) {
+            flog_err(EC_BABEL_PACKET, "Received truncated attributes.");
+	    return false;
+	}
         len = a[i + 1];
-        if(i + len > alen) {
-            zlog_err("Received truncated attributes.");
-            return;
-        }
+        if(i + len + 2 > alen) {
+            flog_err(EC_BABEL_PACKET, "Received truncated attributes.");
+	    return false;
+	}
 
-        if(type == SUBTLV_PADN) {
-            /* Nothing. */
-        } else if(type == SUBTLV_DIVERSITY) {
-            if(len > DIVERSITY_HOPS) {
-                zlog_err("Received overlong channel information (%d > %d).n",
-                         len, DIVERSITY_HOPS);
-                len = DIVERSITY_HOPS;
-            }
-            if(memchr(a + i + 2, 0, len) != NULL) {
-                /* 0 is reserved. */
-                zlog_err("Channel information contains 0!");
-                return;
-            }
-            memset(channels, 0, DIVERSITY_HOPS);
-            memcpy(channels, a + i + 2, len);
-        } else {
-            debugf(BABEL_DEBUG_COMMON,
-                   "Received unknown route attribute %d.", type);
-        }
+	if (type & SUBTLV_MANDATORY) {
+		/*
+		 * RFC 8966 - 4.4
+		 * If the mandatory bit is set, then the whole enclosing
+		 * TLV MUST be silently ignored (except for updating the
+		 * parser state by a Router-Id, Next Hop, or Update TLV,
+		 * as described in the next section).
+		 */
+		debugf(BABEL_DEBUG_COMMON,
+		       "Received Mandatory bit set but this FRR version is not prepared to handle it at this point");
+		return true;
+	} else if (type == SUBTLV_PADN) {
+		/* Nothing. */
+	} else if (type == SUBTLV_DIVERSITY) {
+		if (len > DIVERSITY_HOPS) {
+			flog_err(
+				EC_BABEL_PACKET,
+				"Received overlong channel information (%d > %d).n",
+				len, DIVERSITY_HOPS);
+			len = DIVERSITY_HOPS;
+		}
+		if (memchr(a + i + 2, 0, len) != NULL) {
+			/* 0 is reserved. */
+			flog_err(EC_BABEL_PACKET,
+				 "Channel information contains 0!");
+			return false;
+		}
+		memset(channels, 0, DIVERSITY_HOPS);
+		memcpy(channels, a + i + 2, len);
+	} else {
+		debugf(BABEL_DEBUG_COMMON,
+		       "Received unknown route attribute %d.", type);
+	}
 
-        i += len + 2;
+	i += len + 2;
     }
+    return false;
 }
 
 static int
@@ -180,37 +195,52 @@ parse_hello_subtlv(const unsigned char *a, int alen,
     int type, len, i = 0, ret = 0;
 
     while(i < alen) {
-        type = a[0];
+        type = a[i];
         if(type == SUBTLV_PAD1) {
             i++;
             continue;
         }
 
-        if(i + 1 > alen) {
-            zlog_err("Received truncated sub-TLV on Hello message.");
+        if(i + 1 >= alen) {
+            flog_err(EC_BABEL_PACKET,
+		      "Received truncated sub-TLV on Hello message.");
             return -1;
         }
         len = a[i + 1];
-        if(i + len > alen) {
-            zlog_err("Received truncated sub-TLV on Hello message.");
+        if(i + len + 2 > alen) {
+            flog_err(EC_BABEL_PACKET,
+		      "Received truncated sub-TLV on Hello message.");
             return -1;
         }
 
-        if(type == SUBTLV_PADN) {
-            /* Nothing to do. */
-        } else if(type == SUBTLV_TIMESTAMP) {
-            if(len >= 4) {
-                DO_NTOHL(*hello_send_us, a + i + 2);
-                ret = 1;
-            } else {
-                zlog_err("Received incorrect RTT sub-TLV on Hello message.");
-            }
-        } else {
-            debugf(BABEL_DEBUG_COMMON,
-                   "Received unknown Hello sub-TLV type %d.", type);
-        }
+	if (type & SUBTLV_MANDATORY) {
+		/*
+		 * RFC 8966 4.4
+		 * If the mandatory bit is set, then the whole enclosing
+		 * TLV MUST be silently ignored (except for updating the
+		 * parser state by a Router-Id, Next Hop, or Update TLV, as
+		 * described in the next section).
+		 */
+		debugf(BABEL_DEBUG_COMMON,
+		       "Received subtlv with Mandatory bit, this version of FRR is not prepared to handle this currently");
+		return -2;
+	} else if (type == SUBTLV_PADN) {
+		/* Nothing to do. */
+	} else if (type == SUBTLV_TIMESTAMP) {
+		if (len >= 4) {
+			DO_NTOHL(*hello_send_us, a + i + 2);
+			ret = 1;
+		} else {
+			flog_err(
+				EC_BABEL_PACKET,
+				"Received incorrect RTT sub-TLV on Hello message.");
+		}
+	} else {
+		debugf(BABEL_DEBUG_COMMON,
+		       "Received unknown Hello sub-TLV type %d.", type);
+	}
 
-        i += len + 2;
+	i += len + 2;
     }
     return ret;
 }
@@ -223,19 +253,21 @@ parse_ihu_subtlv(const unsigned char *a, int alen,
     int type, len, i = 0, ret = 0;
 
     while(i < alen) {
-        type = a[0];
+        type = a[i];
         if(type == SUBTLV_PAD1) {
             i++;
             continue;
         }
 
-        if(i + 1 > alen) {
-            zlog_err("Received truncated sub-TLV on IHU message.");
+        if(i + 1 >= alen) {
+            flog_err(EC_BABEL_PACKET,
+		      "Received truncated sub-TLV on IHU message.");
             return -1;
         }
         len = a[i + 1];
-        if(i + len > alen) {
-            zlog_err("Received truncated sub-TLV on IHU message.");
+        if(i + len + 2 > alen) {
+            flog_err(EC_BABEL_PACKET,
+		      "Received truncated sub-TLV on IHU message.");
             return -1;
         }
 
@@ -248,7 +280,8 @@ parse_ihu_subtlv(const unsigned char *a, int alen,
                 ret = 1;
             }
             else {
-                zlog_err("Received incorrect RTT sub-TLV on IHU message.");
+                flog_err(EC_BABEL_PACKET,
+			  "Received incorrect RTT sub-TLV on IHU message.");
             }
         } else {
             debugf(BABEL_DEBUG_COMMON,
@@ -278,15 +311,20 @@ channels_len(unsigned char *channels)
    followed by a sequence of TLVs. TLVs of known types are also checked to meet
    minimum length constraints defined for each. Return 0 for no errors. */
 static int
-babel_packet_examin(const unsigned char *packet, int packetlen)
+babel_packet_examin(const unsigned char *packet, int packetlen, int *blength)
 {
-    unsigned i = 0, bodylen;
+    int i = 0, bodylen;
     const unsigned char *message;
     unsigned char type, len;
 
     if(packetlen < 4 || packet[0] != 42 || packet[1] != 2)
         return 1;
     DO_NTOHS(bodylen, packet + 2);
+    if(bodylen + 4 > packetlen) {
+        debugf(BABEL_DEBUG_COMMON, "Received truncated packet (%d + 4 > %d).",
+                 bodylen, packetlen);
+        return 1;
+    }
     while (i < bodylen){
         message = packet + 4 + i;
         type = message[0];
@@ -294,12 +332,12 @@ babel_packet_examin(const unsigned char *packet, int packetlen)
             i++;
             continue;
         }
-        if(i + 1 > bodylen) {
+        if(i + 2 > bodylen) {
             debugf(BABEL_DEBUG_COMMON,"Received truncated message.");
             return 1;
         }
         len = message[1];
-        if(i + len > bodylen) {
+        if(i + len + 2 > bodylen) {
             debugf(BABEL_DEBUG_COMMON,"Received truncated message.");
             return 1;
         }
@@ -310,6 +348,8 @@ babel_packet_examin(const unsigned char *packet, int packetlen)
         }
         i += len + 2;
     }
+
+    *blength = bodylen;
     return 0;
 }
 
@@ -337,31 +377,25 @@ parse_packet(const unsigned char *from, struct interface *ifp,
     }
 
     if(!linklocal(from)) {
-        zlog_err("Received packet from non-local address %s.",
-                 format_address(from));
+        flog_err(EC_BABEL_PACKET,
+		  "Received packet from non-local address %s.",
+                  format_address(from));
         return;
     }
 
-    if (babel_packet_examin (packet, packetlen)) {
-        zlog_err("Received malformed packet on %s from %s.",
-                 ifp->name, format_address(from));
+    if (babel_packet_examin (packet, packetlen, &bodylen)) {
+        flog_err(EC_BABEL_PACKET,
+		  "Received malformed packet on %s from %s.",
+                  ifp->name, format_address(from));
         return;
     }
 
     neigh = find_neighbour(from, ifp);
     if(neigh == NULL) {
-        zlog_err("Couldn't allocate neighbour.");
+        flog_err(EC_BABEL_PACKET, "Couldn't allocate neighbour.");
         return;
     }
 
-    DO_NTOHS(bodylen, packet + 2);
-
-    if(bodylen + 4 > packetlen) {
-        zlog_err("Received truncated packet (%d + 4 > %d).",
-                 bodylen, packetlen);
-        bodylen = packetlen - 4;
-    }
-
     i = 0;
     while(i < bodylen) {
         message = packet + 4 + i;
@@ -389,27 +423,69 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                    format_address(from), ifp->name);
             /* Nothing right now */
         } else if(type == MESSAGE_HELLO) {
-            unsigned short seqno, interval;
-            int changed;
-            unsigned int timestamp = 0;
-            DO_NTOHS(seqno, message + 4);
-            DO_NTOHS(interval, message + 6);
-            debugf(BABEL_DEBUG_COMMON,"Received hello %d (%d) from %s on %s.",
-                   seqno, interval,
-                   format_address(from), ifp->name);
-            changed = update_neighbour(neigh, seqno, interval);
-            update_neighbour_metric(neigh, changed);
-            if(interval > 0)
-                /* Multiply by 3/2 to allow hellos to expire. */
-                schedule_neighbours_check(interval * 15, 0);
-            /* Sub-TLV handling. */
-            if(len > 8) {
-                if(parse_hello_subtlv(message + 8, len - 6, &timestamp) > 0) {
-                    neigh->hello_send_us = timestamp;
-                    neigh->hello_rtt_receive_time = babel_now;
-                    have_hello_rtt = 1;
-                }
-            }
+		unsigned short seqno, interval, flags;
+		int changed;
+		unsigned int timestamp = 0;
+
+#define BABEL_UNICAST_HELLO 0x8000
+		DO_NTOHS(flags, message + 2);
+
+		/*
+		 * RFC 8966 4.6.5
+		 * All other bits MUST be sent as a 0 and silently
+		 * ignored on reception
+		 */
+		if (CHECK_FLAG(flags, ~BABEL_UNICAST_HELLO)) {
+			debugf(BABEL_DEBUG_COMMON,
+			       "Received Hello from %s on %s that does not have all 0's in the unused section of flags, ignoring",
+			       format_address(from), ifp->name);
+			goto done;
+		}
+
+		/*
+		 * RFC 8966 Appendix F
+		 * TL;DR -> Please ignore Unicast hellos until FRR's
+		 * BABEL is brought up to date
+		 */
+		if (CHECK_FLAG(flags, BABEL_UNICAST_HELLO)) {
+			debugf(BABEL_DEBUG_COMMON,
+			       "Received Unicast Hello from %s on %s that FRR is not prepared to understand yet",
+			       format_address(from), ifp->name);
+			goto done;
+		}
+
+		DO_NTOHS(seqno, message + 4);
+		DO_NTOHS(interval, message + 6);
+		debugf(BABEL_DEBUG_COMMON,
+		       "Received hello %d (%d) from %s on %s.", seqno, interval,
+		       format_address(from), ifp->name);
+
+		/*
+		 * RFC 8966 Appendix F
+		 * TL;DR -> Please ignore any Hello packets with the interval
+		 * field set to 0
+		 */
+		if (interval == 0) {
+			debugf(BABEL_DEBUG_COMMON,
+			       "Received hello from %s on %s should be ignored as that this version of FRR does not know how to properly handle interval == 0",
+			       format_address(from), ifp->name);
+			goto done;
+		}
+
+		changed = update_neighbour(neigh, seqno, interval);
+		update_neighbour_metric(neigh, changed);
+		if (interval > 0)
+			/* Multiply by 3/2 to allow hellos to expire. */
+			schedule_neighbours_check(interval * 15, 0);
+		/* Sub-TLV handling. */
+		if (len > 8) {
+			if (parse_hello_subtlv(message + 8, len - 6,
+					       &timestamp) > 0) {
+				neigh->hello_send_us = timestamp;
+				neigh->hello_rtt_receive_time = babel_now;
+				have_hello_rtt = 1;
+			}
+		}
         } else if(type == MESSAGE_IHU) {
             unsigned short txcost, interval;
             unsigned char address[16];
@@ -467,7 +543,9 @@ parse_packet(const unsigned char *from, struct interface *ifp,
             unsigned char channels[DIVERSITY_HOPS];
             unsigned short interval, seqno, metric;
             int rc, parsed_len;
-            DO_NTOHS(interval, message + 6);
+	    bool ignore_update = false;
+
+	    DO_NTOHS(interval, message + 6);
             DO_NTOHS(seqno, message + 8);
             DO_NTOHS(metric, message + 10);
             if(message[5] == 0 ||
@@ -506,7 +584,8 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                 have_router_id = 1;
             }
             if(!have_router_id && message[2] != 0) {
-                zlog_err("Received prefix with no router id.");
+                flog_err(EC_BABEL_PACKET,
+			  "Received prefix with no router id.");
                 goto fail;
             }
             debugf(BABEL_DEBUG_COMMON,"Received update%s%s for %s from %s on %s.",
@@ -517,7 +596,8 @@ parse_packet(const unsigned char *from, struct interface *ifp,
 
             if(message[2] == 0) {
                 if(metric < 0xFFFF) {
-                    zlog_err("Received wildcard update with finite metric.");
+                    flog_err(EC_BABEL_PACKET,
+			      "Received wildcard update with finite metric.");
                     goto done;
                 }
                 retract_neighbour_routes(neigh);
@@ -551,14 +631,16 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                 }
 
                 if(parsed_len < len)
-                    parse_update_subtlv(message + 2 + parsed_len,
-                                        len - parsed_len, channels);
-            }
+			ignore_update =
+				parse_update_subtlv(message + 2 + parsed_len,
+						    len - parsed_len, channels);
+	    }
 
-            update_route(router_id, prefix, plen, seqno, metric, interval,
-                         neigh, nh,
-                         channels, channels_len(channels));
-        } else if(type == MESSAGE_REQUEST) {
+	    if (!ignore_update)
+		    update_route(router_id, prefix, plen, seqno, metric,
+				 interval, neigh, nh, channels,
+				 channels_len(channels));
+	} else if(type == MESSAGE_REQUEST) {
             unsigned char prefix[16], plen;
             int rc;
             rc = network_prefix(message[2], message[3], 0,
@@ -609,8 +691,9 @@ parse_packet(const unsigned char *from, struct interface *ifp,
         continue;
 
     fail:
-        zlog_err("Couldn't parse packet (%d, %d) from %s on %s.",
-                 message[0], message[1], format_address(from), ifp->name);
+        flog_err(EC_BABEL_PACKET,
+		  "Couldn't parse packet (%d, %d) from %s on %s.",
+                  message[0], message[1], format_address(from), ifp->name);
         goto done;
     }
 
@@ -630,7 +713,7 @@ parse_packet(const unsigned char *from, struct interface *ifp,
             return;
 
         rtt = MAX(0, local_waiting_us - remote_waiting_us);
-        debugf(BABEL_DEBUG_COMMON, "RTT to %s on %s sample result: %d us.\n",
+        debugf(BABEL_DEBUG_COMMON, "RTT to %s on %s sample result: %d us.",
                format_address(from), ifp->name, rtt);
 
         old_rttcost = neighbour_rttcost(neigh);
@@ -662,7 +745,7 @@ static int
 check_bucket(struct interface *ifp)
 {
     babel_interface_nfo *babel_ifp = babel_get_if_nfo(ifp);
-    if(babel_ifp->bucket <= 0) {
+    if(babel_ifp->bucket == 0) {
         int seconds = babel_now.tv_sec - babel_ifp->bucket_time;
         if(seconds > 0) {
             babel_ifp->bucket = MIN(BUCKET_TOKENS_MAX,
@@ -697,8 +780,7 @@ fill_rtt_message(struct interface *ifp)
             DO_HTONL(babel_ifp->sendbuf + babel_ifp->buffered_hello + 10, time);
             return 1;
         } else {
-            zlog_err("No space left for timestamp sub-TLV "
-                     "(this shouldn't happen)");
+            flog_err(EC_BABEL_PACKET, "No space left for timestamp sub-TLV (this shouldn't happen)");
             return -1;
         }
     }
@@ -732,11 +814,11 @@ flushbuf(struct interface *ifp)
                             babel_ifp->sendbuf, babel_ifp->buffered,
                             (struct sockaddr*)&sin6, sizeof(sin6));
             if(rc < 0)
-                zlog_err("send: %s", safe_strerror(errno));
+                flog_err(EC_BABEL_PACKET, "send: %s", safe_strerror(errno));
         } else {
-            zlog_err("Warning: bucket full, dropping packet to %s.",
-                     ifp->name);
-        }
+		flog_err(EC_BABEL_PACKET, "Bucket full, dropping packet to %s.",
+			 ifp->name);
+	}
     }
     VALGRIND_MAKE_MEM_UNDEFINED(babel_ifp->sendbuf, babel_ifp->bufsize);
     babel_ifp->buffered = 0;
@@ -856,7 +938,8 @@ start_unicast_message(struct neighbour *neigh, int type, int len)
     if(!unicast_buffer)
         unicast_buffer = malloc(UNICAST_BUFSIZE);
     if(!unicast_buffer) {
-        zlog_err("malloc(unicast_buffer): %s", safe_strerror(errno));
+        flog_err(EC_BABEL_MEMORY, "malloc(unicast_buffer): %s",
+		  safe_strerror(errno));
         return -1;
     }
 
@@ -992,11 +1075,13 @@ flush_unicast(int dofree)
                         unicast_buffer, unicast_buffered,
                         (struct sockaddr*)&sin6, sizeof(sin6));
         if(rc < 0)
-            zlog_err("send(unicast): %s", safe_strerror(errno));
+            flog_err(EC_BABEL_PACKET, "send(unicast): %s",
+		      safe_strerror(errno));
     } else {
-        zlog_err("Warning: bucket full, dropping unicast packet to %s if %s.",
-                 format_address(unicast_neighbour->address),
-                 unicast_neighbour->ifp->name);
+	    flog_err(EC_BABEL_PACKET,
+		     "Bucket full, dropping unicast packet to %s if %s.",
+		     format_address(unicast_neighbour->address),
+		     unicast_neighbour->ifp->name);
     }
 
  done:
@@ -1098,7 +1183,9 @@ really_send_update(struct interface *ifp,
     if(channels_len >= 0) {
         accumulate_byte(ifp, 2);
         accumulate_byte(ifp, channels_len);
-        accumulate_bytes(ifp, channels, channels_len);
+
+	if (channels && channels_len > 0)
+		accumulate_bytes(ifp, channels, channels_len);
     }
     end_message(ifp, MESSAGE_UPDATE, 10 + (real_plen + 7) / 8 - omit +
                 channels_size);
@@ -1301,7 +1388,8 @@ buffer_update(struct interface *ifp,
     again:
         babel_ifp->buffered_updates = malloc(n *sizeof(struct buffered_update));
         if(babel_ifp->buffered_updates == NULL) {
-            zlog_err("malloc(buffered_updates): %s", safe_strerror(errno));
+            flog_err(EC_BABEL_MEMORY, "malloc(buffered_updates): %s",
+		      safe_strerror(errno));
             if(n > 4) {
                 /* Try again with a tiny buffer. */
                 n = 4;
@@ -1364,7 +1452,7 @@ send_update(struct interface *ifp, int urgent,
             }
             route_stream_done(routes);
         } else {
-            zlog_err("Couldn't allocate route stream.");
+            flog_err(EC_BABEL_MEMORY, "Couldn't allocate route stream.");
         }
         set_timeout(&babel_ifp->update_timeout, babel_ifp->update_interval);
         babel_ifp->last_update_time = babel_now.tv_sec;
@@ -1412,7 +1500,7 @@ send_wildcard_retraction(struct interface *ifp)
 }
 
 void
-update_myseqno()
+update_myseqno(void)
 {
     myseqno = seqno_plus(myseqno, 1);
 }
@@ -1442,7 +1530,7 @@ send_self_update(struct interface *ifp)
         }
         xroute_stream_done(xroutes);
     } else {
-        zlog_err("Couldn't allocate xroute stream.");
+        flog_err(EC_BABEL_MEMORY, "Couldn't allocate xroute stream.");
     }
 }
 
