@@ -35,6 +35,7 @@
 #include "bgpd/bgp_route.h"
 #include "bgpd/bgp_attr.h"
 #include "bgpd/bgp_advertise.h"
+#include "bgpd/bgp_vty.h"
 
 /* Global variable to access damping configuration */
 static struct bgp_damp_config damp[AFI_MAX][SAFI_MAX];
@@ -50,6 +51,12 @@ static int bgp_reuse_index(int penalty, struct bgp_damp_config *bdc)
 	unsigned int i;
 	int index;
 
+	/*
+	 * reuse_limit can't be zero, this is for Coverity
+	 * to bypass division by zero test.
+	 */
+	assert(bdc->reuse_limit);
+
 	i = (int)(((double)penalty / bdc->reuse_limit - 1.0)
 		  * bdc->scale_factor);
 
@@ -106,7 +113,7 @@ int bgp_damp_decay(time_t tdiff, int penalty, struct bgp_damp_config *bdc)
 
 /* Handler of reuse timer event.  Each route in the current reuse-list
    is evaluated.  RFC2439 Section 4.8.7.  */
-static int bgp_reuse_timer(struct thread *t)
+static void bgp_reuse_timer(struct thread *t)
 {
 	struct bgp_damp_info *bdi;
 	struct bgp_damp_info *next;
@@ -118,7 +125,7 @@ static int bgp_reuse_timer(struct thread *t)
 	thread_add_timer(bm->master, bgp_reuse_timer, bdc, DELTA_REUSE,
 			 &bdc->t_reuse);
 
-	t_now = bgp_clock();
+	t_now = monotime(NULL);
 
 	/* 1.  save a pointer to the current zeroth queue head and zero the
 	   list head entry.  */
@@ -148,17 +155,18 @@ static int bgp_reuse_timer(struct thread *t)
 		/* if (figure-of-merit < reuse).  */
 		if (bdi->penalty < bdc->reuse_limit) {
 			/* Reuse the route.  */
-			bgp_path_info_unset_flag(bdi->rn, bdi->path,
+			bgp_path_info_unset_flag(bdi->dest, bdi->path,
 						 BGP_PATH_DAMPED);
 			bdi->suppress_time = 0;
 
 			if (bdi->lastrecord == BGP_RECORD_UPDATE) {
-				bgp_path_info_unset_flag(bdi->rn, bdi->path,
+				bgp_path_info_unset_flag(bdi->dest, bdi->path,
 							 BGP_PATH_HISTORY);
 				bgp_aggregate_increment(
-					bgp, bgp_node_get_prefix(bdi->rn),
+					bgp, bgp_dest_get_prefix(bdi->dest),
 					bdi->path, bdi->afi, bdi->safi);
-				bgp_process(bgp, bdi->rn, bdi->afi, bdi->safi);
+				bgp_process(bgp, bdi->dest, bdi->afi,
+					    bdi->safi);
 			}
 
 			if (bdi->penalty <= bdc->reuse_limit / 2.0)
@@ -170,12 +178,10 @@ static int bgp_reuse_timer(struct thread *t)
 			 * 4.8.6).  */
 			bgp_reuse_list_add(bdi, bdc);
 	}
-
-	return 0;
 }
 
 /* A route becomes unreachable (RFC2439 Section 4.8.2).  */
-int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
+int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_dest *dest,
 		      afi_t afi, safi_t safi, int attr_change)
 {
 	time_t t_now;
@@ -183,7 +189,7 @@ int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
 	unsigned int last_penalty = 0;
 	struct bgp_damp_config *bdc = &damp[afi][safi];
 
-	t_now = bgp_clock();
+	t_now = monotime(NULL);
 
 	/* Processing Unreachable Messages.  */
 	if (path->extra)
@@ -200,7 +206,7 @@ int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
 		bdi = XCALLOC(MTYPE_BGP_DAMP_INFO,
 			      sizeof(struct bgp_damp_info));
 		bdi->path = path;
-		bdi->rn = rn;
+		bdi->dest = dest;
 		bdi->penalty =
 			(attr_change ? DEFAULT_PENALTY / 2 : DEFAULT_PENALTY);
 		bdi->flap = 1;
@@ -226,13 +232,13 @@ int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
 		bdi->flap++;
 	}
 
-	assert((rn == bdi->rn) && (path == bdi->path));
+	assert((dest == bdi->dest) && (path == bdi->path));
 
 	bdi->lastrecord = BGP_RECORD_WITHDRAW;
 	bdi->t_updated = t_now;
 
 	/* Make this route as historical status.  */
-	bgp_path_info_set_flag(rn, path, BGP_PATH_HISTORY);
+	bgp_path_info_set_flag(dest, path, BGP_PATH_HISTORY);
 
 	/* Remove the route from a reuse list if it is on one.  */
 	if (CHECK_FLAG(bdi->path->flags, BGP_PATH_DAMPED)) {
@@ -247,7 +253,7 @@ int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
 	/* If not suppressed before, do annonunce this withdraw and
 	   insert into reuse_list.  */
 	if (bdi->penalty >= bdc->suppress_value) {
-		bgp_path_info_set_flag(rn, path, BGP_PATH_DAMPED);
+		bgp_path_info_set_flag(dest, path, BGP_PATH_DAMPED);
 		bdi->suppress_time = t_now;
 		BGP_DAMP_LIST_DEL(bdc, bdi);
 		bgp_reuse_list_add(bdi, bdc);
@@ -256,8 +262,8 @@ int bgp_damp_withdraw(struct bgp_path_info *path, struct bgp_node *rn,
 	return BGP_DAMP_USED;
 }
 
-int bgp_damp_update(struct bgp_path_info *path, struct bgp_node *rn, afi_t afi,
-		    safi_t safi)
+int bgp_damp_update(struct bgp_path_info *path, struct bgp_dest *dest,
+		    afi_t afi, safi_t safi)
 {
 	time_t t_now;
 	struct bgp_damp_info *bdi;
@@ -267,8 +273,8 @@ int bgp_damp_update(struct bgp_path_info *path, struct bgp_node *rn, afi_t afi,
 	if (!path->extra || !((bdi = path->extra->damp_info)))
 		return BGP_DAMP_USED;
 
-	t_now = bgp_clock();
-	bgp_path_info_unset_flag(rn, path, BGP_PATH_HISTORY);
+	t_now = monotime(NULL);
+	bgp_path_info_unset_flag(dest, path, BGP_PATH_HISTORY);
 
 	bdi->lastrecord = BGP_RECORD_UPDATE;
 	bdi->penalty =
@@ -279,7 +285,7 @@ int bgp_damp_update(struct bgp_path_info *path, struct bgp_node *rn, afi_t afi,
 		status = BGP_DAMP_USED;
 	else if (CHECK_FLAG(bdi->path->flags, BGP_PATH_DAMPED)
 		 && (bdi->penalty < bdc->reuse_limit)) {
-		bgp_path_info_unset_flag(rn, path, BGP_PATH_DAMPED);
+		bgp_path_info_unset_flag(dest, path, BGP_PATH_DAMPED);
 		bgp_reuse_list_delete(bdi, bdc);
 		BGP_DAMP_LIST_ADD(bdc, bdi);
 		bdi->suppress_time = 0;
@@ -312,16 +318,17 @@ void bgp_damp_info_free(struct bgp_damp_info *bdi, int withdraw, afi_t afi,
 	else
 		BGP_DAMP_LIST_DEL(bdc, bdi);
 
-	bgp_path_info_unset_flag(bdi->rn, path,
+	bgp_path_info_unset_flag(bdi->dest, path,
 				 BGP_PATH_HISTORY | BGP_PATH_DAMPED);
 
 	if (bdi->lastrecord == BGP_RECORD_WITHDRAW && withdraw)
-		bgp_path_info_delete(bdi->rn, path);
+		bgp_path_info_delete(bdi->dest, path);
 
 	XFREE(MTYPE_BGP_DAMP_INFO, bdi);
 }
 
-static void bgp_damp_parameter_set(int hlife, int reuse, int sup, int maxsup,
+static void bgp_damp_parameter_set(time_t hlife, unsigned int reuse,
+				   unsigned int sup, time_t maxsup,
 				   struct bgp_damp_config *bdc)
 {
 	double reuse_max_ratio;
@@ -458,10 +465,8 @@ int bgp_damp_disable(struct bgp *bgp, afi_t afi, safi_t safi)
 	if (!CHECK_FLAG(bgp->af_flags[afi][safi], BGP_CONFIG_DAMPENING))
 		return 0;
 
-	/* Cancel reuse thread. */
-	if (bdc->t_reuse)
-		thread_cancel(bdc->t_reuse);
-	bdc->t_reuse = NULL;
+	/* Cancel reuse event. */
+	THREAD_OFF(bdc->t_reuse);
 
 	/* Clean BGP dampening information.  */
 	bgp_damp_info_clean(afi, safi);
@@ -568,7 +573,7 @@ void bgp_damp_info_vty(struct vty *vty, struct bgp_path_info *path, afi_t afi,
 {
 	struct bgp_damp_info *bdi;
 	time_t t_now, t_diff;
-	char timebuf[BGP_UPTIME_LEN];
+	char timebuf[BGP_UPTIME_LEN] = {};
 	int penalty;
 	struct bgp_damp_config *bdc = &damp[afi][safi];
 
@@ -580,11 +585,13 @@ void bgp_damp_info_vty(struct vty *vty, struct bgp_path_info *path, afi_t afi,
 
 	/* If dampening is not enabled or there is no dampening information,
 	   return immediately.  */
-	if (!bdc || !bdi)
+	if (!CHECK_FLAG(path->peer->bgp->af_flags[afi][safi],
+			BGP_CONFIG_DAMPENING) ||
+	    !bdi)
 		return;
 
 	/* Calculate new penalty.  */
-	t_now = bgp_clock();
+	t_now = monotime(NULL);
 	t_diff = t_now - bdi->t_updated;
 	penalty = bgp_damp_decay(t_diff, bdi->penalty, bdc);
 
@@ -634,11 +641,13 @@ const char *bgp_damp_reuse_time_vty(struct vty *vty, struct bgp_path_info *path,
 
 	/* If dampening is not enabled or there is no dampening information,
 	   return immediately.  */
-	if (!bdc || !bdi)
+	if (!CHECK_FLAG(path->peer->bgp->af_flags[afi][safi],
+			BGP_CONFIG_DAMPENING) ||
+	    !bdi)
 		return NULL;
 
 	/* Calculate new penalty.  */
-	t_now = bgp_clock();
+	t_now = monotime(NULL);
 	t_diff = t_now - bdi->t_updated;
 	penalty = bgp_damp_decay(t_diff, bdi->penalty, bdc);
 
@@ -646,9 +655,52 @@ const char *bgp_damp_reuse_time_vty(struct vty *vty, struct bgp_path_info *path,
 				  json);
 }
 
-int bgp_show_dampening_parameters(struct vty *vty, afi_t afi, safi_t safi)
+static int bgp_print_dampening_parameters(struct bgp *bgp, struct vty *vty,
+					  afi_t afi, safi_t safi, bool use_json)
+{
+	if (CHECK_FLAG(bgp->af_flags[afi][safi], BGP_CONFIG_DAMPENING)) {
+		struct bgp_damp_config *bdc = &damp[afi][safi];
+
+		if (use_json) {
+			json_object *json = json_object_new_object();
+
+			json_object_int_add(json, "halfLifeSecs",
+					    bdc->half_life);
+			json_object_int_add(json, "reusePenalty",
+					    bdc->reuse_limit);
+			json_object_int_add(json, "suppressPenalty",
+					    bdc->suppress_value);
+			json_object_int_add(json, "maxSuppressTimeSecs",
+					    bdc->max_suppress_time);
+			json_object_int_add(json, "maxSuppressPenalty",
+					    bdc->ceiling);
+
+			vty_json(vty, json);
+		} else {
+			vty_out(vty, "Half-life time: %lld min\n",
+				(long long)bdc->half_life / 60);
+			vty_out(vty, "Reuse penalty: %d\n", bdc->reuse_limit);
+			vty_out(vty, "Suppress penalty: %d\n",
+				bdc->suppress_value);
+			vty_out(vty, "Max suppress time: %lld min\n",
+				(long long)bdc->max_suppress_time / 60);
+			vty_out(vty, "Max suppress penalty: %u\n",
+				bdc->ceiling);
+			vty_out(vty, "\n");
+		}
+	} else if (!use_json)
+		vty_out(vty, "dampening not enabled for %s\n",
+			get_afi_safi_str(afi, safi, false));
+
+	return CMD_SUCCESS;
+}
+
+int bgp_show_dampening_parameters(struct vty *vty, afi_t afi, safi_t safi,
+				  uint16_t show_flags)
 {
 	struct bgp *bgp;
+	bool use_json = CHECK_FLAG(show_flags, BGP_SHOW_OPT_JSON);
+
 	bgp = bgp_get_default();
 
 	if (bgp == NULL) {
@@ -656,21 +708,39 @@ int bgp_show_dampening_parameters(struct vty *vty, afi_t afi, safi_t safi)
 		return CMD_WARNING;
 	}
 
-	if (CHECK_FLAG(bgp->af_flags[afi][safi], BGP_CONFIG_DAMPENING)) {
-		vty_out(vty, "Half-life time: %lld min\n",
-			(long long)damp[afi][safi].half_life / 60);
-		vty_out(vty, "Reuse penalty: %d\n",
-			damp[afi][safi].reuse_limit);
-		vty_out(vty, "Suppress penalty: %d\n",
-			damp[afi][safi].suppress_value);
-		vty_out(vty, "Max suppress time: %lld min\n",
-			(long long)damp[afi][safi].max_suppress_time / 60);
-		vty_out(vty, "Max suppress penalty: %u\n",
-			damp[afi][safi].ceiling);
-		vty_out(vty, "\n");
-	} else
-		vty_out(vty, "dampening not enabled for %s\n",
-			afi == AFI_IP ? "IPv4" : "IPv6");
+	if (!CHECK_FLAG(show_flags, BGP_SHOW_OPT_AFI_ALL))
+		return bgp_print_dampening_parameters(bgp, vty, afi, safi,
+						      use_json);
 
+	if (CHECK_FLAG(show_flags, BGP_SHOW_OPT_AFI_IP)
+	    || CHECK_FLAG(show_flags, BGP_SHOW_OPT_AFI_IP6)) {
+		afi = CHECK_FLAG(show_flags, BGP_SHOW_OPT_AFI_IP) ? AFI_IP
+								  : AFI_IP6;
+		FOREACH_SAFI (safi) {
+			if (strmatch(get_afi_safi_str(afi, safi, true),
+				     "Unknown"))
+				continue;
+
+			if (!use_json)
+				vty_out(vty, "\nFor address family: %s\n\n",
+					get_afi_safi_str(afi, safi, false));
+
+			bgp_print_dampening_parameters(bgp, vty, afi, safi,
+						       use_json);
+		}
+	} else {
+		FOREACH_AFI_SAFI (afi, safi) {
+			if (strmatch(get_afi_safi_str(afi, safi, true),
+				     "Unknown"))
+				continue;
+
+			if (!use_json)
+				vty_out(vty, "\nFor address family: %s\n",
+					get_afi_safi_str(afi, safi, false));
+
+			bgp_print_dampening_parameters(bgp, vty, afi, safi,
+						       use_json);
+		}
+	}
 	return CMD_SUCCESS;
 }
