@@ -33,7 +33,8 @@
 #include "bgpd/bgp_debug.h"
 #include "bgpd/bgp_errors.h"
 
-static int bgp_fs_nlri_validate(uint8_t *nlri_content, uint32_t len)
+static int bgp_fs_nlri_validate(uint8_t *nlri_content, uint32_t len,
+				afi_t afi)
 {
 	uint32_t offset = 0;
 	int type;
@@ -48,7 +49,15 @@ static int bgp_fs_nlri_validate(uint8_t *nlri_content, uint32_t len)
 			ret = bgp_flowspec_ip_address(
 						BGP_FLOWSPEC_VALIDATE_ONLY,
 						nlri_content + offset,
-						len - offset, NULL, &error);
+						len - offset, NULL, &error,
+						afi, NULL);
+			break;
+		case FLOWSPEC_FLOW_LABEL:
+			if (afi == AFI_IP)
+				return -1;
+			ret = bgp_flowspec_op_decode(BGP_FLOWSPEC_VALIDATE_ONLY,
+						   nlri_content + offset,
+						   len - offset, NULL, &error);
 			break;
 		case FLOWSPEC_IP_PROTOCOL:
 		case FLOWSPEC_PORT:
@@ -86,7 +95,7 @@ static int bgp_fs_nlri_validate(uint8_t *nlri_content, uint32_t len)
 }
 
 int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
-			    struct bgp_nlri *packet, int withdraw)
+			    struct bgp_nlri *packet, bool withdraw)
 {
 	uint8_t *pnt;
 	uint8_t *lim;
@@ -94,7 +103,6 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 	safi_t safi;
 	int psize = 0;
 	struct prefix p;
-	int ret;
 	void *temp;
 
 	/* Start processing the NLRI - there may be multiple in the MP_REACH */
@@ -103,10 +111,12 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 	afi = packet->afi;
 	safi = packet->safi;
 
-	if (afi == AFI_IP6) {
-		flog_err(EC_LIB_DEVELOPMENT, "BGP flowspec IPv6 not supported");
-		return BGP_NLRI_PARSE_ERROR_FLOWSPEC_IPV6_NOT_SUPPORTED;
-	}
+	/*
+	 * All other AFI/SAFI's treat no attribute as a implicit
+	 * withdraw.  Flowspec should as well.
+	 */
+	if (!attr)
+		withdraw = true;
 
 	if (packet->length >= FLOWSPEC_NLRI_SIZELIMIT_EXTENDED) {
 		flog_err(EC_BGP_FLOWSPEC_PACKET,
@@ -117,7 +127,7 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 
 	for (; pnt < lim; pnt += psize) {
 		/* Clear prefix structure. */
-		memset(&p, 0, sizeof(struct prefix));
+		memset(&p, 0, sizeof(p));
 
 		/* All FlowSpec NLRI begin with length. */
 		if (pnt + 1 > lim)
@@ -137,7 +147,14 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 				psize);
 			return BGP_NLRI_PARSE_ERROR_PACKET_OVERFLOW;
 		}
-		if (bgp_fs_nlri_validate(pnt, psize) < 0) {
+
+		if (psize == 0) {
+			flog_err(EC_BGP_FLOWSPEC_PACKET,
+				 "Flowspec NLRI length 0 which makes no sense");
+			return BGP_NLRI_PARSE_ERROR_PACKET_OVERFLOW;
+		}
+
+		if (bgp_fs_nlri_validate(pnt, psize, afi) < 0) {
 			flog_err(
 				EC_BGP_FLOWSPEC_PACKET,
 				"Bad flowspec format or NLRI options not supported");
@@ -147,6 +164,7 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 		p.prefixlen = 0;
 		/* Flowspec encoding is in bytes */
 		p.u.prefix_flowspec.prefixlen = psize;
+		p.u.prefix_flowspec.family = afi2family(afi);
 		temp = XCALLOC(MTYPE_TMP, psize);
 		memcpy(temp, pnt, psize);
 		p.u.prefix_flowspec.ptr = (uintptr_t) temp;
@@ -161,12 +179,14 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 					       p.u.prefix_flowspec.ptr,
 					       p.u.prefix_flowspec.prefixlen,
 					       return_string,
-					       NLRI_STRING_FORMAT_MIN, NULL);
+					       NLRI_STRING_FORMAT_MIN, NULL,
+					       afi);
 			snprintf(ec_string, sizeof(ec_string),
 				 "EC{none}");
-			if (attr && attr->ecommunity) {
-				s = ecommunity_ecom2str(attr->ecommunity,
-						ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
+			if (attr && bgp_attr_get_ecommunity(attr)) {
+				s = ecommunity_ecom2str(
+					bgp_attr_get_ecommunity(attr),
+					ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
 				snprintf(ec_string, sizeof(ec_string),
 					 "EC{%s}",
 					s == NULL ? "none" : s);
@@ -183,21 +203,13 @@ int bgp_nlri_parse_flowspec(struct peer *peer, struct attr *attr,
 		}
 		/* Process the route. */
 		if (!withdraw)
-			ret = bgp_update(peer, &p, 0, attr,
-					 afi, safi,
-					 ZEBRA_ROUTE_BGP, BGP_ROUTE_NORMAL,
-					 NULL, NULL, 0, 0, NULL);
+			bgp_update(peer, &p, 0, attr, afi, safi,
+				   ZEBRA_ROUTE_BGP, BGP_ROUTE_NORMAL, NULL,
+				   NULL, 0, 0, NULL);
 		else
-			ret = bgp_withdraw(peer, &p, 0, attr,
-					   afi, safi,
-					   ZEBRA_ROUTE_BGP, BGP_ROUTE_NORMAL,
-					   NULL, NULL, 0, NULL);
-		if (ret) {
-			flog_err(EC_BGP_FLOWSPEC_INSTALLATION,
-				 "Flowspec NLRI failed to be %s.",
-				 attr ? "added" : "withdrawn");
-			return BGP_NLRI_PARSE_ERROR;
-		}
+			bgp_withdraw(peer, &p, 0, attr, afi, safi,
+				     ZEBRA_ROUTE_BGP, BGP_ROUTE_NORMAL, NULL,
+				     NULL, 0, NULL);
 	}
 	return BGP_NLRI_PARSE_OK;
 }
