@@ -46,7 +46,7 @@
  * Record maximum-paths configuration for BGP instance
  */
 int bgp_maximum_paths_set(struct bgp *bgp, afi_t afi, safi_t safi, int peertype,
-			  uint16_t maxpaths, uint16_t options)
+			  uint16_t maxpaths, bool same_clusterlen)
 {
 	if (!bgp || (afi >= AFI_MAX) || (safi >= SAFI_MAX))
 		return -1;
@@ -54,7 +54,7 @@ int bgp_maximum_paths_set(struct bgp *bgp, afi_t afi, safi_t safi, int peertype,
 	switch (peertype) {
 	case BGP_PEER_IBGP:
 		bgp->maxpaths[afi][safi].maxpaths_ibgp = maxpaths;
-		bgp->maxpaths[afi][safi].ibgp_flags |= options;
+		bgp->maxpaths[afi][safi].same_clusterlen = same_clusterlen;
 		break;
 	case BGP_PEER_EBGP:
 		bgp->maxpaths[afi][safi].maxpaths_ebgp = maxpaths;
@@ -80,7 +80,7 @@ int bgp_maximum_paths_unset(struct bgp *bgp, afi_t afi, safi_t safi,
 	switch (peertype) {
 	case BGP_PEER_IBGP:
 		bgp->maxpaths[afi][safi].maxpaths_ibgp = multipath_num;
-		bgp->maxpaths[afi][safi].ibgp_flags = 0;
+		bgp->maxpaths[afi][safi].same_clusterlen = false;
 		break;
 	case BGP_PEER_EBGP:
 		bgp->maxpaths[afi][safi].maxpaths_ebgp = multipath_num;
@@ -181,6 +181,20 @@ int bgp_path_info_nexthop_cmp(struct bgp_path_info *bpi1,
 		}
 	}
 
+	/*
+	 * If both nexthops are same then check
+	 * if they belong to same VRF
+	 */
+	if (!compare && bpi1->attr->nh_type != NEXTHOP_TYPE_BLACKHOLE) {
+		if (bpi1->extra && bpi1->extra->bgp_orig && bpi2->extra
+		    && bpi2->extra->bgp_orig) {
+			if (bpi1->extra->bgp_orig->vrf_id
+			    != bpi2->extra->bgp_orig->vrf_id) {
+				compare = 1;
+			}
+		}
+	}
+
 	return compare;
 }
 
@@ -264,8 +278,10 @@ void bgp_mp_list_add(struct list *mp_list, struct bgp_path_info *mpinfo)
 static struct bgp_path_info_mpath *bgp_path_info_mpath_new(void)
 {
 	struct bgp_path_info_mpath *new_mpath;
+
 	new_mpath = XCALLOC(MTYPE_BGP_MPATH_INFO,
 			    sizeof(struct bgp_path_info_mpath));
+
 	return new_mpath;
 }
 
@@ -299,8 +315,6 @@ bgp_path_info_mpath_get(struct bgp_path_info *path)
 
 	if (!path->mpath) {
 		mpath = bgp_path_info_mpath_new();
-		if (!mpath)
-			return NULL;
 		path->mpath = mpath;
 		mpath->mp_info = path;
 	}
@@ -411,9 +425,11 @@ static void bgp_path_info_mpath_lb_update(struct bgp_path_info *path, bool set,
 {
 	struct bgp_path_info_mpath *mpath;
 
-	if ((mpath = path->mpath) == NULL) {
-		if (!set)
+	mpath = path->mpath;
+	if (mpath == NULL) {
+		if (!set || (cum_bw == 0 && !all_paths_lb))
 			return;
+
 		mpath = bgp_path_info_mpath_get(path);
 		if (!mpath)
 			return;
@@ -506,7 +522,7 @@ static void bgp_path_info_mpath_attr_set(struct bgp_path_info *path,
  * Compare and sync up the multipath list with the mp_list generated by
  * bgp_best_selection
  */
-void bgp_path_info_mpath_update(struct bgp_node *rn,
+void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 				struct bgp_path_info *new_best,
 				struct bgp_path_info *old_best,
 				struct list *mp_list,
@@ -518,7 +534,6 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 	struct listnode *mp_node, *mp_next_node;
 	struct bgp_path_info *cur_mpath, *new_mpath, *next_mpath, *prev_mpath;
 	int mpath_changed, debug;
-	char nh_buf[2][INET6_ADDRSTRLEN];
 	bool all_paths_lb;
 	char path_buf[PATH_ADDPATH_STR_BUFFER];
 
@@ -530,7 +545,7 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 	old_cum_bw = cum_bw = 0;
 	prev_mpath = new_best;
 	mp_node = listhead(mp_list);
-	debug = bgp_debug_bestpath(rn);
+	debug = bgp_debug_bestpath(dest);
 
 	if (new_best) {
 		mpath_count++;
@@ -552,10 +567,11 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 
 	if (debug)
 		zlog_debug(
-			"%pRN: starting mpath update, newbest %s num candidates %d old-mpath-count %d old-cum-bw u%" PRIu64,
-			rn, new_best ? new_best->peer->host : "NONE",
-			mp_list ? listcount(mp_list) : 0,
-			old_mpath_count, old_cum_bw);
+			"%pRN(%s): starting mpath update, newbest %s num candidates %d old-mpath-count %d old-cum-bw %" PRIu64,
+			bgp_dest_to_rnode(dest), bgp->name_pretty,
+			new_best ? new_best->peer->host : "NONE",
+			mp_list ? listcount(mp_list) : 0, old_mpath_count,
+			old_cum_bw);
 
 	/*
 	 * We perform an ordered walk through both lists in parallel.
@@ -587,8 +603,8 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 
 		if (debug)
 			zlog_debug(
-				"%pRN: comparing candidate %s with existing mpath %s",
-				rn,
+				"%pRN(%s): comparing candidate %s with existing mpath %s",
+				bgp_dest_to_rnode(dest), bgp->name_pretty,
 				tmp_info ? tmp_info->peer->host : "NONE",
 				cur_mpath ? cur_mpath->peer->host : "NONE");
 
@@ -608,30 +624,32 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 				prev_mpath = cur_mpath;
 				mpath_count++;
 				if (ecommunity_linkbw_present(
-					cur_mpath->attr->ecommunity, &bwval))
+					    bgp_attr_get_ecommunity(
+						    cur_mpath->attr),
+					    &bwval))
 					cum_bw += bwval;
 				else
 					all_paths_lb = false;
 				if (debug) {
 					bgp_path_info_path_with_addpath_rx_str(
-						cur_mpath, path_buf);
+						cur_mpath, path_buf,
+						sizeof(path_buf));
 					zlog_debug(
 						"%pRN: %s is still multipath, cur count %d",
-						rn, path_buf, mpath_count);
+						bgp_dest_to_rnode(dest),
+						path_buf, mpath_count);
 				}
 			} else {
 				mpath_changed = 1;
 				if (debug) {
 					bgp_path_info_path_with_addpath_rx_str(
-						cur_mpath, path_buf);
+						cur_mpath, path_buf,
+						sizeof(path_buf));
 					zlog_debug(
-						"%pRN: remove mpath %s nexthop %s, cur count %d",
-						rn, path_buf,
-						inet_ntop(AF_INET,
-							  &cur_mpath->attr
-								   ->nexthop,
-							  nh_buf[0],
-							  sizeof(nh_buf[0])),
+						"%pRN: remove mpath %s nexthop %pI4, cur count %d",
+						bgp_dest_to_rnode(dest),
+						path_buf,
+						&cur_mpath->attr->nexthop,
 						mpath_count);
 				}
 			}
@@ -656,14 +674,11 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 			mpath_changed = 1;
 			if (debug) {
 				bgp_path_info_path_with_addpath_rx_str(
-					cur_mpath, path_buf);
+					cur_mpath, path_buf, sizeof(path_buf));
 				zlog_debug(
-					"%pRN: remove mpath %s nexthop %s, cur count %d",
-					rn, path_buf,
-					inet_ntop(AF_INET,
-						  &cur_mpath->attr->nexthop,
-						  nh_buf[0], sizeof(nh_buf[0])),
-					mpath_count);
+					"%pRN: remove mpath %s nexthop %pI4, cur count %d",
+					bgp_dest_to_rnode(dest), path_buf,
+					&cur_mpath->attr->nexthop, mpath_count);
 			}
 			cur_mpath = next_mpath;
 		} else {
@@ -700,21 +715,21 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 				mpath_changed = 1;
 				mpath_count++;
 				if (ecommunity_linkbw_present(
-					new_mpath->attr->ecommunity, &bwval))
+					    bgp_attr_get_ecommunity(
+						    new_mpath->attr),
+					    &bwval))
 					cum_bw += bwval;
 				else
 					all_paths_lb = false;
 				if (debug) {
 					bgp_path_info_path_with_addpath_rx_str(
-						new_mpath, path_buf);
+						new_mpath, path_buf,
+						sizeof(path_buf));
 					zlog_debug(
-						"%pRN: add mpath %s nexthop %s, cur count %d",
-						rn, path_buf,
-						inet_ntop(AF_INET,
-							  &new_mpath->attr
-								   ->nexthop,
-							  nh_buf[0],
-							  sizeof(nh_buf[0])),
+						"%pRN: add mpath %s nexthop %pI4, cur count %d",
+						bgp_dest_to_rnode(dest),
+						path_buf,
+						&new_mpath->attr->nexthop,
 						mpath_count);
 				}
 			}
@@ -726,7 +741,7 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 		bgp_path_info_mpath_count_set(new_best, mpath_count - 1);
 		if (mpath_count <= 1 ||
 		    !ecommunity_linkbw_present(
-			new_best->attr->ecommunity, &bwval))
+			    bgp_attr_get_ecommunity(new_best->attr), &bwval))
 			all_paths_lb = false;
 		else
 			cum_bw += bwval;
@@ -735,10 +750,9 @@ void bgp_path_info_mpath_update(struct bgp_node *rn,
 
 		if (debug)
 			zlog_debug(
-				"%pRN: New mpath count (incl newbest) %d mpath-change %s"
-				" all_paths_lb %d cum_bw u%" PRIu64,
-				rn, mpath_count,
-				mpath_changed ? "YES" : "NO",
+				"%pRN(%s): New mpath count (incl newbest) %d mpath-change %s all_paths_lb %d cum_bw %" PRIu64,
+				bgp_dest_to_rnode(dest), bgp->name_pretty,
+				mpath_count, mpath_changed ? "YES" : "NO",
 				all_paths_lb, cum_bw);
 
 		if (mpath_changed
@@ -828,11 +842,15 @@ void bgp_path_info_mpath_aggregate_update(struct bgp_path_info *new_best,
 		aspath = aspath_dup(attr.aspath);
 		origin = attr.origin;
 		community =
-			attr.community ? community_dup(attr.community) : NULL;
-		ecomm = (attr.ecommunity) ? ecommunity_dup(attr.ecommunity)
-					  : NULL;
-		lcomm = (attr.lcommunity) ? lcommunity_dup(attr.lcommunity)
-					  : NULL;
+			bgp_attr_get_community(&attr)
+				? community_dup(bgp_attr_get_community(&attr))
+				: NULL;
+		ecomm = (bgp_attr_get_ecommunity(&attr))
+				? ecommunity_dup(bgp_attr_get_ecommunity(&attr))
+				: NULL;
+		lcomm = (bgp_attr_get_lcommunity(&attr))
+				? lcommunity_dup(bgp_attr_get_lcommunity(&attr))
+				: NULL;
 
 		for (mpinfo = bgp_path_info_mpath_first(new_best); mpinfo;
 		     mpinfo = bgp_path_info_mpath_next(mpinfo)) {
@@ -844,57 +862,55 @@ void bgp_path_info_mpath_aggregate_update(struct bgp_path_info *new_best,
 			if (origin < mpinfo->attr->origin)
 				origin = mpinfo->attr->origin;
 
-			if (mpinfo->attr->community) {
+			if (bgp_attr_get_community(mpinfo->attr)) {
 				if (community) {
 					commerge = community_merge(
 						community,
-						mpinfo->attr->community);
+						bgp_attr_get_community(
+							mpinfo->attr));
 					community =
 						community_uniq_sort(commerge);
 					community_free(&commerge);
 				} else
 					community = community_dup(
-						mpinfo->attr->community);
+						bgp_attr_get_community(
+							mpinfo->attr));
 			}
 
-			if (mpinfo->attr->ecommunity) {
+			if (bgp_attr_get_ecommunity(mpinfo->attr)) {
 				if (ecomm) {
 					ecommerge = ecommunity_merge(
-						ecomm,
-						mpinfo->attr->ecommunity);
+						ecomm, bgp_attr_get_ecommunity(
+							       mpinfo->attr));
 					ecomm = ecommunity_uniq_sort(ecommerge);
 					ecommunity_free(&ecommerge);
 				} else
 					ecomm = ecommunity_dup(
-						mpinfo->attr->ecommunity);
+						bgp_attr_get_ecommunity(
+							mpinfo->attr));
 			}
-			if (mpinfo->attr->lcommunity) {
+			if (bgp_attr_get_lcommunity(mpinfo->attr)) {
 				if (lcomm) {
 					lcommerge = lcommunity_merge(
-						lcomm,
-						mpinfo->attr->lcommunity);
+						lcomm, bgp_attr_get_lcommunity(
+							       mpinfo->attr));
 					lcomm = lcommunity_uniq_sort(lcommerge);
 					lcommunity_free(&lcommerge);
 				} else
 					lcomm = lcommunity_dup(
-						mpinfo->attr->lcommunity);
+						bgp_attr_get_lcommunity(
+							mpinfo->attr));
 			}
 		}
 
 		attr.aspath = aspath;
 		attr.origin = origin;
-		if (community) {
-			attr.community = community;
-			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_COMMUNITIES);
-		}
-		if (ecomm) {
-			attr.ecommunity = ecomm;
-			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES);
-		}
-		if (lcomm) {
-			attr.lcommunity = lcomm;
-			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_LARGE_COMMUNITIES);
-		}
+		if (community)
+			bgp_attr_set_community(&attr, community);
+		if (ecomm)
+			bgp_attr_set_ecommunity(&attr, ecomm);
+		if (lcomm)
+			bgp_attr_set_lcommunity(&attr, lcomm);
 
 		/* Zap multipath attr nexthop so we set nexthop to self */
 		attr.nexthop.s_addr = INADDR_ANY;
