@@ -53,6 +53,7 @@
 #define DEBUG_PENDING_DELETE_ROUTE	0
 #define DEBUG_NHL			0
 #define DEBUG_RIB_SL_RD                 0
+#define DEBUG_CLEANUP 0
 
 /* forward decl */
 #if DEBUG_NHL
@@ -268,10 +269,9 @@ static void rfapi_info_free(struct rfapi_info *goner)
 		if (goner->timer) {
 			struct rfapi_rib_tcb *tcb;
 
-			tcb = goner->timer->arg;
-			thread_cancel(goner->timer);
+			tcb = THREAD_ARG(goner->timer);
+			THREAD_OFF(goner->timer);
 			XFREE(MTYPE_RFAPI_RECENT_DELETE, tcb);
-			goner->timer = NULL;
 		}
 		XFREE(MTYPE_RFAPI_INFO, goner);
 	}
@@ -292,9 +292,9 @@ struct rfapi_rib_tcb {
 /*
  * remove route from rib
  */
-static int rfapiRibExpireTimer(struct thread *t)
+static void rfapiRibExpireTimer(struct thread *t)
 {
-	struct rfapi_rib_tcb *tcb = t->arg;
+	struct rfapi_rib_tcb *tcb = THREAD_ARG(t);
 
 	RFAPI_RIB_CHECK_COUNTS(1, 0);
 
@@ -329,8 +329,6 @@ static int rfapiRibExpireTimer(struct thread *t)
 	XFREE(MTYPE_RFAPI_RECENT_DELETE, tcb);
 
 	RFAPI_RIB_CHECK_COUNTS(1, 0);
-
-	return 0;
 }
 
 static void rfapiRibStartTimer(struct rfapi_descriptor *rfd,
@@ -338,17 +336,20 @@ static void rfapiRibStartTimer(struct rfapi_descriptor *rfd,
 			       struct agg_node *rn, /* route node attached to */
 			       int deleted)
 {
-	struct thread *t = ri->timer;
 	struct rfapi_rib_tcb *tcb = NULL;
 
-	if (t) {
-		tcb = t->arg;
-		thread_cancel(t);
-		ri->timer = NULL;
+	if (ri->timer) {
+		tcb = THREAD_ARG(ri->timer);
+		THREAD_OFF(ri->timer);
 	} else {
 		tcb = XCALLOC(MTYPE_RFAPI_RECENT_DELETE,
 			      sizeof(struct rfapi_rib_tcb));
 	}
+#if DEBUG_CLEANUP
+	zlog_debug("%s: rfd %p, rn %p, ri %p, tcb %p", __func__, rfd, rn, ri,
+		   tcb);
+#endif
+
 	tcb->rfd = rfd;
 	tcb->ri = ri;
 	tcb->rn = rn;
@@ -362,10 +363,9 @@ static void rfapiRibStartTimer(struct rfapi_descriptor *rfd,
 
 	vnc_zlog_debug_verbose("%s: rfd %p pfx %pRN life %u", __func__, rfd, rn,
 			       ri->lifetime);
-	ri->timer = NULL;
+
 	thread_add_timer(bm->master, rfapiRibExpireTimer, tcb, ri->lifetime,
 			 &ri->timer);
-	assert(ri->timer);
 }
 
 extern void rfapi_rib_key_init(struct prefix *prefix, /* may be NULL */
@@ -529,6 +529,16 @@ void rfapiRibClear(struct rfapi_descriptor *rfd)
 							    NULL,
 							    (void **)&ri)) {
 
+						if (ri->timer) {
+							struct rfapi_rib_tcb
+								*tcb;
+
+							tcb = THREAD_ARG(
+								ri->timer);
+							THREAD_OFF(ri->timer);
+							XFREE(MTYPE_RFAPI_RECENT_DELETE,
+							      tcb);
+						}
 						rfapi_info_free(ri);
 						skiplist_delete_first(
 							(struct skiplist *)
@@ -578,6 +588,9 @@ void rfapiRibFree(struct rfapi_descriptor *rfd)
 {
 	afi_t afi;
 
+#if DEBUG_CLEANUP
+	zlog_debug("%s: rfd %p", __func__, rfd);
+#endif
 
 	/*
 	 * NB rfd is typically detached from master list, so is not included
@@ -625,7 +638,8 @@ static void rfapiRibBi2Ri(struct bgp_path_info *bpi, struct rfapi_info *ri,
 	ri->lifetime = lifetime;
 
 	/* This loop based on rfapiRouteInfo2NextHopEntry() */
-	for (pEncap = bpi->attr->vnc_subtlvs; pEncap; pEncap = pEncap->next) {
+	for (pEncap = bgp_attr_get_vnc_subtlvs(bpi->attr); pEncap;
+	     pEncap = pEncap->next) {
 		struct bgp_tea_options *hop;
 
 		switch (pEncap->type) {
@@ -646,9 +660,7 @@ static void rfapiRibBi2Ri(struct bgp_path_info *bpi, struct rfapi_info *ri,
 			       pEncap->length - 2);
 			if (hop->length > pEncap->length - 2) {
 				zlog_warn(
-					"%s: VNC subtlv length mismatch: "
-					"RFP option says %d, attr says %d "
-					"(shrinking)",
+					"%s: VNC subtlv length mismatch: RFP option says %d, attr says %d (shrinking)",
 					__func__, hop->length,
 					pEncap->length - 2);
 				hop->length = pEncap->length - 2;
@@ -686,10 +698,11 @@ static void rfapiRibBi2Ri(struct bgp_path_info *bpi, struct rfapi_info *ri,
 		memcpy(&vo->v.l2addr.macaddr, bpi->extra->vnc.import.rd.val + 2,
 		       ETH_ALEN);
 
-		(void)rfapiEcommunityGetLNI(bpi->attr->ecommunity,
+		(void)rfapiEcommunityGetLNI(bgp_attr_get_ecommunity(bpi->attr),
 					    &vo->v.l2addr.logical_net_id);
-		(void)rfapiEcommunityGetEthernetTag(bpi->attr->ecommunity,
-						    &vo->v.l2addr.tag_id);
+		(void)rfapiEcommunityGetEthernetTag(
+			bgp_attr_get_ecommunity(bpi->attr),
+			&vo->v.l2addr.tag_id);
 
 		/* local_nve_id comes from RD */
 		vo->v.l2addr.local_nve_id = bpi->extra->vnc.import.rd.val[1];
@@ -790,7 +803,7 @@ int rfapiRibPreloadBi(
 		skiplist_insert(slRibPt, &ori->rk, ori);
 	}
 
-	ori->last_sent_time = rfapi_time(NULL);
+	ori->last_sent_time = monotime(NULL);
 
 	/*
 	 * poke timer
@@ -803,8 +816,8 @@ int rfapiRibPreloadBi(
 	 * Update last sent time for prefix
 	 */
 	trn = agg_node_get(rfd->rsp_times[afi], p); /* locks trn */
-	trn->info = (void *)(uintptr_t)bgp_clock();
-	if (trn->lock > 1)
+	trn->info = (void *)(uintptr_t)monotime(NULL);
+	if (agg_node_get_lock_count(trn) > 1)
 		agg_unlock_node(trn);
 
 	return 0;
@@ -909,10 +922,6 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 			delete_list = list_new();
 			while (0
 			       == skiplist_first(slRibPt, NULL, (void **)&ri)) {
-
-				char buf[PREFIX_STRLEN];
-				char buf2[PREFIX_STRLEN];
-
 				listnode_add(delete_list, ri);
 				vnc_zlog_debug_verbose(
 					"%s: after listnode_add, delete_list->count=%d",
@@ -923,18 +932,15 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 				if (ri->timer) {
 					struct rfapi_rib_tcb *tcb;
 
-					tcb = ((struct thread *)ri->timer)->arg;
-					thread_cancel(ri->timer);
+					tcb = THREAD_ARG(ri->timer);
+					THREAD_OFF(ri->timer);
 					XFREE(MTYPE_RFAPI_RECENT_DELETE, tcb);
-					ri->timer = NULL;
 				}
 
-				prefix2str(&ri->rk.vn, buf, sizeof(buf));
-				prefix2str(&ri->un, buf2, sizeof(buf2));
 				vnc_zlog_debug_verbose(
-					"%s:   put dl pfx=%pRN vn=%s un=%s cost=%d life=%d vn_options=%p",
-					__func__, pn, buf, buf2, ri->cost,
-					ri->lifetime, ri->vn_options);
+					"%s:   put dl pfx=%pRN vn=%pFX un=%pFX cost=%d life=%d vn_options=%p",
+					__func__, pn, &ri->rk.vn, &ri->un,
+					ri->cost, ri->lifetime, ri->vn_options);
 
 				skiplist_delete_first(slRibPt);
 			}
@@ -1011,11 +1017,9 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 				if (ori->timer) {
 					struct rfapi_rib_tcb *tcb;
 
-					tcb = ((struct thread *)ori->timer)
-						      ->arg;
-					thread_cancel(ori->timer);
+					tcb = THREAD_ARG(ori->timer);
+					THREAD_OFF(ori->timer);
 					XFREE(MTYPE_RFAPI_RECENT_DELETE, tcb);
-					ori->timer = NULL;
 				}
 
 #if DEBUG_PROCESS_PENDING_NODE
@@ -1104,7 +1108,7 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 				rfapiFreeBgpTeaOptionChain(ori->tea_options);
 				ori->tea_options =
 					rfapiOptionsDup(ri->tea_options);
-				ori->last_sent_time = rfapi_time(NULL);
+				ori->last_sent_time = monotime(NULL);
 
 				rfapiFreeRfapiVnOptionChain(ori->vn_options);
 				ori->vn_options =
@@ -1119,9 +1123,6 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 					__func__, ri);
 
 			} else {
-
-				char buf_rd[RD_ADDRSTRLEN];
-
 				/* not found: add new route to RIB */
 				ori = rfapi_info_new();
 				ori->rk = ri->rk;
@@ -1130,7 +1131,7 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 				ori->lifetime = ri->lifetime;
 				ori->tea_options =
 					rfapiOptionsDup(ri->tea_options);
-				ori->last_sent_time = rfapi_time(NULL);
+				ori->last_sent_time = monotime(NULL);
 				ori->vn_options =
 					rfapiVnOptionsDup(ri->vn_options);
 				ori->un_options =
@@ -1144,16 +1145,9 @@ static void process_pending_node(struct bgp *bgp, struct rfapi_descriptor *rfd,
 				}
 				skiplist_insert(slRibPt, &ori->rk, ori);
 
-#if DEBUG_RIB_SL_RD
-				prefix_rd2str(&ori->rk.rd, buf_rd,
-					      sizeof(buf_rd));
-#else
-				buf_rd[0] = 0;
-#endif
-
 				vnc_zlog_debug_verbose(
-					"%s:   nomatch lPendCost item %p in slRibPt, added (rd=%s)",
-					__func__, ri, buf_rd);
+					"%s:   nomatch lPendCost item %p in slRibPt, added (rd=%pRD)",
+					__func__, ri, &ori->rk.rd);
 			}
 
 			/*
@@ -1193,7 +1187,6 @@ callback:
 
 			new = XCALLOC(MTYPE_RFAPI_NEXTHOP,
 				      sizeof(struct rfapi_next_hop_entry));
-			assert(new);
 
 			if (ri->rk.aux_prefix.family) {
 				rfapiQprefix2Rprefix(&ri->rk.aux_prefix,
@@ -1243,8 +1236,8 @@ callback:
 			 */
 			trn = agg_node_get(rfd->rsp_times[afi],
 					   p); /* locks trn */
-			trn->info = (void *)(uintptr_t)bgp_clock();
-			if (trn->lock > 1)
+			trn->info = (void *)(uintptr_t)monotime(NULL);
+			if (agg_node_get_lock_count(trn) > 1)
 				agg_unlock_node(trn);
 
 			rfapiRfapiIpAddr2Str(&new->vn_address, buf, BUFSIZ);
@@ -1283,7 +1276,6 @@ callback:
 				new = XCALLOC(
 					MTYPE_RFAPI_NEXTHOP,
 					sizeof(struct rfapi_next_hop_entry));
-				assert(new);
 
 				if (ri->rk.aux_prefix.family) {
 					rfapiQprefix2Rprefix(&ri->rk.aux_prefix,
@@ -1359,11 +1351,9 @@ callback:
 				if (ri->timer) {
 					struct rfapi_rib_tcb *tcb;
 
-					tcb = ((struct thread *)ri->timer)->arg;
-					thread_cancel(
-						(struct thread *)ri->timer);
+					tcb = THREAD_ARG(ri->timer);
+					THREAD_OFF(ri->timer);
 					XFREE(MTYPE_RFAPI_RECENT_DELETE, tcb);
-					ri->timer = NULL;
 				}
 				RFAPI_RIB_CHECK_COUNTS(0, delete_list->count);
 
@@ -1395,19 +1385,11 @@ callback:
 					rfapiRibStartTimer(rfd, ri, rn, 1);
 					RFAPI_RIB_CHECK_COUNTS(
 						0, delete_list->count);
-					ri->last_sent_time = rfapi_time(NULL);
+					ri->last_sent_time = monotime(NULL);
 #if DEBUG_RIB_SL_RD
-					{
-						char buf_rd[RD_ADDRSTRLEN];
-
-						vnc_zlog_debug_verbose(
-							"%s: move route to recently deleted list, rd=%s",
-							__func__,
-							prefix_rd2str(
-								&ri->rk.rd,
-								buf_rd,
-								sizeof(buf_rd)));
-					}
+					vnc_zlog_debug_verbose(
+						"%s: move route to recently deleted list, rd=%pRD",
+						__func__, &ri->rk.rd);
 #endif
 
 				} else {
@@ -1419,7 +1401,7 @@ callback:
 					rfapiRibStartTimer(rfd, ri_del, rn, 1);
 					RFAPI_RIB_CHECK_COUNTS(
 						0, delete_list->count);
-					ri->last_sent_time = rfapi_time(NULL);
+					ri->last_sent_time = monotime(NULL);
 				}
 			}
 		} else {
@@ -1591,7 +1573,6 @@ void rfapiRibUpdatePendingNode(
 	afi_t afi;
 	uint32_t queued_flag;
 	int count = 0;
-	char buf[PREFIX_STRLEN];
 
 	vnc_zlog_debug_verbose("%s: entry", __func__);
 
@@ -1604,8 +1585,7 @@ void rfapiRibUpdatePendingNode(
 
 	prefix = agg_node_get_prefix(it_node);
 	afi = family2afi(prefix->family);
-	prefix2str(prefix, buf, sizeof(buf));
-	vnc_zlog_debug_verbose("%s: prefix=%s", __func__, buf);
+	vnc_zlog_debug_verbose("%s: prefix=%pFX", __func__, prefix);
 
 	pn = agg_node_get(rfd->rib_pending[afi], prefix);
 	assert(pn);
@@ -1736,7 +1716,6 @@ void rfapiRibUpdatePendingNode(
 
 		urq = XCALLOC(MTYPE_RFAPI_UPDATED_RESPONSE_QUEUE,
 			      sizeof(struct rfapi_updated_responses_queue));
-		assert(urq);
 		if (!rfd->updated_responses_queue)
 			updated_responses_queue_init(rfd);
 
@@ -1811,11 +1790,8 @@ int rfapiRibFTDFilterRecentPrefix(
 
 #ifdef DEBUG_FTD_FILTER_RECENT
 	{
-		char buf_pfx[PREFIX_STRLEN];
-
-		prefix2str(agg_node_get_prefix(it_rn), buf_pfx,
-			   sizeof(buf_pfx));
-		vnc_zlog_debug_verbose("%s: prefix %s", __func__, buf_pfx);
+		vnc_zlog_debug_verbose("%s: prefix %pFX", __func__,
+				       agg_node_get_prefix(it_rn));
 	}
 #endif
 
@@ -1835,7 +1811,7 @@ int rfapiRibFTDFilterRecentPrefix(
 	 */
 	trn = agg_node_get(rfd->rsp_times[afi], p); /* locks trn */
 	prefix_time = (time_t)trn->info;
-	if (trn->lock > 1)
+	if (agg_node_get_lock_count(trn) > 1)
 		agg_unlock_node(trn);
 
 #ifdef DEBUG_FTD_FILTER_RECENT
@@ -1874,7 +1850,7 @@ rfapiRibPreload(struct bgp *bgp, struct rfapi_descriptor *rfd,
 	vnc_zlog_debug_verbose("%s: loading response=%p, use_eth_resolution=%d",
 			       __func__, response, use_eth_resolution);
 
-	new_last_sent_time = rfapi_time(NULL);
+	new_last_sent_time = monotime(NULL);
 
 	for (nhp = response; nhp; nhp = nhp_next) {
 
@@ -1970,27 +1946,24 @@ rfapiRibPreload(struct bgp *bgp, struct rfapi_descriptor *rfd,
 			    && RFAPI_HOST_PREFIX(&rk.aux_prefix)) {
 				/* mark as "none" if nhp->prefix is 0/32 or
 				 * 0/128 */
-				rk.aux_prefix.family = 0;
+				rk.aux_prefix.family = AF_UNSPEC;
 			}
 		}
 
 #if DEBUG_NHL
 		{
-			char str_vn[PREFIX_STRLEN];
 			char str_aux_prefix[PREFIX_STRLEN];
 
-			str_vn[0] = 0;
 			str_aux_prefix[0] = 0;
 
-			prefix2str(&rk.vn, str_vn, sizeof(str_vn));
 			prefix2str(&rk.aux_prefix, str_aux_prefix,
 				   sizeof(str_aux_prefix));
 
 			if (!rk.aux_prefix.family) {
 			}
 			vnc_zlog_debug_verbose(
-				"%s:   rk.vn=%s rk.aux_prefix=%s", __func__,
-				str_vn,
+				"%s:   rk.vn=%pFX rk.aux_prefix=%s", __func__,
+				&rk.vn,
 				(rk.aux_prefix.family ? str_aux_prefix : "-"));
 		}
 		vnc_zlog_debug_verbose(
@@ -2047,7 +2020,7 @@ rfapiRibPreload(struct bgp *bgp, struct rfapi_descriptor *rfd,
 		ri->lifetime = nhp->lifetime;
 		ri->vn_options = rfapiVnOptionsDup(nhp->vn_options);
 		ri->rsp_counter = rfd->rsp_counter;
-		ri->last_sent_time = rfapi_time(NULL);
+		ri->last_sent_time = monotime(NULL);
 
 		if (need_insert) {
 			int rc;
@@ -2070,21 +2043,14 @@ rfapiRibPreload(struct bgp *bgp, struct rfapi_descriptor *rfd,
 		 * update this NVE's timestamp for this prefix
 		 */
 		trn = agg_node_get(rfd->rsp_times[afi], &pfx); /* locks trn */
-		trn->info = (void *)(uintptr_t)bgp_clock();
-		if (trn->lock > 1)
+		trn->info = (void *)(uintptr_t)monotime(NULL);
+		if (agg_node_get_lock_count(trn) > 1)
 			agg_unlock_node(trn);
 
-		{
-			char str_pfx[PREFIX_STRLEN];
-			char str_pfx_vn[PREFIX_STRLEN];
-
-			prefix2str(&pfx, str_pfx, sizeof(str_pfx));
-			prefix2str(&rk.vn, str_pfx_vn, sizeof(str_pfx_vn));
-			vnc_zlog_debug_verbose(
-				"%s:   added pfx=%s nh[vn]=%s, cost=%u, lifetime=%u, allowed=%d",
-				__func__, str_pfx, str_pfx_vn, nhp->prefix.cost,
-				nhp->lifetime, allowed);
-		}
+		vnc_zlog_debug_verbose(
+			"%s:   added pfx=%pFX nh[vn]=%pFX, cost=%u, lifetime=%u, allowed=%d",
+			__func__, &pfx, &rk.vn, nhp->prefix.cost, nhp->lifetime,
+			allowed);
 
 		if (allowed) {
 			if (tail)
@@ -2139,10 +2105,9 @@ void rfapiRibPendingDeleteRoute(struct bgp *bgp, struct rfapi_import_table *it,
 				sl);
 
 			for (cursor = NULL,
-			    rc = skiplist_next(sl, NULL, (void **)&m,
-					       (void **)&cursor);
+			    rc = skiplist_next(sl, NULL, (void **)&m, &cursor);
 			     !rc; rc = skiplist_next(sl, NULL, (void **)&m,
-						     (void **)&cursor)) {
+						     &cursor)) {
 
 #if DEBUG_PENDING_DELETE_ROUTE
 				vnc_zlog_debug_verbose("%s: eth monitor rfd=%p",
@@ -2292,7 +2257,6 @@ static int print_rib_sl(int (*fp)(void *, const char *, ...), struct vty *vty,
 		char str_lifetime[BUFSIZ];
 		char str_age[BUFSIZ];
 		char *p;
-		char str_rd[RD_ADDRSTRLEN];
 
 		++routes_displayed;
 
@@ -2311,7 +2275,7 @@ static int print_rib_sl(int (*fp)(void *, const char *, ...), struct vty *vty,
 		rfapiFormatAge(ri->last_sent_time, str_age, BUFSIZ);
 #else
 		{
-			time_t now = rfapi_time(NULL);
+			time_t now = monotime(NULL);
 			time_t expire =
 				ri->last_sent_time + (time_t)ri->lifetime;
 			/* allow for delayed/async removal */
@@ -2320,14 +2284,9 @@ static int print_rib_sl(int (*fp)(void *, const char *, ...), struct vty *vty,
 		}
 #endif
 
-		str_rd[0] = 0; /* start empty */
-#if DEBUG_RIB_SL_RD
-		prefix_rd2str(&ri->rk.rd, str_rd, sizeof(str_rd));
-#endif
-
-		fp(out, " %c %-20s %-15s %-15s %-4u %-8s %-8s %s\n",
+		fp(out, " %c %-20s %-15s %-15s %-4u %-8s %-8s %pRD\n",
 		   deleted ? 'r' : ' ', *printedprefix ? "" : str_pfx, str_vn,
-		   str_un, ri->cost, str_lifetime, str_age, str_rd);
+		   str_un, ri->cost, str_lifetime, str_age, &ri->rk.rd);
 
 		if (!*printedprefix)
 			*printedprefix = 1;
