@@ -32,8 +32,6 @@
 #ifndef _ZEBRA_EIGRP_CONST_H_
 #define _ZEBRA_EIGRP_CONST_H_
 
-#define FALSE 0
-
 #define EIGRP_NEIGHBOR_DOWN           0
 #define EIGRP_NEIGHBOR_PENDING        1
 #define EIGRP_NEIGHBOR_UP             2
@@ -112,10 +110,8 @@ enum metric_change { METRIC_DECREASE, METRIC_SAME, METRIC_INCREASE };
 #define EIGRP_IFTYPE_NONE 0
 #define EIGRP_IFTYPE_POINTOPOINT 1
 #define EIGRP_IFTYPE_BROADCAST 2
-#define EIGRP_IFTYPE_NBMA 3
-#define EIGRP_IFTYPE_POINTOMULTIPOINT 4
-#define EIGRP_IFTYPE_LOOPBACK 5
-#define EIGRP_IFTYPE_MAX 6
+#define EIGRP_IFTYPE_LOOPBACK 3
+#define EIGRP_IFTYPE_MAX 4
 
 #define EIGRP_IF_ACTIVE                  0
 #define EIGRP_IF_PASSIVE                 1
@@ -126,10 +122,10 @@ enum metric_change { METRIC_DECREASE, METRIC_SAME, METRIC_INCREASE };
 #define EIGRP_TOPOLOGY_TYPE_REMOTE_EXTERNAL     2 // Remote external network
 
 /*EIGRP TT entry flags*/
-#define EIGRP_NEXTHOP_ENTRY_SUCCESSOR_FLAG     (1 << 0)
-#define EIGRP_NEXTHOP_ENTRY_FSUCCESSOR_FLAG    (1 << 1)
-#define EIGRP_NEXTHOP_ENTRY_INTABLE_FLAG       (1 << 2)
-#define EIGRP_NEXTHOP_ENTRY_EXTERNAL_FLAG      (1 << 3)
+#define EIGRP_ROUTE_DESCRIPTOR_SUCCESSOR_FLAG (1 << 0)
+#define EIGRP_ROUTE_DESCRIPTOR_FSUCCESSOR_FLAG (1 << 1)
+#define EIGRP_ROUTE_DESCRIPTOR_INTABLE_FLAG (1 << 2)
+#define EIGRP_ROUTE_DESCRIPTOR_EXTERNAL_FLAG (1 << 3)
 
 /*EIGRP FSM state count, event count*/
 #define EIGRP_FSM_STATE_MAX                  5
@@ -152,7 +148,7 @@ enum eigrp_fsm_states {
 enum eigrp_fsm_events {
 	/*
 	 * Input event other than query from succ,
-	 * FC is not satisified
+	 * FC is not satisfied
 	 */
 	EIGRP_FSM_EVENT_NQ_FCN,
 
@@ -171,7 +167,7 @@ enum eigrp_fsm_events {
 	/* Query from succ while in active state */
 	EIGRP_FSM_EVENT_QACT,
 
-	/* last reply, FC not satisified */
+	/* last reply, FC not satisfied */
 	EIGRP_FSM_EVENT_LR_FCN,
 
 	/*
