@@ -514,7 +514,6 @@ eigrp_snmp_nbr_lookup_next(struct in_addr *nbr_addr, unsigned int *ifindex,
 	struct listnode *node, *nnode, *node2, *nnode2;
 	struct eigrp_interface *ei;
 	struct eigrp_neighbor *nbr;
-	struct route_node *rn;
 	struct eigrp_neighbor *min = NULL;
 	struct eigrp *eigrp;
 
@@ -573,7 +572,7 @@ static struct eigrp_neighbor *eigrpNbrLookup(struct variable *v, oid *name,
 		first = 0;
 		len = *length - v->namelen;
 
-		if (len <= 0)
+		if (len == 0)
 			first = 1;
 
 		if (len > IN_ADDR_SIZE)
@@ -589,8 +588,7 @@ static struct eigrp_neighbor *eigrpNbrLookup(struct variable *v, oid *name,
 
 		if (nbr) {
 			*length = v->namelen + IN_ADDR_SIZE + 1;
-			oid_copy_addr(name + v->namelen, nbr_addr,
-				      IN_ADDR_SIZE);
+			oid_copy_in_addr(name + v->namelen, nbr_addr);
 			name[v->namelen + IN_ADDR_SIZE] = *ifindex;
 			return nbr;
 		}
@@ -620,14 +618,12 @@ static uint8_t *eigrpVpnEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPVPNNAME: /* 2 */
 		/* The name given to the VPN */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
 	}
@@ -681,14 +677,12 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(eigrp->AS);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPNBRCOUNT: /* 2 */
 		/* Neighbor count of this EIGRP instance */
 		if (eigrp)
 			return SNMP_INTEGER(eigrp_neighbor_count(eigrp));
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPHELLOSSENT: /* 3 */
 		/* Hello packets output count */
 		if (eigrp) {
@@ -700,7 +694,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPHELLOSRCVD: /* 4 */
 		/* Hello packets input count */
 		if (eigrp) {
@@ -712,7 +705,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPUPDATESSENT: /* 5 */
 		/* Update packets output count */
 		if (eigrp) {
@@ -724,7 +716,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPUPDATESRCVD: /* 6 */
 		/* Update packets input count */
 		if (eigrp) {
@@ -736,7 +727,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPQUERIESSENT: /* 7 */
 		/* Querry packets output count */
 		if (eigrp) {
@@ -748,7 +738,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPQUERIESRCVD: /* 8 */
 		/* Querry packets input count */
 		if (eigrp) {
@@ -760,7 +749,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPREPLIESSENT: /* 9 */
 		/* Reply packets output count */
 		if (eigrp) {
@@ -772,7 +760,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPREPLIESRCVD: /* 10 */
 		/* Reply packets input count */
 		if (eigrp) {
@@ -784,7 +771,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPACKSSENT: /* 11 */
 		/* Acknowledgement packets output count */
 		if (eigrp) {
@@ -796,7 +782,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPACKSRCVD: /* 12 */
 		/* Acknowledgement packets input count */
 		if (eigrp) {
@@ -808,21 +793,18 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPINPUTQHIGHMARK: /* 13 */
 		/* The highest number of EIGRP packets in the input queue */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPINPUTQDROPS: /* 14 */
 		/* The number of EIGRP packets dropped from the input queue */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPSIAQUERIESSENT: /* 15 */
 		/* SIA querry packets output count */
 		if (eigrp) {
@@ -834,7 +816,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPSIAQUERIESRCVD: /* 16 */
 		/* SIA querry packets input count */
 		if (eigrp) {
@@ -846,7 +827,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(counter);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPASROUTERIDTYPE: /* 17 */
 		/* Whether the router ID is set manually or automatically */
 		if (eigrp)
@@ -856,7 +836,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 				return SNMP_INTEGER(1);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPASROUTERID: /* 18 */
 		/* Router ID for this EIGRP AS */
 		if (eigrp)
@@ -866,7 +845,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 				return SNMP_INTEGER(eigrp->router_id);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPTOPOROUTES: /* 19 */
 		/* The total number of EIGRP derived routes currently existing
 		   in the topology table for the AS */
@@ -874,7 +852,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPHEADSERIAL: /* 20 */
 		/* The serial number of the first route in the internal
 		   sequence for an AS*/
@@ -882,7 +859,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPNEXTSERIAL: /* 21 */
 		/* The serial number that would be assigned to the next new
 		 or changed route in the topology table for the AS*/
@@ -890,7 +866,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPXMITPENDREPLIES: /* 22 */
 		/* Total number of outstanding replies expected to queries
 		   that have been sent to peers in the current AS*/
@@ -898,7 +873,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPXMITDUMMIES: /* 23 */
 		/* Total number of currently existing dummies associated with
 		 * the AS*/
@@ -906,7 +880,6 @@ static uint8_t *eigrpTraffStatsEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
 	}
@@ -918,8 +891,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 				   WriteMethod **write_method)
 {
 	struct eigrp *eigrp;
-	struct eigrp_interface *ei;
-	struct listnode *node, *nnode;
 
 	eigrp = eigrp_lookup();
 
@@ -937,7 +908,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPDESTNET: /* 2 */
 		/* The destination IP network number for a single route in the
 		 * topology table*/
@@ -945,7 +915,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPDESTNETPREFIXLEN: /* 4 */
 		/* The prefix length associated with the destination IP network
 		   address
@@ -954,7 +923,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPACTIVE: /* 5 */
 		/* A value of true(1) indicates the route to the destination
 		   network has failed
@@ -964,7 +932,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPSTUCKINACTIVE: /* 6 */
 		/* A value of true(1) indicates that that this route which is in
 		   active state
@@ -974,14 +941,12 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPDESTSUCCESSORS: /* 7 */
 		/* Next routing hop for a path to the destination IP network */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPFDISTANCE: /* 8 */
 		/* Minimum distance from this router to the destination IP
 		 * network */
@@ -989,7 +954,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPROUTEORIGINTYPE: /* 9 */
 		/* Text string describing the internal origin of the EIGRP route
 		 */
@@ -997,7 +961,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPROUTEORIGINADDRTYPE: /* 10 */
 		/* The format of the IP address defined as the origin of this
 		   topology route entry */
@@ -1005,7 +968,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPROUTEORIGINADDR: /* 11 */
 		/* If the origin of the topology route entry is external to this
 		   router,
@@ -1015,21 +977,18 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPNEXTHOPADDRESSTYPE: /* 12 */
 		/* The format of the next hop IP address */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPNEXTHOPADDRESS: /* 13 */
 		/* Next hop IP address for the route */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPNEXTHOPINTERFACE: /* 14 */
 		/* The interface through which the next hop IP address is
 		 * reached */
@@ -1037,7 +996,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPDISTANCE: /* 15 */
 		/* The computed distance to the destination network entry from
 		 * this router */
@@ -1045,7 +1003,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPREPORTDISTANCE: /* 16 */
 		/* The computed distance to the destination network in the
 		   topology entry
@@ -1054,7 +1011,6 @@ static uint8_t *eigrpTopologyEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
 	}
@@ -1067,7 +1023,6 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 {
 	struct eigrp *eigrp;
 	struct eigrp_interface *ei;
-	struct listnode *node, *nnode;
 	struct eigrp_neighbor *nbr;
 	struct in_addr nbr_addr;
 	unsigned int ifindex;
@@ -1079,7 +1034,7 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 	    == MATCH_FAILED)
 		return NULL;
 
-	memset(&nbr_addr, 0, sizeof(struct in_addr));
+	memset(&nbr_addr, 0, sizeof(nbr_addr));
 	ifindex = 0;
 
 	nbr = eigrpNbrLookup(v, name, length, &nbr_addr, &ifindex, exact);
@@ -1097,7 +1052,6 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPEERADDRTYPE: /* 2 */
 		/* The format of the remote source IP address used by the peer
 		 */
@@ -1105,21 +1059,18 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPEERADDR: /* 3 */
 		/* The source IP address used by the peer */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPEERIFINDEX: /* 4 */
 		/* The ifIndex of the interface on this router */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPHOLDTIME: /* 5 */
 		/* How much time must pass without receiving a hello packet from
 		   this
@@ -1128,7 +1079,6 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPUPTIME: /* 6 */
 		/* The elapsed time since the EIGRP adjacency was first
 		 * established */
@@ -1136,7 +1086,6 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPSRTT: /* 7 */
 		/* The computed smooth round trip time for packets to and from
 		 * the peer */
@@ -1144,42 +1093,36 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRTO: /* 8 */
 		/* The computed retransmission timeout for the peer */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPKTSENQUEUED: /* 9 */
 		/* The number of any EIGRP packets currently enqueued */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPLASTSEQ: /* 10 */
 		/* sequence number of the last EIGRP packet sent to this peer */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPVERSION: /* 11 */
 		/* The EIGRP version information reported by the remote peer */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRETRANS: /* 12 */
 		/* The cumulative number of retransmissions to this peer */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRETRIES: /* 13 */
 		/* The number of times the current unacknowledged packet has
 		 * been retried */
@@ -1187,7 +1130,6 @@ static uint8_t *eigrpPeerEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
 	}
@@ -1199,11 +1141,9 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 				    WriteMethod **write_method)
 {
 	struct eigrp *eigrp;
-	struct eigrp_interface *ei;
 	struct listnode *node, *nnode;
 	struct keychain *keychain;
 	struct list *keylist;
-	int counter;
 
 	eigrp = eigrp_lookup();
 
@@ -1221,7 +1161,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(eigrp_neighbor_count(eigrp));
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPXMITRELIABLEQ: /* 4 */
 		/* The number of EIGRP packets currently waiting in the reliable
 		   transport transmission queue */
@@ -1229,7 +1168,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPXMITUNRELIABLEQ: /* 5 */
 		/* The number of EIGRP packets currently waiting in the
 		   unreliable
@@ -1238,7 +1176,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPMEANSRTT: /* 6 */
 		/* The average of all the computed smooth round trip time values
 		   for a packet to and from all peers established on this
@@ -1247,7 +1184,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPACINGRELIABLE: /* 7 */
 		/* The configured time interval between EIGRP packet
 		 * transmissions */
@@ -1255,7 +1191,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPACINGUNRELIABLE: /* 8 */
 		/* The configured time interval between EIGRP packet
 		   transmissions
@@ -1265,14 +1200,12 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPMFLOWTIMER: /* 9 */
 		/* The configured multicast flow control timer value */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPPENDINGROUTES: /* 10 */
 		/* The number of queued EIGRP routing updates awaiting
 		 * transmission */
@@ -1280,7 +1213,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPHELLOINTERVAL: /* 11 */
 		/* The configured time interval between Hello packet
 		 * transmissions */
@@ -1288,7 +1220,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPXMITNEXTSERIAL: /* 12 */
 		/* The serial number of the next EIGRP packet that is to be
 		   queued
@@ -1297,7 +1228,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPUMCASTS: /* 13 */
 		/* The total number of unreliable EIGRP multicast packets sent
 		   on this interface */
@@ -1305,7 +1235,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRMCASTS: /* 14 */
 		/* The total number of reliable EIGRP multicast packets sent
 		   on this interface */
@@ -1313,7 +1242,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPUUCASTS: /* 15 */
 		/* The total number of unreliable EIGRP unicast packets sent
 		   on this interface */
@@ -1321,7 +1249,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRUCASTS: /* 16 */
 		/* The total number of reliable EIGRP unicast packets sent
 		   on this interface */
@@ -1329,7 +1256,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPMCASTEXCEPTS: /* 17 */
 		/* The total number of EIGRP multicast exception transmissions
 		 */
@@ -1337,7 +1263,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPCRPKTS: /* 18 */
 		/* The total number EIGRP Conditional-Receive packets sent on
 		 * this interface */
@@ -1345,7 +1270,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPACKSSUPPRESSED: /* 19 */
 		/* The total number of individual EIGRP acknowledgement packets
 		   that have been
@@ -1355,7 +1279,6 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPRETRANSSENT: /* 20 */
 		/* The total number EIGRP packet retransmissions sent on the
 		 * interface */
@@ -1363,21 +1286,18 @@ static uint8_t *eigrpInterfaceEntry(struct variable *v, oid *name,
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPOOSRCVD: /* 21 */
 		/* The total number of out-of-sequence EIGRP packets received */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPAUTHMODE: /* 22 */
 		/* The EIGRP authentication mode of the interface */
 		if (eigrp) {
 			return SNMP_INTEGER(1);
 		} else
 			return SNMP_INTEGER(0);
-		break;
 	case EIGRPAUTHKEYCHAIN: /* 23 */
 		/* The name of the authentication key-chain configured
 		   on this interface. */
