@@ -27,6 +27,8 @@
 
 #include "isisd/isis_tlvs.h"
 
+DECLARE_MTYPE(ISIS_ADJACENCY_INFO);
+
 enum isis_adj_usage {
 	ISIS_ADJ_NONE,
 	ISIS_ADJ_LEVEL1,
@@ -86,16 +88,18 @@ struct isis_adjacency {
 	struct in_addr *ipv4_addresses;
 	unsigned int ipv4_address_count;
 	struct in_addr router_address;
-	struct in6_addr *ipv6_addresses;
-	unsigned int ipv6_address_count;
+	struct in6_addr *ll_ipv6_addrs; /* Link local IPv6 neighbor address */
+	unsigned int ll_ipv6_count;
+	struct in6_addr *global_ipv6_addrs; /* Global IPv6 neighbor address */
+	unsigned int global_ipv6_count;
 	struct in6_addr router_address6;
 	uint8_t prio[ISIS_LEVELS];      /* priorityOfNeighbour for DIS */
 	int circuit_t;			/* from hello PDU hdr */
 	int level;			/* level (1 or 2) */
 	enum isis_system_type sys_type; /* neighbourSystemType */
 	uint16_t hold_time;		/* entryRemainingTime */
-	uint32_t last_upd;
-	uint32_t last_flap; /* last time the adj flapped */
+	time_t last_upd;
+	time_t last_flap; /* last time the adj flapped */
 	enum isis_threeway_state threeway_state;
 	uint32_t ext_circuit_id;
 	int flaps;		      /* number of adjacency flaps  */
@@ -103,8 +107,10 @@ struct isis_adjacency {
 	struct isis_circuit *circuit; /* back pointer */
 	uint16_t *mt_set;      /* Topologies this adjacency is valid for */
 	unsigned int mt_count; /* Number of entries in mt_set */
-	struct bfd_session *bfd_session;
+	struct bfd_session_params *bfd_session;
 	struct list *adj_sids; /* Segment Routing Adj-SIDs. */
+	uint32_t snmp_idx;
+	struct listnode *snmp_list_node;
 };
 
 struct isis_threeway_adj;
@@ -113,28 +119,36 @@ struct isis_adjacency *isis_adj_lookup(const uint8_t *sysid,
 				       struct list *adjdb);
 struct isis_adjacency *isis_adj_lookup_snpa(const uint8_t *ssnpa,
 					    struct list *adjdb);
-bool isis_adj_exists(const struct isis_area *area, int level,
-		     const uint8_t *sysid);
+struct isis_adjacency *isis_adj_find(const struct isis_area *area, int level,
+				     const uint8_t *sysid);
 struct isis_adjacency *isis_new_adj(const uint8_t *id, const uint8_t *snpa,
 				    int level, struct isis_circuit *circuit);
 void isis_delete_adj(void *adj);
 void isis_adj_process_threeway(struct isis_adjacency *adj,
 			       struct isis_threeway_adj *tw_adj,
 			       enum isis_adj_usage adj_usage);
-DECLARE_HOOK(isis_adj_state_change_hook, (struct isis_adjacency *adj), (adj))
+DECLARE_HOOK(isis_adj_state_change_hook, (struct isis_adjacency *adj), (adj));
 DECLARE_HOOK(isis_adj_ip_enabled_hook,
-	     (struct isis_adjacency *adj, int family), (adj, family))
+	     (struct isis_adjacency * adj, int family, bool global),
+	     (adj, family, global));
 DECLARE_HOOK(isis_adj_ip_disabled_hook,
-	     (struct isis_adjacency *adj, int family), (adj, family))
+	     (struct isis_adjacency * adj, int family, bool global),
+	     (adj, family, global));
+void isis_log_adj_change(struct isis_adjacency *adj,
+			 enum isis_adj_state old_state,
+			 enum isis_adj_state new_state, const char *reason);
 void isis_adj_state_change(struct isis_adjacency **adj,
 			   enum isis_adj_state state, const char *reason);
 void isis_adj_print(struct isis_adjacency *adj);
 const char *isis_adj_yang_state(enum isis_adj_state state);
-int isis_adj_expire(struct thread *thread);
+void isis_adj_expire(struct thread *thread);
 void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 			char detail);
+void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
+			 char detail);
 void isis_adj_build_neigh_list(struct list *adjdb, struct list *list);
 void isis_adj_build_up_list(struct list *adjdb, struct list *list);
 int isis_adj_usage2levels(enum isis_adj_usage usage);
-
+void isis_bfd_startup_timer(struct thread *thread);
+const char *isis_adj_name(const struct isis_adjacency *adj);
 #endif /* ISIS_ADJACENCY_H */
