@@ -54,9 +54,6 @@
 #include "isisd/isis_dr.h"
 #include "isisd/isis_zebra.h"
 
-DEFINE_MTYPE_STATIC(ISISD, ISIS_MPLS_TE,    "ISIS MPLS_TE parameters");
-DEFINE_MTYPE_STATIC(ISISD, ISIS_PLIST_NAME, "ISIS prefix-list name");
-
 /*
  * XPath: /frr-isisd:isis/instance
  */
@@ -86,12 +83,17 @@ int isis_instance_create(struct nb_cb_create_args *args)
 int isis_instance_destroy(struct nb_cb_destroy_args *args)
 {
 	struct isis_area *area;
+	struct isis *isis;
 
 	if (args->event != NB_EV_APPLY)
 		return NB_OK;
 	area = nb_running_unset_entry(args->dnode);
-
+	isis = area->isis;
 	isis_area_destroy(area);
+
+	if (listcount(isis->area_list) == 0)
+		isis_finish(isis);
+
 	return NB_OK;
 }
 
@@ -334,9 +336,9 @@ int isis_instance_attached_modify(struct nb_cb_modify_args *args)
 }
 
 /*
- * XPath: /frr-isisd:isis/instance/overload
+ * XPath: /frr-isisd:isis/instance/overload/enabled
  */
-int isis_instance_overload_modify(struct nb_cb_modify_args *args)
+int isis_instance_overload_enabled_modify(struct nb_cb_modify_args *args)
 {
 	struct isis_area *area;
 	bool overload;
@@ -346,11 +348,31 @@ int isis_instance_overload_modify(struct nb_cb_modify_args *args)
 
 	area = nb_running_get_entry(args->dnode, NULL, true);
 	overload = yang_dnode_get_bool(args->dnode, NULL);
+	area->overload_configured = overload;
+
 	isis_area_overload_bit_set(area, overload);
 
 	return NB_OK;
 }
 
+/*
+ * XPath: /frr-isisd:isis/instance/overload/on-startup
+ */
+int isis_instance_overload_on_startup_modify(struct nb_cb_modify_args *args)
+{
+	struct isis_area *area;
+	uint32_t overload_time;
+
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	overload_time = yang_dnode_get_uint32(args->dnode, NULL);
+	area = nb_running_get_entry(args->dnode, NULL, true);
+	isis_area_overload_on_startup_set(area, overload_time);
+
+	return NB_OK;
+}
+
 /*
  * XPath: /frr-isisd:isis/instance/metric-style
  */
@@ -393,30 +415,11 @@ int isis_instance_purge_originator_modify(struct nb_cb_modify_args *args)
  */
 int isis_instance_lsp_mtu_modify(struct nb_cb_modify_args *args)
 {
-	struct listnode *node;
-	struct isis_circuit *circuit;
 	uint16_t lsp_mtu = yang_dnode_get_uint16(args->dnode, NULL);
 	struct isis_area *area;
 
 	switch (args->event) {
 	case NB_EV_VALIDATE:
-		area = nb_running_get_entry(args->dnode, NULL, false);
-		if (!area)
-			break;
-		for (ALL_LIST_ELEMENTS_RO(area->circuit_list, node, circuit)) {
-			if (circuit->state != C_STATE_INIT
-			    && circuit->state != C_STATE_UP)
-				continue;
-			if (lsp_mtu > isis_circuit_pdu_size(circuit)) {
-				snprintf(
-					args->errmsg, args->errmsg_len,
-					"ISIS area contains circuit %s, which has a maximum PDU size of %zu",
-					circuit->interface->name,
-					isis_circuit_pdu_size(circuit));
-				return NB_ERR_VALIDATION;
-			}
-		}
-		break;
 	case NB_EV_PREPARE:
 	case NB_EV_ABORT:
 		break;
@@ -1806,45 +1809,13 @@ int isis_instance_log_adjacency_changes_modify(struct nb_cb_modify_args *args)
  */
 int isis_instance_mpls_te_create(struct nb_cb_create_args *args)
 {
-	struct listnode *node;
 	struct isis_area *area;
-	struct isis_circuit *circuit;
 
 	if (args->event != NB_EV_APPLY)
 		return NB_OK;
 
 	area = nb_running_get_entry(args->dnode, NULL, true);
-	if (area->mta == NULL) {
-
-		struct mpls_te_area *new;
-
-		zlog_debug("ISIS-TE(%s): Initialize MPLS Traffic Engineering",
-			   area->area_tag);
-
-		new = XCALLOC(MTYPE_ISIS_MPLS_TE, sizeof(struct mpls_te_area));
-
-		/* Initialize MPLS_TE structure */
-		new->status = enable;
-		new->level = 0;
-		new->inter_as = off;
-		new->interas_areaid.s_addr = 0;
-		new->router_id.s_addr = 0;
-		new->ted = ls_ted_new(1, "ISIS", 0);
-		if (!new->ted)
-			zlog_warn("Unable to create Link State Data Base");
-
-		area->mta = new;
-	} else {
-		area->mta->status = enable;
-	}
-
-	/* Initialize Link State Database */
-	if (area->mta->ted)
-		isis_te_init_ted(area);
-
-	/* Update Extended TLVs according to Interface link parameters */
-	for (ALL_LIST_ELEMENTS_RO(area->circuit_list, node, circuit))
-		isis_link_params_update(circuit, circuit->interface);
+	isis_mpls_te_create(area);
 
 	/* Reoriginate STD_TE & GMPLS circuits */
 	lsp_regenerate_schedule(area, area->is_type, 0);
@@ -1854,35 +1825,16 @@ int isis_instance_mpls_te_create(struct nb_cb_create_args *args)
 
 int isis_instance_mpls_te_destroy(struct nb_cb_destroy_args *args)
 {
-	struct listnode *node;
 	struct isis_area *area;
-	struct isis_circuit *circuit;
 
 	if (args->event != NB_EV_APPLY)
 		return NB_OK;
 
 	area = nb_running_get_entry(args->dnode, NULL, true);
-	if (IS_MPLS_TE(area->mta))
-		area->mta->status = disable;
-	else
+	if (!IS_MPLS_TE(area->mta))
 		return NB_OK;
 
-	/* Remove Link State Database */
-	ls_ted_del_all(area->mta->ted);
-
-	/* Flush LSP if circuit engage */
-	for (ALL_LIST_ELEMENTS_RO(area->circuit_list, node, circuit)) {
-		if (!IS_EXT_TE(circuit->ext))
-			continue;
-
-		/* disable MPLS_TE Circuit keeping SR one's */
-		if (IS_SUBTLV(circuit->ext, EXT_ADJ_SID))
-			circuit->ext->status = EXT_ADJ_SID;
-		else if (IS_SUBTLV(circuit->ext, EXT_LAN_ADJ_SID))
-			circuit->ext->status = EXT_LAN_ADJ_SID;
-		else
-			circuit->ext->status = 0;
-	}
+	isis_mpls_te_disable(area);
 
 	/* Reoriginate STD_TE & GMPLS circuits */
 	lsp_regenerate_schedule(area, area->is_type, 0);
@@ -2552,43 +2504,14 @@ int isis_instance_mpls_ldp_sync_holddown_modify(struct nb_cb_modify_args *args)
  */
 int lib_interface_isis_create(struct nb_cb_create_args *args)
 {
-	struct isis_area *area = NULL;
 	struct interface *ifp;
 	struct isis_circuit *circuit = NULL;
 	const char *area_tag = yang_dnode_get_string(args->dnode, "./area-tag");
-	uint32_t min_mtu, actual_mtu;
 
 	switch (args->event) {
 	case NB_EV_PREPARE:
 	case NB_EV_ABORT:
-		break;
 	case NB_EV_VALIDATE:
-		/* check if interface mtu is sufficient. If the area has not
-		 * been created yet, assume default MTU for the area
-		 */
-		ifp = nb_running_get_entry(args->dnode, NULL, false);
-		/* zebra might not know yet about the MTU - nothing we can do */
-		if (!ifp || ifp->mtu == 0)
-			break;
-		actual_mtu =
-			if_is_broadcast(ifp) ? ifp->mtu - LLC_LEN : ifp->mtu;
-
-		area = isis_area_lookup(area_tag, ifp->vrf->vrf_id);
-		if (area)
-			min_mtu = area->lsp_mtu;
-		else
-#ifndef FABRICD
-			min_mtu = yang_get_default_uint16(
-				"/frr-isisd:isis/instance/lsp/mtu");
-#else
-			min_mtu = DEFAULT_LSP_MTU;
-#endif /* ifndef FABRICD */
-		if (actual_mtu < min_mtu) {
-			snprintf(args->errmsg, args->errmsg_len,
-				 "Interface %s has MTU %u, minimum MTU for the area is %u",
-				 ifp->name, actual_mtu, min_mtu);
-			return NB_ERR_VALIDATION;
-		}
 		break;
 	case NB_EV_APPLY:
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
@@ -3118,10 +3041,6 @@ int lib_interface_isis_disable_three_way_handshake_modify(
 	return NB_OK;
 }
 
-/*
- * XPath:
- * /frr-interface:lib/interface/frr-isisd:isis/multi-topology/ipv4-unicast
- */
 static int lib_interface_isis_multi_topology_common(
 	enum nb_event event, const struct lyd_node *dnode, char *errmsg,
 	size_t errmsg_len, uint16_t mtid)
@@ -3152,12 +3071,16 @@ static int lib_interface_isis_multi_topology_common(
 	return NB_OK;
 }
 
-int lib_interface_isis_multi_topology_ipv4_unicast_modify(
+/*
+ * XPath:
+ * /frr-interface:lib/interface/frr-isisd:isis/multi-topology/standard
+ */
+int lib_interface_isis_multi_topology_standard_modify(
 	struct nb_cb_modify_args *args)
 {
 	return lib_interface_isis_multi_topology_common(
 		args->event, args->dnode, args->errmsg, args->errmsg_len,
-		ISIS_MT_IPV4_UNICAST);
+		ISIS_MT_STANDARD);
 }
 
 /*
