@@ -30,9 +30,17 @@
 #include "smux.h"
 #include "memory.h"
 #include "linklist.h"
-#include "version.h"
+#include "lib/version.h"
+#include "lib_errors.h"
+#include "hook.h"
+#include "libfrr.h"
+#include "xref.h"
 
-static int agentx_enabled = 0;
+XREF_SETUP();
+
+DEFINE_HOOK(agentx_enabled, (), ());
+
+static bool agentx_enabled = false;
 
 static struct thread_master *agentx_tm;
 static struct thread *timeout_thr = NULL;
@@ -40,30 +48,58 @@ static struct list *events = NULL;
 
 static void agentx_events_update(void);
 
-static int agentx_timeout(struct thread *t)
+static void agentx_timeout(struct thread *t)
 {
-	timeout_thr = NULL;
-
 	snmp_timeout();
 	run_alarms();
 	netsnmp_check_outstanding_agent_requests();
 	agentx_events_update();
-	return 0;
 }
 
-static int agentx_read(struct thread *t)
+static void agentx_read(struct thread *t)
 {
 	fd_set fds;
+	int flags, new_flags = 0;
+	int nonblock = false;
 	struct listnode *ln = THREAD_ARG(t);
+	struct thread **thr = listgetdata(ln);
+	XFREE(MTYPE_TMP, thr);
 	list_delete_node(events, ln);
 
+	/* fix for non blocking socket */
+	flags = fcntl(THREAD_FD(t), F_GETFL, 0);
+	if (-1 == flags) {
+		flog_err(EC_LIB_SYSTEM_CALL, "Failed to get FD settings fcntl: %s(%d)",
+			 strerror(errno), errno);
+		return;
+	}
+
+	if (flags & O_NONBLOCK)
+		nonblock = true;
+	else
+		new_flags = fcntl(THREAD_FD(t), F_SETFL, flags | O_NONBLOCK);
+
+	if (new_flags == -1)
+		flog_err(EC_LIB_SYSTEM_CALL, "Failed to set snmp fd non blocking: %s(%d)",
+			 strerror(errno), errno);
+
 	FD_ZERO(&fds);
 	FD_SET(THREAD_FD(t), &fds);
 	snmp_read(&fds);
 
+	/* Reset the flag */
+	if (!nonblock) {
+		new_flags = fcntl(THREAD_FD(t), F_SETFL, flags);
+
+		if (new_flags == -1)
+			flog_err(
+				EC_LIB_SYSTEM_CALL,
+				"Failed to set snmp fd back to original settings: %s(%d)",
+				strerror(errno), errno);
+	}
+
 	netsnmp_check_outstanding_agent_requests();
 	agentx_events_update();
-	return 0;
 }
 
 static void agentx_events_update(void)
@@ -73,23 +109,22 @@ static void agentx_events_update(void)
 	struct timeval timeout = {.tv_sec = 0, .tv_usec = 0};
 	fd_set fds;
 	struct listnode *ln;
-	struct thread *thr;
+	struct thread **thr;
 	int fd, thr_fd;
 
-	THREAD_OFF(timeout_thr);
+	thread_cancel(&timeout_thr);
 
 	FD_ZERO(&fds);
 	snmp_select_info(&maxfd, &fds, &timeout, &block);
 
 	if (!block) {
-		timeout_thr = NULL;
 		thread_add_timer_tv(agentx_tm, agentx_timeout, NULL, &timeout,
 				    &timeout_thr);
 	}
 
 	ln = listhead(events);
 	thr = ln ? listgetdata(ln) : NULL;
-	thr_fd = thr ? THREAD_FD(thr) : -1;
+	thr_fd = thr ? THREAD_FD(*thr) : -1;
 
 	/* "two-pointer" / two-list simultaneous iteration
 	 * ln/thr/thr_fd point to the next existing event listener to hit while
@@ -100,19 +135,20 @@ static void agentx_events_update(void)
 			struct listnode *nextln = listnextnode(ln);
 			if (!FD_ISSET(fd, &fds)) {
 				thread_cancel(thr);
+				XFREE(MTYPE_TMP, thr);
 				list_delete_node(events, ln);
 			}
 			ln = nextln;
 			thr = ln ? listgetdata(ln) : NULL;
-			thr_fd = thr ? THREAD_FD(thr) : -1;
+			thr_fd = thr ? THREAD_FD(*thr) : -1;
 		}
 		/* need listener, but haven't hit one where it would be */
 		else if (FD_ISSET(fd, &fds)) {
 			struct listnode *newln;
-			thr = NULL;
-			thread_add_read(agentx_tm, agentx_read, NULL, fd, &thr);
+			thr = XCALLOC(MTYPE_TMP, sizeof(struct thread *));
+
 			newln = listnode_add_before(events, ln, thr);
-			thr->arg = newln;
+			thread_add_read(agentx_tm, agentx_read, newln, fd, thr);
 		}
 	}
 
@@ -120,16 +156,22 @@ static void agentx_events_update(void)
 	 */
 	while (ln) {
 		struct listnode *nextln = listnextnode(ln);
-		thread_cancel(listgetdata(ln));
+		thr = listgetdata(ln);
+		thread_cancel(thr);
+		XFREE(MTYPE_TMP, thr);
 		list_delete_node(events, ln);
 		ln = nextln;
 	}
 }
 
 /* AgentX node. */
-static struct cmd_node agentx_node = {SMUX_NODE,
-				      "", /* AgentX has no interface. */
-				      1};
+static int config_write_agentx(struct vty *vty);
+static struct cmd_node agentx_node = {
+	.name = "smux",
+	.node = SMUX_NODE,
+	.prompt = "",
+	.config_write = config_write_agentx,
+};
 
 /* Logging NetSNMP messages */
 static int agentx_log_callback(int major, int minor, void *serverarg,
@@ -141,19 +183,20 @@ static int agentx_log_callback(int major, int minor, void *serverarg,
 		msg[strlen(msg) - 1] = '\0';
 	switch (slm->priority) {
 	case LOG_EMERG:
-		zlog_err("snmp[emerg]: %s", msg ? msg : slm->msg);
+		flog_err(EC_LIB_SNMP, "snmp[emerg]: %s", msg ? msg : slm->msg);
 		break;
 	case LOG_ALERT:
-		zlog_err("snmp[alert]: %s", msg ? msg : slm->msg);
+		flog_err(EC_LIB_SNMP, "snmp[alert]: %s", msg ? msg : slm->msg);
 		break;
 	case LOG_CRIT:
-		zlog_err("snmp[crit]: %s", msg ? msg : slm->msg);
+		flog_err(EC_LIB_SNMP, "snmp[crit]: %s", msg ? msg : slm->msg);
 		break;
 	case LOG_ERR:
-		zlog_err("snmp[err]: %s", msg ? msg : slm->msg);
+		flog_err(EC_LIB_SNMP, "snmp[err]: %s", msg ? msg : slm->msg);
 		break;
 	case LOG_WARNING:
-		zlog_warn("snmp[warning]: %s", msg ? msg : slm->msg);
+		flog_warn(EC_LIB_SNMP, "snmp[warning]: %s",
+			  msg ? msg : slm->msg);
 		break;
 	case LOG_NOTICE:
 		zlog_notice("snmp[notice]: %s", msg ? msg : slm->msg);
@@ -185,10 +228,10 @@ DEFUN (agentx_enable,
 		init_snmp(FRR_SMUX_NAME);
 		events = list_new();
 		agentx_events_update();
-		agentx_enabled = 1;
-		return CMD_SUCCESS;
+		agentx_enabled = true;
+		hook_call(agentx_enabled);
 	}
-	vty_out(vty, "SNMP AgentX already enabled\n");
+
 	return CMD_SUCCESS;
 }
 
@@ -204,6 +247,18 @@ DEFUN (no_agentx,
 	return CMD_WARNING_CONFIG_FAILED;
 }
 
+static int smux_disable(void)
+{
+	agentx_enabled = false;
+
+	return 0;
+}
+
+bool smux_enabled(void)
+{
+	return agentx_enabled;
+}
+
 void smux_init(struct thread_master *tm)
 {
 	agentx_tm = tm;
@@ -215,9 +270,21 @@ void smux_init(struct thread_master *tm)
 			       agentx_log_callback, NULL);
 	init_agent(FRR_SMUX_NAME);
 
-	install_node(&agentx_node, config_write_agentx);
+	install_node(&agentx_node);
 	install_element(CONFIG_NODE, &agentx_enable_cmd);
 	install_element(CONFIG_NODE, &no_agentx_cmd);
+
+	hook_register(frr_early_fini, smux_disable);
+}
+
+void smux_agentx_enable(void)
+{
+	if (!agentx_enabled) {
+		init_snmp(FRR_SMUX_NAME);
+		events = list_new();
+		agentx_events_update();
+		agentx_enabled = true;
+	}
 }
 
 void smux_register_mib(const char *descr, struct variable *var, size_t width,
@@ -226,14 +293,31 @@ void smux_register_mib(const char *descr, struct variable *var, size_t width,
 	register_mib(descr, var, width, num, name, namelen);
 }
 
-int smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
-	      size_t enamelen, const oid *name, size_t namelen,
-	      const oid *iname, size_t inamelen,
-	      const struct trap_object *trapobj, size_t trapobjlen,
-	      uint8_t sptrap)
+void smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
+	       size_t enamelen, const oid *name, size_t namelen,
+	       const oid *iname, size_t inamelen,
+	       const struct trap_object *trapobj, size_t trapobjlen,
+	       uint8_t sptrap)
+{
+	struct index_oid trap_index[1];
+
+	/* copy the single index into the multi-index format */
+	oid_copy(trap_index[0].indexname, iname, inamelen);
+	trap_index[0].indexlen = inamelen;
+
+	smux_trap_multi_index(vp, vp_len, ename, enamelen, name, namelen,
+			      trap_index, array_size(trap_index), trapobj,
+			      trapobjlen, sptrap);
+}
+
+int smux_trap_multi_index(struct variable *vp, size_t vp_len, const oid *ename,
+			  size_t enamelen, const oid *name, size_t namelen,
+			  struct index_oid *iname, size_t index_len,
+			  const struct trap_object *trapobj, size_t trapobjlen,
+			  uint8_t sptrap)
 {
 	oid objid_snmptrap[] = {1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0};
-	size_t objid_snmptrap_len = sizeof objid_snmptrap / sizeof(oid);
+	size_t objid_snmptrap_len = sizeof(objid_snmptrap) / sizeof(oid);
 	oid notification_oid[MAX_OID_LEN];
 	size_t notification_oid_len;
 	unsigned int i;
@@ -260,6 +344,13 @@ int smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
 		size_t val_len;
 		WriteMethod *wm = NULL;
 		struct variable cvp;
+		unsigned int iindex;
+		/*
+		 * this allows the behaviour of smux_trap with a singe index
+		 * for all objects to be maintained whilst allowing traps which
+		 * have different indices per object to be supported
+		 */
+		iindex = (index_len == 1) ? 0 : i;
 
 		/* Make OID. */
 		if (trapobj[i].namelen > 0) {
@@ -267,8 +358,10 @@ int smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
 			onamelen = trapobj[i].namelen;
 			oid_copy(oid, name, namelen);
 			oid_copy(oid + namelen, trapobj[i].name, onamelen);
-			oid_copy(oid + namelen + onamelen, iname, inamelen);
-			oid_len = namelen + onamelen + inamelen;
+			oid_copy(oid + namelen + onamelen,
+				 iname[iindex].indexname,
+				 iname[iindex].indexlen);
+			oid_len = namelen + onamelen + iname[iindex].indexlen;
 		} else {
 			/* Scalar object */
 			onamelen = trapobj[i].namelen * (-1);
@@ -294,6 +387,7 @@ int smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
 			cvp.magic = vp[j].magic;
 			cvp.acl = vp[j].acl;
 			cvp.findVar = vp[j].findVar;
+
 			/* Grab the result. */
 			val = cvp.findVar(&cvp, oid, &oid_len, 1, &val_len,
 					  &wm);
@@ -313,4 +407,9 @@ int smux_trap(struct variable *vp, size_t vp_len, const oid *ename,
 	return 1;
 }
 
+void smux_events_update(void)
+{
+	agentx_events_update();
+}
+
 #endif /* SNMP_AGENTX */
