@@ -26,7 +26,7 @@
 #include "command_match.h"
 #include "memory.h"
 
-DEFINE_MTYPE_STATIC(LIB, CMD_MATCHSTACK, "Command Match Stack")
+DEFINE_MTYPE_STATIC(LIB, CMD_MATCHSTACK, "Command Match Stack");
 
 #ifdef TRACE_MATCHER
 #define TM 1
@@ -42,7 +42,7 @@ DEFINE_MTYPE_STATIC(LIB, CMD_MATCHSTACK, "Command Match Stack")
 
 /* matcher helper prototypes */
 static int add_nexthops(struct list *, struct graph_node *,
-			struct graph_node **, size_t);
+			struct graph_node **, size_t, bool);
 
 static enum matcher_rv command_match_r(struct graph_node *, vector,
 				       unsigned int, struct graph_node **,
@@ -79,6 +79,13 @@ static enum match_type match_variable(struct cmd_token *, const char *);
 
 static enum match_type match_mac(const char *, bool);
 
+static bool is_neg(vector vline, size_t idx)
+{
+	if (idx >= vector_active(vline) || !vector_slot(vline, idx))
+		return false;
+	return !strcmp(vector_slot(vline, idx), "no");
+}
+
 enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 			      struct list **argv, const struct cmd_element **el)
 {
@@ -88,7 +95,7 @@ enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 
 	// prepend a dummy token to match that pesky start node
 	vector vvline = vector_init(vline->alloced + 1);
-	vector_set_index(vvline, 0, (void *)XSTRDUP(MTYPE_TMP, "dummy"));
+	vector_set_index(vvline, 0, XSTRDUP(MTYPE_TMP, "dummy"));
 	memcpy(vvline->index + 1, vline->index,
 	       sizeof(void *) * vline->alloced);
 	vvline->active = vline->active + 1;
@@ -195,7 +202,7 @@ static enum matcher_rv command_match_r(struct graph_node *start, vector vline,
 	enum matcher_rv status = MATCHER_NO_MATCH;
 
 	// get the minimum match level that can count as a full match
-	struct cmd_token *token = start->data;
+	struct cmd_token *copy, *token = start->data;
 	enum match_type minmatch = min_match_level(token->type);
 
 	/* check history/stack of tokens
@@ -248,7 +255,7 @@ static enum matcher_rv command_match_r(struct graph_node *start, vector vline,
 
 	// get all possible nexthops
 	struct list *next = list_new();
-	add_nexthops(next, start, NULL, 0);
+	add_nexthops(next, start, NULL, 0, is_neg(vline, 1));
 
 	// determine the best match
 	for (ALL_LIST_ELEMENTS_RO(next, ln, gn)) {
@@ -326,15 +333,15 @@ static enum matcher_rv command_match_r(struct graph_node *start, vector vline,
 	}
 	if (*currbest) {
 		// copy token, set arg and prepend to currbest
-		struct cmd_token *token = start->data;
-		struct cmd_token *copy = cmd_token_dup(token);
+		token = start->data;
+		copy = cmd_token_dup(token);
 		copy->arg = XSTRDUP(MTYPE_CMD_ARG, input_token);
 		listnode_add_before(*currbest, (*currbest)->head, copy);
 	} else if (n + 1 == vector_active(vline) && status == MATCHER_NO_MATCH)
 		status = MATCHER_INCOMPLETE;
 
 	// cleanup
-	list_delete_and_null(&next);
+	list_delete(&next);
 
 	return status;
 }
@@ -349,6 +356,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 {
 	// pointer to next input token to match
 	char *input_token;
+	bool neg = is_neg(vline, 0);
 
 	struct list *
 		current =
@@ -363,11 +371,11 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 
 	// add all children of start node to list
 	struct graph_node *start = vector_slot(graph->nodes, 0);
-	add_nexthops(next, start, &start, 0);
+	add_nexthops(next, start, &start, 0, neg);
 
 	unsigned int idx;
 	for (idx = 0; idx < vector_active(vline) && next->count > 0; idx++) {
-		list_delete_and_null(&current);
+		list_delete(&current);
 		current = next;
 		next = list_new();
 		next->del = stack_del;
@@ -387,8 +395,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 		for (ALL_LIST_ELEMENTS_RO(current, node, gstack)) {
 			struct cmd_token *token = gstack[0]->data;
 
-			if (token->attr == CMD_ATTR_HIDDEN
-			    || token->attr == CMD_ATTR_DEPRECATED)
+			if (token->attr & CMD_ATTR_HIDDEN)
 				continue;
 
 			enum match_type minmatch = min_match_level(token->type);
@@ -428,7 +435,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 					listnode_add(next, newstack);
 				} else if (matchtype >= minmatch)
 					add_nexthops(next, gstack[0], gstack,
-						     idx + 1);
+						     idx + 1, neg);
 				break;
 			default:
 				trace_matcher("no_match\n");
@@ -458,8 +465,8 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 		}
 	}
 
-	list_delete_and_null(&current);
-	list_delete_and_null(&next);
+	list_delete(&current);
+	list_delete(&next);
 
 	return mrv;
 }
@@ -478,7 +485,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
  * output, instead of direct node pointers!
  */
 static int add_nexthops(struct list *list, struct graph_node *node,
-			struct graph_node **stack, size_t stackpos)
+			struct graph_node **stack, size_t stackpos, bool neg)
 {
 	int added = 0;
 	struct graph_node *child;
@@ -494,8 +501,13 @@ static int add_nexthops(struct list *list, struct graph_node *node,
 			if (j != stackpos)
 				continue;
 		}
+
+		if (token->type == NEG_ONLY_TKN && !neg)
+			continue;
+
 		if (token->type >= SPECIAL_TKN && token->type != END_TKN) {
-			added += add_nexthops(list, child, stack, stackpos);
+			added +=
+				add_nexthops(list, child, stack, stackpos, neg);
 		} else {
 			if (stack) {
 				nextstack = XMALLOC(
@@ -608,11 +620,14 @@ static struct cmd_token *disambiguate_tokens(struct cmd_token *first,
 static struct list *disambiguate(struct list *first, struct list *second,
 				 vector vline, unsigned int n)
 {
+	assert(first != NULL);
+	assert(second != NULL);
 	// doesn't make sense for these to be inequal length
 	assert(first->count == second->count);
 	assert(first->count == vector_active(vline) - n + 1);
 
-	struct listnode *fnode = listhead(first), *snode = listhead(second);
+	struct listnode *fnode = listhead_unchecked(first),
+			*snode = listhead_unchecked(second);
 	struct cmd_token *ftok = listgetdata(fnode), *stok = listgetdata(snode),
 			 *best = NULL;
 
@@ -649,7 +664,7 @@ static void del_arglist(struct list *list)
 	list_delete_node(list, tail);
 
 	// delete the rest of the list as usual
-	list_delete_and_null(&list);
+	list_delete(&list);
 }
 
 /*---------- token level matching functions ----------*/
@@ -711,7 +726,7 @@ static enum match_type match_ipv4(const char *str)
 				dots++;
 				break;
 			}
-			if (!isdigit((int)*str))
+			if (!isdigit((unsigned char)*str))
 				return no_match;
 
 			str++;
@@ -720,8 +735,13 @@ static enum match_type match_ipv4(const char *str)
 		if (str - sp > 3)
 			return no_match;
 
-		strncpy(buf, sp, str - sp);
-		if (atoi(buf) > 255)
+		memcpy(buf, sp, str - sp);
+
+		int v = atoi(buf);
+
+		if (v > 255)
+			return no_match;
+		if (v > 0 && buf[0] == '0')
 			return no_match;
 
 		nums++;
@@ -762,7 +782,7 @@ static enum match_type match_ipv4_prefix(const char *str)
 				break;
 			}
 
-			if (!isdigit((int)*str))
+			if (!isdigit((unsigned char)*str))
 				return no_match;
 
 			str++;
@@ -771,8 +791,13 @@ static enum match_type match_ipv4_prefix(const char *str)
 		if (str - sp > 3)
 			return no_match;
 
-		strncpy(buf, sp, str - sp);
-		if (atoi(buf) > 255)
+		memcpy(buf, sp, str - sp);
+
+		int v = atoi(buf);
+
+		if (v > 255)
+			return no_match;
+		if (v > 0 && buf[0] == '0')
 			return no_match;
 
 		if (dots == 3) {
@@ -794,13 +819,13 @@ static enum match_type match_ipv4_prefix(const char *str)
 
 	sp = str;
 	while (*str != '\0') {
-		if (!isdigit((int)*str))
+		if (!isdigit((unsigned char)*str))
 			return no_match;
 
 		str++;
 	}
 
-	if (atoi(sp) > 32)
+	if (atoi(sp) > IPV4_MAX_BITLEN)
 		return no_match;
 
 	return exact_match;
@@ -935,7 +960,7 @@ static enum match_type match_ipv6_prefix(const char *str, bool prefix)
 	if (*endptr != '\0')
 		return no_match;
 
-	if (mask < 0 || mask > 128)
+	if (mask < 0 || mask > IPV6_MAX_BITLEN)
 		return no_match;
 
 	return exact_match;
