@@ -14,6 +14,10 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
@@ -29,8 +33,9 @@
 #include "command.h"
 #include "json.h"
 #include "linklist.h"
+#include "frr_pthread.h"
 
-DEFINE_MTYPE_STATIC(LIB, ERRINFO, "error information")
+DEFINE_MTYPE_STATIC(LIB, ERRINFO, "error information");
 
 /*
  * Thread-specific key for temporary storage of allocated ferr.
@@ -57,10 +62,10 @@ static void err_key_fini(void)
 /*
  * Global shared hash table holding reference text for all defined errors.
  */
-pthread_mutex_t refs_mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t refs_mtx = PTHREAD_MUTEX_INITIALIZER;
 struct hash *refs;
 
-static int ferr_hash_cmp(const void *a, const void *b)
+static bool ferr_hash_cmp(const void *a, const void *b)
 {
 	const struct log_ref *f_a = a;
 	const struct log_ref *f_b = b;
@@ -68,9 +73,9 @@ static int ferr_hash_cmp(const void *a, const void *b)
 	return f_a->code == f_b->code;
 }
 
-static inline unsigned int ferr_hash_key(void *a)
+static inline unsigned int ferr_hash_key(const void *a)
 {
-	struct log_ref *f = a;
+	const struct log_ref *f = a;
 
 	return f->code;
 }
@@ -79,14 +84,12 @@ void log_ref_add(struct log_ref *ref)
 {
 	uint32_t i = 0;
 
-	pthread_mutex_lock(&refs_mtx);
-	{
+	frr_with_mutex (&refs_mtx) {
 		while (ref[i].code != END_FERR) {
-			hash_get(refs, &ref[i], hash_alloc_intern);
+			(void)hash_get(refs, &ref[i], hash_alloc_intern);
 			i++;
 		}
 	}
-	pthread_mutex_unlock(&refs_mtx);
 }
 
 struct log_ref *log_ref_get(uint32_t code)
@@ -95,11 +98,9 @@ struct log_ref *log_ref_get(uint32_t code)
 	struct log_ref *ref;
 
 	holder.code = code;
-	pthread_mutex_lock(&refs_mtx);
-	{
+	frr_with_mutex (&refs_mtx) {
 		ref = hash_lookup(refs, &holder);
 	}
-	pthread_mutex_unlock(&refs_mtx);
 
 	return ref;
 }
@@ -114,16 +115,16 @@ void log_ref_display(struct vty *vty, uint32_t code, bool json)
 	if (json)
 		top = json_object_new_object();
 
-	pthread_mutex_lock(&refs_mtx);
-	{
+	frr_with_mutex (&refs_mtx) {
 		errlist = code ? list_new() : hash_to_list(refs);
 	}
-	pthread_mutex_unlock(&refs_mtx);
 
 	if (code) {
 		ref = log_ref_get(code);
 		if (!ref) {
-			vty_out(vty, "Code %"PRIu32" - Unknown\n", code);
+			if (top)
+				json_object_free(top);
+			list_delete(&errlist);
 			return;
 		}
 		listnode_add(errlist, ref);
@@ -133,7 +134,7 @@ void log_ref_display(struct vty *vty, uint32_t code, bool json)
 		if (json) {
 			char key[11];
 
-			snprintf(key, sizeof(key), "%"PRIu32, ref->code);
+			snprintf(key, sizeof(key), "%u", ref->code);
 			obj = json_object_new_object();
 			json_object_string_add(obj, "title", ref->title);
 			json_object_string_add(obj, "description",
@@ -145,10 +146,10 @@ void log_ref_display(struct vty *vty, uint32_t code, bool json)
 			char pbuf[256];
 			char ubuf[256];
 
-			snprintf(pbuf, sizeof(pbuf), "\nError %"PRIu32" - %s",
+			snprintf(pbuf, sizeof(pbuf), "\nError %u - %s",
 				 ref->code, ref->title);
 			memset(ubuf, '=', strlen(pbuf));
-			ubuf[sizeof(ubuf) - 1] = '\0';
+			ubuf[strlen(pbuf)] = '\0';
 
 			vty_out(vty, "%s\n%s\n", pbuf, ubuf);
 			vty_out(vty, "Description:\n%s\n\n", ref->description);
@@ -156,19 +157,13 @@ void log_ref_display(struct vty *vty, uint32_t code, bool json)
 		}
 	}
 
-	if (json) {
-		const char *str = json_object_to_json_string_ext(
-			top, JSON_C_TO_STRING_PRETTY);
-		vty_out(vty, "%s\n", str);
-		json_object_free(top);
-	}
-
-	list_delete_and_null(&errlist);
+	vty_json(vty, top);
+	list_delete(&errlist);
 }
 
 DEFUN_NOSH(show_error_code,
 	   show_error_code_cmd,
-	   "show error <(1-4294967296)|all> [json]",
+	   "show error <(1-4294967295)|all> [json]",
 	   SHOW_STR
 	   "Information on errors\n"
 	   "Error code to get info about\n"
@@ -187,27 +182,27 @@ DEFUN_NOSH(show_error_code,
 
 void log_ref_init(void)
 {
-	pthread_mutex_lock(&refs_mtx);
-	{
+	frr_with_mutex (&refs_mtx) {
 		refs = hash_create(ferr_hash_key, ferr_hash_cmp,
 				   "Error Reference Texts");
 	}
-	pthread_mutex_unlock(&refs_mtx);
-
-	install_element(VIEW_NODE, &show_error_code_cmd);
 }
 
 void log_ref_fini(void)
 {
-	pthread_mutex_lock(&refs_mtx);
-	{
+	frr_with_mutex (&refs_mtx) {
 		hash_clean(refs, NULL);
 		hash_free(refs);
 		refs = NULL;
 	}
-	pthread_mutex_unlock(&refs_mtx);
 }
 
+void log_ref_vty_init(void)
+{
+	install_element(VIEW_NODE, &show_error_code_cmd);
+}
+
+
 const struct ferr *ferr_get_last(ferr_r errval)
 {
 	struct ferr *last_error = pthread_getspecific(errkey);
@@ -224,6 +219,7 @@ ferr_r ferr_clear(void)
 	return ferr_ok();
 }
 
+PRINTFRR(7, 0)
 static ferr_r ferr_set_va(const char *file, int line, const char *func,
 			  enum ferr_kind kind, const char *pathname,
 			  int errno_val, const char *text, va_list va)
