@@ -49,13 +49,12 @@ int main(int argc, char **argv)
 
 	/* Library inits. */
 	cmd_init(1);
-	host.name = strdup("test");
-	host.domainname = strdup("testdomainname");
+	cmd_hostname_set("test");
+	cmd_domainname_set("testdomainname");
 
 	vty_init(master, true);
 	lib_cmd_init();
-	yang_init(true);
-	nb_init(master, NULL, 0);
+	nb_init(master, NULL, 0, false);
 
 	vty_stdio(vty_do_exit);
 
