@@ -64,7 +64,7 @@ const char *lookup_msg(const struct message *mz, int kz, const char *nf)
 }
 
 /* For time string format. */
-size_t quagga_timestamp(int timestamp_precision, char *buf, size_t buflen)
+size_t frr_timestamp(int timestamp_precision, char *buf, size_t buflen)
 {
 	static struct {
 		time_t last;
@@ -161,8 +161,9 @@ void zlog_signal(int signo, const char *action, void *siginfo_v,
 	if (!tc)
 		bprintfrr(&fb, "no thread information available\n");
 	else
-		bprintfrr(&fb, "in thread %s scheduled from %s:%d\n",
-			  tc->funcname, tc->schedfrom, tc->schedfrom_line);
+		bprintfrr(&fb, "in thread %s scheduled from %s:%d %s()\n",
+			  tc->xref->funcname, tc->xref->xref.file,
+			  tc->xref->xref.line, tc->xref->xref.func);
 
 	zlog_sigsafe(fb.buf, fb.pos - fb.buf);
 }
@@ -179,6 +180,9 @@ void zlog_backtrace_sigsafe(int priority, void *program_counter)
 	unw_word_t ip, off, sp;
 	Dl_info dlinfo;
 
+	memset(&uc, 0, sizeof(uc));
+	memset(&cursor, 0, sizeof(cursor));
+
 	unw_getcontext(&uc);
 	unw_init_local(&cursor, &uc);
 	while (unw_step(&cursor) > 0) {
@@ -235,7 +239,7 @@ void zlog_backtrace(int priority)
 {
 #ifdef HAVE_LIBUNWIND
 	char buf[100];
-	unw_cursor_t cursor;
+	unw_cursor_t cursor = {};
 	unw_context_t uc;
 	unw_word_t ip, off, sp;
 	Dl_info dlinfo;
@@ -273,8 +277,7 @@ void zlog_backtrace(int priority)
 	if (size <= 0 || (size_t)size > array_size(array)) {
 		flog_err_sys(
 			EC_LIB_SYSTEM_CALL,
-			"Cannot get backtrace, returned invalid # of frames %d "
-			"(valid range is between 1 and %lu)",
+			"Cannot get backtrace, returned invalid # of frames %d (valid range is between 1 and %lu)",
 			size, (unsigned long)(array_size(array)));
 		return;
 	}
@@ -301,24 +304,13 @@ void zlog_thread_info(int log_level)
 
 	if (tc)
 		zlog(log_level,
-		     "Current thread function %s, scheduled from "
-		     "file %s, line %u",
-		     tc->funcname, tc->schedfrom, tc->schedfrom_line);
+		     "Current thread function %s, scheduled from file %s, line %u in %s()",
+		     tc->xref->funcname, tc->xref->xref.file,
+		     tc->xref->xref.line, tc->xref->xref.func);
 	else
 		zlog(log_level, "Current thread not known/applicable");
 }
 
-void _zlog_assert_failed(const char *assertion, const char *file,
-			 unsigned int line, const char *function)
-{
-	zlog(LOG_CRIT, "Assertion `%s' failed in file %s, line %u, function %s",
-	     assertion, file, line, (function ? function : "?"));
-	zlog_backtrace(LOG_CRIT);
-	zlog_thread_info(LOG_CRIT);
-	log_memstats(stderr, "log");
-	abort();
-}
-
 void memory_oom(size_t size, const char *name)
 {
 	zlog(LOG_CRIT,
@@ -345,6 +337,7 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_INTERFACE_UP),
 	DESC_ENTRY(ZEBRA_INTERFACE_DOWN),
 	DESC_ENTRY(ZEBRA_INTERFACE_SET_MASTER),
+	DESC_ENTRY(ZEBRA_INTERFACE_SET_PROTODOWN),
 	DESC_ENTRY(ZEBRA_ROUTE_ADD),
 	DESC_ENTRY(ZEBRA_ROUTE_DELETE),
 	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_OWNER),
@@ -363,9 +356,6 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_INTERFACE_NBR_ADDRESS_ADD),
 	DESC_ENTRY(ZEBRA_INTERFACE_NBR_ADDRESS_DELETE),
 	DESC_ENTRY(ZEBRA_INTERFACE_BFD_DEST_UPDATE),
-	DESC_ENTRY(ZEBRA_IMPORT_ROUTE_REGISTER),
-	DESC_ENTRY(ZEBRA_IMPORT_ROUTE_UNREGISTER),
-	DESC_ENTRY(ZEBRA_IMPORT_CHECK_UPDATE),
 	DESC_ENTRY(ZEBRA_BFD_DEST_REGISTER),
 	DESC_ENTRY(ZEBRA_BFD_DEST_DEREGISTER),
 	DESC_ENTRY(ZEBRA_BFD_DEST_UPDATE),
@@ -381,11 +371,14 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_BFD_CLIENT_DEREGISTER),
 	DESC_ENTRY(ZEBRA_INTERFACE_ENABLE_RADV),
 	DESC_ENTRY(ZEBRA_INTERFACE_DISABLE_RADV),
-	DESC_ENTRY(ZEBRA_IPV4_NEXTHOP_LOOKUP_MRIB),
+	DESC_ENTRY(ZEBRA_NEXTHOP_LOOKUP_MRIB),
 	DESC_ENTRY(ZEBRA_INTERFACE_LINK_PARAMS),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_ADD),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_DELETE),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_REPLACE),
+	DESC_ENTRY(ZEBRA_SR_POLICY_SET),
+	DESC_ENTRY(ZEBRA_SR_POLICY_DELETE),
+	DESC_ENTRY(ZEBRA_SR_POLICY_NOTIFY_STATUS),
 	DESC_ENTRY(ZEBRA_IPMR_ROUTE_STATS),
 	DESC_ENTRY(ZEBRA_LABEL_MANAGER_CONNECT),
 	DESC_ENTRY(ZEBRA_LABEL_MANAGER_CONNECT_ASYNC),
@@ -394,12 +387,16 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_FEC_REGISTER),
 	DESC_ENTRY(ZEBRA_FEC_UNREGISTER),
 	DESC_ENTRY(ZEBRA_FEC_UPDATE),
-	DESC_ENTRY(ZEBRA_ADVERTISE_ALL_VNI),
 	DESC_ENTRY(ZEBRA_ADVERTISE_DEFAULT_GW),
 	DESC_ENTRY(ZEBRA_ADVERTISE_SVI_MACIP),
 	DESC_ENTRY(ZEBRA_ADVERTISE_SUBNET),
+	DESC_ENTRY(ZEBRA_ADVERTISE_ALL_VNI),
 	DESC_ENTRY(ZEBRA_LOCAL_ES_ADD),
 	DESC_ENTRY(ZEBRA_LOCAL_ES_DEL),
+	DESC_ENTRY(ZEBRA_REMOTE_ES_VTEP_ADD),
+	DESC_ENTRY(ZEBRA_REMOTE_ES_VTEP_DEL),
+	DESC_ENTRY(ZEBRA_LOCAL_ES_EVI_ADD),
+	DESC_ENTRY(ZEBRA_LOCAL_ES_EVI_DEL),
 	DESC_ENTRY(ZEBRA_VNI_ADD),
 	DESC_ENTRY(ZEBRA_VNI_DEL),
 	DESC_ENTRY(ZEBRA_L3VNI_ADD),
@@ -442,8 +439,40 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_MLAG_CLIENT_REGISTER),
 	DESC_ENTRY(ZEBRA_MLAG_CLIENT_UNREGISTER),
 	DESC_ENTRY(ZEBRA_MLAG_FORWARD_MSG),
+	DESC_ENTRY(ZEBRA_NHG_ADD),
+	DESC_ENTRY(ZEBRA_NHG_DEL),
+	DESC_ENTRY(ZEBRA_NHG_NOTIFY_OWNER),
+	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_ADD),
+	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_DEL),
+	DESC_ENTRY(ZEBRA_SRV6_LOCATOR_ADD),
+	DESC_ENTRY(ZEBRA_SRV6_LOCATOR_DELETE),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_LOCATOR_CHUNK),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_RELEASE_LOCATOR_CHUNK),
 	DESC_ENTRY(ZEBRA_ERROR),
-	DESC_ENTRY(ZEBRA_CLIENT_CAPABILITIES)};
+	DESC_ENTRY(ZEBRA_CLIENT_CAPABILITIES),
+	DESC_ENTRY(ZEBRA_OPAQUE_MESSAGE),
+	DESC_ENTRY(ZEBRA_OPAQUE_REGISTER),
+	DESC_ENTRY(ZEBRA_OPAQUE_UNREGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_DISCOVER),
+	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_REQUEST),
+	DESC_ENTRY(ZEBRA_CLIENT_CLOSE_NOTIFY),
+	DESC_ENTRY(ZEBRA_NHRP_NEIGH_ADDED),
+	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REMOVED),
+	DESC_ENTRY(ZEBRA_NHRP_NEIGH_GET),
+	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REGISTER),
+	DESC_ENTRY(ZEBRA_NHRP_NEIGH_UNREGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_IP_ADD),
+	DESC_ENTRY(ZEBRA_NEIGH_IP_DEL),
+	DESC_ENTRY(ZEBRA_CONFIGURE_ARP),
+	DESC_ENTRY(ZEBRA_GRE_GET),
+	DESC_ENTRY(ZEBRA_GRE_UPDATE),
+	DESC_ENTRY(ZEBRA_GRE_SOURCE_SET),
+	DESC_ENTRY(ZEBRA_TC_QDISC_INSTALL),
+	DESC_ENTRY(ZEBRA_TC_QDISC_UNINSTALL),
+	DESC_ENTRY(ZEBRA_TC_CLASS_ADD),
+	DESC_ENTRY(ZEBRA_TC_CLASS_DELETE),
+	DESC_ENTRY(ZEBRA_TC_FILTER_ADD),
+	DESC_ENTRY(ZEBRA_TC_FILTER_DELETE)};
 #undef DESC_ENTRY
 
 static const struct zebra_desc_table unknown = {0, "unknown", '?'};
@@ -462,8 +491,7 @@ static const struct zebra_desc_table *zroute_lookup(unsigned int zroute)
 	for (i = 0; i < array_size(route_types); i++) {
 		if (zroute == route_types[i].type) {
 			zlog_warn(
-				"internal error: route type table out of order "
-				"while searching for %u, please notify developers",
+				"internal error: route type table out of order while searching for %u, please notify developers",
 				zroute);
 			return &route_types[i];
 		}
