@@ -21,7 +21,9 @@
 
 #include <zebra.h>
 
+#include "command.h"
 #include "prefix.h"
+#include "ipaddr.h"
 #include "vty.h"
 #include "sockunion.h"
 #include "memory.h"
@@ -29,9 +31,10 @@
 #include "jhash.h"
 #include "lib_errors.h"
 #include "printfrr.h"
+#include "vxlan.h"
 
-DEFINE_MTYPE_STATIC(LIB, PREFIX, "Prefix")
-DEFINE_MTYPE_STATIC(LIB, PREFIX_FLOWSPEC, "Prefix Flowspec")
+DEFINE_MTYPE_STATIC(LIB, PREFIX, "Prefix");
+DEFINE_MTYPE_STATIC(LIB, PREFIX_FLOWSPEC, "Prefix Flowspec");
 
 /* Maskbit. */
 static const uint8_t maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
@@ -75,10 +78,10 @@ bool is_mcast_mac(const struct ethaddr *mac)
 	return false;
 }
 
-unsigned int prefix_bit(const uint8_t *prefix, const uint16_t prefixlen)
+unsigned int prefix_bit(const uint8_t *prefix, const uint16_t bit_index)
 {
-	unsigned int offset = prefixlen / 8;
-	unsigned int shift = 7 - (prefixlen % 8);
+	unsigned int offset = bit_index / 8;
+	unsigned int shift = 7 - (bit_index % 8);
 
 	return (prefix[offset] >> shift) & 1;
 }
@@ -111,7 +114,7 @@ const char *family2str(int family)
 	return "?";
 }
 
-/* Address Famiy Identifier to Address Family converter. */
+/* Address Family Identifier to Address Family converter. */
 int afi2family(afi_t afi)
 {
 	if (afi == AFI_IP)
@@ -135,6 +138,23 @@ afi_t family2afi(int family)
 	return 0;
 }
 
+const char *afi2str_lower(afi_t afi)
+{
+	switch (afi) {
+	case AFI_IP:
+		return "ipv4";
+	case AFI_IP6:
+		return "ipv6";
+	case AFI_L2VPN:
+		return "l2vpn";
+	case AFI_MAX:
+	case AFI_UNSPEC:
+		return "bad-value";
+	}
+
+	assert(!"Reached end of function we should never reach");
+}
+
 const char *afi2str(afi_t afi)
 {
 	switch (afi) {
@@ -175,8 +195,10 @@ const char *safi2str(safi_t safi)
 }
 
 /* If n includes p prefix then return 1 else return 0. */
-int prefix_match(const struct prefix *n, const struct prefix *p)
+int prefix_match(union prefixconstptr unet, union prefixconstptr upfx)
 {
+	const struct prefix *n = unet.p;
+	const struct prefix *p = upfx.p;
 	int offset;
 	int shift;
 	const uint8_t *np, *pp;
@@ -187,6 +209,10 @@ int prefix_match(const struct prefix *n, const struct prefix *p)
 
 	if (n->family == AF_FLOWSPEC) {
 		/* prefixlen is unused. look at fs prefix len */
+		if (n->u.prefix_flowspec.family !=
+		    p->u.prefix_flowspec.family)
+			return 0;
+
 		if (n->u.prefix_flowspec.prefixlen >
 		    p->u.prefix_flowspec.prefixlen)
 			return 0;
@@ -268,9 +294,11 @@ int evpn_type5_prefix_match(const struct prefix *n, const struct prefix *p)
 }
 
 /* If n includes p then return 1 else return 0. Prefix mask is not considered */
-int prefix_match_network_statement(const struct prefix *n,
-				   const struct prefix *p)
+int prefix_match_network_statement(union prefixconstptr unet,
+				   union prefixconstptr upfx)
 {
+	const struct prefix *n = unet.p;
+	const struct prefix *p = upfx.p;
 	int offset;
 	int shift;
 	const uint8_t *np, *pp;
@@ -324,6 +352,8 @@ void prefix_copy(union prefixptr udest, union prefixconstptr usrc)
 		len = src->u.prefix_flowspec.prefixlen;
 		dest->u.prefix_flowspec.prefixlen =
 			src->u.prefix_flowspec.prefixlen;
+		dest->u.prefix_flowspec.family =
+			src->u.prefix_flowspec.family;
 		dest->family = src->family;
 		temp = XCALLOC(MTYPE_PREFIX_FLOWSPEC, len);
 		dest->u.prefix_flowspec.ptr = (uintptr_t)temp;
@@ -373,6 +403,9 @@ int prefix_same(union prefixconstptr up1, union prefixconstptr up2)
 				    sizeof(struct evpn_addr)))
 				return 1;
 		if (p1->family == AF_FLOWSPEC) {
+			if (p1->u.prefix_flowspec.family !=
+			    p2->u.prefix_flowspec.family)
+				return 0;
 			if (p1->u.prefix_flowspec.prefixlen !=
 			    p2->u.prefix_flowspec.prefixlen)
 				return 0;
@@ -414,6 +447,10 @@ int prefix_cmp(union prefixconstptr up1, union prefixconstptr up2)
 		pp1 = (const uint8_t *)p1->u.prefix_flowspec.ptr;
 		pp2 = (const uint8_t *)p2->u.prefix_flowspec.ptr;
 
+		if (p1->u.prefix_flowspec.family !=
+		    p2->u.prefix_flowspec.family)
+			return 1;
+
 		if (p1->u.prefix_flowspec.prefixlen !=
 		    p2->u.prefix_flowspec.prefixlen)
 			return numcmp(p1->u.prefix_flowspec.prefixlen,
@@ -457,8 +494,10 @@ int prefix_cmp(union prefixconstptr up1, union prefixconstptr up2)
  * address families don't match, return -1; otherwise the return value is
  * in range 0 ... maximum prefix length for the address family.
  */
-int prefix_common_bits(const struct prefix *p1, const struct prefix *p2)
+int prefix_common_bits(union prefixconstptr ua, union prefixconstptr ub)
 {
+	const struct prefix *p1 = ua.p;
+	const struct prefix *p2 = ub.p;
 	int pos, bit;
 	int length = 0;
 	uint8_t xor ;
@@ -494,8 +533,10 @@ int prefix_common_bits(const struct prefix *p1, const struct prefix *p2)
 }
 
 /* Return prefix family type string. */
-const char *prefix_family_str(const struct prefix *p)
+const char *prefix_family_str(union prefixconstptr pu)
 {
+	const struct prefix *p = pu.p;
+
 	if (p->family == AF_INET)
 		return "inet";
 	if (p->family == AF_INET6)
@@ -562,7 +603,7 @@ int str2prefix_ipv4(const char *str, struct prefix_ipv4 *p)
 
 		/* Get prefix length. */
 		plen = (uint8_t)atoi(++pnt);
-		if (plen > IPV4_MAX_PREFIXLEN)
+		if (plen > IPV4_MAX_BITLEN)
 			return 0;
 
 		p->family = AF_INET;
@@ -800,14 +841,16 @@ void apply_mask_ipv6(struct prefix_ipv6 *p)
 	}
 }
 
-void apply_mask(struct prefix *p)
+void apply_mask(union prefixptr pu)
 {
+	struct prefix *p = pu.p;
+
 	switch (p->family) {
 	case AF_INET:
-		apply_mask_ipv4((struct prefix_ipv4 *)p);
+		apply_mask_ipv4(pu.p4);
 		break;
 	case AF_INET6:
-		apply_mask_ipv6((struct prefix_ipv6 *)p);
+		apply_mask_ipv6(pu.p6);
 		break;
 	default:
 		break;
@@ -853,8 +896,10 @@ void prefix2sockunion(const struct prefix *p, union sockunion *su)
 		       sizeof(struct in6_addr));
 }
 
-int prefix_blen(const struct prefix *p)
+int prefix_blen(union prefixconstptr pu)
 {
+	const struct prefix *p = pu.p;
+
 	switch (p->family) {
 	case AF_INET:
 		return IPV4_MAX_BYTELEN;
@@ -895,7 +940,18 @@ int str2prefix(const char *str, struct prefix *p)
 static const char *prefixevpn_ead2str(const struct prefix_evpn *p, char *str,
 				      int size)
 {
-	snprintf(str, size, "Unsupported EVPN prefix");
+	uint8_t family;
+	char buf[ESI_STR_LEN];
+	char buf1[INET6_ADDRSTRLEN];
+
+	family = IS_IPADDR_V4(&p->prefix.ead_addr.ip) ? AF_INET : AF_INET6;
+	snprintf(str, size, "[%d]:[%u]:[%s]:[%d]:[%s]:[%u]",
+		 p->prefix.route_type, p->prefix.ead_addr.eth_tag,
+		 esi_to_str(&p->prefix.ead_addr.esi, buf, sizeof(buf)),
+		 (family == AF_INET) ? IPV4_MAX_BITLEN : IPV6_MAX_BITLEN,
+		 inet_ntop(family, &p->prefix.ead_addr.ip.ipaddr_v4, buf1,
+			   sizeof(buf1)),
+		 p->prefix.ead_addr.frag_id);
 	return str;
 }
 
@@ -903,27 +959,24 @@ static const char *prefixevpn_macip2str(const struct prefix_evpn *p, char *str,
 					int size)
 {
 	uint8_t family;
-	char buf[PREFIX2STR_BUFFER];
-	char buf2[ETHER_ADDR_STRLEN];
+	char buf1[ETHER_ADDR_STRLEN];
+	char buf2[PREFIX2STR_BUFFER];
 
 	if (is_evpn_prefix_ipaddr_none(p))
-		snprintf(str, size, "[%d]:[%s]/%d",
-			 p->prefix.route_type,
-			 prefix_mac2str(&p->prefix.macip_addr.mac,
-					buf2, sizeof(buf2)),
-			 p->prefixlen);
+		snprintf(str, size, "[%d]:[%d]:[%d]:[%s]", p->prefix.route_type,
+			 p->prefix.macip_addr.eth_tag, 8 * ETH_ALEN,
+			 prefix_mac2str(&p->prefix.macip_addr.mac, buf1,
+					sizeof(buf1)));
 	else {
-		family = is_evpn_prefix_ipaddr_v4(p)
-				 ? AF_INET
-				 : AF_INET6;
-		snprintf(str, size, "[%d]:[%s]:[%s]/%d",
-			 p->prefix.route_type,
-			 prefix_mac2str(&p->prefix.macip_addr.mac,
-					buf2, sizeof(buf2)),
-			 inet_ntop(family,
-				   &p->prefix.macip_addr.ip.ip.addr,
-				   buf, PREFIX2STR_BUFFER),
-			 p->prefixlen);
+		family = is_evpn_prefix_ipaddr_v4(p) ? AF_INET : AF_INET6;
+		snprintf(str, size, "[%d]:[%d]:[%d]:[%s]:[%d]:[%s]",
+			 p->prefix.route_type, p->prefix.macip_addr.eth_tag,
+			 8 * ETH_ALEN,
+			 prefix_mac2str(&p->prefix.macip_addr.mac, buf1,
+					sizeof(buf1)),
+			 family == AF_INET ? IPV4_MAX_BITLEN : IPV6_MAX_BITLEN,
+			 inet_ntop(family, &p->prefix.macip_addr.ip.ip.addr,
+				   buf2, PREFIX2STR_BUFFER));
 	}
 	return str;
 }
@@ -932,28 +985,32 @@ static const char *prefixevpn_imet2str(const struct prefix_evpn *p, char *str,
 				       int size)
 {
 	uint8_t family;
-	char buf[PREFIX2STR_BUFFER];
+	char buf[INET6_ADDRSTRLEN];
+
+	family = IS_IPADDR_V4(&p->prefix.imet_addr.ip) ? AF_INET : AF_INET6;
+	snprintf(str, size, "[%d]:[%d]:[%d]:[%s]", p->prefix.route_type,
+		 p->prefix.imet_addr.eth_tag,
+		 (family == AF_INET) ? IPV4_MAX_BITLEN : IPV6_MAX_BITLEN,
+		 inet_ntop(family, &p->prefix.imet_addr.ip.ipaddr_v4, buf,
+			   sizeof(buf)));
 
-	family = is_evpn_prefix_ipaddr_v4(p)
-			 ? AF_INET
-			 : AF_INET6;
-	snprintf(str, size, "[%d]:[%s]/%d", p->prefix.route_type,
-		 inet_ntop(family,
-			   &p->prefix.imet_addr.ip.ip.addr, buf,
-			   PREFIX2STR_BUFFER),
-		 p->prefixlen);
 	return str;
 }
 
 static const char *prefixevpn_es2str(const struct prefix_evpn *p, char *str,
 				     int size)
 {
+	uint8_t family;
 	char buf[ESI_STR_LEN];
+	char buf1[INET6_ADDRSTRLEN];
 
-	snprintf(str, size, "[%d]:[%s]:[%s]/%d", p->prefix.route_type,
+	family = IS_IPADDR_V4(&p->prefix.es_addr.ip) ? AF_INET : AF_INET6;
+	snprintf(str, size, "[%d]:[%s]:[%d]:[%s]", p->prefix.route_type,
 		 esi_to_str(&p->prefix.es_addr.esi, buf, sizeof(buf)),
-		 inet_ntoa(p->prefix.es_addr.ip.ipaddr_v4),
-		 p->prefixlen);
+		 (family == AF_INET) ? IPV4_MAX_BITLEN : IPV6_MAX_BITLEN,
+		 inet_ntop(family, &p->prefix.es_addr.ip.ipaddr_v4, buf1,
+			   sizeof(buf1)));
+
 	return str;
 }
 
@@ -961,19 +1018,14 @@ static const char *prefixevpn_prefix2str(const struct prefix_evpn *p, char *str,
 					 int size)
 {
 	uint8_t family;
-	char buf[PREFIX2STR_BUFFER];
+	char buf[INET6_ADDRSTRLEN];
 
-	family = is_evpn_prefix_ipaddr_v4(p)
-			 ? AF_INET
-			 : AF_INET6;
-	snprintf(str, size, "[%d]:[%u][%s/%d]/%d",
-		 p->prefix.route_type,
+	family = IS_IPADDR_V4(&p->prefix.prefix_addr.ip) ? AF_INET : AF_INET6;
+	snprintf(str, size, "[%d]:[%d]:[%d]:[%s]", p->prefix.route_type,
 		 p->prefix.prefix_addr.eth_tag,
-		 inet_ntop(family,
-			   &p->prefix.prefix_addr.ip.ip.addr, buf,
-			   PREFIX2STR_BUFFER),
 		 p->prefix.prefix_addr.ip_prefix_length,
-		 p->prefixlen);
+		 inet_ntop(family, &p->prefix.prefix_addr.ip.ipaddr_v4, buf,
+			   sizeof(buf)));
 	return str;
 }
 
@@ -981,15 +1033,15 @@ static const char *prefixevpn2str(const struct prefix_evpn *p, char *str,
 				  int size)
 {
 	switch (p->prefix.route_type) {
-	case 1:
+	case BGP_EVPN_AD_ROUTE:
 		return prefixevpn_ead2str(p, str, size);
-	case 2:
+	case BGP_EVPN_MAC_IP_ROUTE:
 		return prefixevpn_macip2str(p, str, size);
-	case 3:
+	case BGP_EVPN_IMET_ROUTE:
 		return prefixevpn_imet2str(p, str, size);
-	case 4:
+	case BGP_EVPN_ES_ROUTE:
 		return prefixevpn_es2str(p, str, size);
-	case 5:
+	case BGP_EVPN_IP_PREFIX_ROUTE:
 		return prefixevpn_prefix2str(p, str, size);
 	default:
 		snprintf(str, size, "Unsupported EVPN prefix");
@@ -1013,7 +1065,8 @@ const char *prefix2str(union prefixconstptr pu, char *str, int size)
 		l = strlen(buf);
 		buf[l++] = '/';
 		byte = p->prefixlen;
-		if ((tmp = p->prefixlen - 100) >= 0) {
+		tmp = p->prefixlen - 100;
+		if (tmp >= 0) {
 			buf[l++] = '1';
 			z = true;
 			byte = tmp;
@@ -1049,6 +1102,26 @@ const char *prefix2str(union prefixconstptr pu, char *str, int size)
 	return str;
 }
 
+static ssize_t prefixhost2str(struct fbuf *fbuf, union prefixconstptr pu)
+{
+	const struct prefix *p = pu.p;
+	char buf[PREFIX2STR_BUFFER];
+
+	switch (p->family) {
+	case AF_INET:
+	case AF_INET6:
+		inet_ntop(p->family, &p->u.prefix, buf, sizeof(buf));
+		return bputs(fbuf, buf);
+
+	case AF_ETHERNET:
+		prefix_mac2str(&p->u.prefix_eth, buf, sizeof(buf));
+		return bputs(fbuf, buf);
+
+	default:
+		return bprintfrr(fbuf, "{prefix.af=%dPF}", p->family);
+	}
+}
+
 void prefix_mcast_inet4_dump(const char *onfail, struct in_addr addr,
 		char *buf, int buf_size)
 {
@@ -1107,7 +1180,7 @@ void apply_classful_mask_ipv4(struct prefix_ipv4 *p)
 
 	destination = ntohl(p->prefix.s_addr);
 
-	if (p->prefixlen == IPV4_MAX_PREFIXLEN)
+	if (p->prefixlen == IPV4_MAX_BITLEN)
 		;
 	/* do nothing for host routes */
 	else if (IN_CLASSC(destination)) {
@@ -1127,19 +1200,20 @@ in_addr_t ipv4_broadcast_addr(in_addr_t hostaddr, int masklen)
 	struct in_addr mask;
 
 	masklen2ip(masklen, &mask);
-	return (masklen != IPV4_MAX_PREFIXLEN - 1) ?
-		/* normal case */
-		(hostaddr | ~mask.s_addr)
-		   :
-		/* For prefix 31 return 255.255.255.255 (RFC3021) */
-		htonl(0xFFFFFFFF);
+	return (masklen != IPV4_MAX_BITLEN - 1)
+		       ?
+		       /* normal case */
+		       (hostaddr | ~mask.s_addr)
+		       :
+		       /* For prefix 31 return 255.255.255.255 (RFC3021) */
+		       htonl(0xFFFFFFFF);
 }
 
 /* Utility function to convert ipv4 netmask to prefixes
    ex.) "1.1.0.0" "255.255.0.0" => "1.1.0.0/16"
    ex.) "1.0.0.0" NULL => "1.0.0.0/8"                   */
 int netmask_str2prefix_str(const char *net_str, const char *mask_str,
-			   char *prefix_str)
+			   char *prefix_str, size_t prefix_str_len)
 {
 	struct in_addr network;
 	struct in_addr mask;
@@ -1172,20 +1246,11 @@ int netmask_str2prefix_str(const char *net_str, const char *mask_str,
 			return 0;
 	}
 
-	sprintf(prefix_str, "%s/%d", net_str, prefixlen);
+	snprintf(prefix_str, prefix_str_len, "%s/%d", net_str, prefixlen);
 
 	return 1;
 }
 
-/* Utility function for making IPv6 address string. */
-const char *inet6_ntoa(struct in6_addr addr)
-{
-	static char buf[INET6_ADDRSTRLEN];
-
-	inet_ntop(AF_INET6, &addr, buf, INET6_ADDRSTRLEN);
-	return buf;
-}
-
 /* converts to internal representation of mac address
  * returns 1 on success, 0 otherwise
  * format accepted: AA:BB:CC:DD:EE:FF
@@ -1316,47 +1381,249 @@ char *esi_to_str(const esi_t *esi, char *buf, int size)
 	return ptr;
 }
 
-printfrr_ext_autoreg_p("I4", printfrr_i4)
-static ssize_t printfrr_i4(char *buf, size_t bsz, const char *fmt,
-			   int prec, const void *ptr)
+char *evpn_es_df_alg2str(uint8_t df_alg, char *buf, int buf_len)
 {
-	inet_ntop(AF_INET, ptr, buf, bsz);
-	return 2;
+	switch (df_alg) {
+	case EVPN_MH_DF_ALG_SERVICE_CARVING:
+		snprintf(buf, buf_len, "service-carving");
+		break;
+
+	case EVPN_MH_DF_ALG_HRW:
+		snprintf(buf, buf_len, "HRW");
+		break;
+
+	case EVPN_MH_DF_ALG_PREF:
+		snprintf(buf, buf_len, "preference");
+		break;
+
+	default:
+		snprintf(buf, buf_len, "unknown %u", df_alg);
+		break;
+	}
+
+	return buf;
 }
 
-printfrr_ext_autoreg_p("I6", printfrr_i6)
-static ssize_t printfrr_i6(char *buf, size_t bsz, const char *fmt,
-			   int prec, const void *ptr)
+bool ipv4_unicast_valid(const struct in_addr *addr)
 {
-	inet_ntop(AF_INET6, ptr, buf, bsz);
-	return 2;
+	in_addr_t ip = ntohl(addr->s_addr);
+
+	if (IPV4_CLASS_D(ip))
+		return false;
+
+	if (IPV4_CLASS_E(ip)) {
+		if (cmd_allow_reserved_ranges_get())
+			return true;
+		else
+			return false;
+	}
+
+	return true;
 }
 
-printfrr_ext_autoreg_p("FX", printfrr_pfx)
-static ssize_t printfrr_pfx(char *buf, size_t bsz, const char *fmt,
-			    int prec, const void *ptr)
+static int ipaddr2prefix(const struct ipaddr *ip, uint16_t prefixlen,
+			 struct prefix *p)
 {
-	prefix2str(ptr, buf, bsz);
-	return 2;
+	switch (ip->ipa_type) {
+	case (IPADDR_V4):
+		p->family = AF_INET;
+		p->u.prefix4 = ip->ipaddr_v4;
+		p->prefixlen = prefixlen;
+		break;
+	case (IPADDR_V6):
+		p->family = AF_INET6;
+		p->u.prefix6 = ip->ipaddr_v6;
+		p->prefixlen = prefixlen;
+		break;
+	case (IPADDR_NONE):
+		p->family = AF_UNSPEC;
+		break;
+	}
+
+	return 0;
 }
 
-printfrr_ext_autoreg_p("SG4", printfrr_psg)
-static ssize_t printfrr_psg(char *buf, size_t bsz, const char *fmt,
-			    int prec, const void *ptr)
+/*
+ * Convert type-2 and type-5 evpn route prefixes into the more
+ * general ipv4/ipv6 prefix types so we can match prefix lists
+ * and such.
+ */
+int evpn_prefix2prefix(const struct prefix *evpn, struct prefix *to)
+{
+	const struct evpn_addr *addr;
+
+	if (evpn->family != AF_EVPN)
+		return -1;
+
+	addr = &evpn->u.prefix_evpn;
+
+	switch (addr->route_type) {
+	case BGP_EVPN_MAC_IP_ROUTE:
+		if (IS_IPADDR_V4(&addr->macip_addr.ip))
+			ipaddr2prefix(&addr->macip_addr.ip, IPV4_MAX_BITLEN,
+				      to);
+		else if (IS_IPADDR_V6(&addr->macip_addr.ip))
+			ipaddr2prefix(&addr->macip_addr.ip, IPV6_MAX_BITLEN,
+				      to);
+		else
+			return -1; /* mac only? */
+
+		break;
+	case BGP_EVPN_IP_PREFIX_ROUTE:
+		ipaddr2prefix(&addr->prefix_addr.ip,
+			      addr->prefix_addr.ip_prefix_length, to);
+		break;
+	default:
+		return -1;
+	}
+
+	return 0;
+}
+
+printfrr_ext_autoreg_p("EA", printfrr_ea);
+static ssize_t printfrr_ea(struct fbuf *buf, struct printfrr_eargs *ea,
+			   const void *ptr)
+{
+	const struct ethaddr *mac = ptr;
+	char cbuf[ETHER_ADDR_STRLEN];
+
+	if (!mac)
+		return bputs(buf, "(null)");
+
+	/* need real length even if buffer is too short */
+	prefix_mac2str(mac, cbuf, sizeof(cbuf));
+	return bputs(buf, cbuf);
+}
+
+printfrr_ext_autoreg_p("IA", printfrr_ia);
+static ssize_t printfrr_ia(struct fbuf *buf, struct printfrr_eargs *ea,
+			   const void *ptr)
+{
+	const struct ipaddr *ipa = ptr;
+	char cbuf[INET6_ADDRSTRLEN];
+	bool use_star = false;
+
+	if (ea->fmt[0] == 's') {
+		use_star = true;
+		ea->fmt++;
+	}
+
+	if (!ipa || !ipa->ipa_type)
+		return bputs(buf, "(null)");
+
+	if (use_star) {
+		struct in_addr zero4 = {};
+		struct in6_addr zero6 = {};
+
+		switch (ipa->ipa_type) {
+		case IPADDR_V4:
+			if (!memcmp(&ipa->ip.addr, &zero4, sizeof(zero4)))
+				return bputch(buf, '*');
+			break;
+
+		case IPADDR_V6:
+			if (!memcmp(&ipa->ip.addr, &zero6, sizeof(zero6)))
+				return bputch(buf, '*');
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	ipaddr2str(ipa, cbuf, sizeof(cbuf));
+	return bputs(buf, cbuf);
+}
+
+printfrr_ext_autoreg_p("I4", printfrr_i4);
+static ssize_t printfrr_i4(struct fbuf *buf, struct printfrr_eargs *ea,
+			   const void *ptr)
+{
+	char cbuf[INET_ADDRSTRLEN];
+	bool use_star = false;
+	struct in_addr zero = {};
+
+	if (ea->fmt[0] == 's') {
+		use_star = true;
+		ea->fmt++;
+	}
+
+	if (!ptr)
+		return bputs(buf, "(null)");
+
+	if (use_star && !memcmp(ptr, &zero, sizeof(zero)))
+		return bputch(buf, '*');
+
+	inet_ntop(AF_INET, ptr, cbuf, sizeof(cbuf));
+	return bputs(buf, cbuf);
+}
+
+printfrr_ext_autoreg_p("I6", printfrr_i6);
+static ssize_t printfrr_i6(struct fbuf *buf, struct printfrr_eargs *ea,
+			   const void *ptr)
+{
+	char cbuf[INET6_ADDRSTRLEN];
+	bool use_star = false;
+	struct in6_addr zero = {};
+
+	if (ea->fmt[0] == 's') {
+		use_star = true;
+		ea->fmt++;
+	}
+
+	if (!ptr)
+		return bputs(buf, "(null)");
+
+	if (use_star && !memcmp(ptr, &zero, sizeof(zero)))
+		return bputch(buf, '*');
+
+	inet_ntop(AF_INET6, ptr, cbuf, sizeof(cbuf));
+	return bputs(buf, cbuf);
+}
+
+printfrr_ext_autoreg_p("FX", printfrr_pfx);
+static ssize_t printfrr_pfx(struct fbuf *buf, struct printfrr_eargs *ea,
+			    const void *ptr)
+{
+	bool host_only = false;
+
+	if (ea->fmt[0] == 'h') {
+		ea->fmt++;
+		host_only = true;
+	}
+
+	if (!ptr)
+		return bputs(buf, "(null)");
+
+	if (host_only)
+		return prefixhost2str(buf, (struct prefix *)ptr);
+	else {
+		char cbuf[PREFIX_STRLEN];
+
+		prefix2str(ptr, cbuf, sizeof(cbuf));
+		return bputs(buf, cbuf);
+	}
+}
+
+printfrr_ext_autoreg_p("PSG4", printfrr_psg);
+static ssize_t printfrr_psg(struct fbuf *buf, struct printfrr_eargs *ea,
+			    const void *ptr)
 {
 	const struct prefix_sg *sg = ptr;
-	struct fbuf fb = { .buf = buf, .pos = buf, .len = bsz - 1 };
+	ssize_t ret = 0;
+
+	if (!sg)
+		return bputs(buf, "(null)");
 
 	if (sg->src.s_addr == INADDR_ANY)
-		bprintfrr(&fb, "(*,");
+		ret += bputs(buf, "(*,");
 	else
-		bprintfrr(&fb, "(%pI4,", &sg->src);
+		ret += bprintfrr(buf, "(%pI4,", &sg->src);
 
 	if (sg->grp.s_addr == INADDR_ANY)
-		bprintfrr(&fb, "*)");
+		ret += bputs(buf, "*)");
 	else
-		bprintfrr(&fb, "%pI4)", &sg->grp);
+		ret += bprintfrr(buf, "%pI4)", &sg->grp);
 
-	fb.pos[0] = '\0';
-	return 3;
+	return ret;
 }
