@@ -20,15 +20,21 @@
 
 #include <zebra.h>
 
-#ifdef SUNOS_5
-#include <ifaddrs.h>
-#endif
-
 #include "log.h"
 #include "sockopt.h"
 #include "sockunion.h"
 #include "lib_errors.h"
 
+#if (defined(__FreeBSD__)                                                      \
+     && ((__FreeBSD_version >= 500022 && __FreeBSD_version < 700000)           \
+	 || (__FreeBSD_version < 500000 && __FreeBSD_version >= 440000)))      \
+	|| (defined(__NetBSD__) && defined(__NetBSD_Version__)                 \
+	    && __NetBSD_Version__ >= 106010000)                                \
+	|| defined(__OpenBSD__) || defined(__APPLE__)                          \
+	|| defined(__DragonFly__) || defined(__sun)
+#define HAVE_BSD_STRUCT_IP_MREQ_HACK
+#endif
+
 void setsockopt_so_recvbuf(int sock, int size)
 {
 	int orig_req = size;
@@ -116,7 +122,7 @@ int setsockopt_ipv6_pktinfo(int sock, int val)
 	if (ret < 0)
 		flog_err(EC_LIB_SOCKET, "can't setsockopt IPV6_PKTINFO : %s",
 			 safe_strerror(errno));
-#endif /* INIA_IPV6 */
+#endif /* IANA_IPV6 */
 	return ret;
 }
 
@@ -262,13 +268,9 @@ int setsockopt_ipv4_multicast(int sock, int optname, struct in_addr if_addr,
 	    && (errno == EADDRINUSE)) {
 		/* see above: handle possible problem when interface comes back
 		 * up */
-		char buf[1][INET_ADDRSTRLEN];
 		zlog_info(
-			"setsockopt_ipv4_multicast attempting to drop and "
-			"re-add (fd %d, mcast %s, ifindex %u)",
-			sock, inet_ntop(AF_INET, &mreqn.imr_multiaddr, buf[0],
-					sizeof(buf[0])),
-			ifindex);
+			"setsockopt_ipv4_multicast attempting to drop and re-add (fd %d, mcast %pI4, ifindex %u)",
+			sock, &mreqn.imr_multiaddr, ifindex);
 		setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, (void *)&mreqn,
 			   sizeof(mreqn));
 		ret = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
@@ -304,13 +306,9 @@ int setsockopt_ipv4_multicast(int sock, int optname, struct in_addr if_addr,
 	    && (errno == EADDRINUSE)) {
 		/* see above: handle possible problem when interface comes back
 		 * up */
-		char buf[1][INET_ADDRSTRLEN];
 		zlog_info(
-			"setsockopt_ipv4_multicast attempting to drop and "
-			"re-add (fd %d, mcast %s, ifindex %u)",
-			sock, inet_ntop(AF_INET, &mreq.imr_multiaddr, buf[0],
-					sizeof(buf[0])),
-			ifindex);
+			"setsockopt_ipv4_multicast attempting to drop and re-add (fd %d, mcast %pI4, ifindex %u)",
+			sock, &mreq.imr_multiaddr, ifindex);
 		setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, (void *)&mreq,
 			   sizeof(mreq));
 		ret = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
@@ -353,35 +351,6 @@ int setsockopt_ipv4_multicast_if(int sock, struct in_addr if_addr,
 
 	return setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (void *)&m,
 			  sizeof(m));
-#elif defined(SUNOS_5)
-	char ifname[IF_NAMESIZE];
-	struct ifaddrs *ifa, *ifap;
-	struct in_addr ifaddr;
-
-	if (if_indextoname(ifindex, ifname) == NULL)
-		return -1;
-
-	if (getifaddrs(&ifa) != 0)
-		return -1;
-
-	for (ifap = ifa; ifap != NULL; ifap = ifap->ifa_next) {
-		struct sockaddr_in *sa;
-
-		if (strcmp(ifap->ifa_name, ifname) != 0)
-			continue;
-		if (ifap->ifa_addr->sa_family != AF_INET)
-			continue;
-		sa = (struct sockaddr_in *)ifap->ifa_addr;
-		memcpy(&ifaddr, &sa->sin_addr, sizeof(ifaddr));
-		break;
-	}
-
-	freeifaddrs(ifa);
-	if (!ifap) /* This means we did not find an IP */
-		return -1;
-
-	return setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (void *)&ifaddr,
-			  sizeof(ifaddr));
 #else
 #error "Unsupported multicast API"
 #endif
@@ -404,14 +373,14 @@ static int setsockopt_ipv4_ifindex(int sock, ifindex_t val)
 	int ret;
 
 #if defined(IP_PKTINFO)
-	if ((ret = setsockopt(sock, IPPROTO_IP, IP_PKTINFO, &val, sizeof(val)))
-	    < 0)
+	ret = setsockopt(sock, IPPROTO_IP, IP_PKTINFO, &val, sizeof(val));
+	if (ret < 0)
 		flog_err(EC_LIB_SOCKET,
 			 "Can't set IP_PKTINFO option for fd %d to %d: %s",
 			 sock, val, safe_strerror(errno));
 #elif defined(IP_RECVIF)
-	if ((ret = setsockopt(sock, IPPROTO_IP, IP_RECVIF, &val, sizeof(val)))
-	    < 0)
+	ret = setsockopt(sock, IPPROTO_IP, IP_RECVIF, &val, sizeof(val));
+	if (ret < 0)
 		flog_err(EC_LIB_SOCKET,
 			 "Can't set IP_RECVIF option for fd %d to %d: %s", sock,
 			 val, safe_strerror(errno));
@@ -485,15 +454,9 @@ static ifindex_t getsockopt_ipv4_ifindex(struct msghdr *msgh)
 
 /* retrieval based on IP_RECVIF */
 
-#ifndef SUNOS_5
 	/* BSD systems use a sockaddr_dl as the control message payload. */
 	struct sockaddr_dl *sdl;
-#else
-	/* SUNOS_5 uses an integer with the index. */
-	ifindex_t *ifindex_p;
-#endif /* SUNOS_5 */
 
-#ifndef SUNOS_5
 	/* BSD */
 	sdl = (struct sockaddr_dl *)getsockopt_cmsg_data(msgh, IPPROTO_IP,
 							 IP_RECVIF);
@@ -501,18 +464,6 @@ static ifindex_t getsockopt_ipv4_ifindex(struct msghdr *msgh)
 		ifindex = sdl->sdl_index;
 	else
 		ifindex = 0;
-#else
-	/*
-	 * Solaris.  On Solaris 8, IP_RECVIF is defined, but the call to
-	 * enable it fails with errno=99, and the struct msghdr has
-	 * controllen 0.
-	 */
-	ifindex_p = (uint_t *)getsockopt_cmsg_data(msgh, IPPROTO_IP, IP_RECVIF);
-	if (ifindex_p != NULL)
-		ifindex = *ifindex_p;
-	else
-		ifindex = 0;
-#endif /* SUNOS_5 */
 
 #else
 /*
@@ -638,7 +589,7 @@ int sockopt_tcp_signature_ext(int sock, union sockunion *su, uint16_t prefixlen,
 
 		/* If this does not work, then all users of this sockopt will
 		 * need to
-		 * differentiate between IPv4 and IPv6, and keep seperate
+		 * differentiate between IPv4 and IPv6, and keep separate
 		 * sockets for
 		 * each.
 		 *
@@ -682,12 +633,8 @@ int sockopt_tcp_signature_ext(int sock, union sockunion *su, uint16_t prefixlen,
 
 #endif /* GNU_LINUX */
 
-	if ((ret = setsockopt(sock, IPPROTO_TCP, optname, &md5sig,
-			      sizeof(md5sig)))
-	    < 0) {
-		/* ENOENT is harmless.  It is returned when we clear a password
-		   for which
-		   one was not previously set. */
+	ret = setsockopt(sock, IPPROTO_TCP, optname, &md5sig, sizeof(md5sig));
+	if (ret < 0) {
 		if (ENOENT == errno)
 			ret = 0;
 		else
@@ -710,3 +657,88 @@ int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
 {
 	return sockopt_tcp_signature_ext(sock, su, 0, password);
 }
+
+/* set TCP mss value to socket */
+int sockopt_tcp_mss_set(int sock, int tcp_maxseg)
+{
+	int ret = 0;
+	socklen_t tcp_maxseg_len = sizeof(tcp_maxseg);
+
+	ret = setsockopt(sock, IPPROTO_TCP, TCP_MAXSEG, &tcp_maxseg,
+			 tcp_maxseg_len);
+	if (ret != 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt(%d): %s", __func__, sock,
+			     safe_strerror(errno));
+	}
+
+	return ret;
+}
+
+/* get TCP mss value synced by socket */
+int sockopt_tcp_mss_get(int sock)
+{
+	int ret = 0;
+	int tcp_maxseg = 0;
+	socklen_t tcp_maxseg_len = sizeof(tcp_maxseg);
+
+	ret = getsockopt(sock, IPPROTO_TCP, TCP_MAXSEG, &tcp_maxseg,
+			 &tcp_maxseg_len);
+	if (ret != 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: getsockopt(%d): %s", __func__, sock,
+			     safe_strerror(errno));
+		return 0;
+	}
+
+	return tcp_maxseg;
+}
+
+int setsockopt_tcp_keepalive(int sock, uint16_t keepalive_idle,
+			     uint16_t keepalive_intvl,
+			     uint16_t keepalive_probes)
+{
+	int val = 1;
+
+	if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &val, sizeof(val)) < 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt SO_KEEPALIVE (%d): %s",
+			     __func__, sock, safe_strerror(errno));
+		return -1;
+	}
+
+#if defined __OpenBSD__
+	return 0;
+#else
+	/* Send first probe after keepalive_idle seconds */
+	val = keepalive_idle;
+	if (setsockopt(sock, IPPROTO_TCP, TCP_KEEPIDLE, &val, sizeof(val)) <
+	    0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt TCP_KEEPIDLE (%d): %s",
+			     __func__, sock, safe_strerror(errno));
+		return -1;
+	}
+
+	/* Set interval between two probes */
+	val = keepalive_intvl;
+	if (setsockopt(sock, IPPROTO_TCP, TCP_KEEPINTVL, &val, sizeof(val)) <
+	    0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt TCP_KEEPINTVL (%d): %s",
+			     __func__, sock, safe_strerror(errno));
+		return -1;
+	}
+
+	/* Set maximum probes */
+	val = keepalive_probes;
+	if (setsockopt(sock, IPPROTO_TCP, TCP_KEEPCNT, &val, sizeof(val)) < 0) {
+		flog_err_sys(EC_LIB_SYSTEM_CALL,
+			     "%s failed: setsockopt TCP_KEEPCNT (%d): %s",
+			     __func__, sock, safe_strerror(errno));
+		return -1;
+	}
+
+	return 0;
+#endif
+}
