@@ -14,26 +14,34 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#define FRR_COMPILING_TABLE_C
+
 #include <zebra.h>
 
 #include "prefix.h"
 #include "table.h"
 #include "memory.h"
 #include "sockunion.h"
+#include "libfrr_trace.h"
 
-DEFINE_MTYPE(LIB, ROUTE_TABLE, "Route table")
-DEFINE_MTYPE(LIB, ROUTE_NODE, "Route node")
+DEFINE_MTYPE_STATIC(LIB, ROUTE_TABLE, "Route table");
+DEFINE_MTYPE(LIB, ROUTE_NODE, "Route node");
 
-static void route_node_delete(struct route_node *);
 static void route_table_free(struct route_table *);
 
+static int route_table_hash_cmp(const struct route_node *a,
+				const struct route_node *b)
+{
+	return prefix_cmp(&a->p, &b->p);
+}
 
+DECLARE_HASH(rn_hash_node, struct route_node, nodehash, route_table_hash_cmp,
+	     prefix_hash_key);
 /*
  * route_table_init_with_delegate
  */
@@ -44,6 +52,7 @@ route_table_init_with_delegate(route_table_delegate_t *delegate)
 
 	rt = XCALLOC(MTYPE_ROUTE_TABLE, sizeof(struct route_table));
 	rt->delegate = delegate;
+	rn_hash_node_init(&rt->hash);
 	return rt;
 }
 
@@ -69,6 +78,8 @@ static struct route_node *route_node_set(struct route_table *table,
 	prefix_copy(&node->p, prefix);
 	node->table = table;
 
+	rn_hash_node_add(&node->table->hash, node);
+
 	return node;
 }
 
@@ -109,7 +120,9 @@ static void route_table_free(struct route_table *rt)
 		node = node->parent;
 
 		tmp_node->table->count--;
-		tmp_node->lock = 0; /* to cause assert if unlocked after this */
+		tmp_node->lock =
+			0; /* to cause assert if unlocked after this */
+		rn_hash_node_del(&rt->hash, tmp_node);
 		route_node_free(rt, tmp_node);
 
 		if (node != NULL) {
@@ -124,25 +137,32 @@ static void route_table_free(struct route_table *rt)
 
 	assert(rt->count == 0);
 
+	rn_hash_node_fini(&rt->hash);
 	XFREE(MTYPE_ROUTE_TABLE, rt);
 	return;
 }
 
 /* Utility mask array. */
-static const u_char maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
-				 0xf8, 0xfc, 0xfe, 0xff};
+static const uint8_t maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
+				  0xf8, 0xfc, 0xfe, 0xff};
 
 /* Common prefix route genaration. */
 static void route_common(const struct prefix *n, const struct prefix *p,
 			 struct prefix *new)
 {
 	int i;
-	u_char diff;
-	u_char mask;
+	uint8_t diff;
+	uint8_t mask;
+	const uint8_t *np;
+	const uint8_t *pp;
+	uint8_t *newp;
 
-	const u_char *np = (const u_char *)&n->u.prefix;
-	const u_char *pp = (const u_char *)&p->u.prefix;
-	u_char *newp = (u_char *)&new->u.prefix;
+	if (n->family == AF_FLOWSPEC)
+		return prefix_copy(new, p);
+	np = (const uint8_t *)&n->u.prefix;
+	pp = (const uint8_t *)&p->u.prefix;
+
+	newp = &new->u.prefix;
 
 	for (i = 0; i < p->prefixlen / 8; i++) {
 		if (np[i] == pp[i])
@@ -172,27 +192,11 @@ static void set_link(struct route_node *node, struct route_node *new)
 	new->parent = node;
 }
 
-/* Lock node. */
-struct route_node *route_lock_node(struct route_node *node)
-{
-	node->lock++;
-	return node;
-}
-
-/* Unlock node. */
-void route_unlock_node(struct route_node *node)
-{
-	assert(node->lock > 0);
-	node->lock--;
-
-	if (node->lock == 0)
-		route_node_delete(node);
-}
-
 /* Find matched prefix. */
-struct route_node *route_node_match(const struct route_table *table,
-				    const struct prefix *p)
+struct route_node *route_node_match(struct route_table *table,
+				    union prefixconstptr pu)
 {
+	const struct prefix *p = pu.p;
 	struct route_node *node;
 	struct route_node *matched;
 
@@ -219,83 +223,81 @@ struct route_node *route_node_match(const struct route_table *table,
 	return NULL;
 }
 
-struct route_node *route_node_match_ipv4(const struct route_table *table,
+struct route_node *route_node_match_ipv4(struct route_table *table,
 					 const struct in_addr *addr)
 {
 	struct prefix_ipv4 p;
 
-	memset(&p, 0, sizeof(struct prefix_ipv4));
+	memset(&p, 0, sizeof(p));
 	p.family = AF_INET;
-	p.prefixlen = IPV4_MAX_PREFIXLEN;
+	p.prefixlen = IPV4_MAX_BITLEN;
 	p.prefix = *addr;
 
 	return route_node_match(table, (struct prefix *)&p);
 }
 
-struct route_node *route_node_match_ipv6(const struct route_table *table,
+struct route_node *route_node_match_ipv6(struct route_table *table,
 					 const struct in6_addr *addr)
 {
 	struct prefix_ipv6 p;
 
-	memset(&p, 0, sizeof(struct prefix_ipv6));
+	memset(&p, 0, sizeof(p));
 	p.family = AF_INET6;
-	p.prefixlen = IPV6_MAX_PREFIXLEN;
+	p.prefixlen = IPV6_MAX_BITLEN;
 	p.prefix = *addr;
 
-	return route_node_match(table, (struct prefix *)&p);
+	return route_node_match(table, &p);
 }
 
 /* Lookup same prefix node.  Return NULL when we can't find route. */
-struct route_node *route_node_lookup(const struct route_table *table,
-				     const struct prefix *p)
+struct route_node *route_node_lookup(struct route_table *table,
+				     union prefixconstptr pu)
 {
-	struct route_node *node;
-	u_char prefixlen = p->prefixlen;
-	const u_char *prefix = &p->u.prefix;
+	struct route_node rn, *node;
+	prefix_copy(&rn.p, pu.p);
+	apply_mask(&rn.p);
 
-	node = table->top;
-
-	while (node && node->p.prefixlen <= prefixlen
-	       && prefix_match(&node->p, p)) {
-		if (node->p.prefixlen == prefixlen)
-			return node->info ? route_lock_node(node) : NULL;
-
-		node = node->link[prefix_bit(prefix, node->p.prefixlen)];
-	}
-
-	return NULL;
+	node = rn_hash_node_find(&table->hash, &rn);
+	return (node && node->info) ? route_lock_node(node) : NULL;
 }
 
 /* Lookup same prefix node.  Return NULL when we can't find route. */
-struct route_node *route_node_lookup_maynull(const struct route_table *table,
-					     const struct prefix *p)
+struct route_node *route_node_lookup_maynull(struct route_table *table,
+					     union prefixconstptr pu)
 {
-	struct route_node *node;
-	u_char prefixlen = p->prefixlen;
-	const u_char *prefix = &p->u.prefix;
+	struct route_node rn, *node;
+	prefix_copy(&rn.p, pu.p);
+	apply_mask(&rn.p);
 
-	node = table->top;
-
-	while (node && node->p.prefixlen <= prefixlen
-	       && prefix_match(&node->p, p)) {
-		if (node->p.prefixlen == prefixlen)
-			return route_lock_node(node);
-
-		node = node->link[prefix_bit(prefix, node->p.prefixlen)];
-	}
-
-	return NULL;
+	node = rn_hash_node_find(&table->hash, &rn);
+	return node ? route_lock_node(node) : NULL;
 }
 
 /* Add node to routing table. */
-struct route_node *route_node_get(struct route_table *const table,
-				  const struct prefix *p)
+struct route_node *route_node_get(struct route_table *table,
+				  union prefixconstptr pu)
 {
+	if (frrtrace_enabled(frr_libfrr, route_node_get)) {
+		char buf[PREFIX2STR_BUFFER];
+		prefix2str(pu, buf, sizeof(buf));
+		frrtrace(2, frr_libfrr, route_node_get, table, buf);
+	}
+
+	struct route_node search;
+	struct prefix *p = &search.p;
+
+	prefix_copy(p, pu.p);
+	apply_mask(p);
+
 	struct route_node *new;
 	struct route_node *node;
 	struct route_node *match;
-	u_char prefixlen = p->prefixlen;
-	const u_char *prefix = &p->u.prefix;
+	uint16_t prefixlen = p->prefixlen;
+	const uint8_t *prefix = &p->u.prefix;
+
+	node = rn_hash_node_find(&table->hash, &search);
+	if (node && node->info)
+		return route_lock_node(node);
 
 	match = NULL;
 	node = table->top;
@@ -320,6 +322,7 @@ struct route_node *route_node_get(struct route_table *const table,
 		new->p.family = p->family;
 		new->table = table;
 		set_link(new, node);
+		rn_hash_node_add(&table->hash, new);
 
 		if (match)
 			set_link(match, new);
@@ -340,7 +343,7 @@ struct route_node *route_node_get(struct route_table *const table,
 }
 
 /* Delete node from the routing table. */
-static void route_node_delete(struct route_node *node)
+void route_node_delete(struct route_node *node)
 {
 	struct route_node *child;
 	struct route_node *parent;
@@ -371,6 +374,8 @@ static void route_node_delete(struct route_node *node)
 
 	node->table->count--;
 
+	rn_hash_node_del(&node->table->hash, node);
+
 	/* WARNING: FRAGILE CODE!
 	 * route_node_free may have the side effect of free'ing the entire
 	 * table.
@@ -388,7 +393,7 @@ static void route_node_delete(struct route_node *node)
 		route_node_delete(parent);
 }
 
-/* Get fist node and lock it.  This function is useful when one want
+/* Get first node and lock it.  This function is useful when one wants
    to lookup all the node exist in the routing table. */
 struct route_node *route_top(struct route_table *table)
 {
@@ -439,7 +444,7 @@ struct route_node *route_next(struct route_node *node)
 
 /* Unlock current node and lock next node until limit. */
 struct route_node *route_next_until(struct route_node *node,
-				    struct route_node *limit)
+				    const struct route_node *limit)
 {
 	struct route_node *next;
 	struct route_node *start;
@@ -474,7 +479,7 @@ struct route_node *route_next_until(struct route_node *node,
 	return NULL;
 }
 
-unsigned long route_table_count(const struct route_table *table)
+unsigned long route_table_count(struct route_table *table)
 {
 	return table->count;
 }
@@ -533,7 +538,8 @@ struct route_table *route_table_init(void)
  *          0 if the prefixes are identical (p1 == p2)
  *         +1 if p1 occurs after p2 (p1 > p2)
  */
-int route_table_prefix_iter_cmp(struct prefix *p1, struct prefix *p2)
+int route_table_prefix_iter_cmp(const struct prefix *p1,
+				const struct prefix *p2)
 {
 	struct prefix common_space;
 	struct prefix *common = &common_space;
@@ -608,7 +614,8 @@ static struct route_node *route_get_subtree_next(struct route_node *node)
  * @see route_table_get_next
  */
 static struct route_node *
-route_table_get_next_internal(const struct route_table *table, struct prefix *p)
+route_table_get_next_internal(struct route_table *table,
+			      const struct prefix *p)
 {
 	struct route_node *node, *tmp_node;
 	int cmp;
@@ -708,9 +715,10 @@ route_table_get_next_internal(const struct route_table *table, struct prefix *p)
  * Find the node that occurs after the given prefix in order of
  * iteration.
  */
-struct route_node *route_table_get_next(const struct route_table *table,
-					struct prefix *p)
+struct route_node *route_table_get_next(struct route_table *table,
+					union prefixconstptr pu)
 {
+	const struct prefix *p = pu.p;
 	struct route_node *node;
 
 	node = route_table_get_next_internal(table, p);
