@@ -28,9 +28,9 @@
 #include "command.h"
 #include "log.h"
 
-DEFINE_MTYPE(LIB, WORK_QUEUE, "Work queue")
-DEFINE_MTYPE_STATIC(LIB, WORK_QUEUE_ITEM, "Work queue item")
-DEFINE_MTYPE_STATIC(LIB, WORK_QUEUE_NAME, "Work queue name string")
+DEFINE_MTYPE(LIB, WORK_QUEUE, "Work queue");
+DEFINE_MTYPE_STATIC(LIB, WORK_QUEUE_ITEM, "Work queue item");
+DEFINE_MTYPE_STATIC(LIB, WORK_QUEUE_NAME, "Work queue name string");
 
 /* master list of work_queues */
 static struct list _work_queues;
@@ -81,9 +81,6 @@ struct work_queue *work_queue_new(struct thread_master *m,
 
 	new = XCALLOC(MTYPE_WORK_QUEUE, sizeof(struct work_queue));
 
-	if (new == NULL)
-		return new;
-
 	new->name = XSTRDUP(MTYPE_WORK_QUEUE_NAME, queue_name);
 	new->master = m;
 	SET_FLAG(new->flags, WQ_UNPLUGGED);
@@ -94,17 +91,19 @@ struct work_queue *work_queue_new(struct thread_master *m,
 
 	new->cycles.granularity = WORK_QUEUE_MIN_GRANULARITY;
 
-	/* Default values, can be overriden by caller */
+	/* Default values, can be overridden by caller */
 	new->spec.hold = WORK_QUEUE_DEFAULT_HOLD;
 	new->spec.yield = THREAD_YIELD_TIME_SLOT;
+	new->spec.retry = WORK_QUEUE_DEFAULT_RETRY;
 
 	return new;
 }
 
-void work_queue_free_original(struct work_queue *wq)
+void work_queue_free_and_null(struct work_queue **wqp)
 {
-	if (wq->thread != NULL)
-		thread_cancel(wq->thread);
+	struct work_queue *wq = *wqp;
+
+	THREAD_OFF(wq->thread);
 
 	while (!work_queue_empty(wq)) {
 		struct work_queue_item *item = work_queue_last_item(wq);
@@ -116,30 +115,34 @@ void work_queue_free_original(struct work_queue *wq)
 
 	XFREE(MTYPE_WORK_QUEUE_NAME, wq->name);
 	XFREE(MTYPE_WORK_QUEUE, wq);
-	return;
-}
 
-void work_queue_free_and_null(struct work_queue **wq)
-{
-	work_queue_free_original(*wq);
-	*wq = NULL;
+	*wqp = NULL;
 }
 
 bool work_queue_is_scheduled(struct work_queue *wq)
 {
-	return (wq->thread != NULL);
+	return thread_is_scheduled(wq->thread);
 }
 
 static int work_queue_schedule(struct work_queue *wq, unsigned int delay)
 {
 	/* if appropriate, schedule work queue thread */
-	if (CHECK_FLAG(wq->flags, WQ_UNPLUGGED) && (wq->thread == NULL)
-	    && !work_queue_empty(wq)) {
-		wq->thread = NULL;
-		thread_add_timer_msec(wq->master, work_queue_run, wq, delay,
-				      &wq->thread);
+	if (CHECK_FLAG(wq->flags, WQ_UNPLUGGED) &&
+	    !thread_is_scheduled(wq->thread) && !work_queue_empty(wq)) {
+		/* Schedule timer if there's a delay, otherwise just schedule
+		 * as an 'event'
+		 */
+		if (delay > 0) {
+			thread_add_timer_msec(wq->master, work_queue_run, wq,
+					      delay, &wq->thread);
+			thread_ignore_late_timer(wq->thread);
+		} else
+			thread_add_event(wq->master, work_queue_run, wq, 0,
+					 &wq->thread);
+
 		/* set thread yield time, if needed */
-		if (wq->thread && wq->spec.yield != THREAD_YIELD_TIME_SLOT)
+		if (thread_is_scheduled(wq->thread) &&
+		    wq->spec.yield != THREAD_YIELD_TIME_SLOT)
 			thread_set_yield_time(wq->thread, wq->spec.yield);
 		return 1;
 	} else
@@ -152,10 +155,7 @@ void work_queue_add(struct work_queue *wq, void *data)
 
 	assert(wq);
 
-	if (!(item = work_queue_item_new(wq))) {
-		zlog_err("%s: unable to get new queue item", __func__);
-		return;
-	}
+	item = work_queue_item_new(wq);
 
 	item->data = data;
 	work_queue_item_enqueue(wq, item);
@@ -213,10 +213,7 @@ void workqueue_cmd_init(void)
  */
 void work_queue_plug(struct work_queue *wq)
 {
-	if (wq->thread)
-		thread_cancel(wq->thread);
-
-	wq->thread = NULL;
+	THREAD_OFF(wq->thread);
 
 	UNSET_FLAG(wq->flags, WQ_UNPLUGGED);
 }
@@ -236,16 +233,15 @@ void work_queue_unplug(struct work_queue *wq)
  * will reschedule itself if required,
  * otherwise work_queue_item_add
  */
-int work_queue_run(struct thread *thread)
+void work_queue_run(struct thread *thread)
 {
 	struct work_queue *wq;
 	struct work_queue_item *item, *titem;
-	wq_item_status ret;
+	wq_item_status ret = WQ_SUCCESS;
 	unsigned int cycles = 0;
 	char yielded = 0;
 
 	wq = THREAD_ARG(thread);
-	wq->thread = NULL;
 
 	assert(wq);
 
@@ -272,13 +268,10 @@ int work_queue_run(struct thread *thread)
 		wq->cycles.granularity = WORK_QUEUE_MIN_GRANULARITY;
 
 	STAILQ_FOREACH_SAFE (item, &wq->items, wq, titem) {
-		assert(item && item->data);
+		assert(item->data);
 
 		/* dont run items which are past their allowed retries */
 		if (item->ran > wq->spec.max_retries) {
-			/* run error handler, if any */
-			if (wq->spec.errorfunc)
-				wq->spec.errorfunc(wq, item->data);
 			work_queue_item_remove(wq, item);
 			continue;
 		}
@@ -321,10 +314,6 @@ int work_queue_run(struct thread *thread)
 		case WQ_RETRY_NOW:
 		/* a RETRY_NOW that gets here has exceeded max_tries, same as
 		 * ERROR */
-		case WQ_ERROR: {
-			if (wq->spec.errorfunc)
-				wq->spec.errorfunc(wq, item);
-		}
 		/* fallthru */
 		case WQ_SUCCESS:
 		default: {
@@ -375,16 +364,14 @@ stats:
 	if (yielded)
 		wq->yields++;
 
-#if 0
-  printf ("%s: cycles %d, new: best %d, worst %d\n",
-            __func__, cycles, wq->cycles.best, wq->cycles.granularity);
-#endif
-
 	/* Is the queue done yet? If it is, call the completion callback. */
-	if (!work_queue_empty(wq))
-		work_queue_schedule(wq, 0);
-	else if (wq->spec.completion_func)
-		wq->spec.completion_func(wq);
+	if (!work_queue_empty(wq)) {
+		if (ret == WQ_RETRY_LATER ||
+		    ret == WQ_QUEUE_BLOCKED)
+			work_queue_schedule(wq, wq->spec.retry);
+		else
+			work_queue_schedule(wq, 0);
 
-	return 0;
+	} else if (wq->spec.completion_func)
+		wq->spec.completion_func(wq);
 }
