@@ -7,6 +7,10 @@
  * (at your option) any later version.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <unistd.h>
 
 #include "zebra.h"
@@ -14,17 +18,17 @@
 #include "getopt.h"
 #include "thread.h"
 #include "sigevent.h"
-#include "version.h"
+#include "lib/version.h"
 #include "log.h"
 #include "memory.h"
-#include "memory_vty.h"
 #include "command.h"
 #include "libfrr.h"
+#include "filter.h"
 
 #include "nhrpd.h"
-#include "netlink.h"
+#include "nhrp_errors.h"
 
-DEFINE_MGROUP(NHRPD, "NHRP")
+DEFINE_MGROUP(NHRPD, "NHRP");
 
 unsigned int debug_flags = 0;
 
@@ -50,9 +54,11 @@ struct zebra_privs_t nhrpd_privs = {
 	.vty_group = VTY_GROUP,
 #endif
 	.caps_p = _caps_p,
-	.cap_num_p = ZEBRA_NUM_OF(_caps_p),
+	.cap_num_p = array_size(_caps_p),
+	.cap_num_i = 0
 };
 
+
 static void parse_arguments(int argc, char **argv)
 {
 	int opt;
@@ -67,7 +73,6 @@ static void parse_arguments(int argc, char **argv)
 			break;
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 }
@@ -82,13 +87,13 @@ static void nhrp_request_stop(void)
 	debugf(NHRP_DEBUG_COMMON, "Exiting...");
 	frr_early_fini();
 
-	nhrp_shortcut_terminate();
+	vrf_terminate();
 	nhrp_nhs_terminate();
 	nhrp_zebra_terminate();
 	vici_terminate();
 	evmgr_terminate();
 	nhrp_vc_terminate();
-	vrf_terminate();
+	nhrp_shortcut_terminate();
 
 	debugf(NHRP_DEBUG_COMMON, "Done.");
 	frr_fini();
@@ -96,7 +101,7 @@ static void nhrp_request_stop(void)
 	exit(0);
 }
 
-static struct quagga_signal_t sighandlers[] = {
+static struct frr_signal_t sighandlers[] = {
 	{
 		.signal = SIGUSR1,
 		.handler = &nhrp_sigusr1,
@@ -111,13 +116,21 @@ static struct quagga_signal_t sighandlers[] = {
 	},
 };
 
+static const struct frr_yang_module_info *const nhrpd_yang_modules[] = {
+	&frr_filter_info,
+	&frr_interface_info,
+	&frr_vrf_info,
+};
+
 FRR_DAEMON_INFO(nhrpd, NHRP, .vty_port = NHRP_VTY_PORT,
 
 		.proghelp = "Implementation of the NHRP routing protocol.",
 
 		.signals = sighandlers, .n_signals = array_size(sighandlers),
 
-		.privs = &nhrpd_privs, )
+		.privs = &nhrpd_privs, .yang_modules = nhrpd_yang_modules,
+		.n_yang_modules = array_size(nhrpd_yang_modules),
+);
 
 int main(int argc, char **argv)
 {
@@ -128,19 +141,26 @@ int main(int argc, char **argv)
 
 	/* Library inits. */
 	master = frr_init();
+	nhrp_error_init();
 	vrf_init(NULL, NULL, NULL, NULL);
 	nhrp_interface_init();
-	resolver_init();
+	resolver_init(master);
 
-	/* Run with elevated capabilities, as for all netlink activity
-	 * we need privileges anyway. */
+	/*
+	 * Run with elevated capabilities, as for all netlink activity
+	 * we need privileges anyway.
+	 * The assert is for clang SA code where it does
+	 * not see the change function being set in lib
+	 */
+	assert(nhrpd_privs.change);
 	nhrpd_privs.change(ZPRIVS_RAISE);
 
-	netlink_init();
 	evmgr_init();
 	nhrp_vc_init();
 	nhrp_packet_init();
 	vici_init();
+	if_zapi_callbacks(nhrp_ifp_create, nhrp_ifp_up,
+			  nhrp_ifp_down, nhrp_ifp_destroy);
 	nhrp_zebra_init();
 	nhrp_shortcut_init();
 
