@@ -26,15 +26,6 @@ static struct cmd_node zebra_node = {
 	.config_write = nhrp_config_write,
 };
 
-static int interface_config_write(struct vty *vty);
-static struct cmd_node nhrp_interface_node = {
-	.name = "interface",
-	.node = INTERFACE_NODE,
-	.parent_node = CONFIG_NODE,
-	.prompt = "%s(config-if)# ",
-	.config_write = interface_config_write,
-};
-
 #define NHRP_DEBUG_FLAGS_CMD "<all|common|event|interface|kernel|route|vici>"
 
 #define NHRP_DEBUG_FLAGS_STR                                                   \
@@ -135,6 +126,8 @@ DEFUN_NOSH(show_debugging_nhrp, show_debugging_nhrp_cmd,
 			debug_flags_desc[i].str);
 	}
 
+	cmd_show_lib_debugs(vty);
+
 	return CMD_SUCCESS;
 }
 
@@ -187,6 +180,9 @@ static int nhrp_config_write(struct vty *vty)
 	if (netlink_nflog_group) {
 		vty_out(vty, "nhrp nflog-group %d\n", netlink_nflog_group);
 	}
+	if (netlink_mcast_nflog_group)
+		vty_out(vty, "nhrp multicast-nflog-group %d\n",
+			netlink_mcast_nflog_group);
 
 	return 0;
 }
@@ -257,6 +253,31 @@ DEFUN(no_nhrp_nflog_group, no_nhrp_nflog_group_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(nhrp_multicast_nflog_group, nhrp_multicast_nflog_group_cmd,
+	"nhrp multicast-nflog-group (1-65535)",
+	NHRP_STR
+	"Specify NFLOG group number for Multicast Packets\n"
+	"NFLOG group number\n")
+{
+	uint32_t nfgroup;
+
+	nfgroup = strtoul(argv[2]->arg, NULL, 10);
+	netlink_mcast_set_nflog_group(nfgroup);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(no_nhrp_multicast_nflog_group, no_nhrp_multicast_nflog_group_cmd,
+	"no nhrp multicast-nflog-group [(1-65535)]",
+	NO_STR
+	NHRP_STR
+	"Specify NFLOG group number\n"
+	"NFLOG group number\n")
+{
+	netlink_mcast_set_nflog_group(0);
+	return CMD_SUCCESS;
+}
+
 DEFUN(tunnel_protection, tunnel_protection_cmd,
 	"tunnel protection vici profile PROFILE [fallback-profile FALLBACK]",
 	"NHRP/GRE integration\n"
@@ -494,28 +515,42 @@ DEFUN(if_nhrp_map, if_nhrp_map_cmd,
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	afi_t afi = cmd_to_afi(argv[0]);
 	union sockunion proto_addr, nbma_addr;
+	struct nhrp_cache_config *cc;
 	struct nhrp_cache *c;
+	enum nhrp_cache_type type;
 
 	if (str2sockunion(argv[3]->arg, &proto_addr) < 0
 	    || afi2family(afi) != sockunion_family(&proto_addr))
 		return nhrp_vty_return(vty, NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH);
 
+	if (strmatch(argv[4]->text, "local"))
+		type = NHRP_CACHE_LOCAL;
+	else {
+		if (str2sockunion(argv[4]->arg, &nbma_addr) < 0)
+			return nhrp_vty_return(vty, NHRP_ERR_FAIL);
+		type = NHRP_CACHE_STATIC;
+	}
+	cc = nhrp_cache_config_get(ifp, &proto_addr, 1);
+	if (!cc)
+		return nhrp_vty_return(vty, NHRP_ERR_FAIL);
+	cc->nbma = nbma_addr;
+	cc->type = type;
+	/* gre layer not ready */
+	if (ifp->ifindex == IFINDEX_INTERNAL)
+		return CMD_SUCCESS;
+
 	c = nhrp_cache_get(ifp, &proto_addr, 1);
 	if (!c)
 		return nhrp_vty_return(vty, NHRP_ERR_FAIL);
 
 	c->map = 1;
-	if (strmatch(argv[4]->text, "local")) {
+	if (type == NHRP_CACHE_LOCAL)
 		nhrp_cache_update_binding(c, NHRP_CACHE_LOCAL, 0, NULL, 0,
-					  NULL);
-	} else {
-		if (str2sockunion(argv[4]->arg, &nbma_addr) < 0)
-			return nhrp_vty_return(vty, NHRP_ERR_FAIL);
+					  NULL, NULL);
+	else
 		nhrp_cache_update_binding(c, NHRP_CACHE_STATIC, 0,
 					  nhrp_peer_get(ifp, &nbma_addr), 0,
-					  NULL);
-	}
-
+					  NULL, NULL);
 	return CMD_SUCCESS;
 }
 
@@ -533,21 +568,76 @@ DEFUN(if_no_nhrp_map, if_no_nhrp_map_cmd,
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	afi_t afi = cmd_to_afi(argv[1]);
 	union sockunion proto_addr, nbma_addr;
+	struct nhrp_cache_config *cc;
 	struct nhrp_cache *c;
 
 	if (str2sockunion(argv[4]->arg, &proto_addr) < 0
 	    || afi2family(afi) != sockunion_family(&proto_addr))
 		return nhrp_vty_return(vty, NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH);
 
+	cc = nhrp_cache_config_get(ifp, &proto_addr, 0);
+	if (!cc)
+		return nhrp_vty_return(vty, NHRP_ERR_FAIL);
+	nhrp_cache_config_free(cc);
+
 	c = nhrp_cache_get(ifp, &proto_addr, 0);
+	/* silently return */
 	if (!c || !c->map)
-		return nhrp_vty_return(vty, NHRP_ERR_ENTRY_NOT_FOUND);
+		return CMD_SUCCESS;
 
 	nhrp_cache_update_binding(c, c->cur.type, -1,
-				  nhrp_peer_get(ifp, &nbma_addr), 0, NULL);
+				  nhrp_peer_get(ifp, &nbma_addr), 0, NULL,
+				  NULL);
 	return CMD_SUCCESS;
 }
 
+DEFUN(if_nhrp_map_multicast, if_nhrp_map_multicast_cmd,
+	AFI_CMD " nhrp map multicast <A.B.C.D|X:X::X:X|dynamic>",
+	AFI_STR
+	NHRP_STR
+	"Multicast NBMA Configuration\n"
+	"Use this NBMA mapping for multicasts\n"
+	"IPv4 NBMA address\n"
+	"IPv6 NBMA address\n"
+	"Dynamically learn destinations from client registrations on hub\n")
+{
+	VTY_DECLVAR_CONTEXT(interface, ifp);
+	afi_t afi = cmd_to_afi(argv[0]);
+	union sockunion nbma_addr;
+	int ret;
+
+	if (str2sockunion(argv[4]->arg, &nbma_addr) < 0)
+		sockunion_family(&nbma_addr) = AF_UNSPEC;
+
+	ret = nhrp_multicast_add(ifp, afi, &nbma_addr);
+
+	return nhrp_vty_return(vty, ret);
+}
+
+DEFUN(if_no_nhrp_map_multicast, if_no_nhrp_map_multicast_cmd,
+	"no " AFI_CMD " nhrp map multicast <A.B.C.D|X:X::X:X|dynamic>",
+	NO_STR
+	AFI_STR
+	NHRP_STR
+	"Multicast NBMA Configuration\n"
+	"Use this NBMA mapping for multicasts\n"
+	"IPv4 NBMA address\n"
+	"IPv6 NBMA address\n"
+	"Dynamically learn destinations from client registrations on hub\n")
+{
+	VTY_DECLVAR_CONTEXT(interface, ifp);
+	afi_t afi = cmd_to_afi(argv[1]);
+	union sockunion nbma_addr;
+	int ret;
+
+	if (str2sockunion(argv[5]->arg, &nbma_addr) < 0)
+		sockunion_family(&nbma_addr) = AF_UNSPEC;
+
+	ret = nhrp_multicast_del(ifp, afi, &nbma_addr);
+
+	return nhrp_vty_return(vty, ret);
+}
+
 DEFUN(if_nhrp_nhs, if_nhrp_nhs_cmd,
 	AFI_CMD " nhrp nhs <A.B.C.D|X:X::X:X|dynamic> nbma <A.B.C.D|FQDN>",
 	AFI_STR
@@ -608,7 +698,7 @@ static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
 {
 	struct info_ctx *ctx = pctx;
 	struct vty *vty = ctx->vty;
-	char buf[2][SU_ADDRSTRLEN];
+	char buf[3][SU_ADDRSTRLEN];
 	struct json_object *json = NULL;
 
 	if (ctx->afi != family2afi(sockunion_family(&c->remote_addr)))
@@ -616,17 +706,42 @@ static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
 
 
 	if (!ctx->count && !ctx->json) {
-		vty_out(vty, "%-8s %-8s %-24s %-24s %-6s %s\n", "Iface", "Type",
-			"Protocol", "NBMA", "Flags", "Identity");
+		vty_out(vty, "%-8s %-8s %-24s %-24s %-24s %-6s %s\n", "Iface",
+			"Type", "Protocol", "NBMA", "Claimed NBMA", "Flags",
+			"Identity");
 	}
 	ctx->count++;
 
 	sockunion2str(&c->remote_addr, buf[0], sizeof(buf[0]));
-	if (c->cur.peer)
-		sockunion2str(&c->cur.peer->vc->remote.nbma,
-			      buf[1], sizeof(buf[1]));
-	else
-		snprintf(buf[1], sizeof(buf[1]), "-");
+	if (c->cur.type == NHRP_CACHE_LOCAL) {
+		struct nhrp_interface *nifp = c->ifp->info;
+
+		if (sockunion_family(&nifp->nbma) != AF_UNSPEC) {
+			sockunion2str(&nifp->nbma, buf[1], sizeof(buf[1]));
+			sockunion2str(&nifp->nbma, buf[2], sizeof(buf[2]));
+		} else {
+			snprintf(buf[1], sizeof(buf[1]), "-");
+			snprintf(buf[2], sizeof(buf[2]), "-");
+		}
+
+		/* if we are behind NAT then update NBMA field */
+		if (sockunion_family(&nifp->nat_nbma) != AF_UNSPEC)
+			sockunion2str(&nifp->nat_nbma, buf[1], sizeof(buf[1]));
+	} else {
+		if (c->cur.peer)
+			sockunion2str(&c->cur.peer->vc->remote.nbma,
+				      buf[1], sizeof(buf[1]));
+		else
+			snprintf(buf[1], sizeof(buf[1]), "-");
+
+		if (c->cur.peer
+		    && sockunion_family(&c->cur.remote_nbma_claimed)
+		    != AF_UNSPEC)
+			sockunion2str(&c->cur.remote_nbma_claimed,
+				      buf[2], sizeof(buf[2]));
+		else
+			snprintf(buf[2], sizeof(buf[2]), "-");
+	}
 
 	if (ctx->json) {
 		json = json_object_new_object();
@@ -635,6 +750,7 @@ static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
 				       nhrp_cache_type_str[c->cur.type]);
 		json_object_string_add(json, "protocol", buf[0]);
 		json_object_string_add(json, "nbma", buf[1]);
+		json_object_string_add(json, "claimed_nbma", buf[2]);
 
 		if (c->used)
 			json_object_boolean_true_add(json, "used");
@@ -660,9 +776,10 @@ static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
 		json_object_array_add(ctx->json, json);
 		return;
 	}
-	vty_out(ctx->vty, "%-8s %-8s %-24s %-24s %c%c%c    %s\n", c->ifp->name,
+	vty_out(ctx->vty, "%-8s %-8s %-24s %-24s %-24s %c%c%c    %s\n",
+		c->ifp->name,
 		nhrp_cache_type_str[c->cur.type],
-		buf[0], buf[1],
+		buf[0], buf[1], buf[2],
 		c->used ? 'U' : ' ', c->t_timeout ? 'T' : ' ',
 		c->t_auth ? 'A' : ' ',
 		c->cur.peer ? c->cur.peer->vc->remote.id : "-");
@@ -683,8 +800,8 @@ static void show_ip_nhrp_nhs(struct nhrp_nhs *n, struct nhrp_registration *reg,
 	ctx->count++;
 
 	if (reg && reg->peer)
-		sockunion2str(&reg->peer->vc->remote.nbma,
-			      buf[0], sizeof(buf[0]));
+		sockunion2str(&reg->peer->vc->remote.nbma, buf[0],
+			      sizeof(buf[0]));
 	else
 		snprintf(buf[0], sizeof(buf[0]), "-");
 	sockunion2str(reg ? &reg->proto_addr : &n->proto_addr, buf[1],
@@ -720,6 +837,7 @@ static void show_ip_nhrp_shortcut(struct nhrp_shortcut *s, void *pctx)
 	ctx->count++;
 
 	c = s->cache;
+	buf2[0] = '\0';
 	if (c)
 		sockunion2str(&c->remote_addr, buf2, sizeof(buf2));
 	prefix2str(s->p, buf1, sizeof(buf1));
@@ -871,10 +989,7 @@ DEFUN(show_ip_nhrp, show_ip_nhrp_cmd,
 	if (uj) {
 		json_object_object_add(json_vrf_path, "attr", json_vrf);
 		json_object_object_add(json_vrf_path, "table", ctx.json);
-		vty_out(vty, "%s",
-			json_object_to_json_string_ext(
-			       json_vrf_path, JSON_C_TO_STRING_PRETTY));
-		json_object_free(json_vrf_path);
+		vty_json(vty, json_vrf_path);
 	}
 	return ret;
 }
@@ -888,18 +1003,15 @@ static void show_dmvpn_entry(struct nhrp_vc *vc, void *ctx)
 {
 	struct dmvpn_cfg *ctxt = ctx;
 	struct vty *vty;
-	char buf[2][SU_ADDRSTRLEN];
 	struct json_object *json = NULL;
 
 	if (!ctxt || !ctxt->vty)
 		return;
 	vty = ctxt->vty;
-	sockunion2str(&vc->local.nbma, buf[0], sizeof(buf[0]));
-	sockunion2str(&vc->remote.nbma, buf[1], sizeof(buf[1]));
 	if (ctxt->json) {
 		json = json_object_new_object();
-		json_object_string_add(json, "src", buf[0]);
-		json_object_string_add(json, "dst", buf[1]);
+		json_object_string_addf(json, "src", "%pSU", &vc->local.nbma);
+		json_object_string_addf(json, "dst", "%pSU", &vc->remote.nbma);
 
 		if (notifier_active(&vc->notifier_list))
 			json_object_boolean_true_add(json, "notifierActive");
@@ -910,9 +1022,10 @@ static void show_dmvpn_entry(struct nhrp_vc *vc, void *ctx)
 		json_object_string_add(json, "identity", vc->remote.id);
 		json_object_array_add(ctxt->json, json);
 	} else {
-		vty_out(vty, "%-24s %-24s %c      %-4d %-24s\n",
-			buf[0], buf[1], notifier_active(&vc->notifier_list) ?
-			'n' : ' ', vc->ipsec, vc->remote.id);
+		vty_out(vty, "%-24pSU %-24pSU %c      %-4d %-24s\n",
+			&vc->local.nbma, &vc->remote.nbma,
+			notifier_active(&vc->notifier_list) ? 'n' : ' ',
+			vc->ipsec, vc->remote.id);
 	}
 }
 
@@ -936,12 +1049,8 @@ DEFUN(show_dmvpn, show_dmvpn_cmd,
 		ctxt.json = json_path;
 	}
 	nhrp_vc_foreach(show_dmvpn_entry, &ctxt);
-	if (uj) {
-		vty_out(vty, "%s",
-			json_object_to_json_string_ext(
-			       json_path, JSON_C_TO_STRING_PRETTY));
-		json_object_free(json_path);
-	}
+	if (uj)
+		vty_json(vty, json_path);
 	return CMD_SUCCESS;
 }
 
@@ -949,8 +1058,10 @@ static void clear_nhrp_cache(struct nhrp_cache *c, void *data)
 {
 	struct info_ctx *ctx = data;
 	if (c->cur.type <= NHRP_CACHE_DYNAMIC) {
-		nhrp_cache_update_binding(c, c->cur.type, -1, NULL, 0, NULL);
-		ctx->count++;
+		nhrp_cache_update_binding(c, c->cur.type, -1, NULL, 0, NULL,
+					  NULL);
+		if (ctx)
+			ctx->count++;
 	}
 }
 
@@ -980,6 +1091,12 @@ DEFUN(clear_nhrp, clear_nhrp_cmd,
 			nhrp_cache_foreach(ifp, clear_nhrp_cache, &ctx);
 	} else {
 		nhrp_shortcut_foreach(ctx.afi, clear_nhrp_shortcut, &ctx);
+		/* Clear cache also because when a shortcut is cleared then its
+		 * cache entry should be cleared as well (otherwise traffic
+		 * continues via the shortcut path)
+		 */
+		FOR_ALL_INTERFACES (vrf, ifp)
+			nhrp_cache_foreach(ifp, clear_nhrp_cache, NULL);
 	}
 
 	if (!ctx.count) {
@@ -997,23 +1114,20 @@ struct write_map_ctx {
 	const char *aficmd;
 };
 
-static void interface_config_write_nhrp_map(struct nhrp_cache *c, void *data)
+static void interface_config_write_nhrp_map(struct nhrp_cache_config *c,
+					    void *data)
 {
 	struct write_map_ctx *ctx = data;
 	struct vty *vty = ctx->vty;
-	char buf[2][SU_ADDRSTRLEN];
 
-	if (!c->map)
-		return;
 	if (sockunion_family(&c->remote_addr) != ctx->family)
 		return;
 
-	vty_out(vty, " %s nhrp map %s %s\n", ctx->aficmd,
-		sockunion2str(&c->remote_addr, buf[0], sizeof(buf[0])),
-		c->cur.type == NHRP_CACHE_LOCAL
-			? "local"
-			: sockunion2str(&c->cur.peer->vc->remote.nbma, buf[1],
-					sizeof(buf[1])));
+	vty_out(vty, " %s nhrp map %pSU ", ctx->aficmd, &c->remote_addr);
+	if (c->type == NHRP_CACHE_LOCAL)
+		vty_out(vty, "local\n");
+	else
+		vty_out(vty, "%pSU\n", &c->nbma);
 }
 
 static int interface_config_write(struct vty *vty)
@@ -1023,13 +1137,13 @@ static int interface_config_write(struct vty *vty)
 	struct interface *ifp;
 	struct nhrp_interface *nifp;
 	struct nhrp_nhs *nhs;
+	struct nhrp_multicast *mcast;
 	const char *aficmd;
 	afi_t afi;
-	char buf[SU_ADDRSTRLEN];
 	int i;
 
 	FOR_ALL_INTERFACES (vrf, ifp) {
-		vty_frame(vty, "interface %s\n", ifp->name);
+		if_vty_config_start(vty, ifp);
 		if (ifp->desc)
 			vty_out(vty, " description %s\n", ifp->desc);
 
@@ -1076,25 +1190,31 @@ static int interface_config_write(struct vty *vty)
 				.family = afi2family(afi),
 				.aficmd = aficmd,
 			};
-			nhrp_cache_foreach(ifp, interface_config_write_nhrp_map,
-					   &mapctx);
+			nhrp_cache_config_foreach(
+				ifp, interface_config_write_nhrp_map, &mapctx);
 
-			list_for_each_entry(nhs, &ad->nhslist_head,
-					    nhslist_entry)
-			{
-				vty_out(vty, " %s nhrp nhs %s nbma %s\n",
-					aficmd,
-					sockunion_family(&nhs->proto_addr)
-							== AF_UNSPEC
-						? "dynamic"
-						: sockunion2str(
-							  &nhs->proto_addr, buf,
-							  sizeof(buf)),
-					nhs->nbma_fqdn);
+			frr_each (nhrp_nhslist, &ad->nhslist_head, nhs) {
+				vty_out(vty, " %s nhrp nhs ", aficmd);
+				if (sockunion_family(&nhs->proto_addr)
+				   == AF_UNSPEC)
+					vty_out(vty, "dynamic");
+				else
+					vty_out(vty, "%pSU", &nhs->proto_addr);
+				vty_out(vty, " nbma %s\n", nhs->nbma_fqdn);
+			}
+
+			frr_each (nhrp_mcastlist, &ad->mcastlist_head, mcast) {
+				vty_out(vty, " %s nhrp map multicast ", aficmd);
+				if (sockunion_family(&mcast->nbma_addr)
+				   == AF_UNSPEC)
+					vty_out(vty, "dynamic\n");
+				else
+					vty_out(vty, "%pSU\n",
+						&mcast->nbma_addr);
 			}
 		}
 
-		vty_endframe(vty, "!\n");
+		if_vty_config_end(vty);
 	}
 
 	return 0;
@@ -1109,11 +1229,12 @@ void nhrp_config_init(void)
 	access_list_init();
 
 	/* global commands */
-	install_element(VIEW_NODE, &show_debugging_nhrp_cmd);
 	install_element(VIEW_NODE, &show_ip_nhrp_cmd);
 	install_element(VIEW_NODE, &show_dmvpn_cmd);
 	install_element(ENABLE_NODE, &clear_nhrp_cmd);
 
+	install_element(ENABLE_NODE, &show_debugging_nhrp_cmd);
+
 	install_element(ENABLE_NODE, &debug_nhrp_cmd);
 	install_element(ENABLE_NODE, &no_debug_nhrp_cmd);
 
@@ -1124,11 +1245,13 @@ void nhrp_config_init(void)
 	install_element(CONFIG_NODE, &no_nhrp_event_socket_cmd);
 	install_element(CONFIG_NODE, &nhrp_nflog_group_cmd);
 	install_element(CONFIG_NODE, &no_nhrp_nflog_group_cmd);
+	install_element(CONFIG_NODE, &nhrp_multicast_nflog_group_cmd);
+	install_element(CONFIG_NODE, &no_nhrp_multicast_nflog_group_cmd);
+
+	vrf_cmd_init(NULL);
 
 	/* interface specific commands */
-	install_node(&nhrp_interface_node);
-
-	if_cmd_init();
+	if_cmd_init(interface_config_write);
 	install_element(INTERFACE_NODE, &tunnel_protection_cmd);
 	install_element(INTERFACE_NODE, &no_tunnel_protection_cmd);
 	install_element(INTERFACE_NODE, &tunnel_source_cmd);
@@ -1145,6 +1268,8 @@ void nhrp_config_init(void)
 	install_element(INTERFACE_NODE, &if_no_nhrp_reg_flags_cmd);
 	install_element(INTERFACE_NODE, &if_nhrp_map_cmd);
 	install_element(INTERFACE_NODE, &if_no_nhrp_map_cmd);
+	install_element(INTERFACE_NODE, &if_nhrp_map_multicast_cmd);
+	install_element(INTERFACE_NODE, &if_no_nhrp_map_multicast_cmd);
 	install_element(INTERFACE_NODE, &if_nhrp_nhs_cmd);
 	install_element(INTERFACE_NODE, &if_no_nhrp_nhs_cmd);
 }
