@@ -23,6 +23,9 @@
 
 #include "hook.h"
 
+/* Forward declaration(s). */
+struct ospf6_area;
+
 /* Debug option */
 extern unsigned char conf_debug_ospf6_neighbor;
 #define OSPF6_DEBUG_NEIGHBOR_STATE   0x01
@@ -32,10 +35,42 @@ extern unsigned char conf_debug_ospf6_neighbor;
 #define IS_OSPF6_DEBUG_NEIGHBOR(level)                                         \
 	(conf_debug_ospf6_neighbor & OSPF6_DEBUG_NEIGHBOR_##level)
 
+struct ospf6_helper_info {
+
+	/* Grace interval received from
+	 * Restarting Router.
+	 */
+	uint32_t recvd_grace_period;
+
+	/* Grace interval used for grace
+	 * gracetimer.
+	 */
+	uint32_t actual_grace_period;
+
+	/* Grace timer,This Router acts as
+	 * helper until this timer until
+	 * this timer expires.
+	 */
+	struct thread *t_grace_timer;
+
+	/* Helper status */
+	uint32_t gr_helper_status;
+
+	/* Helper exit reason*/
+	uint32_t helper_exit_reason;
+
+	/* Planned/Unplanned restart*/
+	uint32_t gr_restart_reason;
+
+
+	/* Helper rejected reason */
+	uint32_t rejected_reason;
+};
+
 /* Neighbor structure */
 struct ospf6_neighbor {
 	/* Neighbor Router ID String */
-	char name[32];
+	char name[36];
 
 	/* OSPFv3 Interface this neighbor belongs to */
 	struct ospf6_interface *ospf6_if;
@@ -47,8 +82,12 @@ struct ospf6_neighbor {
 	uint32_t state_change;
 	struct timeval last_changed;
 
+	/* last received hello */
+	struct timeval last_hello;
+	uint32_t hello_in;
+
 	/* Neighbor Router ID */
-	uint32_t router_id;
+	in_addr_t router_id;
 
 	/* Neighbor Interface ID */
 	ifindex_t ifindex;
@@ -56,15 +95,15 @@ struct ospf6_neighbor {
 	/* Router Priority of this neighbor */
 	uint8_t priority;
 
-	uint32_t drouter;
-	uint32_t bdrouter;
-	uint32_t prev_drouter;
-	uint32_t prev_bdrouter;
+	in_addr_t drouter;
+	in_addr_t bdrouter;
+	in_addr_t prev_drouter;
+	in_addr_t prev_bdrouter;
 
 	/* Options field (Capability) */
 	char options[3];
 
-	/* IPaddr of I/F on our side link */
+	/* IPaddr of I/F on neighbour's link */
 	struct in6_addr linklocal_addr;
 
 	/* For Database Exchange */
@@ -89,14 +128,31 @@ struct ospf6_neighbor {
 	/* Inactivity timer */
 	struct thread *inactivity_timer;
 
+	/* Timer to release the last dbdesc packet */
+	struct thread *last_dbdesc_release_timer;
+
 	/* Thread for sending message */
 	struct thread *thread_send_dbdesc;
 	struct thread *thread_send_lsreq;
 	struct thread *thread_send_lsupdate;
 	struct thread *thread_send_lsack;
+	struct thread *thread_exchange_done;
+	struct thread *thread_adj_ok;
+	struct thread *event_loading_done;
 
 	/* BFD information */
-	void *bfd_info;
+	struct bfd_session_params *bfd_session;
+
+	/* ospf6 graceful restart HELPER info */
+	struct ospf6_helper_info gr_helper_info;
+
+	/* seqnum_h/l is used to compare sequence
+	 * number in received packet Auth header
+	 */
+	uint32_t seqnum_h[OSPF6_MESSAGE_TYPE_MAX];
+	uint32_t seqnum_l[OSPF6_MESSAGE_TYPE_MAX];
+	bool auth_present;
+	bool lls_present;
 };
 
 /* Neighbor state */
@@ -123,11 +179,7 @@ struct ospf6_neighbor {
 #define OSPF6_NEIGHBOR_EVENT_INACTIVITY_TIMER    10
 #define OSPF6_NEIGHBOR_EVENT_MAX_EVENT           11
 
-static const char *ospf6_neighbor_event_str[] = {
-	"NoEvent",      "HelloReceived", "2-WayReceived",   "NegotiationDone",
-	"ExchangeDone", "LoadingDone",   "AdjOK?",	  "SeqNumberMismatch",
-	"BadLSReq",     "1-WayReceived", "InactivityTimer",
-};
+extern const char *const ospf6_neighbor_event_str[];
 
 static inline const char *ospf6_neighbor_event_string(int event)
 {
@@ -138,31 +190,33 @@ static inline const char *ospf6_neighbor_event_string(int event)
 	return OSPF6_NEIGHBOR_UNKNOWN_EVENT_STRING;
 }
 
-extern const char *ospf6_neighbor_state_str[];
+extern const char *const ospf6_neighbor_state_str[];
 
 
 /* Function Prototypes */
 int ospf6_neighbor_cmp(void *va, void *vb);
 void ospf6_neighbor_dbex_init(struct ospf6_neighbor *on);
 
-struct ospf6_neighbor *ospf6_neighbor_lookup(uint32_t,
-					     struct ospf6_interface *);
-struct ospf6_neighbor *ospf6_neighbor_create(uint32_t,
-					     struct ospf6_interface *);
-void ospf6_neighbor_delete(struct ospf6_neighbor *);
+struct ospf6_neighbor *ospf6_neighbor_lookup(uint32_t router_id,
+					     struct ospf6_interface *oi);
+struct ospf6_neighbor *ospf6_area_neighbor_lookup(struct ospf6_area *area,
+						  uint32_t router_id);
+struct ospf6_neighbor *ospf6_neighbor_create(uint32_t router_id,
+					     struct ospf6_interface *oi);
+void ospf6_neighbor_delete(struct ospf6_neighbor *on);
 
 /* Neighbor event */
-extern int hello_received(struct thread *);
-extern int twoway_received(struct thread *);
-extern int negotiation_done(struct thread *);
-extern int exchange_done(struct thread *);
-extern int loading_done(struct thread *);
-extern int adj_ok(struct thread *);
-extern int seqnumber_mismatch(struct thread *);
-extern int bad_lsreq(struct thread *);
-extern int oneway_received(struct thread *);
-extern int inactivity_timer(struct thread *);
-extern void ospf6_check_nbr_loading(struct ospf6_neighbor *);
+extern void hello_received(struct thread *thread);
+extern void twoway_received(struct thread *thread);
+extern void negotiation_done(struct thread *thread);
+extern void exchange_done(struct thread *thread);
+extern void loading_done(struct thread *thread);
+extern void adj_ok(struct thread *thread);
+extern void seqnumber_mismatch(struct thread *thread);
+extern void bad_lsreq(struct thread *thread);
+extern void oneway_received(struct thread *thread);
+extern void inactivity_timer(struct thread *thread);
+extern void ospf6_check_nbr_loading(struct ospf6_neighbor *on);
 
 extern void ospf6_neighbor_init(void);
 extern int config_write_ospf6_debug_neighbor(struct vty *vty);
@@ -170,6 +224,6 @@ extern void install_element_ospf6_debug_neighbor(void);
 
 DECLARE_HOOK(ospf6_neighbor_change,
 	     (struct ospf6_neighbor * on, int state, int next_state),
-	     (on, state, next_state))
+	     (on, state, next_state));
 
 #endif /* OSPF6_NEIGHBOR_H */
