@@ -325,7 +325,7 @@ static void ospf6_nexthop_calc(struct ospf6_vertex *w, struct ospf6_vertex *v,
 static int ospf6_spf_install(struct ospf6_vertex *v,
 			     struct ospf6_route_table *result_table)
 {
-	struct ospf6_route *route, *parent_route;
+	struct ospf6_route *route;
 	struct ospf6_vertex *prev;
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
@@ -345,7 +345,12 @@ static int ospf6_spf_install(struct ospf6_vertex *v,
 			zlog_debug(
 				"  another path found to route %pFX lsa %s, merge",
 				&route->prefix, v->lsa->name);
-		ospf6_spf_merge_nexthops_to_route(route, v);
+
+		/* merging the parent's nexthop information to the child's
+		 * if the parent is not the root of the tree.
+		 */
+		if (!ospf6_merge_parents_nh_to_child(v, route, result_table))
+			ospf6_spf_merge_nexthops_to_route(route, v);
 
 		prev = (struct ospf6_vertex *)route->route_option;
 		assert(prev->hops <= v->hops);
@@ -411,13 +416,7 @@ static int ospf6_spf_install(struct ospf6_vertex *v,
 	 * installed,
 	 * its parent's route's nexthops have already been installed.
 	 */
-	if (v->parent && v->parent->hops) {
-		parent_route =
-			ospf6_route_lookup(&v->parent->vertex_id, result_table);
-		if (parent_route) {
-			ospf6_route_merge_nexthops(route, parent_route);
-		}
-	}
+	ospf6_merge_parents_nh_to_child(v, route, result_table);
 
 	if (v->parent)
 		listnode_add_sort(v->parent->child_list, v);
@@ -605,7 +604,7 @@ static void ospf6_spf_log_database(struct ospf6_area *oa)
 	zlog_debug("%s", buffer);
 }
 
-static int ospf6_spf_calculation_thread(struct thread *t)
+static void ospf6_spf_calculation_thread(struct thread *t)
 {
 	struct ospf6_area *oa;
 	struct ospf6 *ospf6;
@@ -615,7 +614,6 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 	char rbuf[32];
 
 	ospf6 = (struct ospf6 *)THREAD_ARG(t);
-	ospf6->t_spf_calc = NULL;
 
 	/* execute SPF calculation */
 	monotime(&start);
@@ -681,7 +679,6 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 
 	ospf6->last_spf_reason = ospf6->spf_reason;
 	ospf6_reset_spf_reason(ospf6);
-	return 0;
 }
 
 /* Add schedule for SPF calculation.  To avoid frequenst SPF calc, we
@@ -704,7 +701,7 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 	}
 
 	/* SPF calculation timer is already scheduled. */
-	if (ospf6->t_spf_calc) {
+	if (thread_is_scheduled(ospf6->t_spf_calc)) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME))
 			zlog_debug(
 				"SPF: calculation timer is already scheduled: %p",
@@ -741,7 +738,7 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 	if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME))
 		zlog_debug("SPF: Rescheduling in %ld msec", delay);
 
-	ospf6->t_spf_calc = NULL;
+	THREAD_OFF(ospf6->t_spf_calc);
 	thread_add_timer_msec(master, ospf6_spf_calculation_thread, ospf6,
 			      delay, &ospf6->t_spf_calc);
 }
@@ -1245,7 +1242,7 @@ int ospf6_ase_calculate_route(struct ospf6 *ospf6, struct ospf6_lsa *lsa,
 	return 0;
 }
 
-static int ospf6_ase_calculate_timer(struct thread *t)
+static void ospf6_ase_calculate_timer(struct thread *t)
 {
 	struct ospf6 *ospf6;
 	struct ospf6_lsa *lsa;
@@ -1254,7 +1251,6 @@ static int ospf6_ase_calculate_timer(struct thread *t)
 	uint16_t type;
 
 	ospf6 = THREAD_ARG(t);
-	ospf6->t_ase_calc = NULL;
 
 	/* Calculate external route for each AS-external-LSA */
 	type = htons(OSPF6_LSTYPE_AS_EXTERNAL);
@@ -1283,8 +1279,6 @@ static int ospf6_ase_calculate_timer(struct thread *t)
 		ospf6_zebra_gr_disable(ospf6);
 		ospf6->gr_info.finishing_restart = false;
 	}
-
-	return 0;
 }
 
 void ospf6_ase_calculate_timer_add(struct ospf6 *ospf6)
@@ -1295,3 +1289,20 @@ void ospf6_ase_calculate_timer_add(struct ospf6 *ospf6)
 	thread_add_timer(master, ospf6_ase_calculate_timer, ospf6,
 			 OSPF6_ASE_CALC_INTERVAL, &ospf6->t_ase_calc);
 }
+
+bool ospf6_merge_parents_nh_to_child(struct ospf6_vertex *v,
+				     struct ospf6_route *route,
+				     struct ospf6_route_table *result_table)
+{
+	struct ospf6_route *parent_route;
+
+	if (v->parent && v->parent->hops) {
+		parent_route =
+			ospf6_route_lookup(&v->parent->vertex_id, result_table);
+		if (parent_route) {
+			ospf6_route_merge_nexthops(route, parent_route);
+			return true;
+		}
+	}
+	return false;
+}
