@@ -27,6 +27,7 @@
 #include "nexthop_group.h"
 #include "nexthop_group_private.h"
 #include "log.h"
+#include "json.h"
 #include "debug.h"
 #include "pbr.h"
 
@@ -35,9 +36,7 @@
 #include "pbrd/pbr_zebra.h"
 #include "pbrd/pbr_vty.h"
 #include "pbrd/pbr_debug.h"
-#ifndef VTYSH_EXTRACT_PL
 #include "pbrd/pbr_vty_clippy.c"
-#endif
 
 DEFUN_NOSH(pbr_map, pbr_map_cmd, "pbr-map PBRMAP seq (1-700)",
 	   "Create pbr-map or enter pbr-map command mode\n"
@@ -136,6 +135,14 @@ DEFPY(pbr_map_match_src, pbr_map_match_src_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (pbrms->dst && prefix->family != pbrms->dst->family) {
+		vty_out(vty, "Cannot mismatch families within match src/dst\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	pbrms->family = prefix->family;
 
 	if (!no) {
@@ -164,6 +171,14 @@ DEFPY(pbr_map_match_dst, pbr_map_match_dst_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (pbrms->src && prefix->family != pbrms->src->family) {
+		vty_out(vty, "Cannot mismatch families within match src/dst\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	pbrms->family = prefix->family;
 
 	if (!no) {
@@ -182,6 +197,181 @@ DEFPY(pbr_map_match_dst, pbr_map_match_dst_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFPY(pbr_map_match_ip_proto, pbr_map_match_ip_proto_cmd,
+      "[no] match ip-protocol [tcp|udp]$ip_proto",
+      NO_STR
+      "Match the rest of the command\n"
+      "Choose an ip-protocol\n"
+      "Match on tcp flows\n"
+      "Match on udp flows\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+	struct protoent *p;
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no) {
+		if (!ip_proto) {
+			vty_out(vty, "Unable to convert (null) to proto id\n");
+			return CMD_WARNING;
+		}
+
+		p = getprotobyname(ip_proto);
+		if (!p) {
+			vty_out(vty, "Unable to convert %s to proto id\n",
+				ip_proto);
+			return CMD_WARNING;
+		}
+
+		pbrms->ip_proto = p->p_proto;
+	} else
+		pbrms->ip_proto = 0;
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_match_src_port, pbr_map_match_src_port_cmd,
+      "[no] match src-port (1-65535)$port",
+      NO_STR
+      "Match the rest of the command\n"
+      "Choose the source port to use\n"
+      "The Source Port\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no) {
+		if (pbrms->src_prt == port)
+			return CMD_SUCCESS;
+		else
+			pbrms->src_prt = port;
+	} else
+		pbrms->src_prt = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_match_dst_port, pbr_map_match_dst_port_cmd,
+      "[no] match dst-port (1-65535)$port",
+      NO_STR
+      "Match the rest of the command\n"
+      "Choose the destination port to use\n"
+      "The Destination Port\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no) {
+		if (pbrms->dst_prt == port)
+			return CMD_SUCCESS;
+		else
+			pbrms->dst_prt = port;
+	} else
+		pbrms->dst_prt = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_match_dscp, pbr_map_match_dscp_cmd,
+      "[no] match dscp DSCP$dscp",
+      NO_STR
+      "Match the rest of the command\n"
+      "Match based on IP DSCP field\n"
+      "DSCP value (below 64) or standard codepoint name\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+	char dscpname[100];
+	uint8_t rawDscp;
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	/* Discriminate dscp enums (cs0, cs1 etc.) and numbers */
+	bool isANumber = true;
+	for (int i = 0; i < (int)strlen(dscp); i++) {
+		/* Letters are not numbers */
+		if (!isdigit(dscp[i]))
+			isANumber = false;
+
+		/* Lowercase the dscp enum (if needed) */
+		if (isupper(dscp[i]))
+			dscpname[i] = tolower(dscp[i]);
+		else
+			dscpname[i] = dscp[i];
+	}
+	dscpname[strlen(dscp)] = '\0';
+
+	if (isANumber) {
+		/* dscp passed is a regular number */
+		long dscpAsNum = strtol(dscp, NULL, 0);
+
+		if (dscpAsNum > PBR_DSFIELD_DSCP >> 2) {
+			/* Refuse to install on overflow */
+			vty_out(vty, "dscp (%s) must be less than 64\n", dscp);
+			return CMD_WARNING_CONFIG_FAILED;
+		}
+		rawDscp = dscpAsNum;
+	} else {
+		/* check dscp if it is an enum like cs0 */
+		rawDscp = pbr_map_decode_dscp_enum(dscpname);
+		if (rawDscp > PBR_DSFIELD_DSCP) {
+			vty_out(vty, "Invalid dscp value: %s\n", dscpname);
+			return CMD_WARNING_CONFIG_FAILED;
+		}
+	}
+
+	if (!no) {
+		if (((pbrms->dsfield & PBR_DSFIELD_DSCP) >> 2) == rawDscp)
+			return CMD_SUCCESS;
+
+		/* Set the DSCP bits of the DSField */
+		pbrms->dsfield =
+			(pbrms->dsfield & ~PBR_DSFIELD_DSCP) | (rawDscp << 2);
+	} else {
+		pbrms->dsfield &= ~PBR_DSFIELD_DSCP;
+	}
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_match_ecn, pbr_map_match_ecn_cmd,
+      "[no] match ecn (0-3)$ecn",
+      NO_STR
+      "Match the rest of the command\n"
+      "Match based on IP ECN field\n"
+      "Explicit Congestion Notification\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no) {
+		if ((pbrms->dsfield & PBR_DSFIELD_ECN) == ecn)
+			return CMD_SUCCESS;
+
+		/* Set the ECN bits of the DSField */
+		pbrms->dsfield = (pbrms->dsfield & ~PBR_DSFIELD_ECN) | ecn;
+	} else {
+		pbrms->dsfield &= ~PBR_DSFIELD_ECN;
+	}
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
 DEFPY(pbr_map_match_mark, pbr_map_match_mark_cmd,
 	"[no] match mark (1-4294967295)$mark",
 	NO_STR
@@ -191,8 +381,11 @@ DEFPY(pbr_map_match_mark, pbr_map_match_mark_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 #ifndef GNU_LINUX
-	vty_out(vty, "pbr marks are not supported on this platform");
+	vty_out(vty, "pbr marks are not supported on this platform\n");
 	return CMD_WARNING_CONFIG_FAILED;
 #endif
 
@@ -241,6 +434,94 @@ static void pbrms_clear_set_config(struct pbr_map_sequence *pbrms)
 	pbrms->nhs_installed = false;
 }
 
+
+DEFPY(pbr_map_action_queue_id, pbr_map_action_queue_id_cmd,
+      "[no] set queue-id <(1-65535)$queue_id>",
+      NO_STR
+      "Set the rest of the command\n"
+      "Set based on egress port queue id\n"
+      "A valid value in range 1..65535 \n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no)
+		pbrms->action_queue_id = queue_id;
+	else if ((uint32_t)queue_id == pbrms->action_queue_id)
+		pbrms->action_queue_id = PBR_MAP_UNDEFINED_QUEUE_ID;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_action_pcp, pbr_map_action_pcp_cmd, "[no] set pcp <(0-7)$pcp>",
+      NO_STR
+      "Set the rest of the command\n"
+      "Set based on 802.1p Priority Code Point (PCP) value\n"
+      "A valid value in range 0..7\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no)
+		pbrms->action_pcp = pcp;
+	else if (pcp == pbrms->action_pcp)
+		pbrms->action_pcp = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_action_vlan_id, pbr_map_action_vlan_id_cmd,
+      "[no] set vlan <(1-4094)$vlan_id>",
+      NO_STR
+      "Set the rest of the command\n"
+      "Set action for VLAN tagging\n"
+      "A valid value in range 1..4094\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no)
+		pbrms->action_vlan_id = vlan_id;
+	else if (pbrms->action_vlan_id == vlan_id)
+		pbrms->action_vlan_id = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(pbr_map_action_strip_vlan, pbr_map_action_strip_vlan_cmd,
+      "[no] strip vlan",
+      NO_STR
+      "Strip the vlan tags from frame\n"
+      "Strip any inner vlan tag \n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
+	if (!no)
+		pbrms->action_vlan_flags = PBR_MAP_STRIP_INNER_ANY;
+	else
+		pbrms->action_vlan_flags = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+
 DEFPY(pbr_map_nexthop_group, pbr_map_nexthop_group_cmd,
       "set nexthop-group NHGNAME$name",
       "Set for the PBR-MAP\n"
@@ -250,6 +531,9 @@ DEFPY(pbr_map_nexthop_group, pbr_map_nexthop_group_cmd,
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 	struct nexthop_group_cmd *nhgc;
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	nhgc = nhgc_find(name);
 	if (!nhgc) {
 		vty_out(vty, "Specified nexthop-group %s does not exist\n",
@@ -264,7 +548,8 @@ DEFPY(pbr_map_nexthop_group, pbr_map_nexthop_group_cmd,
 	/* This is new/replacement config */
 	pbrms_clear_set_config(pbrms);
 
-	pbrms->nhgrp_name = XSTRDUP(MTYPE_TMP, name);
+	pbr_nht_set_seq_nhg(pbrms, name);
+
 	pbr_map_check(pbrms, true);
 
 	return CMD_SUCCESS;
@@ -279,6 +564,9 @@ DEFPY(no_pbr_map_nexthop_group, no_pbr_map_nexthop_group_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	pbrms_clear_set_config(pbrms);
 
 	return CMD_SUCCESS;
@@ -305,13 +593,16 @@ DEFPY(pbr_map_nexthop, pbr_map_nexthop_cmd,
 	struct nexthop nhop;
 	struct nexthop *nh = NULL;
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	if (vrf_name)
 		vrf = vrf_lookup_by_name(vrf_name);
 	else
 		vrf = vrf_lookup_by_id(VRF_DEFAULT);
 
 	if (!vrf) {
-		vty_out(vty, "Specified: %s is non-existent\n", vrf_name);
+		vty_out(vty, "Specified VRF: %s is non-existent\n", vrf_name);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
@@ -319,13 +610,43 @@ DEFPY(pbr_map_nexthop, pbr_map_nexthop_cmd,
 	nhop.vrf_id = vrf->vrf_id;
 
 	if (intf) {
-		nhop.ifindex = ifname2ifindex(intf, vrf->vrf_id);
-		if (nhop.ifindex == IFINDEX_INTERNAL) {
-			vty_out(vty,
-				"Specified Intf %s does not exist in vrf: %s\n",
-				intf, vrf->name);
+		struct interface *ifp = NULL;
+		struct interface *ifptmp;
+		struct vrf *vrftmp;
+		int count = 0;
+
+		if (vrf_is_backend_netns() && vrf_name) {
+			ifp = if_lookup_by_name_vrf(intf, vrf);
+		} else {
+			RB_FOREACH (vrftmp, vrf_name_head, &vrfs_by_name) {
+				ifptmp = if_lookup_by_name_vrf(intf, vrftmp);
+				if (ifptmp) {
+					ifp = ifptmp;
+					count++;
+					if (!vrf_is_backend_netns())
+						break;
+				}
+			}
+		}
+
+		if (!ifp) {
+			vty_out(vty, "Specified Intf %s does not exist\n",
+				intf);
 			return CMD_WARNING_CONFIG_FAILED;
 		}
+		if (count > 1) {
+			vty_out(vty,
+				"Specified Intf %s exists in multiple VRFs\n",
+				intf);
+			vty_out(vty, "You must specify the nexthop-vrf\n");
+			return CMD_WARNING_CONFIG_FAILED;
+		}
+		if (ifp->vrf->vrf_id != vrf->vrf_id)
+			vty_out(vty,
+				"Specified Intf %s is not in vrf %s but is in vrf %s, using actual vrf\n",
+				ifp->name, vrf->name, ifp->vrf->name);
+		nhop.ifindex = ifp->ifindex;
+		nhop.vrf_id = ifp->vrf->vrf_id;
 	}
 
 	if (addr) {
@@ -397,6 +718,9 @@ DEFPY(no_pbr_map_nexthop, no_pbr_map_nexthop_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	pbrms_clear_set_config(pbrms);
 
 	return CMD_SUCCESS;
@@ -411,6 +735,9 @@ DEFPY(pbr_map_vrf, pbr_map_vrf_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	/*
 	 * If an equivalent set vrf * exists, just return success.
 	 */
@@ -449,6 +776,9 @@ DEFPY(no_pbr_map_vrf, no_pbr_map_vrf_cmd,
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
+	if (!pbrms)
+		return CMD_WARNING_CONFIG_FAILED;
+
 	pbrms_clear_set_config(pbrms);
 
 	return CMD_SUCCESS;
@@ -521,10 +851,36 @@ DEFPY (show_pbr,
 	return CMD_SUCCESS;
 }
 
+static void
+pbrms_nexthop_group_write_individual_nexthop(
+	struct vty *vty, const struct pbr_map_sequence *pbrms)
+{
+	struct pbr_nexthop_group_cache find;
+	struct pbr_nexthop_group_cache *pnhgc;
+	struct pbr_nexthop_cache lookup;
+	struct pbr_nexthop_cache *pnhc;
+
+	memset(&find, 0, sizeof(find));
+	strlcpy(find.name, pbrms->internal_nhg_name, sizeof(find.name));
+
+	pnhgc = hash_lookup(pbr_nhg_hash, &find);
+	assert(pnhgc);
+
+	lookup.nexthop = *pbrms->nhg->nexthop;
+	pnhc = hash_lookup(pnhgc->nhh, &lookup);
+
+	nexthop_group_write_nexthop_simple(
+		vty, pbrms->nhg->nexthop,
+		pnhc->nexthop.ifindex != 0 ? pnhc->intf_name : NULL);
+	if (pnhc->nexthop.vrf_id != VRF_DEFAULT)
+		vty_out(vty, " nexthop-vrf %s", pnhc->vrf_name);
+
+	vty_out(vty, "\n");
+}
+
 static void vty_show_pbrms(struct vty *vty,
 			   const struct pbr_map_sequence *pbrms, bool detail)
 {
-	char buf[PREFIX_STRLEN];
 	char rbuf[64];
 
 	if (pbrms->reason)
@@ -541,41 +897,68 @@ static void vty_show_pbrms(struct vty *vty,
 			pbrms->installed ? "yes" : "no",
 			pbrms->reason ? rbuf : "Valid");
 
+	if (pbrms->ip_proto) {
+		struct protoent *p;
+
+		p = getprotobynumber(pbrms->ip_proto);
+		vty_out(vty, "        IP Protocol Match: %s\n", p->p_name);
+	}
+
 	if (pbrms->src)
-		vty_out(vty, "        SRC Match: %s\n",
-			prefix2str(pbrms->src, buf, sizeof(buf)));
+		vty_out(vty, "        SRC IP Match: %pFX\n", pbrms->src);
 	if (pbrms->dst)
-		vty_out(vty, "        DST Match: %s\n",
-			prefix2str(pbrms->dst, buf, sizeof(buf)));
+		vty_out(vty, "        DST IP Match: %pFX\n", pbrms->dst);
+	if (pbrms->src_prt)
+		vty_out(vty, "        SRC Port Match: %u\n", pbrms->src_prt);
+	if (pbrms->dst_prt)
+		vty_out(vty, "        DST Port Match: %u\n", pbrms->dst_prt);
+	if (pbrms->dsfield & PBR_DSFIELD_DSCP)
+		vty_out(vty, "        DSCP Match: %u\n",
+			(pbrms->dsfield & PBR_DSFIELD_DSCP) >> 2);
+	if (pbrms->dsfield & PBR_DSFIELD_ECN)
+		vty_out(vty, "        ECN Match: %u\n",
+			pbrms->dsfield & PBR_DSFIELD_ECN);
 	if (pbrms->mark)
 		vty_out(vty, "        MARK Match: %u\n", pbrms->mark);
 
+	if (pbrms->action_queue_id != PBR_MAP_UNDEFINED_QUEUE_ID)
+		vty_out(vty, "        Set Queue ID %u\n",
+			pbrms->action_queue_id);
+
+	if (pbrms->action_vlan_id != 0)
+		vty_out(vty, "        Set VLAN ID %u\n", pbrms->action_vlan_id);
+	if (pbrms->action_vlan_flags == PBR_MAP_STRIP_INNER_ANY)
+		vty_out(vty, "        Strip VLAN ID\n");
+	if (pbrms->action_pcp)
+		vty_out(vty, "        Set PCP %u\n", pbrms->action_pcp);
+
+
 	if (pbrms->nhgrp_name) {
 		vty_out(vty, "        Nexthop-Group: %s\n", pbrms->nhgrp_name);
 
 		if (detail)
 			vty_out(vty,
-				"          Installed: %u(%d) Tableid: %d\n",
+				"          Installed: %u(%d) Tableid: %u\n",
 				pbrms->nhs_installed,
 				pbr_nht_get_installed(pbrms->nhgrp_name),
 				pbr_nht_get_table(pbrms->nhgrp_name));
 		else
-			vty_out(vty, "          Installed: %s Tableid: %d\n",
+			vty_out(vty, "          Installed: %s Tableid: %u\n",
 				pbr_nht_get_installed(pbrms->nhgrp_name) ? "yes"
 									 : "no",
 				pbr_nht_get_table(pbrms->nhgrp_name));
 
 	} else if (pbrms->nhg) {
 		vty_out(vty, "        ");
-		nexthop_group_write_nexthop(vty, pbrms->nhg->nexthop);
+		pbrms_nexthop_group_write_individual_nexthop(vty, pbrms);
 		if (detail)
 			vty_out(vty,
-				"          Installed: %u(%d) Tableid: %d\n",
+				"          Installed: %u(%d) Tableid: %u\n",
 				pbrms->nhs_installed,
 				pbr_nht_get_installed(pbrms->internal_nhg_name),
 				pbr_nht_get_table(pbrms->internal_nhg_name));
 		else
-			vty_out(vty, "          Installed: %s Tableid: %d\n",
+			vty_out(vty, "          Installed: %s Tableid: %u\n",
 				pbr_nht_get_installed(pbrms->internal_nhg_name)
 					? "yes"
 					: "no",
@@ -590,6 +973,62 @@ static void vty_show_pbrms(struct vty *vty,
 	}
 }
 
+static void vty_json_pbrms(json_object *j, struct vty *vty,
+			   const struct pbr_map_sequence *pbrms)
+{
+	json_object *jpbrm, *nexthop_group;
+	char *nhg_name = pbrms->nhgrp_name ? pbrms->nhgrp_name
+					   : pbrms->internal_nhg_name;
+	char rbuf[64];
+
+	jpbrm = json_object_new_object();
+
+	json_object_int_add(jpbrm, "id", pbrms->unique);
+
+	if (pbrms->reason)
+		pbr_map_reason_string(pbrms->reason, rbuf, sizeof(rbuf));
+
+	json_object_int_add(jpbrm, "sequenceNumber", pbrms->seqno);
+	json_object_int_add(jpbrm, "ruleNumber", pbrms->ruleno);
+	json_object_boolean_add(jpbrm, "vrfUnchanged", pbrms->vrf_unchanged);
+	json_object_boolean_add(jpbrm, "installed",
+				pbr_nht_get_installed(nhg_name));
+	json_object_string_add(jpbrm, "installedReason",
+			       pbrms->reason ? rbuf : "Valid");
+
+	if (nhg_name) {
+		nexthop_group = json_object_new_object();
+
+		json_object_int_add(nexthop_group, "tableId",
+				    pbr_nht_get_table(nhg_name));
+		json_object_string_add(nexthop_group, "name", nhg_name);
+		json_object_boolean_add(nexthop_group, "installed",
+					pbr_nht_get_installed(nhg_name));
+		json_object_int_add(nexthop_group, "installedInternally",
+				    pbrms->nhs_installed);
+
+		json_object_object_add(jpbrm, "nexthopGroup", nexthop_group);
+	}
+
+	if (pbrms->vrf_lookup)
+		json_object_string_add(jpbrm, "vrfName", pbrms->vrf_name);
+
+	if (pbrms->src)
+		json_object_string_addf(jpbrm, "matchSrc", "%pFX", pbrms->src);
+	if (pbrms->dst)
+		json_object_string_addf(jpbrm, "matchDst", "%pFX", pbrms->dst);
+	if (pbrms->mark)
+		json_object_int_add(jpbrm, "matchMark", pbrms->mark);
+	if (pbrms->dsfield & PBR_DSFIELD_DSCP)
+		json_object_int_add(jpbrm, "matchDscp",
+				    (pbrms->dsfield & PBR_DSFIELD_DSCP) >> 2);
+	if (pbrms->dsfield & PBR_DSFIELD_ECN)
+		json_object_int_add(jpbrm, "matchEcn",
+				    pbrms->dsfield & PBR_DSFIELD_ECN);
+
+	json_object_array_add(j, jpbrm);
+}
+
 static void vty_show_pbr_map(struct vty *vty, const struct pbr_map *pbrm,
 			     bool detail)
 {
@@ -603,67 +1042,147 @@ static void vty_show_pbr_map(struct vty *vty, const struct pbr_map *pbrm,
 		vty_show_pbrms(vty, pbrms, detail);
 }
 
+static void vty_json_pbr_map(json_object *j, struct vty *vty,
+			     const struct pbr_map *pbrm)
+{
+	struct pbr_map_sequence *pbrms;
+	struct listnode *node;
+	json_object *jpbrms;
+
+	json_object_string_add(j, "name", pbrm->name);
+	json_object_boolean_add(j, "valid", pbrm->valid);
+
+	jpbrms = json_object_new_array();
+
+	for (ALL_LIST_ELEMENTS_RO(pbrm->seqnumbers, node, pbrms))
+		vty_json_pbrms(jpbrms, vty, pbrms);
+
+	json_object_object_add(j, "policies", jpbrms);
+}
+
 DEFPY (show_pbr_map,
 	show_pbr_map_cmd,
-	"show pbr map [NAME$name] [detail$detail]",
+	"show pbr map [NAME$name] [detail$detail] [json$json]",
 	SHOW_STR
 	PBR_STR
 	"PBR Map\n"
 	"PBR Map Name\n"
-	"Detailed information\n")
+	"Detailed information\n"
+	JSON_STR)
 {
 	struct pbr_map *pbrm;
+	json_object *j = NULL;
+
+	if (json)
+		j = json_object_new_array();
 
 	RB_FOREACH (pbrm, pbr_map_entry_head, &pbr_maps) {
+		json_object *this_map = NULL;
 		if (name && strcmp(name, pbrm->name) != 0)
 			continue;
 
+		if (j)
+			this_map = json_object_new_object();
+
+		if (this_map) {
+			vty_json_pbr_map(this_map, vty, pbrm);
+
+			json_object_array_add(j, this_map);
+			continue;
+		}
+
 		vty_show_pbr_map(vty, pbrm, detail);
 	}
+
+	if (j)
+		vty_json(vty, j);
+
 	return CMD_SUCCESS;
 }
 
 DEFPY(show_pbr_nexthop_group,
       show_pbr_nexthop_group_cmd,
-      "show pbr nexthop-groups [WORD$word]",
+      "show pbr nexthop-groups [WORD$word] [json$json]",
       SHOW_STR
       PBR_STR
       "Nexthop Groups\n"
-      "Optional Name of the nexthop group\n")
+      "Optional Name of the nexthop group\n"
+      JSON_STR)
 {
-	pbr_nht_show_nexthop_group(vty, word);
+	json_object *j = NULL;
+
+	if (json)
+		j = json_object_new_array();
+
+	if (j) {
+		pbr_nht_json_nexthop_group(j, word);
+
+		vty_json(vty, j);
+	} else
+		pbr_nht_show_nexthop_group(vty, word);
+
 
 	return CMD_SUCCESS;
 }
 
 DEFPY (show_pbr_interface,
 	show_pbr_interface_cmd,
-	"show pbr interface [NAME$name]",
+	"show pbr interface [NAME$name] [json$json]",
 	SHOW_STR
 	PBR_STR
 	"PBR Interface\n"
-	"PBR Interface Name\n")
+	"PBR Interface Name\n"
+	JSON_STR)
 {
 	struct interface *ifp;
 	struct vrf *vrf;
 	struct pbr_interface *pbr_ifp;
+	json_object *j = NULL;
+
+	if (json)
+		j = json_object_new_array();
 
 	RB_FOREACH(vrf, vrf_name_head, &vrfs_by_name) {
 		FOR_ALL_INTERFACES(vrf, ifp) {
 			struct pbr_map *pbrm;
+			json_object *this_iface = NULL;
 
-			if (!ifp->info)
-				continue;
+			if (j)
+				this_iface = json_object_new_object();
 
-			if (name && strcmp(ifp->name, name) != 0)
+			if (!ifp->info) {
+				json_object_free(this_iface);
 				continue;
+			}
+
+			if (name && strcmp(ifp->name, name) != 0) {
+				json_object_free(this_iface);
+				continue;
+			}
 
 			pbr_ifp = ifp->info;
 
-			if (strcmp(pbr_ifp->mapname, "") == 0)
+			if (strcmp(pbr_ifp->mapname, "") == 0) {
+				json_object_free(this_iface);
 				continue;
+			}
 
 			pbrm = pbrm_find(pbr_ifp->mapname);
+
+			if (this_iface) {
+				json_object_string_add(this_iface, "name",
+						       ifp->name);
+				json_object_int_add(this_iface, "index",
+						    ifp->ifindex);
+				json_object_string_add(this_iface, "policy",
+						       pbr_ifp->mapname);
+				json_object_boolean_add(this_iface, "valid",
+							pbrm);
+
+				json_object_array_add(j, this_iface);
+				continue;
+			}
+
 			vty_out(vty, "  %s(%d) with pbr-policy %s", ifp->name,
 				ifp->ifindex, pbr_ifp->mapname);
 			if (!pbrm)
@@ -672,6 +1191,9 @@ DEFPY (show_pbr_interface,
 		}
 	}
 
+	if (j)
+		vty_json(vty, j);
+
 	return CMD_SUCCESS;
 }
 
@@ -724,21 +1246,14 @@ DEFUN_NOSH(show_debugging_pbr,
 
 	pbr_debug_config_write_helper(vty, false);
 
+	cmd_show_lib_debugs(vty);
+
 	return CMD_SUCCESS;
 }
 
 /* ------------------------------------------------------------------------- */
 
 
-static int pbr_interface_config_write(struct vty *vty);
-static struct cmd_node interface_node = {
-	.name = "interface",
-	.node = INTERFACE_NODE,
-	.parent_node = CONFIG_NODE,
-	.prompt = "%s(config-if)# ",
-	.config_write = pbr_interface_config_write,
-};
-
 static int pbr_interface_config_write(struct vty *vty)
 {
 	struct interface *ifp;
@@ -746,18 +1261,14 @@ static int pbr_interface_config_write(struct vty *vty)
 
 	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
 		FOR_ALL_INTERFACES (vrf, ifp) {
-			if (vrf->vrf_id == VRF_DEFAULT)
-				vty_frame(vty, "interface %s\n", ifp->name);
-			else
-				vty_frame(vty, "interface %s vrf %s\n",
-					  ifp->name, vrf->name);
+			if_vty_config_start(vty, ifp);
 
 			if (ifp->desc)
 				vty_out(vty, " description %s\n", ifp->desc);
 
 			pbr_map_write_interfaces(vty, ifp);
 
-			vty_endframe(vty, "!\n");
+			if_vty_config_end(vty);
 		}
 	}
 
@@ -778,21 +1289,50 @@ static int pbr_vty_map_config_write_sequence(struct vty *vty,
 					     struct pbr_map *pbrm,
 					     struct pbr_map_sequence *pbrms)
 {
-	char buff[PREFIX_STRLEN];
-
 	vty_out(vty, "pbr-map %s seq %u\n", pbrm->name, pbrms->seqno);
 
 	if (pbrms->src)
-		vty_out(vty, " match src-ip %s\n",
-			prefix2str(pbrms->src, buff, sizeof(buff)));
+		vty_out(vty, " match src-ip %pFX\n", pbrms->src);
 
 	if (pbrms->dst)
-		vty_out(vty, " match dst-ip %s\n",
-			prefix2str(pbrms->dst, buff, sizeof(buff)));
+		vty_out(vty, " match dst-ip %pFX\n", pbrms->dst);
+
+	if (pbrms->src_prt)
+		vty_out(vty, " match src-port %u\n", pbrms->src_prt);
+	if (pbrms->dst_prt)
+		vty_out(vty, " match dst-port %u\n", pbrms->dst_prt);
+
+	if (pbrms->ip_proto) {
+		struct protoent *p;
+
+		p = getprotobynumber(pbrms->ip_proto);
+		vty_out(vty, " match ip-protocol %s\n", p->p_name);
+	}
+
+	if (pbrms->dsfield & PBR_DSFIELD_DSCP)
+		vty_out(vty, " match dscp %u\n",
+			(pbrms->dsfield & PBR_DSFIELD_DSCP) >> 2);
+
+	if (pbrms->dsfield & PBR_DSFIELD_ECN)
+		vty_out(vty, " match ecn %u\n",
+			pbrms->dsfield & PBR_DSFIELD_ECN);
 
 	if (pbrms->mark)
 		vty_out(vty, " match mark %u\n", pbrms->mark);
 
+
+	if (pbrms->action_queue_id != PBR_MAP_UNDEFINED_QUEUE_ID)
+		vty_out(vty, " set queue-id %d\n", pbrms->action_queue_id);
+
+	if (pbrms->action_pcp)
+		vty_out(vty, " set pcp %d\n", pbrms->action_pcp);
+
+	if (pbrms->action_vlan_id)
+		vty_out(vty, " set vlan %u\n", pbrms->action_vlan_id);
+
+	if (pbrms->action_vlan_flags == PBR_MAP_STRIP_INNER_ANY)
+		vty_out(vty, " strip vlan any\n");
+
 	if (pbrms->vrf_unchanged)
 		vty_out(vty, " set vrf unchanged\n");
 
@@ -804,9 +1344,10 @@ static int pbr_vty_map_config_write_sequence(struct vty *vty,
 
 	if (pbrms->nhg) {
 		vty_out(vty, " set ");
-		nexthop_group_write_nexthop(vty, pbrms->nhg->nexthop);
+		pbrms_nexthop_group_write_individual_nexthop(vty, pbrms);
 	}
 
+	vty_out(vty, "exit\n");
 	vty_out(vty, "!\n");
 	return 1;
 }
@@ -846,20 +1387,23 @@ static const struct cmd_variable_handler pbr_map_name[] = {
 	}
 };
 
+extern struct zebra_privs_t pbr_privs;
+
 void pbr_vty_init(void)
 {
 	cmd_variable_handler_register(pbr_map_name);
 
-	install_node(&interface_node);
-	if_cmd_init();
+	vrf_cmd_init(NULL);
+
+	if_cmd_init(pbr_interface_config_write);
 
 	install_node(&pbr_map_node);
 
 	/* debug */
 	install_node(&debug_node);
-	install_element(VIEW_NODE, &debug_pbr_cmd);
+	install_element(ENABLE_NODE, &debug_pbr_cmd);
 	install_element(CONFIG_NODE, &debug_pbr_cmd);
-	install_element(VIEW_NODE, &show_debugging_pbr_cmd);
+	install_element(ENABLE_NODE, &show_debugging_pbr_cmd);
 
 	install_default(PBRMAP_NODE);
 
@@ -868,9 +1412,18 @@ void pbr_vty_init(void)
 	install_element(CONFIG_NODE, &pbr_set_table_range_cmd);
 	install_element(CONFIG_NODE, &no_pbr_set_table_range_cmd);
 	install_element(INTERFACE_NODE, &pbr_policy_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_ip_proto_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_src_port_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_dst_port_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_match_src_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_match_dst_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_dscp_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_ecn_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_match_mark_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_action_queue_id_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_action_strip_vlan_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_action_vlan_id_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_action_pcp_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_nexthop_group_cmd);
 	install_element(PBRMAP_NODE, &no_pbr_map_nexthop_group_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_nexthop_cmd);
