@@ -22,6 +22,7 @@
 #include <zebra.h>
 
 #include "pimd.h"
+#include "pim_instance.h"
 #include "pim_util.h"
 #include "pim_sock.h"
 #include "pim_rp.h"
@@ -66,7 +67,6 @@ static bool mtrace_fwd_info_weak(struct pim_instance *pim,
 	struct pim_nexthop nexthop;
 	struct interface *ifp_in;
 	struct in_addr nh_addr;
-	char nexthop_str[INET_ADDRSTRLEN];
 
 	nh_addr.s_addr = INADDR_ANY;
 
@@ -82,13 +82,9 @@ static bool mtrace_fwd_info_weak(struct pim_instance *pim,
 		zlog_debug("mtrace pim_nexthop_lookup OK");
 
 	if (PIM_DEBUG_MTRACE)
-		zlog_warn("mtrace next_hop=%s",
-			  inet_ntop(nexthop.mrib_nexthop_addr.family,
-				    &nexthop.mrib_nexthop_addr.u.prefix,
-				    nexthop_str, sizeof(nexthop_str)));
+		zlog_debug("mtrace next_hop=%pPAs", &nexthop.mrib_nexthop_addr);
 
-	if (nexthop.mrib_nexthop_addr.family == AF_INET)
-		nh_addr = nexthop.mrib_nexthop_addr.u.prefix4;
+	nh_addr = nexthop.mrib_nexthop_addr;
 
 	ifp_in = nexthop.interface;
 
@@ -109,21 +105,20 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 			    struct igmp_mtrace_rsp *rspp,
 			    struct interface **ifpp)
 {
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct pim_upstream *up;
 	struct interface *ifp_in;
 	struct in_addr nh_addr;
 	uint32_t total;
-	char up_str[INET_ADDRSTRLEN];
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src = mtracep->src_addr;
 	sg.grp = mtracep->grp_addr;
 
 	up = pim_upstream_find(pim, &sg);
 
 	if (!up) {
-		sg.src.s_addr = INADDR_ANY;
+		sg.src = PIMADDR_ANY;
 		up = pim_upstream_find(pim, &sg);
 	}
 
@@ -138,13 +133,11 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	}
 
 	ifp_in = up->rpf.source_nexthop.interface;
-	nh_addr = up->rpf.source_nexthop.mrib_nexthop_addr.u.prefix4;
+	nh_addr = up->rpf.source_nexthop.mrib_nexthop_addr;
 	total = htonl(MTRACE_UNKNOWN_COUNT);
 
 	if (PIM_DEBUG_MTRACE)
-		zlog_debug("fwd_info: upstream next hop=%s",
-			   inet_ntop(AF_INET, &(nh_addr), up_str,
-				     sizeof(up_str)));
+		zlog_debug("fwd_info: upstream next hop=%pI4", &nh_addr);
 
 	if (up->channel_oil)
 		total = up->channel_oil->cc.pktcnt;
@@ -160,7 +153,7 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	rspp->rtg_proto = MTRACE_RTG_PROTO_PIM;
 
 	/* 6.2.2. 4. Fill in ... S, and Src Mask */
-	if (sg.src.s_addr != INADDR_ANY) {
+	if (!pim_addr_is_any(sg.src)) {
 		rspp->s = 1;
 		rspp->src_mask = MTRACE_SRC_MASK_SOURCE;
 	} else {
@@ -198,31 +191,19 @@ static void mtrace_rsp_init(struct igmp_mtrace_rsp *mtrace_rspp)
 static void mtrace_rsp_debug(uint32_t qry_id, int rsp,
 			     struct igmp_mtrace_rsp *mrspp)
 {
-	char inc_str[INET_ADDRSTRLEN];
-	char out_str[INET_ADDRSTRLEN];
-	char prv_str[INET_ADDRSTRLEN];
+	struct in_addr incoming = mrspp->incoming;
+	struct in_addr outgoing = mrspp->outgoing;
+	struct in_addr prev_hop = mrspp->prev_hop;
 
 	zlog_debug(
-		"Rx mt(%d) qid=%ud arr=%x in=%s out=%s prev=%s proto=%d fwd=%d",
-		rsp, ntohl(qry_id), mrspp->arrival,
-		inet_ntop(AF_INET, &(mrspp->incoming), inc_str,
-			  sizeof(inc_str)),
-		inet_ntop(AF_INET, &(mrspp->outgoing), out_str,
-			  sizeof(out_str)),
-		inet_ntop(AF_INET, &(mrspp->prev_hop), prv_str,
-			  sizeof(prv_str)),
-		mrspp->rtg_proto, mrspp->fwd_code);
+		"Rx mt(%d) qid=%ud arr=%x in=%pI4 out=%pI4 prev=%pI4 proto=%d fwd=%d",
+		rsp, ntohl(qry_id), mrspp->arrival, &incoming, &outgoing,
+		&prev_hop, mrspp->rtg_proto, mrspp->fwd_code);
 }
 
 static void mtrace_debug(struct pim_interface *pim_ifp,
 			 struct igmp_mtrace *mtracep, int mtrace_len)
 {
-	char inc_str[INET_ADDRSTRLEN];
-	char grp_str[INET_ADDRSTRLEN];
-	char src_str[INET_ADDRSTRLEN];
-	char dst_str[INET_ADDRSTRLEN];
-	char rsp_str[INET_ADDRSTRLEN];
-
 	struct in_addr ga, sa, da, ra;
 
 	ga = mtracep->grp_addr;
@@ -231,21 +212,10 @@ static void mtrace_debug(struct pim_interface *pim_ifp,
 	ra = mtracep->rsp_addr;
 
 	zlog_debug(
-		"Rx mtrace packet incoming on %s: "
-		"hops=%d type=%d size=%d, grp=%s, src=%s,"
-		" dst=%s rsp=%s ttl=%d qid=%ud",
-		inet_ntop(AF_INET, &(pim_ifp->primary_address), inc_str,
-			  sizeof(inc_str)),
-		mtracep->hops, mtracep->type, mtrace_len,
-		inet_ntop(AF_INET, &ga, grp_str,
-			  sizeof(grp_str)),
-		inet_ntop(AF_INET, &sa, src_str,
-			  sizeof(src_str)),
-		inet_ntop(AF_INET, &da, dst_str,
-			  sizeof(dst_str)),
-		inet_ntop(AF_INET, &ra, rsp_str,
-			  sizeof(rsp_str)),
-		mtracep->rsp_ttl, ntohl(mtracep->qry_id));
+		"Rx mtrace packet incoming on %pI4: hops=%d type=%d size=%d, grp=%pI4, src=%pI4, dst=%pI4 rsp=%pI4 ttl=%d qid=%ud",
+		&pim_ifp->primary_address, mtracep->hops, mtracep->type,
+		mtrace_len, &ga, &sa, &da, &ra, mtracep->rsp_ttl,
+		ntohl(mtracep->qry_id));
 	if (mtrace_len > (int)sizeof(struct igmp_mtrace)) {
 
 		int i;
@@ -255,8 +225,7 @@ static void mtrace_debug(struct pim_interface *pim_ifp,
 		if ((responses % sizeof(struct igmp_mtrace_rsp)) != 0)
 			if (PIM_DEBUG_MTRACE)
 				zlog_debug(
-					"Mtrace response block of wrong"
-					" length");
+					"Mtrace response block of wrong length");
 
 		responses = responses / sizeof(struct igmp_mtrace_rsp);
 
@@ -273,8 +242,8 @@ static uint32_t query_arrival_time(void)
 
 	if (gettimeofday(&tv, NULL) < 0) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn("Query arrival time lookup failed: errno=%d: %s",
-				  errno, safe_strerror(errno));
+			zlog_debug("Query arrival time lookup failed: errno=%d: %s",
+				   errno, safe_strerror(errno));
 		return 0;
 	}
 	/* not sure second offset correct, as I get different value */
@@ -293,8 +262,6 @@ static int mtrace_send_packet(struct interface *ifp,
 	ssize_t sent;
 	int ret;
 	int fd;
-	char if_str[INET_ADDRSTRLEN];
-	char rsp_str[INET_ADDRSTRLEN];
 	uint8_t ttl;
 
 	memset(&to, 0, sizeof(to));
@@ -304,13 +271,11 @@ static int mtrace_send_packet(struct interface *ifp,
 
 	if (PIM_DEBUG_MTRACE) {
 		struct in_addr if_addr;
+		struct in_addr rsp_addr = mtracep->rsp_addr;
 
 		if_addr = mtrace_primary_address(ifp);
-		zlog_debug(
-			"Sending mtrace packet to %s on %s",
-			inet_ntop(AF_INET, &mtracep->rsp_addr, rsp_str,
-				  sizeof(rsp_str)),
-			inet_ntop(AF_INET, &if_addr, if_str, sizeof(if_str)));
+		zlog_debug("Sending mtrace packet to %pI4 on %pI4", &rsp_addr,
+			   &if_addr);
 	}
 
 	fd = pim_socket_raw(IPPROTO_IGMP);
@@ -339,7 +304,7 @@ static int mtrace_send_packet(struct interface *ifp,
 
 		if (ret < 0) {
 			if (PIM_DEBUG_MTRACE)
-				zlog_warn("Failed to set socket multicast TTL");
+				zlog_debug("Failed to set socket multicast TTL");
 			ret = -1;
 			goto close_fd;
 		}
@@ -357,18 +322,15 @@ static int mtrace_send_packet(struct interface *ifp,
 			       sizeof(group_str));
 		if (sent < 0) {
 			if (PIM_DEBUG_MTRACE)
-				zlog_warn(
-					"Send mtrace request failed for %s on"
-					"%s: group=%s msg_size=%zd: errno=%d: "
-					" %s",
+				zlog_debug(
+					"Send mtrace request failed for %s on%s: group=%s msg_size=%zd: errno=%d:  %s",
 					dst_str, ifp->name, group_str,
 					mtrace_buf_len, errno,
 					safe_strerror(errno));
 		} else {
 			if (PIM_DEBUG_MTRACE)
-				zlog_warn(
-					"Send mtrace request failed for %s on"
-					" %s: group=%s msg_size=%zd: sent=%zd",
+				zlog_debug(
+					"Send mtrace request failed for %s on %s: group=%s msg_size=%zd: sent=%zd",
 					dst_str, ifp->name, group_str,
 					mtrace_buf_len, sent);
 		}
@@ -403,6 +365,20 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 	if (ip_hdr->ip_ttl-- <= 1)
 		return -1;
 
+	if (interface == NULL) {
+		memset(&nexthop, 0, sizeof(nexthop));
+		if (!pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0)) {
+			if (PIM_DEBUG_MTRACE)
+				zlog_debug(
+					"Dropping mtrace packet, no route to destination");
+			return -1;
+		}
+
+		if_out = nexthop.interface;
+	} else {
+		if_out = interface;
+	}
+
 	ip_hdr->ip_sum = in_cksum(ip_hdr, ip_hdr->ip_hl * 4);
 
 	fd = pim_socket_raw(IPPROTO_RAW);
@@ -412,22 +388,6 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 
 	pim_socket_ip_hdr(fd);
 
-	if (interface == NULL) {
-		memset(&nexthop, 0, sizeof(nexthop));
-		if (!pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0)) {
-			close(fd);
-			if (PIM_DEBUG_MTRACE)
-				zlog_warn(
-					"Dropping mtrace packet, "
-					"no route to destination");
-			return -1;
-		}
-
-		if_out = nexthop.interface;
-	} else {
-		if_out = interface;
-	}
-
 	ret = pim_socket_bind(fd, if_out);
 
 	if (ret < 0) {
@@ -447,16 +407,15 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 
 	if (sent < 0) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Failed to forward mtrace packet:"
-				" sendto errno=%d, %s",
+			zlog_debug(
+				"Failed to forward mtrace packet: sendto errno=%d, %s",
 				errno, safe_strerror(errno));
 		return -1;
 	}
 
 	if (PIM_DEBUG_MTRACE) {
-		zlog_debug("Fwd mtrace packet len=%u to %s ttl=%u",
-			   ntohs(ip_hdr->ip_len), inet_ntoa(ip_hdr->ip_dst),
+		zlog_debug("Fwd mtrace packet len=%u to %pI4 ttl=%u",
+			   ntohs(ip_hdr->ip_len), &ip_hdr->ip_dst,
 			   ip_hdr->ip_ttl);
 	}
 
@@ -465,14 +424,14 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 
 static int mtrace_mc_forward_packet(struct pim_instance *pim, struct ip *ip_hdr)
 {
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct channel_oil *c_oil;
 	struct listnode *chnode;
 	struct listnode *chnextnode;
 	struct pim_ifchannel *ch = NULL;
 	int ret = -1;
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.grp = ip_hdr->ip_dst;
 
 	c_oil = pim_find_channel_oil(pim, &sg);
@@ -480,10 +439,9 @@ static int mtrace_mc_forward_packet(struct pim_instance *pim, struct ip *ip_hdr)
 	if (c_oil == NULL) {
 		if (PIM_DEBUG_MTRACE) {
 			zlog_debug(
-				"Dropping mtrace multicast packet "
-				"len=%u to %s ttl=%u",
+				"Dropping mtrace multicast packet len=%u to %pI4 ttl=%u",
 				ntohs(ip_hdr->ip_len),
-				inet_ntoa(ip_hdr->ip_dst), ip_hdr->ip_ttl);
+				&ip_hdr->ip_dst, ip_hdr->ip_ttl);
 		}
 		return -1;
 	}
@@ -517,25 +475,25 @@ static int mtrace_send_mc_response(struct pim_instance *pim,
 				   struct igmp_mtrace *mtracep,
 				   size_t mtrace_len)
 {
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct channel_oil *c_oil;
 	struct listnode *chnode;
 	struct listnode *chnextnode;
 	struct pim_ifchannel *ch = NULL;
 	int ret = -1;
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.grp = mtracep->rsp_addr;
 
 	c_oil = pim_find_channel_oil(pim, &sg);
 
 	if (c_oil == NULL) {
 		if (PIM_DEBUG_MTRACE) {
+			struct in_addr rsp_addr = mtracep->rsp_addr;
+
 			zlog_debug(
-				"Dropping mtrace multicast response packet "
-				"len=%u to %s",
-				(unsigned int)mtrace_len,
-				inet_ntoa(mtracep->rsp_addr));
+				"Dropping mtrace multicast response packet len=%u to %pI4",
+				(unsigned int)mtrace_len, &rsp_addr);
 		}
 		return -1;
 	}
@@ -570,7 +528,6 @@ static int mtrace_send_response(struct pim_instance *pim,
 
 	if (IPV4_CLASS_DE(ntohl(mtracep->rsp_addr.s_addr))) {
 		struct pim_rpf *p_rpf;
-		char grp_str[INET_ADDRSTRLEN];
 
 		if (pim_rp_i_am_rp(pim, mtracep->rsp_addr))
 			return mtrace_send_mc_response(pim, mtracep,
@@ -579,11 +536,11 @@ static int mtrace_send_response(struct pim_instance *pim,
 		p_rpf = pim_rp_g(pim, mtracep->rsp_addr);
 
 		if (p_rpf == NULL) {
-			if (PIM_DEBUG_MTRACE)
-				zlog_warn("mtrace no RP for %s",
-					  inet_ntop(AF_INET,
-						    &(mtracep->rsp_addr),
-						    grp_str, sizeof(grp_str)));
+			if (PIM_DEBUG_MTRACE) {
+				struct in_addr rsp_addr = mtracep->rsp_addr;
+
+				zlog_debug("mtrace no RP for %pI4", &rsp_addr);
+			}
 			return -1;
 		}
 		nexthop = p_rpf->source_nexthop;
@@ -594,9 +551,8 @@ static int mtrace_send_response(struct pim_instance *pim,
 		/* TODO: should use unicast rib lookup */
 		if (!pim_nexthop_lookup(pim, &nexthop, mtracep->rsp_addr, 1)) {
 			if (PIM_DEBUG_MTRACE)
-				zlog_warn(
-					"Dropped response qid=%ud, no route to "
-					"response address",
+				zlog_debug(
+					"Dropped response qid=%ud, no route to response address",
 					mtracep->qry_id);
 			return -1;
 		}
@@ -606,7 +562,7 @@ static int mtrace_send_response(struct pim_instance *pim,
 				  mtracep->rsp_addr, mtracep->grp_addr);
 }
 
-int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
+int igmp_mtrace_recv_qry_req(struct gm_sock *igmp, struct ip *ip_hdr,
 			     struct in_addr from, const char *from_str,
 			     char *igmp_msg, int igmp_msg_len)
 {
@@ -638,15 +594,14 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	 * if applicable
 	 */
 	if (!IPV4_CLASS_DE(ntohl(ip_hdr->ip_dst.s_addr)))
-		if (!if_lookup_exact_address(&ip_hdr->ip_dst, AF_INET,
-					     pim->vrf_id))
+		if (!if_address_is_local(&ip_hdr->ip_dst, AF_INET,
+					 pim->vrf->vrf_id))
 			return mtrace_forward_packet(pim, ip_hdr);
 
 	if (igmp_msg_len < (int)sizeof(struct igmp_mtrace)) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace packet from %s on %s: too short,"
-				" len=%d, min=%zu",
+			zlog_debug(
+				"Recv mtrace packet from %s on %s: too short, len=%d, min=%zu",
 				from_str, ifp->name, igmp_msg_len,
 				sizeof(struct igmp_mtrace));
 		return -1;
@@ -662,15 +617,14 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	if (recv_checksum != checksum) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace packet from %s on %s: checksum"
-				" mismatch: received=%x computed=%x",
+			zlog_debug(
+				"Recv mtrace packet from %s on %s: checksum mismatch: received=%x computed=%x",
 				from_str, ifp->name, recv_checksum, checksum);
 		return -1;
 	}
 
 	/* Collecting IGMP Rx stats */
-	igmp->rx_stats.mtrace_req++;
+	igmp->igmp_stats.mtrace_req++;
 
 	if (PIM_DEBUG_MTRACE)
 		mtrace_debug(pim_ifp, mtracep, igmp_msg_len);
@@ -688,8 +642,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 			if (IPV4_CLASS_DE(ntohl(ip_hdr->ip_dst.s_addr))) {
 				if (PIM_DEBUG_MTRACE)
 					zlog_debug(
-						"Dropping multicast query "
-						"on wrong interface");
+						"Dropping multicast query on wrong interface");
 				return -1;
 			}
 			/* Unicast query on wrong interface */
@@ -700,8 +653,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 		if (qry_id == mtracep->qry_id && qry_src == from.s_addr) {
 			if (PIM_DEBUG_MTRACE)
 				zlog_debug(
-					"Dropping multicast query with "
-					"duplicate source and id");
+					"Dropping multicast query with duplicate source and id");
 			return -1;
 		}
 		qry_id = mtracep->qry_id;
@@ -715,14 +667,13 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 			last_rsp_ind = r_len / sizeof(struct igmp_mtrace_rsp);
 		if (last_rsp_ind > MTRACE_MAX_HOPS) {
 			if (PIM_DEBUG_MTRACE)
-				zlog_warn("Mtrace request of excessive size");
+				zlog_debug("Mtrace request of excessive size");
 			return -1;
 		}
 	} else {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace packet from %s on %s: "
-				"invalid length %d",
+			zlog_debug(
+				"Recv mtrace packet from %s on %s: invalid length %d",
 				from_str, ifp->name, igmp_msg_len);
 		return -1;
 	}
@@ -731,10 +682,9 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	if (IPV4_CLASS_DE(ntohl(ip_hdr->ip_dst.s_addr))
 	    && !IPV4_MC_LINKLOCAL(ntohl(ip_hdr->ip_dst.s_addr))) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace packet from %s on %s:"
-				" not link-local multicast %s",
-				from_str, ifp->name, inet_ntoa(ip_hdr->ip_dst));
+			zlog_debug(
+				"Recv mtrace packet from %s on %s: not link-local multicast %pI4",
+				from_str, ifp->name, &ip_hdr->ip_dst);
 		return -1;
 	}
 
@@ -819,7 +769,8 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	}
 
 	/* 6.2.2 8. If this router is the Rendez-vous Point */
-	if (pim_rp_i_am_rp(pim, mtracep->grp_addr)) {
+	if (mtracep->grp_addr.s_addr != INADDR_ANY &&
+	    pim_rp_i_am_rp(pim, mtracep->grp_addr)) {
 		mtrace_rsp_set_fwd_code(rspp, MTRACE_FWD_CODE_REACHED_RP);
 		/* 7.7.1. PIM-SM ...RP has not performed source-specific join */
 		if (rspp->src_mask == MTRACE_SRC_MASK_GROUP)
@@ -847,7 +798,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 }
 
 /* 6.3. Traceroute responses */
-int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
+int igmp_mtrace_recv_response(struct gm_sock *igmp, struct ip *ip_hdr,
 			      struct in_addr from, const char *from_str,
 			      char *igmp_msg, int igmp_msg_len)
 {
@@ -865,9 +816,8 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	if (igmp_msg_len < (int)sizeof(struct igmp_mtrace)) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace packet from %s on %s: too short,"
-				" len=%d, min=%zu",
+			zlog_debug(
+				"Recv mtrace packet from %s on %s: too short, len=%d, min=%zu",
 				from_str, ifp->name, igmp_msg_len,
 				sizeof(struct igmp_mtrace));
 		return -1;
@@ -883,9 +833,8 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	if (recv_checksum != checksum) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(
-				"Recv mtrace response from %s on %s: checksum"
-				" mismatch: received=%x computed=%x",
+			zlog_debug(
+				"Recv mtrace response from %s on %s: checksum mismatch: received=%x computed=%x",
 				from_str, ifp->name, recv_checksum, checksum);
 		return -1;
 	}
@@ -893,7 +842,7 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 	mtracep->checksum = checksum;
 
 	/* Collecting IGMP Rx stats */
-	igmp->rx_stats.mtrace_rsp++;
+	igmp->igmp_stats.mtrace_rsp++;
 
 	if (PIM_DEBUG_MTRACE)
 		mtrace_debug(pim_ifp, mtracep, igmp_msg_len);
