@@ -39,18 +39,18 @@ char *pim_channel_oil_dump(struct channel_oil *c_oil, char *buf, size_t size)
 {
 	char *out;
 	struct interface *ifp;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	int i;
 
-	sg.src = c_oil->oil.mfcc_origin;
-	sg.grp = c_oil->oil.mfcc_mcastgrp;
-	ifp = pim_if_find_by_vif_index(c_oil->pim, c_oil->oil.mfcc_parent);
-	snprintf(buf, size, "%s IIF: %s, OIFS: ", pim_str_sg_dump(&sg),
-		 ifp ? ifp->name : "(?)");
+	sg.src = *oil_origin(c_oil);
+	sg.grp = *oil_mcastgrp(c_oil);
+	ifp = pim_if_find_by_vif_index(c_oil->pim, *oil_parent(c_oil));
+	snprintfrr(buf, size, "%pSG IIF: %s, OIFS: ", &sg,
+		   ifp ? ifp->name : "(?)");
 
 	out = buf + strlen(buf);
 	for (i = 0; i < MAXVIFS; i++) {
-		if (c_oil->oil.mfcc_ttls[i] != 0) {
+		if (oil_if_has(c_oil, i) != 0) {
 			ifp = pim_if_find_by_vif_index(c_oil->pim, i);
 			snprintf(out, buf + size - out, "%s ",
 				 ifp ? ifp->name : "(?)");
@@ -61,25 +61,19 @@ char *pim_channel_oil_dump(struct channel_oil *c_oil, char *buf, size_t size)
 	return buf;
 }
 
-int pim_channel_oil_compare(const struct channel_oil *c1,
-			    const struct channel_oil *c2)
+int pim_channel_oil_compare(const struct channel_oil *cc1,
+			    const struct channel_oil *cc2)
 {
-	if (ntohl(c1->oil.mfcc_mcastgrp.s_addr)
-	    < ntohl(c2->oil.mfcc_mcastgrp.s_addr))
-		return -1;
-
-	if (ntohl(c1->oil.mfcc_mcastgrp.s_addr)
-	    > ntohl(c2->oil.mfcc_mcastgrp.s_addr))
-		return 1;
-
-	if (ntohl(c1->oil.mfcc_origin.s_addr)
-	    < ntohl(c2->oil.mfcc_origin.s_addr))
-		return -1;
-
-	if (ntohl(c1->oil.mfcc_origin.s_addr)
-	    > ntohl(c2->oil.mfcc_origin.s_addr))
-		return 1;
+	struct channel_oil *c1 = (struct channel_oil *)cc1;
+	struct channel_oil *c2 = (struct channel_oil *)cc2;
+	int rv;
 
+	rv = pim_addr_cmp(*oil_mcastgrp(c1), *oil_mcastgrp(c2));
+	if (rv)
+		return rv;
+	rv = pim_addr_cmp(*oil_origin(c1), *oil_origin(c2));
+	if (rv)
+		return rv;
 	return 0;
 }
 
@@ -104,13 +98,13 @@ void pim_channel_oil_free(struct channel_oil *c_oil)
 }
 
 struct channel_oil *pim_find_channel_oil(struct pim_instance *pim,
-					 struct prefix_sg *sg)
+					 pim_sgaddr *sg)
 {
 	struct channel_oil *c_oil = NULL;
 	struct channel_oil lookup;
 
-	lookup.oil.mfcc_mcastgrp = sg->grp;
-	lookup.oil.mfcc_origin = sg->src;
+	*oil_mcastgrp(&lookup) = sg->grp;
+	*oil_origin(&lookup) = sg->src;
 
 	c_oil = rb_pim_oil_find(&pim->channel_oil_head, &lookup);
 
@@ -118,8 +112,7 @@ struct channel_oil *pim_find_channel_oil(struct pim_instance *pim,
 }
 
 struct channel_oil *pim_channel_oil_add(struct pim_instance *pim,
-					struct prefix_sg *sg,
-					const char *name)
+					pim_sgaddr *sg, const char *name)
 {
 	struct channel_oil *c_oil;
 
@@ -145,17 +138,17 @@ struct channel_oil *pim_channel_oil_add(struct pim_instance *pim,
 
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
-				"%s(%s): Existing oil for %pSG4 Ref Count: %d (Post Increment)",
+				"%s(%s): Existing oil for %pSG Ref Count: %d (Post Increment)",
 				__func__, name, sg, c_oil->oil_ref_count);
 		return c_oil;
 	}
 
 	c_oil = XCALLOC(MTYPE_PIM_CHANNEL_OIL, sizeof(*c_oil));
 
-	c_oil->oil.mfcc_mcastgrp = sg->grp;
-	c_oil->oil.mfcc_origin = sg->src;
+	*oil_mcastgrp(c_oil) = sg->grp;
+	*oil_origin(c_oil) = sg->src;
 
-	c_oil->oil.mfcc_parent = MAXVIFS;
+	*oil_parent(c_oil) = MAXVIFS;
 	c_oil->oil_ref_count = 1;
 	c_oil->installed = 0;
 	c_oil->up = pim_upstream_find(pim, sg);
@@ -164,21 +157,45 @@ struct channel_oil *pim_channel_oil_add(struct pim_instance *pim,
 	rb_pim_oil_add(&pim->channel_oil_head, c_oil);
 
 	if (PIM_DEBUG_MROUTE)
-		zlog_debug("%s(%s): c_oil %s add",
-				__func__, name, pim_str_sg_dump(sg));
+		zlog_debug("%s(%s): c_oil %pSG add", __func__, name, sg);
 
 	return c_oil;
 }
 
+
+/*
+ * Clean up mroute and channel oil created for dropping pkts from directly
+ * connected source when the interface was non DR.
+ */
+void pim_clear_nocache_state(struct pim_interface *pim_ifp)
+{
+	struct channel_oil *c_oil;
+
+	frr_each_safe (rb_pim_oil, &pim_ifp->pim->channel_oil_head, c_oil) {
+
+		if ((!c_oil->up) ||
+		    !(PIM_UPSTREAM_FLAG_TEST_SRC_NOCACHE(c_oil->up->flags)))
+			continue;
+
+		if (*oil_parent(c_oil) != pim_ifp->mroute_vif_index)
+			continue;
+
+		THREAD_OFF(c_oil->up->t_ka_timer);
+		PIM_UPSTREAM_FLAG_UNSET_SRC_NOCACHE(c_oil->up->flags);
+		PIM_UPSTREAM_FLAG_UNSET_SRC_STREAM(c_oil->up->flags);
+		pim_upstream_del(pim_ifp->pim, c_oil->up, __func__);
+	}
+}
+
 struct channel_oil *pim_channel_oil_del(struct channel_oil *c_oil,
 					const char *name)
 {
 	if (PIM_DEBUG_MROUTE) {
-		struct prefix_sg sg = {.src = c_oil->oil.mfcc_mcastgrp,
-				       .grp = c_oil->oil.mfcc_origin};
+		pim_sgaddr sg = {.src = *oil_origin(c_oil),
+				 .grp = *oil_mcastgrp(c_oil)};
 
 		zlog_debug(
-			"%s(%s): Del oil for %pSG4, Ref Count: %d (Predecrement)",
+			"%s(%s): Del oil for %pSG, Ref Count: %d (Predecrement)",
 			__func__, name, &sg, c_oil->oil_ref_count);
 	}
 	--c_oil->oil_ref_count;
@@ -219,34 +236,30 @@ int pim_channel_del_oif(struct channel_oil *channel_oil, struct interface *oif,
 {
 	struct pim_interface *pim_ifp;
 
-	zassert(channel_oil);
-	zassert(oif);
+	assert(channel_oil);
+	assert(oif);
 
 	pim_ifp = oif->info;
 
+	assertf(pim_ifp->mroute_vif_index >= 0,
+		"trying to del OIF %s with VIF (%d)", oif->name,
+		pim_ifp->mroute_vif_index);
+
 	/*
 	 * Don't do anything if we've been asked to remove a source
 	 * that is not actually on it.
 	 */
 	if (!(channel_oil->oif_flags[pim_ifp->mroute_vif_index] & proto_mask)) {
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s %s: no existing protocol mask %u(%u) for requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%s,%s)",
+				"%s %s: no existing protocol mask %u(%u) for requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%pPAs,%pPAs)",
 				__FILE__, __func__, proto_mask,
 				channel_oil
 					->oif_flags[pim_ifp->mroute_vif_index],
 				oif->name, pim_ifp->mroute_vif_index,
-				channel_oil->oil
-					.mfcc_ttls[pim_ifp->mroute_vif_index],
-				source_str, group_str);
+				oil_if_has(channel_oil, pim_ifp->mroute_vif_index),
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 		return 0;
 	}
@@ -256,44 +269,29 @@ int pim_channel_del_oif(struct channel_oil *channel_oil, struct interface *oif,
 	if (channel_oil->oif_flags[pim_ifp->mroute_vif_index] &
 			PIM_OIF_FLAG_PROTO_ANY) {
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s %s: other protocol masks remain for requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%s,%s)",
+				"%s %s: other protocol masks remain for requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%pPAs,%pPAs)",
 				__FILE__, __func__, oif->name,
 				pim_ifp->mroute_vif_index,
-				channel_oil->oil
-					.mfcc_ttls[pim_ifp->mroute_vif_index],
-				source_str, group_str);
+				oil_if_has(channel_oil, pim_ifp->mroute_vif_index),
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 		return 0;
 	}
 
-	channel_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index] = 0;
+	oil_if_set(channel_oil, pim_ifp->mroute_vif_index, 0);
 	/* clear mute; will be re-evaluated when the OIF becomes valid again */
 	channel_oil->oif_flags[pim_ifp->mroute_vif_index] &= ~PIM_OIF_FLAG_MUTE;
 
 	if (pim_upstream_mroute_add(channel_oil, __func__)) {
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s %s: could not remove output interface %s (vif_index=%d) for channel (S,G)=(%s,%s)",
+				"%s %s: could not remove output interface %s (vif_index=%d) for channel (S,G)=(%pPAs,%pPAs)",
 				__FILE__, __func__, oif->name,
-				pim_ifp->mroute_vif_index, source_str,
-				group_str);
+				pim_ifp->mroute_vif_index,
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 		return -1;
 	}
@@ -301,16 +299,12 @@ int pim_channel_del_oif(struct channel_oil *channel_oil, struct interface *oif,
 	--channel_oil->oil_size;
 
 	if (PIM_DEBUG_MROUTE) {
-		char group_str[INET_ADDRSTRLEN];
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<group?>", channel_oil->oil.mfcc_mcastgrp,
-			       group_str, sizeof(group_str));
-		pim_inet4_dump("<source?>", channel_oil->oil.mfcc_origin,
-			       source_str, sizeof(source_str));
 		zlog_debug(
-			"%s(%s): (S,G)=(%s,%s): proto_mask=%u IIF:%d OIF=%s vif_index=%d",
-			__func__, caller, source_str, group_str, proto_mask,
-			channel_oil->oil.mfcc_parent, oif->name,
+			"%s(%s): (S,G)=(%pPAs,%pPAs): proto_mask=%u IIF:%d OIF=%s vif_index=%d",
+			__func__, caller, oil_origin(channel_oil),
+			oil_mcastgrp(channel_oil),
+			proto_mask,
+			*oil_parent(channel_oil), oif->name,
 			pim_ifp->mroute_vif_index);
 	}
 
@@ -328,8 +322,8 @@ void pim_channel_del_inherited_oif(struct channel_oil *c_oil,
 	/* if an inherited OIF is being removed join-desired can change
 	 * if the inherited OIL is now empty and KAT is running
 	 */
-	if (up && up->sg.src.s_addr != INADDR_ANY &&
-			pim_upstream_empty_inherited_olist(up))
+	if (up && !pim_addr_is_any(up->sg.src) &&
+	    pim_upstream_empty_inherited_olist(up))
 		pim_upstream_update_join_desired(up->pim, up);
 }
 
@@ -375,6 +369,20 @@ static bool pim_channel_eval_oif_mute(struct channel_oil *c_oil,
 		return do_mute;
 	}
 
+	if (PIM_I_am_DualActive(pim_ifp)) {
+		struct pim_upstream *starup = c_oil->up->parent;
+		if (PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(c_oil->up->flags)
+		    && (PIM_UPSTREAM_FLAG_TEST_MLAG_NON_DF(c_oil->up->flags)))
+			do_mute = true;
+
+		/* In case entry is (S,G), Negotiation happens at (*.G) */
+		if (starup
+
+		    && PIM_UPSTREAM_FLAG_TEST_MLAG_INTERFACE(starup->flags)
+		    && (PIM_UPSTREAM_FLAG_TEST_MLAG_NON_DF(starup->flags)))
+			do_mute = true;
+		return do_mute;
+	}
 	return do_mute;
 }
 
@@ -385,7 +393,7 @@ void pim_channel_update_oif_mute(struct channel_oil *c_oil,
 	bool new_mute;
 
 	/* If pim_ifp is not a part of the OIL there is nothing to do */
-	if (!c_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index])
+	if (!oil_if_has(c_oil, pim_ifp->mroute_vif_index))
 		return;
 
 	old_mute = !!(c_oil->oif_flags[pim_ifp->mroute_vif_index] &
@@ -412,9 +420,11 @@ static void pim_channel_update_mute(struct channel_oil *c_oil)
 	struct pim_interface *pim_reg_ifp;
 	struct pim_interface *vxlan_ifp;
 
-	pim_reg_ifp = c_oil->pim->regiface->info;
-	if (pim_reg_ifp)
-		pim_channel_update_oif_mute(c_oil, pim_reg_ifp);
+	if (c_oil->pim->regiface) {
+		pim_reg_ifp = c_oil->pim->regiface->info;
+		if (pim_reg_ifp)
+			pim_channel_update_oif_mute(c_oil, pim_reg_ifp);
+	}
 	vxlan_ifp = pim_vxlan_get_term_ifp(c_oil->pim);
 	if (vxlan_ifp)
 		pim_channel_update_oif_mute(c_oil, vxlan_ifp);
@@ -437,25 +447,23 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 
 	pim_ifp = oif->info;
 
+	assertf(pim_ifp->mroute_vif_index >= 0,
+		"trying to add OIF %s with VIF (%d)", oif->name,
+		pim_ifp->mroute_vif_index);
+
 	/* Prevent single protocol from subscribing same interface to
 	   channel (S,G) multiple times */
 	if (channel_oil->oif_flags[pim_ifp->mroute_vif_index] & proto_mask) {
+		channel_oil->oif_flags[pim_ifp->mroute_vif_index] |= proto_mask;
+
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s %s: existing protocol mask %u requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%s,%s)",
+				"%s %s: existing protocol mask %u requested OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%pPAs,%pPAs)",
 				__FILE__, __func__, proto_mask, oif->name,
 				pim_ifp->mroute_vif_index,
-				channel_oil->oil
-					.mfcc_ttls[pim_ifp->mroute_vif_index],
-				source_str, group_str);
+				oil_if_has(channel_oil, pim_ifp->mroute_vif_index),
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 		return -3;
 	}
@@ -473,38 +481,21 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 		channel_oil->oif_flags[pim_ifp->mroute_vif_index] |= proto_mask;
 		/* Check the OIF really exists before returning, and only log
 		   warning otherwise */
-		if (channel_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index] < 1) {
-			{
-				char group_str[INET_ADDRSTRLEN];
-				char source_str[INET_ADDRSTRLEN];
-				pim_inet4_dump("<group?>",
-					       channel_oil->oil.mfcc_mcastgrp,
-					       group_str, sizeof(group_str));
-				pim_inet4_dump("<source?>",
-					       channel_oil->oil.mfcc_origin,
-					       source_str, sizeof(source_str));
-				zlog_warn(
-					"%s %s: new protocol mask %u requested nonexistent OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%s,%s)",
-					__FILE__, __func__, proto_mask,
-					oif->name, pim_ifp->mroute_vif_index,
-					channel_oil->oil.mfcc_ttls
-						[pim_ifp->mroute_vif_index],
-					source_str, group_str);
-			}
+		if (oil_if_has(channel_oil, pim_ifp->mroute_vif_index) < 1) {
+			zlog_warn(
+				"%s %s: new protocol mask %u requested nonexistent OIF %s (vif_index=%d, min_ttl=%d) for channel (S,G)=(%pPAs,%pPAs)",
+				__FILE__, __func__, proto_mask, oif->name,
+				pim_ifp->mroute_vif_index,
+				oil_if_has(channel_oil, pim_ifp->mroute_vif_index),
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s(%s): (S,G)=(%s,%s): proto_mask=%u OIF=%s vif_index=%d added to 0x%x",
-				__func__, caller, source_str, group_str,
+				"%s(%s): (S,G)=(%pPAs,%pPAs): proto_mask=%u OIF=%s vif_index=%d added to 0x%x",
+				__func__, caller, oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil),
 				proto_mask, oif->name,
 				pim_ifp->mroute_vif_index,
 				channel_oil
@@ -513,29 +504,21 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 		return 0;
 	}
 
-	old_ttl = channel_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index];
+	old_ttl = oil_if_has(channel_oil, pim_ifp->mroute_vif_index);
 
 	if (old_ttl > 0) {
 		if (PIM_DEBUG_MROUTE) {
-			char group_str[INET_ADDRSTRLEN];
-			char source_str[INET_ADDRSTRLEN];
-			pim_inet4_dump("<group?>",
-				       channel_oil->oil.mfcc_mcastgrp,
-				       group_str, sizeof(group_str));
-			pim_inet4_dump("<source?>",
-				       channel_oil->oil.mfcc_origin, source_str,
-				       sizeof(source_str));
 			zlog_debug(
-				"%s %s: interface %s (vif_index=%d) is existing output for channel (S,G)=(%s,%s)",
+				"%s %s: interface %s (vif_index=%d) is existing output for channel (S,G)=(%pPAs,%pPAs)",
 				__FILE__, __func__, oif->name,
-				pim_ifp->mroute_vif_index, source_str,
-				group_str);
+				pim_ifp->mroute_vif_index,
+				oil_origin(channel_oil),
+				oil_mcastgrp(channel_oil));
 		}
 		return -4;
 	}
 
-	channel_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index] =
-		PIM_MROUTE_MIN_TTL;
+	oil_if_set(channel_oil, pim_ifp->mroute_vif_index, PIM_MROUTE_MIN_TTL);
 
 	/* Some OIFs are held in a muted state i.e. the PIM state machine
 	 * decided to include the OIF but additional status check such as
@@ -552,26 +535,19 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 	/* channel_oil->oil.mfcc_parent != MAXVIFS indicate this entry is not
 	 * valid to get installed in kernel.
 	 */
-	if (channel_oil->oil.mfcc_parent != MAXVIFS) {
+	if (*oil_parent(channel_oil) != MAXVIFS) {
 		if (pim_upstream_mroute_add(channel_oil, __func__)) {
 			if (PIM_DEBUG_MROUTE) {
-				char group_str[INET_ADDRSTRLEN];
-				char source_str[INET_ADDRSTRLEN];
-				pim_inet4_dump("<group?>",
-				      channel_oil->oil.mfcc_mcastgrp,
-				      group_str, sizeof(group_str));
-				pim_inet4_dump("<source?>",
-				      channel_oil->oil.mfcc_origin, source_str,
-				      sizeof(source_str));
 				zlog_debug(
-					"%s %s: could not add output interface %s (vif_index=%d) for channel (S,G)=(%s,%s)",
+					"%s %s: could not add output interface %s (vif_index=%d) for channel (S,G)=(%pPAs,%pPAs)",
 					__FILE__, __func__, oif->name,
-					pim_ifp->mroute_vif_index, source_str,
-					group_str);
+					pim_ifp->mroute_vif_index,
+					oil_origin(channel_oil),
+					oil_mcastgrp(channel_oil));
 			}
 
-			channel_oil->oil.mfcc_ttls[pim_ifp->mroute_vif_index]
-				= old_ttl;
+			oil_if_set(channel_oil, pim_ifp->mroute_vif_index,
+				   old_ttl);
 			return -5;
 		}
 	}
@@ -582,15 +558,11 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 	channel_oil->oif_flags[pim_ifp->mroute_vif_index] |= proto_mask;
 
 	if (PIM_DEBUG_MROUTE) {
-		char group_str[INET_ADDRSTRLEN];
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<group?>", channel_oil->oil.mfcc_mcastgrp,
-			       group_str, sizeof(group_str));
-		pim_inet4_dump("<source?>", channel_oil->oil.mfcc_origin,
-			       source_str, sizeof(source_str));
 		zlog_debug(
-			"%s(%s): (S,G)=(%s,%s): proto_mask=%u OIF=%s vif_index=%d: DONE",
-			__func__, caller, source_str, group_str, proto_mask,
+			"%s(%s): (S,G)=(%pPAs,%pPAs): proto_mask=%u OIF=%s vif_index=%d: DONE",
+			__func__, caller, oil_origin(channel_oil),
+			oil_mcastgrp(channel_oil),
+			proto_mask,
 			oif->name, pim_ifp->mroute_vif_index);
 	}
 
@@ -599,7 +571,7 @@ int pim_channel_add_oif(struct channel_oil *channel_oil, struct interface *oif,
 
 int pim_channel_oil_empty(struct channel_oil *c_oil)
 {
-	static struct mfcctl null_oil;
+	static struct channel_oil null_oil;
 
 	if (!c_oil)
 		return 1;
@@ -608,6 +580,10 @@ int pim_channel_oil_empty(struct channel_oil *c_oil)
 	 * non-NULL.
 	 * pimreg device (in all vrfs) uses a vifi of
 	 * 0 (PIM_OIF_PIM_REGISTER_VIF) so we simply mfcc_ttls[0] */
-	return !memcmp(&c_oil->oil.mfcc_ttls[1], &null_oil.mfcc_ttls[1],
-		sizeof(null_oil.mfcc_ttls) - sizeof(null_oil.mfcc_ttls[0]));
+	if (oil_if_has(c_oil, 0))
+		oil_if_set(&null_oil, 0, 1);
+	else
+		oil_if_set(&null_oil, 0, 0);
+
+	return !oil_if_cmp(&c_oil->oil, &null_oil.oil);
 }
