@@ -35,22 +35,22 @@
 #define PIM_SOCK_ERR_NAME    (-10) /* Socket name (getsockname) */
 #define PIM_SOCK_ERR_BIND    (-11) /* Can't bind to interface */
 
+struct pim_instance;
+
 int pim_socket_bind(int fd, struct interface *ifp);
 void pim_socket_ip_hdr(int fd);
 int pim_socket_raw(int protocol);
-int pim_socket_mcast(int protocol, struct in_addr ifaddr, struct interface *ifp,
+int pim_socket_mcast(int protocol, pim_addr ifaddr, struct interface *ifp,
 		     uint8_t loop);
-int pim_socket_join(int fd, struct in_addr group, struct in_addr ifaddr,
-		    ifindex_t ifindex);
-int pim_socket_join_source(int fd, ifindex_t ifindex, struct in_addr group_addr,
-			   struct in_addr source_addr, const char *ifname);
+int pim_socket_join(int fd, pim_addr group, pim_addr ifaddr, ifindex_t ifindex,
+		    struct pim_interface *pim_ifp);
 int pim_socket_recvfromto(int fd, uint8_t *buf, size_t len,
-			  struct sockaddr_in *from, socklen_t *fromlen,
-			  struct sockaddr_in *to, socklen_t *tolen,
+			  struct sockaddr_storage *from, socklen_t *fromlen,
+			  struct sockaddr_storage *to, socklen_t *tolen,
 			  ifindex_t *ifindex);
 
-int pim_socket_mcastloop_get(int fd);
-
 int pim_socket_getsockname(int fd, struct sockaddr *name, socklen_t *namelen);
 
+int pim_reg_sock(void);
+
 #endif /* PIM_SOCK_H */
