@@ -23,27 +23,24 @@
 #include <zebra.h>
 #include "zclient.h"
 
-#include "pim_igmp.h"
 #include "pim_ifchannel.h"
 
 void pim_zebra_init(void);
 void pim_zebra_zclient_update(struct vty *vty);
 
-void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index);
 void pim_scan_oil(struct pim_instance *pim_matcher);
 
-void igmp_anysource_forward_start(struct pim_instance *pim,
-				  struct igmp_group *group);
-void igmp_anysource_forward_stop(struct igmp_group *group);
-
-void igmp_source_forward_start(struct pim_instance *pim,
-			       struct igmp_source *source);
-void igmp_source_forward_stop(struct igmp_source *source);
-void igmp_source_forward_reevaluate_all(struct pim_instance *pim);
-
 void pim_forward_start(struct pim_ifchannel *ch);
-void pim_forward_stop(struct pim_ifchannel *ch, bool install_it);
+void pim_forward_stop(struct pim_ifchannel *ch);
 
 void sched_rpf_cache_refresh(struct pim_instance *pim);
 struct zclient *pim_zebra_zclient_get(void);
+
+void pim_zebra_update_all_interfaces(struct pim_instance *pim);
+void pim_zebra_upstream_rpf_changed(struct pim_instance *pim,
+				    struct pim_upstream *up,
+				    struct pim_rpf *old);
+
+void pim_zebra_interface_set_master(struct interface *vrf,
+				    struct interface *ifp);
 #endif /* PIM_ZEBRA_H */
