@@ -56,6 +56,8 @@ DEFUN_NOSH (show_debugging_ripng,
 	if (IS_RIPNG_DEBUG_ZEBRA)
 		vty_out(vty, "  RIPng zebra debugging is on\n");
 
+	cmd_show_lib_debugs(vty);
+
 	return CMD_SUCCESS;
 }
 
@@ -218,7 +220,7 @@ void ripng_debug_init(void)
 
 	install_node(&debug_node);
 
-	install_element(VIEW_NODE, &show_debugging_ripng_cmd);
+	install_element(ENABLE_NODE, &show_debugging_ripng_cmd);
 
 	install_element(ENABLE_NODE, &debug_ripng_events_cmd);
 	install_element(ENABLE_NODE, &debug_ripng_packet_cmd);
