@@ -40,8 +40,8 @@
 
 #define VRRP_LOGPFX "[CORE] "
 
-DEFINE_MTYPE_STATIC(VRRPD, VRRP_IP, "VRRP IP address")
-DEFINE_MTYPE_STATIC(VRRPD, VRRP_RTR, "VRRP Router")
+DEFINE_MTYPE_STATIC(VRRPD, VRRP_IP, "VRRP IP address");
+DEFINE_MTYPE_STATIC(VRRPD, VRRP_RTR, "VRRP Router");
 
 /* statics */
 struct hash *vrrp_vrouters_hash;
@@ -216,7 +216,7 @@ static struct vrrp_vrouter *vrrp_lookup_by_if_mvl(struct interface *mvl_ifp)
 		return NULL;
 	}
 
-	p = if_lookup_by_index(mvl_ifp->link_ifindex, mvl_ifp->vrf_id);
+	p = if_lookup_by_index(mvl_ifp->link_ifindex, mvl_ifp->vrf->vrf_id);
 
 	if (!p) {
 		DEBUGD(&vrrp_dbg_zebra,
@@ -405,7 +405,7 @@ static bool vrrp_has_ip(struct vrrp_vrouter *vr, struct ipaddr *ip)
 	struct ipaddr *iter;
 
 	for (ALL_LIST_ELEMENTS_RO(r->addrs, ln, iter))
-		if (!memcmp(&iter->ip, &ip->ip, IPADDRSZ(ip)))
+		if (!ipaddr_cmp(iter, ip))
 			return true;
 
 	return false;
@@ -484,7 +484,7 @@ int vrrp_del_ip(struct vrrp_vrouter *vr, struct ipaddr *ip)
 		return 0;
 
 	for (ALL_LIST_ELEMENTS(r->addrs, ln, nn, iter))
-		if (!memcmp(&iter->ip, &ip->ip, IPADDRSZ(ip)))
+		if (!ipaddr_cmp(iter, ip))
 			list_delete_node(r->addrs, ln);
 
 	/*
@@ -544,7 +544,7 @@ static bool vrrp_attach_interface(struct vrrp_router *r)
 
 	size_t ifps_cnt =
 		if_lookup_by_hwaddr(r->vmac.octet, sizeof(r->vmac.octet), &ifps,
-				    r->vr->ifp->vrf_id);
+				    r->vr->ifp->vrf->vrf_id);
 
 	/*
 	 * Filter to only those macvlan interfaces whose parent is the base
@@ -647,6 +647,8 @@ struct vrrp_vrouter *vrrp_vrouter_create(struct interface *ifp, uint8_t vrid,
 	vr->priority = vd.priority;
 	vr->preempt_mode = vd.preempt_mode;
 	vr->accept_mode = vd.accept_mode;
+	vr->checksum_with_ipv4_pseudoheader =
+		vd.checksum_with_ipv4_pseudoheader;
 	vr->shutdown = vd.shutdown;
 
 	vr->v4 = vrrp_router_create(vr, AF_INET);
@@ -654,7 +656,7 @@ struct vrrp_vrouter *vrrp_vrouter_create(struct interface *ifp, uint8_t vrid,
 
 	vrrp_set_advertisement_interval(vr, vd.advertisement_interval);
 
-	hash_get(vrrp_vrouters_hash, vr, hash_alloc_intern);
+	(void)hash_get(vrrp_vrouters_hash, vr, hash_alloc_intern);
 
 	return vr;
 }
@@ -669,6 +671,9 @@ void vrrp_vrouter_destroy(struct vrrp_vrouter *vr)
 
 struct vrrp_vrouter *vrrp_lookup(const struct interface *ifp, uint8_t vrid)
 {
+	if (!ifp)
+		return NULL;
+
 	struct vrrp_vrouter vr;
 
 	vr.vrid = vrid;
@@ -681,8 +686,8 @@ struct vrrp_vrouter *vrrp_lookup(const struct interface *ifp, uint8_t vrid)
 
 /* Forward decls */
 static void vrrp_change_state(struct vrrp_router *r, int to);
-static int vrrp_adver_timer_expire(struct thread *thread);
-static int vrrp_master_down_timer_expire(struct thread *thread);
+static void vrrp_adver_timer_expire(struct thread *thread);
+static void vrrp_master_down_timer_expire(struct thread *thread);
 
 /*
  * Finds the first connected address of the appropriate family on a VRRP
@@ -700,7 +705,6 @@ static int vrrp_master_down_timer_expire(struct thread *thread);
  */
 static int vrrp_bind_to_primary_connected(struct vrrp_router *r)
 {
-	char ipstr[INET6_ADDRSTRLEN];
 	struct interface *ifp;
 
 	/*
@@ -754,20 +758,15 @@ static int vrrp_bind_to_primary_connected(struct vrrp_router *r)
 	if (bind(r->sock_tx, (const struct sockaddr *)&su, sizeof(su)) < 0) {
 		zlog_err(
 			VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-			"Failed to bind Tx socket to primary IP address %s: %s",
-			r->vr->vrid, family2str(r->family),
-			inet_ntop(r->family,
-				  (const void *)&c->address->u.prefix, ipstr,
-				  sizeof(ipstr)),
+			"Failed to bind Tx socket to primary IP address %pFX: %s",
+			r->vr->vrid, family2str(r->family), c->address,
 			safe_strerror(errno));
 		ret = -1;
 	} else {
 		DEBUGD(&vrrp_dbg_sock,
 		       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-		       "Bound Tx socket to primary IP address %s",
-		       r->vr->vrid, family2str(r->family),
-		       inet_ntop(r->family, (const void *)&c->address->u.prefix,
-				 ipstr, sizeof(ipstr)));
+		       "Bound Tx socket to primary IP address %pFX",
+		       r->vr->vrid, family2str(r->family), c->address);
 	}
 
 	return ret;
@@ -795,7 +794,8 @@ static void vrrp_send_advertisement(struct vrrp_router *r)
 
 	pktsz = vrrp_pkt_adver_build(&pkt, &r->src, r->vr->version, r->vr->vrid,
 				     r->priority, r->vr->advertisement_interval,
-				     r->addrs->count, (struct ipaddr **)&addrs);
+				     r->addrs->count, (struct ipaddr **)&addrs,
+				     r->vr->checksum_with_ipv4_pseudoheader);
 
 	if (DEBUG_MODE_CHECK(&vrrp_dbg_pkt, DEBUG_MODE_ALL))
 		zlog_hexdump(pkt, (size_t)pktsz);
@@ -860,15 +860,14 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 	vrrp_pkt_adver_dump(dumpbuf, sizeof(dumpbuf), pkt);
 	DEBUGD(&vrrp_dbg_proto,
 	       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-	       "Received VRRP Advertisement from %s:\n%s",
+	       "Received VRRP Advertisement from %s: %s",
 	       r->vr->vrid, family2str(r->family), sipstr, dumpbuf);
 
 	/* Check that VRID matches our configured VRID */
 	if (pkt->hdr.vrid != r->vr->vrid) {
 		DEBUGD(&vrrp_dbg_proto,
 		       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-		       "Datagram invalid: Advertisement contains VRID %" PRIu8
-		       " which does not match our instance",
+		       "Datagram invalid: Advertisement contains VRID %hhu which does not match our instance",
 		       r->vr->vrid, family2str(r->family), pkt->hdr.vrid);
 		return -1;
 	}
@@ -888,8 +887,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 	if (r->vr->version == 2 && !adveq) {
 		DEBUGD(&vrrp_dbg_proto,
 		       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-		       "Datagram invalid: Received advertisement with advertisement interval %" PRIu8
-		       " unequal to our configured value %u",
+		       "Datagram invalid: Received advertisement with advertisement interval %hhu unequal to our configured value %u",
 		       r->vr->vrid, family2str(r->family),
 		       pkt->hdr.v2.adver_int,
 		       MAX(r->vr->advertisement_interval / 100, 1));
@@ -901,8 +899,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 	if (pkt->hdr.naddr != r->addrs->count)
 		DEBUGD(&vrrp_dbg_proto,
 		       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-		       "Datagram has %" PRIu8
-		       " addresses, but this VRRP instance has %u",
+		       "Datagram has %hhu addresses, but this VRRP instance has %u",
 		       r->vr->vrid, family2str(r->family), pkt->hdr.naddr,
 		       r->addrs->count);
 
@@ -912,7 +909,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 
 	switch (r->fsm.state) {
 	case VRRP_STATE_MASTER:
-		addrcmp = memcmp(&src->ip, &r->src.ip, IPADDRSZ(src));
+		addrcmp = ipaddr_cmp(src, &r->src);
 
 		if (pkt->hdr.priority == 0) {
 			vrrp_send_advertisement(r);
@@ -926,8 +923,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 			       && addrcmp > 0)) {
 			zlog_info(
 				VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-				"Received advertisement from %s w/ priority %" PRIu8
-				"; switching to Backup",
+				"Received advertisement from %s w/ priority %hhu; switching to Backup",
 				r->vr->vrid, family2str(r->family), sipstr,
 				pkt->hdr.priority);
 			THREAD_OFF(r->t_adver_timer);
@@ -946,8 +942,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 			/* Discard advertisement */
 			DEBUGD(&vrrp_dbg_proto,
 			       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-			       "Discarding advertisement from %s (%" PRIu8
-			       " <= %" PRIu8 " & %s <= %s)",
+			       "Discarding advertisement from %s (%hhu <= %hhu & %s <= %s)",
 			       r->vr->vrid, family2str(r->family), sipstr,
 			       pkt->hdr.priority, r->priority, sipstr, dipstr);
 		}
@@ -975,8 +970,7 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 			/* Discard advertisement */
 			DEBUGD(&vrrp_dbg_proto,
 			       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-			       "Discarding advertisement from %s (%" PRIu8
-			       " < %" PRIu8 " & preempt = true)",
+			       "Discarding advertisement from %s (%hhu < %hhu & preempt = true)",
 			       r->vr->vrid, family2str(r->family), sipstr,
 			       pkt->hdr.priority, r->priority);
 		}
@@ -995,9 +989,9 @@ static int vrrp_recv_advertisement(struct vrrp_router *r, struct ipaddr *src,
 /*
  * Read and process next IPvX datagram.
  */
-static int vrrp_read(struct thread *thread)
+static void vrrp_read(struct thread *thread)
 {
-	struct vrrp_router *r = thread->arg;
+	struct vrrp_router *r = THREAD_ARG(thread);
 
 	struct vrrp_pkt *pkt;
 	ssize_t pktsize;
@@ -1039,8 +1033,10 @@ static int vrrp_read(struct thread *thread)
 		zlog_hexdump(r->ibuf, nbytes);
 	}
 
-	pktsize = vrrp_pkt_parse_datagram(r->family, r->vr->version, &m, nbytes,
-					  &src, &pkt, errbuf, sizeof(errbuf));
+	pktsize = vrrp_pkt_parse_datagram(
+		r->family, r->vr->version,
+		r->vr->checksum_with_ipv4_pseudoheader, &m, nbytes, &src, &pkt,
+		errbuf, sizeof(errbuf));
 
 	if (pktsize < 0)
 		DEBUGD(&vrrp_dbg_pkt,
@@ -1057,8 +1053,6 @@ done:
 
 	if (resched)
 		thread_add_read(master, vrrp_read, r, r->sock_rx, &r->t_read);
-
-	return 0;
 }
 
 /*
@@ -1095,9 +1089,9 @@ static int vrrp_socket(struct vrrp_router *r)
 
 	frr_with_privs(&vrrp_privs) {
 		r->sock_rx = vrf_socket(r->family, SOCK_RAW, IPPROTO_VRRP,
-					r->vr->ifp->vrf_id, NULL);
+					r->vr->ifp->vrf->vrf_id, NULL);
 		r->sock_tx = vrf_socket(r->family, SOCK_RAW, IPPROTO_VRRP,
-					r->vr->ifp->vrf_id, NULL);
+					r->vr->ifp->vrf->vrf_id, NULL);
 	}
 
 	if (r->sock_rx < 0 || r->sock_tx < 0) {
@@ -1114,7 +1108,7 @@ static int vrrp_socket(struct vrrp_router *r)
 	 * Bind Tx socket to macvlan device - necessary for VRF support,
 	 * otherwise the kernel will select the vrf device
 	 */
-	if (r->vr->ifp->vrf_id != VRF_DEFAULT) {
+	if (r->vr->ifp->vrf->vrf_id != VRF_DEFAULT) {
 		frr_with_privs (&vrrp_privs) {
 			ret = setsockopt(r->sock_tx, SOL_SOCKET,
 					 SO_BINDTODEVICE, r->mvl_ifp->name,
@@ -1492,9 +1486,9 @@ static void vrrp_change_state(struct vrrp_router *r, int to)
 /*
  * Called when Adver_Timer expires.
  */
-static int vrrp_adver_timer_expire(struct thread *thread)
+static void vrrp_adver_timer_expire(struct thread *thread)
 {
-	struct vrrp_router *r = thread->arg;
+	struct vrrp_router *r = THREAD_ARG(thread);
 
 	DEBUGD(&vrrp_dbg_proto,
 	       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
@@ -1515,16 +1509,14 @@ static int vrrp_adver_timer_expire(struct thread *thread)
 			 r->vr->vrid, family2str(r->family),
 			 vrrp_state_names[r->fsm.state]);
 	}
-
-	return 0;
 }
 
 /*
  * Called when Master_Down_Timer expires.
  */
-static int vrrp_master_down_timer_expire(struct thread *thread)
+static void vrrp_master_down_timer_expire(struct thread *thread)
 {
-	struct vrrp_router *r = thread->arg;
+	struct vrrp_router *r = THREAD_ARG(thread);
 
 	zlog_info(VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
 		  "Master_Down_Timer expired",
@@ -1534,8 +1526,6 @@ static int vrrp_master_down_timer_expire(struct thread *thread)
 			      r->vr->advertisement_interval * CS2MS,
 			      &r->t_adver_timer);
 	vrrp_change_state(r, VRRP_STATE_MASTER);
-
-	return 0;
 }
 
 /*
@@ -1723,7 +1713,6 @@ static void vrrp_autoconfig_autoaddrupdate(struct vrrp_router *r)
 	struct listnode *ln;
 	struct connected *c = NULL;
 	bool is_v6_ll;
-	char ipbuf[INET6_ADDRSTRLEN];
 
 	if (!r->mvl_ifp)
 		return;
@@ -1736,12 +1725,10 @@ static void vrrp_autoconfig_autoaddrupdate(struct vrrp_router *r)
 		is_v6_ll = (c->address->family == AF_INET6
 			    && IN6_IS_ADDR_LINKLOCAL(&c->address->u.prefix6));
 		if (c->address->family == r->family && !is_v6_ll) {
-			inet_ntop(r->family, &c->address->u.prefix, ipbuf,
-				  sizeof(ipbuf));
 			DEBUGD(&vrrp_dbg_auto,
 			       VRRP_LOGPFX VRRP_LOGPFX_VRID VRRP_LOGPFX_FAM
-			       "Adding %s",
-			       r->vr->vrid, family2str(r->family), ipbuf);
+			       "Adding %pFX",
+			       r->vr->vrid, family2str(r->family), c->address);
 			if (r->family == AF_INET)
 				vrrp_add_ipv4(r->vr, c->address->u.prefix4);
 			else if (r->vr->version == 3)
@@ -1766,7 +1753,7 @@ vrrp_autoconfig_autocreate(struct interface *mvl_ifp)
 	struct interface *p;
 	struct vrrp_vrouter *vr;
 
-	p = if_lookup_by_index(mvl_ifp->link_ifindex, mvl_ifp->vrf_id);
+	p = if_lookup_by_index(mvl_ifp->link_ifindex, mvl_ifp->vrf->vrf_id);
 
 	if (!p)
 		return NULL;
@@ -2353,11 +2340,11 @@ int vrrp_config_write_global(struct vty *vty)
 
 	/* FIXME: needs to be udpated for full YANG conversion. */
 	if (vd.priority != VRRP_DEFAULT_PRIORITY && ++writes)
-		vty_out(vty, "vrrp default priority %" PRIu8 "\n", vd.priority);
+		vty_out(vty, "vrrp default priority %hhu\n", vd.priority);
 
 	if (vd.advertisement_interval != VRRP_DEFAULT_ADVINT && ++writes)
 		vty_out(vty,
-			"vrrp default advertisement-interval %" PRIu16 "\n",
+			"vrrp default advertisement-interval %u\n",
 			vd.advertisement_interval * CS2MS);
 
 	if (vd.preempt_mode != VRRP_DEFAULT_PREEMPT && ++writes)
@@ -2368,6 +2355,12 @@ int vrrp_config_write_global(struct vty *vty)
 		vty_out(vty, "%svrrp default accept\n",
 			!vd.accept_mode ? "no " : "");
 
+	if (vd.checksum_with_ipv4_pseudoheader !=
+		    VRRP_DEFAULT_CHECKSUM_WITH_IPV4_PSEUDOHEADER &&
+	    ++writes)
+		vty_out(vty, "%svrrp default checksum-with-ipv4-pseudoheader\n",
+			!vd.checksum_with_ipv4_pseudoheader ? "no " : "");
+
 	if (vd.shutdown != VRRP_DEFAULT_SHUTDOWN && ++writes)
 		vty_out(vty, "%svrrp default shutdown\n",
 			!vd.shutdown ? "no " : "");
@@ -2380,7 +2373,7 @@ static unsigned int vrrp_hash_key(const void *arg)
 	const struct vrrp_vrouter *vr = arg;
 	char key[IFNAMSIZ + 64];
 
-	snprintf(key, sizeof(key), "%s@%" PRIu8, vr->ifp->name, vr->vrid);
+	snprintf(key, sizeof(key), "%s@%u", vr->ifp->name, vr->vrid);
 
 	return string_hash_make(key);
 }
@@ -2408,12 +2401,14 @@ void vrrp_init(void)
 	vd.preempt_mode = yang_get_default_bool("%s/preempt", VRRP_XPATH_FULL);
 	vd.accept_mode =
 		yang_get_default_bool("%s/accept-mode", VRRP_XPATH_FULL);
+	vd.checksum_with_ipv4_pseudoheader = yang_get_default_bool(
+		"%s/checksum-with-ipv4-pseudoheader", VRRP_XPATH_FULL);
 	vd.shutdown = VRRP_DEFAULT_SHUTDOWN;
 
 	vrrp_autoconfig_version = 3;
 	vrrp_vrouters_hash = hash_create(&vrrp_hash_key, vrrp_hash_cmp,
 					 "VRRP virtual router hash");
-	vrf_init(NULL, NULL, NULL, NULL, NULL);
+	vrf_init(NULL, NULL, NULL, NULL);
 }
 
 void vrrp_fini(void)
