@@ -40,12 +40,31 @@ static int lib_interface_vrrp_vrrp_group_create(struct nb_cb_create_args *args)
 	uint8_t version = 3;
 	struct vrrp_vrouter *vr;
 
-	if (args->event != NB_EV_APPLY)
-		return NB_OK;
-
-	ifp = nb_running_get_entry(args->dnode, NULL, true);
 	vrid = yang_dnode_get_uint8(args->dnode, "./virtual-router-id");
 	version = yang_dnode_get_enum(args->dnode, "./version");
+
+	switch (args->event) {
+	case NB_EV_VALIDATE:
+		ifp = nb_running_get_entry(args->dnode, NULL, false);
+		if (ifp) {
+			vr = vrrp_lookup(ifp, vrid);
+			if (vr && vr->autoconf) {
+				snprintf(
+					args->errmsg, args->errmsg_len,
+					"Virtual Router with ID %d already exists on interface '%s'; created by VRRP autoconfiguration",
+					vrid, ifp->name);
+				return NB_ERR_VALIDATION;
+			}
+		}
+		return NB_OK;
+	case NB_EV_PREPARE:
+	case NB_EV_ABORT:
+		return NB_OK;
+	case NB_EV_APPLY:
+		break;
+	}
+
+	ifp = nb_running_get_entry(args->dnode, NULL, true);
 	vr = vrrp_vrouter_create(ifp, vrid, version);
 	nb_running_set_entry(args->dnode, vr);
 
@@ -103,7 +122,7 @@ lib_interface_vrrp_vrrp_group_get_keys(struct nb_cb_get_keys_args *args)
 	const struct vrrp_vrouter *vr = args->list_entry;
 
 	args->keys->num = 1;
-	snprintf(args->keys->key[0], sizeof(args->keys->key[0]), "%" PRIu32,
+	snprintf(args->keys->key[0], sizeof(args->keys->key[0]), "%u",
 		 vr->vrid);
 
 	return NB_OK;
@@ -227,11 +246,8 @@ lib_interface_vrrp_vrrp_group_v4_source_address_get_elem(
 {
 	const struct vrrp_vrouter *vr = args->list_entry;
 	struct yang_data *val = NULL;
-	struct ipaddr ip;
 
-	memset(&ip, 0x00, sizeof(ip));
-
-	if (memcmp(&vr->v4->src.ipaddr_v4, &ip.ipaddr_v4, sizeof(ip.ipaddr_v4)))
+	if (!ipaddr_is_zero(&vr->v4->src))
 		val = yang_data_new_ip(args->xpath, &vr->v4->src);
 
 	return val;
@@ -391,7 +407,7 @@ lib_interface_vrrp_vrrp_group_v6_source_address_get_elem(
 	const struct vrrp_vrouter *vr = args->list_entry;
 	struct yang_data *val = NULL;
 
-	if (ipaddr_isset(&vr->v6->src))
+	if (!ipaddr_is_zero(&vr->v6->src))
 		val = yang_data_new_ip(args->xpath, &vr->v6->src);
 
 	return val;
@@ -586,6 +602,26 @@ lib_interface_vrrp_vrrp_group_shutdown_modify(struct nb_cb_modify_args *args)
 	return NB_OK;
 }
 
+/*
+ * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/checksum-with-
+ *        ipv4-pseudoheader
+ */
+static int lib_interface_vrrp_vrrp_group_checksum_with_ipv4_pseudoheader_modify(
+	struct nb_cb_modify_args *args)
+{
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	struct vrrp_vrouter *vr;
+	bool checksum_with_ipv4_ph;
+
+	vr = nb_running_get_entry(args->dnode, NULL, true);
+	checksum_with_ipv4_ph = yang_dnode_get_bool(args->dnode, NULL);
+	vr->checksum_with_ipv4_pseudoheader = checksum_with_ipv4_ph;
+
+	return NB_OK;
+}
+
 /* clang-format off */
 const struct frr_yang_module_info frr_vrrpd_info = {
 	.name = "frr-vrrpd",
@@ -627,6 +663,13 @@ const struct frr_yang_module_info frr_vrrpd_info = {
 				.modify = lib_interface_vrrp_vrrp_group_accept_mode_modify,
 			}
 		},
+		{
+			.xpath = "/frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/checksum-with-ipv4-pseudoheader",
+			.cbs = {
+				.modify = lib_interface_vrrp_vrrp_group_checksum_with_ipv4_pseudoheader_modify,
+				.cli_show = cli_show_checksum_with_ipv4_pseudoheader,
+			}
+		},
 		{
 			.xpath = "/frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/advertisement-interval",
 			.cbs = {
