@@ -33,9 +33,7 @@
 #include "vrrp_debug.h"
 #include "vrrp_vty.h"
 #include "vrrp_zebra.h"
-#ifndef VTYSH_EXTRACT_PL
 #include "vrrpd/vrrp_vty_clippy.c"
-#endif
 
 
 #define VRRP_STR "Virtual Router Redundancy Protocol\n"
@@ -52,7 +50,7 @@
 /*
  * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group
  */
-DEFPY(vrrp_vrid,
+DEFPY_YANG(vrrp_vrid,
       vrrp_vrid_cmd,
       "[no] vrrp (1-255)$vrid [version (2-3)]",
       NO_STR
@@ -75,7 +73,7 @@ DEFPY(vrrp_vrid,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_vrrp(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
+void cli_show_vrrp(struct vty *vty, const struct lyd_node *dnode, bool show_defaults)
 {
 	const char *vrid = yang_dnode_get_string(dnode, "./virtual-router-id");
 	const char *ver = yang_dnode_get_string(dnode, "./version");
@@ -89,7 +87,7 @@ void cli_show_vrrp(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
 /*
  * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/shutdown
  */
-DEFPY(vrrp_shutdown,
+DEFPY_YANG(vrrp_shutdown,
       vrrp_shutdown_cmd,
       "[no] vrrp (1-255)$vrid shutdown",
       NO_STR
@@ -103,7 +101,7 @@ DEFPY(vrrp_shutdown,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_shutdown(struct vty *vty, struct lyd_node *dnode,
+void cli_show_shutdown(struct vty *vty, const struct lyd_node *dnode,
 		       bool show_defaults)
 {
 	const char *vrid = yang_dnode_get_string(dnode, "../virtual-router-id");
@@ -115,13 +113,13 @@ void cli_show_shutdown(struct vty *vty, struct lyd_node *dnode,
 /*
  * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/priority
  */
-DEFPY(vrrp_priority,
+DEFPY_YANG(vrrp_priority,
       vrrp_priority_cmd,
       "vrrp (1-255)$vrid priority (1-254)",
       VRRP_STR
       VRRP_VRID_STR
       VRRP_PRIORITY_STR
-      "Priority value")
+      "Priority value\n")
 {
 	nb_cli_enqueue_change(vty, "./priority", NB_OP_MODIFY, priority_str);
 
@@ -131,21 +129,21 @@ DEFPY(vrrp_priority,
 /*
  * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/priority
  */
-DEFPY(no_vrrp_priority,
+DEFPY_YANG(no_vrrp_priority,
       no_vrrp_priority_cmd,
       "no vrrp (1-255)$vrid priority [(1-254)]",
       NO_STR
       VRRP_STR
       VRRP_VRID_STR
       VRRP_PRIORITY_STR
-      "Priority value")
+      "Priority value\n")
 {
 	nb_cli_enqueue_change(vty, "./priority", NB_OP_MODIFY, NULL);
 
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_priority(struct vty *vty, struct lyd_node *dnode,
+void cli_show_priority(struct vty *vty, const struct lyd_node *dnode,
 		       bool show_defaults)
 {
 	const char *vrid = yang_dnode_get_string(dnode, "../virtual-router-id");
@@ -158,11 +156,11 @@ void cli_show_priority(struct vty *vty, struct lyd_node *dnode,
  * XPath:
  * /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/advertisement-interval
  */
-DEFPY(vrrp_advertisement_interval,
+DEFPY_YANG(vrrp_advertisement_interval,
       vrrp_advertisement_interval_cmd,
       "vrrp (1-255)$vrid advertisement-interval (10-40950)",
       VRRP_STR VRRP_VRID_STR VRRP_ADVINT_STR
-      "Advertisement interval in milliseconds; must be multiple of 10")
+      "Advertisement interval in milliseconds; must be multiple of 10\n")
 {
 	char val[20];
 
@@ -179,11 +177,11 @@ DEFPY(vrrp_advertisement_interval,
  * XPath:
  * /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/advertisement-interval
  */
-DEFPY(no_vrrp_advertisement_interval,
+DEFPY_YANG(no_vrrp_advertisement_interval,
       no_vrrp_advertisement_interval_cmd,
       "no vrrp (1-255)$vrid advertisement-interval [(10-40950)]",
       NO_STR VRRP_STR VRRP_VRID_STR VRRP_ADVINT_STR
-      "Advertisement interval in milliseconds; must be multiple of 10")
+      "Advertisement interval in milliseconds; must be multiple of 10\n")
 {
 	nb_cli_enqueue_change(vty, "./advertisement-interval", NB_OP_MODIFY,
 			      NULL);
@@ -191,7 +189,7 @@ DEFPY(no_vrrp_advertisement_interval,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_advertisement_interval(struct vty *vty, struct lyd_node *dnode,
+void cli_show_advertisement_interval(struct vty *vty, const struct lyd_node *dnode,
 				     bool show_defaults)
 {
 	const char *vrid = yang_dnode_get_string(dnode, "../virtual-router-id");
@@ -205,7 +203,7 @@ void cli_show_advertisement_interval(struct vty *vty, struct lyd_node *dnode,
  * XPath:
  * /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/v4/virtual-address
  */
-DEFPY(vrrp_ip,
+DEFPY_YANG(vrrp_ip,
       vrrp_ip_cmd,
       "[no] vrrp (1-255)$vrid ip A.B.C.D",
       NO_STR
@@ -220,7 +218,7 @@ DEFPY(vrrp_ip,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_ip(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
+void cli_show_ip(struct vty *vty, const struct lyd_node *dnode, bool show_defaults)
 {
 	const char *vrid =
 		yang_dnode_get_string(dnode, "../../virtual-router-id");
@@ -233,7 +231,7 @@ void cli_show_ip(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
  * XPath:
  * /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/v6/virtual-address
  */
-DEFPY(vrrp_ip6,
+DEFPY_YANG(vrrp_ip6,
       vrrp_ip6_cmd,
       "[no] vrrp (1-255)$vrid ipv6 X:X::X:X",
       NO_STR
@@ -248,7 +246,7 @@ DEFPY(vrrp_ip6,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_ipv6(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
+void cli_show_ipv6(struct vty *vty, const struct lyd_node *dnode, bool show_defaults)
 {
 	const char *vrid =
 		yang_dnode_get_string(dnode, "../../virtual-router-id");
@@ -260,7 +258,7 @@ void cli_show_ipv6(struct vty *vty, struct lyd_node *dnode, bool show_defaults)
 /*
  * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/preempt
  */
-DEFPY(vrrp_preempt,
+DEFPY_YANG(vrrp_preempt,
       vrrp_preempt_cmd,
       "[no] vrrp (1-255)$vrid preempt",
       NO_STR
@@ -274,7 +272,7 @@ DEFPY(vrrp_preempt,
 	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
 }
 
-void cli_show_preempt(struct vty *vty, struct lyd_node *dnode,
+void cli_show_preempt(struct vty *vty, const struct lyd_node *dnode,
 		      bool show_defaults)
 {
 	const char *vrid = yang_dnode_get_string(dnode, "../virtual-router-id");
@@ -283,8 +281,37 @@ void cli_show_preempt(struct vty *vty, struct lyd_node *dnode,
 	vty_out(vty, " %svrrp %s preempt\n", pre ? "" : "no ", vrid);
 }
 
+/*
+ * XPath: /frr-interface:lib/interface/frr-vrrpd:vrrp/vrrp-group/checksum-with-
+ *        ipv4-pseudoheader
+ */
+DEFPY_YANG(vrrp_checksum_with_ipv4_pseudoheader,
+      vrrp_checksum_with_ipv4_pseudoheader_cmd,
+      "[no] vrrp (1-255)$vrid checksum-with-ipv4-pseudoheader",
+      NO_STR
+      VRRP_STR
+      VRRP_VRID_STR
+      "Checksum mode in VRRPv3\n")
+{
+	nb_cli_enqueue_change(vty, "./checksum-with-ipv4-pseudoheader",
+			      NB_OP_MODIFY, no ? "false" : "true");
+
+	return nb_cli_apply_changes(vty, VRRP_XPATH_ENTRY, vrid);
+}
+
+void cli_show_checksum_with_ipv4_pseudoheader(struct vty *vty,
+					      const struct lyd_node *dnode,
+					      bool show_defaults)
+{
+	const char *vrid = yang_dnode_get_string(dnode, "../virtual-router-id");
+	const bool pre = yang_dnode_get_bool(dnode, NULL);
+
+	vty_out(vty, " %svrrp %s checksum-with-ipv4-pseudoheader\n",
+		pre ? "" : "no ", vrid);
+}
+
 /* XXX: yang conversion */
-DEFPY(vrrp_autoconfigure,
+DEFPY_YANG(vrrp_autoconfigure,
       vrrp_autoconfigure_cmd,
       "[no] vrrp autoconfigure [version (2-3)]",
       NO_STR
@@ -304,9 +331,9 @@ DEFPY(vrrp_autoconfigure,
 }
 
 /* XXX: yang conversion */
-DEFPY(vrrp_default,
+DEFPY_YANG(vrrp_default,
       vrrp_default_cmd,
-      "[no] vrrp default <advertisement-interval$adv (10-40950)$advint|preempt$p|priority$prio (1-254)$prioval|shutdown$s>",
+      "[no] vrrp default <advertisement-interval$adv (10-40950)$advint|preempt$p|priority$prio (1-254)$prioval|checksum-with-ipv4-pseudoheader$ipv4ph|shutdown$s>",
       NO_STR
       VRRP_STR
       "Configure defaults for new VRRP instances\n"
@@ -315,6 +342,7 @@ DEFPY(vrrp_default,
       "Preempt mode\n"
       VRRP_PRIORITY_STR
       "Priority value\n"
+      "Checksum mode in VRRPv3\n"
       "Force VRRP router into administrative shutdown\n")
 {
 	if (adv) {
@@ -331,6 +359,8 @@ DEFPY(vrrp_default,
 		vd.preempt_mode = !no;
 	if (prio)
 		vd.priority = no ? VRRP_DEFAULT_PRIORITY : prioval;
+	if (ipv4ph)
+		vd.checksum_with_ipv4_pseudoheader = !no;
 	if (s)
 		vd.shutdown = !no;
 
@@ -376,6 +406,8 @@ static struct json_object *vrrp_build_json(struct vrrp_vrouter *vr)
 	json_object_boolean_add(j, "shutdown", vr->shutdown);
 	json_object_boolean_add(j, "preemptMode", vr->preempt_mode);
 	json_object_boolean_add(j, "acceptMode", vr->accept_mode);
+	json_object_boolean_add(j, "checksumWithIpv4Pseudoheader",
+				vr->checksum_with_ipv4_pseudoheader);
 	json_object_string_add(j, "interface", vr->ifp->name);
 	json_object_int_add(j, "advertisementInterval",
 			    vr->advertisement_interval * CS2MS);
@@ -470,8 +502,8 @@ static void vrrp_show(struct vty *vty, struct vrrp_vrouter *vr)
 
 	struct ttable *tt = ttable_new(&ttable_styles[TTSTYLE_BLANK]);
 
-	ttable_add_row(tt, "%s|%" PRIu32, "Virtual Router ID", vr->vrid);
-	ttable_add_row(tt, "%s|%" PRIu8, "Protocol Version", vr->version);
+	ttable_add_row(tt, "%s|%u", "Virtual Router ID", vr->vrid);
+	ttable_add_row(tt, "%s|%hhu", "Protocol Version", vr->version);
 	ttable_add_row(tt, "%s|%s", "Autoconfigured",
 		       vr->autoconf ? "Yes" : "No");
 	ttable_add_row(tt, "%s|%s", "Shutdown", vr->shutdown ? "Yes" : "No");
@@ -492,38 +524,40 @@ static void vrrp_show(struct vty *vty, struct vrrp_vrouter *vr)
 	ttable_add_row(tt, "%s|%s", "Virtual MAC (v6)", ethstr6);
 	ttable_add_row(tt, "%s|%s", "Status (v4)", stastr4);
 	ttable_add_row(tt, "%s|%s", "Status (v6)", stastr6);
-	ttable_add_row(tt, "%s|%" PRIu8, "Priority", vr->priority);
-	ttable_add_row(tt, "%s|%" PRIu8, "Effective Priority (v4)",
+	ttable_add_row(tt, "%s|%hhu", "Priority", vr->priority);
+	ttable_add_row(tt, "%s|%hhu", "Effective Priority (v4)",
 		       vr->v4->priority);
-	ttable_add_row(tt, "%s|%" PRIu8, "Effective Priority (v6)",
+	ttable_add_row(tt, "%s|%hhu", "Effective Priority (v6)",
 		       vr->v6->priority);
 	ttable_add_row(tt, "%s|%s", "Preempt Mode",
 		       vr->preempt_mode ? "Yes" : "No");
 	ttable_add_row(tt, "%s|%s", "Accept Mode",
 		       vr->accept_mode ? "Yes" : "No");
+	ttable_add_row(tt, "%s|%s", "Checksum with IPv4 Pseudoheader",
+		       vr->checksum_with_ipv4_pseudoheader ? "Yes" : "No");
 	ttable_add_row(tt, "%s|%d ms", "Advertisement Interval",
 		       vr->advertisement_interval * CS2MS);
-	ttable_add_row(tt, "%s|%d ms",
-		       "Master Advertisement Interval (v4)",
+	ttable_add_row(tt, "%s|%d ms (stale)",
+		       "Master Advertisement Interval (v4) Rx",
 		       vr->v4->master_adver_interval * CS2MS);
-	ttable_add_row(tt, "%s|%d ms",
-		       "Master Advertisement Interval (v6)",
+	ttable_add_row(tt, "%s|%d ms (stale)",
+		       "Master Advertisement Interval (v6) Rx",
 		       vr->v6->master_adver_interval * CS2MS);
-	ttable_add_row(tt, "%s|%" PRIu32, "Advertisements Tx (v4)",
+	ttable_add_row(tt, "%s|%u", "Advertisements Tx (v4)",
 		       vr->v4->stats.adver_tx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "Advertisements Tx (v6)",
+	ttable_add_row(tt, "%s|%u", "Advertisements Tx (v6)",
 		       vr->v6->stats.adver_tx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "Advertisements Rx (v4)",
+	ttable_add_row(tt, "%s|%u", "Advertisements Rx (v4)",
 		       vr->v4->stats.adver_rx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "Advertisements Rx (v6)",
+	ttable_add_row(tt, "%s|%u", "Advertisements Rx (v6)",
 		       vr->v6->stats.adver_rx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "Gratuitous ARP Tx (v4)",
+	ttable_add_row(tt, "%s|%u", "Gratuitous ARP Tx (v4)",
 		       vr->v4->stats.garp_tx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "Neigh. Adverts Tx (v6)",
+	ttable_add_row(tt, "%s|%u", "Neigh. Adverts Tx (v6)",
 		       vr->v6->stats.una_tx_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "State transitions (v4)",
+	ttable_add_row(tt, "%s|%u", "State transitions (v4)",
 		       vr->v4->stats.trans_cnt);
-	ttable_add_row(tt, "%s|%" PRIu32, "State transitions (v6)",
+	ttable_add_row(tt, "%s|%u", "State transitions (v6)",
 		       vr->v6->stats.trans_cnt);
 	ttable_add_row(tt, "%s|%d ms", "Skew Time (v4)",
 		       vr->v4->skew_time * CS2MS);
@@ -583,7 +617,7 @@ static int vrrp_instance_display_sort_cmp(const void **d1, const void **d2)
 
 /* clang-format off */
 
-DEFPY(vrrp_vrid_show,
+DEFPY_YANG(vrrp_vrid_show,
       vrrp_vrid_show_cmd,
       "show vrrp [interface INTERFACE$ifn] [(1-255)$vrid] [json$json]",
       SHOW_STR
@@ -624,7 +658,7 @@ DEFPY(vrrp_vrid_show,
 	return CMD_SUCCESS;
 }
 
-DEFPY(vrrp_vrid_show_summary,
+DEFPY_YANG(vrrp_vrid_show_summary,
       vrrp_vrid_show_summary_cmd,
       "show vrrp [interface INTERFACE$ifn] [(1-255)$vrid] summary",
       SHOW_STR
@@ -653,7 +687,7 @@ DEFPY(vrrp_vrid_show_summary,
 			continue;
 
 		ttable_add_row(
-			tt, "%s|%" PRIu8 "|%" PRIu8 "|%d|%d|%s|%s",
+			tt, "%s|%u|%hhu|%d|%d|%s|%s",
 			vr->ifp->name, vr->vrid, vr->priority,
 			vr->v4->addrs->count, vr->v6->addrs->count,
 			vr->v4->fsm.state == VRRP_STATE_MASTER ? "Master"
@@ -674,7 +708,7 @@ DEFPY(vrrp_vrid_show_summary,
 }
 
 
-DEFPY(debug_vrrp,
+DEFPY_YANG(debug_vrrp,
       debug_vrrp_cmd,
       "[no] debug vrrp [{protocol$proto|autoconfigure$ac|packets$pkt|sockets$sock|ndisc$ndisc|arp$arp|zebra$zebra}]",
       NO_STR
@@ -710,48 +744,13 @@ DEFUN_NOSH (show_debugging_vrrp,
 
 	vrrp_debug_status_write(vty);
 
+	cmd_show_lib_debugs(vty);
+
 	return CMD_SUCCESS;
 }
 
 /* clang-format on */
 
-/*
- * Write per interface VRRP config.
- */
-static int vrrp_config_write_interface(struct vty *vty)
-{
-	struct vrf *vrf;
-	int write = 0;
-
-	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
-		struct interface *ifp;
-
-		FOR_ALL_INTERFACES (vrf, ifp) {
-			struct lyd_node *dnode;
-
-			dnode = yang_dnode_get(
-				running_config->dnode,
-				"/frr-interface:lib/interface[name='%s'][vrf='%s']",
-				ifp->name, vrf->name);
-			if (dnode == NULL)
-				continue;
-
-			write = 1;
-			nb_cli_show_dnode_cmds(vty, dnode, false);
-		}
-	}
-
-	return write;
-}
-
-static struct cmd_node interface_node = {
-	.name = "interface",
-	.node = INTERFACE_NODE,
-	.parent_node = CONFIG_NODE,
-	.prompt = "%s(config-if)# ",
-	.config_write = vrrp_config_write_interface,
-};
-
 static struct cmd_node debug_node = {
 	.name = "debug",
 	.node = DEBUG_NODE,
@@ -769,14 +768,14 @@ static struct cmd_node vrrp_node = {
 void vrrp_vty_init(void)
 {
 	install_node(&debug_node);
-	install_node(&interface_node);
 	install_node(&vrrp_node);
-	if_cmd_init();
+	vrf_cmd_init(NULL);
+	if_cmd_init_default();
 
 	install_element(VIEW_NODE, &vrrp_vrid_show_cmd);
 	install_element(VIEW_NODE, &vrrp_vrid_show_summary_cmd);
-	install_element(VIEW_NODE, &show_debugging_vrrp_cmd);
-	install_element(VIEW_NODE, &debug_vrrp_cmd);
+	install_element(ENABLE_NODE, &show_debugging_vrrp_cmd);
+	install_element(ENABLE_NODE, &debug_vrrp_cmd);
 	install_element(CONFIG_NODE, &debug_vrrp_cmd);
 	install_element(CONFIG_NODE, &vrrp_autoconfigure_cmd);
 	install_element(CONFIG_NODE, &vrrp_default_cmd);
@@ -789,4 +788,6 @@ void vrrp_vty_init(void)
 	install_element(INTERFACE_NODE, &vrrp_ip_cmd);
 	install_element(INTERFACE_NODE, &vrrp_ip6_cmd);
 	install_element(INTERFACE_NODE, &vrrp_preempt_cmd);
+	install_element(INTERFACE_NODE,
+			&vrrp_checksum_with_ipv4_pseudoheader_cmd);
 }
