@@ -26,7 +26,6 @@
 #include "thread.h"
 #include "filter.h"
 #include "memory.h"
-#include "zebra_memory.h"
 #include "prefix.h"
 #include "log.h"
 #include "plist.h"
@@ -35,6 +34,7 @@
 #include "vrf.h"
 #include "libfrr.h"
 #include "routemap.h"
+#include "routing_nb.h"
 
 #include "zebra/zebra_router.h"
 #include "zebra/zebra_errors.h"
@@ -55,10 +55,10 @@
 #include "zebra/zebra_vxlan.h"
 #include "zebra/zebra_routemap.h"
 #include "zebra/zebra_nb.h"
-
-#if defined(HANDLE_NETLINK_FUZZING)
-#include "zebra/kernel_netlink.h"
-#endif /* HANDLE_NETLINK_FUZZING */
+#include "zebra/zebra_opaque.h"
+#include "zebra/zebra_srte.h"
+#include "zebra/zebra_srv6.h"
+#include "zebra/zebra_srv6_vty.h"
 
 #define ZEBRA_PTM_SUPPORT
 
@@ -71,29 +71,30 @@ struct thread_master *master;
 /* Route retain mode flag. */
 int retain_mode = 0;
 
-/* Allow non-quagga entities to delete quagga routes */
-int allow_delete = 0;
-
 int graceful_restart;
 
 bool v6_rr_semantics = false;
 
+/* Receive buffer size for kernel control sockets */
+#define RCVBUFSIZE_MIN 4194304
 #ifdef HAVE_NETLINK
-/* Receive buffer size for netlink socket */
-uint32_t nl_rcvbufsize = 4194304;
-#endif /* HAVE_NETLINK */
+uint32_t rcvbufsize = RCVBUFSIZE_MIN;
+#else
+uint32_t rcvbufsize = 128 * 1024;
+#endif
 
 #define OPTION_V6_RR_SEMANTICS 2000
+#define OPTION_ASIC_OFFLOAD    2001
+
 /* Command line options. */
 const struct option longopts[] = {
 	{"batch", no_argument, NULL, 'b'},
 	{"allow_delete", no_argument, NULL, 'a'},
-	{"keep_kernel", no_argument, NULL, 'k'},
 	{"socket", required_argument, NULL, 'z'},
 	{"ecmp", required_argument, NULL, 'e'},
 	{"retain", no_argument, NULL, 'r'},
-	{"vrfdefaultname", required_argument, NULL, 'o'},
 	{"graceful_restart", required_argument, NULL, 'K'},
+	{"asic-offload", optional_argument, NULL, OPTION_ASIC_OFFLOAD},
 #ifdef HAVE_NETLINK
 	{"vrfwnetns", no_argument, NULL, 'n'},
 	{"nl-bufsize", required_argument, NULL, 's'},
@@ -101,8 +102,12 @@ const struct option longopts[] = {
 #endif /* HAVE_NETLINK */
 	{0}};
 
-zebra_capabilities_t _caps_p[] = {
-	ZCAP_NET_ADMIN, ZCAP_SYS_ADMIN, ZCAP_NET_RAW,
+zebra_capabilities_t _caps_p[] = {ZCAP_NET_ADMIN, ZCAP_SYS_ADMIN,
+				  ZCAP_NET_RAW,
+#ifdef HAVE_DPDK
+				  ZCAP_IPC_LOCK,  ZCAP_READ_SEARCH,
+				  ZCAP_SYS_RAWIO
+#endif
 };
 
 /* zebra privileges to run with */
@@ -151,33 +156,42 @@ static void sigint(void)
 
 	frr_early_fini();
 
+	/* Stop the opaque module pthread */
+	zebra_opaque_stop();
+
 	zebra_dplane_pre_finish();
 
 	/* Clean up GR related info. */
 	zebra_gr_stale_client_cleanup(zrouter.stale_client_list);
 	list_delete_all_node(zrouter.stale_client_list);
 
+	/* Clean up zapi clients and server module */
 	for (ALL_LIST_ELEMENTS(zrouter.client_list, ln, nn, client))
 		zserv_close_client(client);
 
 	zserv_close();
 	list_delete_all_node(zrouter.client_list);
 
+	/* Once all the zclients are cleaned up, clean up the opaque module */
+	zebra_opaque_finish();
+
 	zebra_ptm_finish();
 
-	if (retain_mode)
+	if (retain_mode) {
+		zebra_nhg_mark_keep();
 		RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
 			zvrf = vrf->info;
 			if (zvrf)
 				SET_FLAG(zvrf->flags, ZEBRA_VRF_RETAIN);
 		}
+	}
+
 	if (zrouter.lsp_process_q)
 		work_queue_free_and_null(&zrouter.lsp_process_q);
 
 	vrf_terminate();
-	rtadv_terminate();
 
-	ns_walk_func(zebra_ns_early_shutdown);
+	ns_walk_func(zebra_ns_early_shutdown, NULL, NULL);
 	zebra_ns_notify_close();
 
 	access_list_reset();
@@ -190,6 +204,8 @@ static void sigint(void)
 	 */
 	zebra_routemap_finish();
 
+	rib_update_finish();
+
 	list_delete(&zrouter.client_list);
 
 	/* Indicate that all new dplane work has been enqueued. When that
@@ -203,18 +219,19 @@ static void sigint(void)
  * Final shutdown step for the zebra main thread. This is run after all
  * async update processing has completed.
  */
-int zebra_finalize(struct thread *dummy)
+void zebra_finalize(struct thread *dummy)
 {
 	zlog_info("Zebra final shutdown");
 
-	/* Final shutdown of ns resources */
-	ns_walk_func(zebra_ns_final_shutdown);
-
 	/* Stop dplane thread and finish any cleanup */
 	zebra_dplane_shutdown();
 
+	/* Final shutdown of ns resources */
+	ns_walk_func(zebra_ns_final_shutdown, NULL, NULL);
+
 	zebra_router_terminate();
 
+	ns_terminate();
 	frr_fini();
 	exit(0);
 }
@@ -225,7 +242,7 @@ static void sigusr1(void)
 	zlog_rotate();
 }
 
-struct quagga_signal_t zebra_signals[] = {
+struct frr_signal_t zebra_signals[] = {
 	{
 		.signal = SIGHUP,
 		.handler = &sighup,
@@ -245,40 +262,38 @@ struct quagga_signal_t zebra_signals[] = {
 };
 
 static const struct frr_yang_module_info *const zebra_yang_modules[] = {
+	&frr_filter_info,
 	&frr_interface_info,
 	&frr_route_map_info,
 	&frr_zebra_info,
 	&frr_vrf_info,
+	&frr_routing_info,
+	&frr_zebra_route_map_info,
 };
 
 FRR_DAEMON_INFO(
 	zebra, ZEBRA, .vty_port = ZEBRA_VTY_PORT, .flags = FRR_NO_ZCLIENT,
 
 	.proghelp =
-		"Daemon which manages kernel routing table management "
-		"and\nredistribution between different routing protocols.",
+		"Daemon which manages kernel routing table management and\nredistribution between different routing protocols.",
 
 	.signals = zebra_signals, .n_signals = array_size(zebra_signals),
 
 	.privs = &zserv_privs,
 
 	.yang_modules = zebra_yang_modules,
-	.n_yang_modules = array_size(zebra_yang_modules), )
+	.n_yang_modules = array_size(zebra_yang_modules),
+);
 
 /* Main startup routine. */
 int main(int argc, char **argv)
 {
 	// int batch_mode = 0;
 	char *zserv_path = NULL;
-	char *vrf_default_name_configured = NULL;
 	struct sockaddr_storage dummy;
 	socklen_t dummylen;
-#if defined(HANDLE_ZAPI_FUZZING)
-	char *zapi_fuzzing = NULL;
-#endif /* HANDLE_ZAPI_FUZZING */
-#if defined(HANDLE_NETLINK_FUZZING)
-	char *netlink_fuzzing = NULL;
-#endif /* HANDLE_NETLINK_FUZZING */
+	bool asic_offload = false;
+	bool notify_on_ack = true;
 
 	graceful_restart = 0;
 	vrf_configure_backend(VRF_BACKEND_VRF_LITE);
@@ -286,16 +301,10 @@ int main(int argc, char **argv)
 	frr_preinit(&zebra_di, argc, argv);
 
 	frr_opt_add(
-		"baz:e:o:rK:"
+		"baz:e:rK:s:"
 #ifdef HAVE_NETLINK
-		"s:n"
+		"n"
 #endif
-#if defined(HANDLE_ZAPI_FUZZING)
-		"c:"
-#endif /* HANDLE_ZAPI_FUZZING */
-#if defined(HANDLE_NETLINK_FUZZING)
-		"w:"
-#endif /* HANDLE_NETLINK_FUZZING */
 		,
 		longopts,
 		"  -b, --batch              Runs in batch mode\n"
@@ -303,19 +312,15 @@ int main(int argc, char **argv)
 		"  -z, --socket             Set path of zebra socket\n"
 		"  -e, --ecmp               Specify ECMP to use.\n"
 		"  -r, --retain             When program terminates, retain added route by zebra.\n"
-		"  -o, --vrfdefaultname     Set default VRF name.\n"
 		"  -K, --graceful_restart   Graceful restart at the kernel level, timer in seconds for expiration\n"
+		"  -A, --asic-offload       FRR is interacting with an asic underneath the linux kernel\n"
 #ifdef HAVE_NETLINK
-		"  -n, --vrfwnetns          Use NetNS as VRF backend\n"
 		"  -s, --nl-bufsize         Set netlink receive buffer size\n"
+		"  -n, --vrfwnetns          Use NetNS as VRF backend\n"
 		"      --v6-rr-semantics    Use v6 RR semantics\n"
+#else
+		"  -s,                      Set kernel socket receive buffer size\n"
 #endif /* HAVE_NETLINK */
-#if defined(HANDLE_ZAPI_FUZZING)
-		"  -c <file>                Bypass normal startup and use this file for testing of zapi\n"
-#endif /* HANDLE_ZAPI_FUZZING */
-#if defined(HANDLE_NETLINK_FUZZING)
-		"  -w <file>                Bypass normal startup and use this file for testing of netlink input\n"
-#endif /* HANDLE_NETLINK_FUZZING */
 	);
 
 	while (1) {
@@ -331,7 +336,7 @@ int main(int argc, char **argv)
 			// batch_mode = 1;
 			break;
 		case 'a':
-			allow_delete = 1;
+			zrouter.allow_delete = true;
 			break;
 		case 'e': {
 			unsigned long int parsed_multipath =
@@ -348,9 +353,6 @@ int main(int argc, char **argv)
 			zrouter.multipath_num = parsed_multipath;
 			break;
 		}
-		case 'o':
-			vrf_default_name_configured = optarg;
-			break;
 		case 'z':
 			zserv_path = optarg;
 			if (!frr_zclient_addr(&dummy, &dummylen, optarg)) {
@@ -366,58 +368,51 @@ int main(int argc, char **argv)
 		case 'K':
 			graceful_restart = atoi(optarg);
 			break;
-#ifdef HAVE_NETLINK
 		case 's':
-			nl_rcvbufsize = atoi(optarg);
+			rcvbufsize = atoi(optarg);
+			if (rcvbufsize < RCVBUFSIZE_MIN)
+				fprintf(stderr,
+					"Rcvbufsize is smaller than recommended value: %d\n",
+					RCVBUFSIZE_MIN);
 			break;
+#ifdef HAVE_NETLINK
 		case 'n':
 			vrf_configure_backend(VRF_BACKEND_NETNS);
 			break;
 		case OPTION_V6_RR_SEMANTICS:
 			v6_rr_semantics = true;
 			break;
+		case OPTION_ASIC_OFFLOAD:
+			if (!strcmp(optarg, "notify_on_offload"))
+				notify_on_ack = false;
+			if (!strcmp(optarg, "notify_on_ack"))
+				notify_on_ack = true;
+			asic_offload = true;
+			break;
 #endif /* HAVE_NETLINK */
-#if defined(HANDLE_ZAPI_FUZZING)
-		case 'c':
-			zapi_fuzzing = optarg;
-			break;
-#endif /* HANDLE_ZAPI_FUZZING */
-#if defined(HANDLE_NETLINK_FUZZING)
-		case 'w':
-			netlink_fuzzing = optarg;
-			/* This ensures we are aren't writing any of the
-			 * startup netlink messages that happen when we
-			 * just want to read.
-			 */
-			netlink_read = true;
-			break;
-#endif /* HANDLE_NETLINK_FUZZING */
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
 	zrouter.master = frr_init();
 
 	/* Zebra related initialize. */
-	zebra_router_init();
+	zebra_router_init(asic_offload, notify_on_ack);
 	zserv_init();
 	rib_init();
 	zebra_if_init();
 	zebra_debug_init();
-	router_id_cmd_init();
 
 	/*
 	 * Initialize NS( and implicitly the VRF module), and make kernel
 	 * routing socket. */
-	zebra_ns_init((const char *)vrf_default_name_configured);
+	zebra_ns_init();
+	router_id_cmd_init();
 	zebra_vty_init();
 	access_list_init();
 	prefix_list_init();
-#if defined(HAVE_RTADV)
 	rtadv_cmd_init();
-#endif
 /* PTM socket */
 #ifdef ZEBRA_PTM_SUPPORT
 	zebra_ptm_init();
@@ -427,9 +422,13 @@ int main(int argc, char **argv)
 	zebra_mpls_vty_init();
 	zebra_pw_vty_init();
 	zebra_pbr_init();
+	zebra_opaque_init();
+	zebra_srte_init();
+	zebra_srv6_init();
+	zebra_srv6_vty_init();
 
-/* For debug purpose. */
-/* SET_FLAG (zebra_debug_event, ZEBRA_DEBUG_EVENT); */
+	/* For debug purpose. */
+	/* SET_FLAG (zebra_debug_event, ZEBRA_DEBUG_EVENT); */
 
 	/* Process the configuration file. Among other configuration
 	*  directives we can meet those installing static routes. Such
@@ -449,8 +448,8 @@ int main(int argc, char **argv)
 	* we have to have route_read() called before.
 	*/
 	zrouter.startup_time = monotime(NULL);
-	thread_add_timer(zrouter.master, rib_sweep_route,
-			 NULL, graceful_restart, NULL);
+	thread_add_timer(zrouter.master, rib_sweep_route, NULL,
+			 graceful_restart, &zrouter.sweeper);
 
 	/* Needed for BSD routing socket. */
 	pid = getpid();
@@ -458,6 +457,9 @@ int main(int argc, char **argv)
 	/* Start dataplane system */
 	zebra_dplane_start();
 
+	/* Start the ted module, before zserv */
+	zebra_opaque_start();
+
 	/* Start Zebra API server */
 	zserv_start(zserv_path);
 
@@ -473,20 +475,6 @@ int main(int argc, char **argv)
 	/* Error init */
 	zebra_error_init();
 
-#if defined(HANDLE_ZAPI_FUZZING)
-	if (zapi_fuzzing) {
-		zserv_read_file(zapi_fuzzing);
-		exit(0);
-	}
-#endif /* HANDLE_ZAPI_FUZZING */
-#if defined(HANDLE_NETLINK_FUZZING)
-	if (netlink_fuzzing) {
-		netlink_read_init(netlink_fuzzing);
-		exit(0);
-	}
-#endif /* HANDLE_NETLINK_FUZZING */
-
-
 	frr_run(zrouter.master);
 
 	/* Not reached... */
