@@ -42,41 +42,7 @@
 
 #include "zebra/zebra_fpm_private.h"
 #include "zebra/zebra_vxlan_private.h"
-
-/*
- * addr_to_a
- *
- * Returns string representation of an address of the given AF.
- */
-static inline const char *addr_to_a(uint8_t af, void *addr)
-{
-	if (!addr)
-		return "<No address>";
-
-	switch (af) {
-
-	case AF_INET:
-		return inet_ntoa(*((struct in_addr *)addr));
-	case AF_INET6:
-		return inet6_ntoa(*((struct in6_addr *)addr));
-	default:
-		return "<Addr in unknown AF>";
-	}
-}
-
-/*
- * prefix_addr_to_a
- *
- * Convience wrapper that returns a human-readable string for the
- * address in a prefix.
- */
-static const char *prefix_addr_to_a(struct prefix *prefix)
-{
-	if (!prefix)
-		return "<No address>";
-
-	return addr_to_a(prefix->family, &prefix->u.prefix);
-}
+#include "zebra/interface.h"
 
 /*
  * af_addr_size
@@ -143,13 +109,15 @@ struct fpm_nh_encap_info_t {
 };
 
 /*
- * netlink_nh_info_t
+ * netlink_nh_info
  *
  * Holds information about a single nexthop for netlink. These info
  * structures are transient and may contain pointers into rib
  * data structures for convenience.
  */
-typedef struct netlink_nh_info_t_ {
+struct netlink_nh_info {
+	/* Weight of the nexthop ( for unequal cost ECMP ) */
+	uint8_t weight;
 	uint32_t if_index;
 	union g_addr *gateway;
 
@@ -160,14 +128,15 @@ typedef struct netlink_nh_info_t_ {
 	int recursive;
 	enum nexthop_types_t type;
 	struct fpm_nh_encap_info_t encap_info;
-} netlink_nh_info_t;
+};
 
 /*
- * netlink_route_info_t
+ * netlink_route_info
  *
  * A structure for holding information for a netlink route message.
  */
-typedef struct netlink_route_info_t_ {
+struct netlink_route_info {
+	uint32_t nlmsg_pid;
 	uint16_t nlmsg_type;
 	uint8_t rtm_type;
 	uint32_t rtm_table;
@@ -180,9 +149,9 @@ typedef struct netlink_route_info_t_ {
 	/*
 	 * Nexthop structures
 	 */
-	netlink_nh_info_t nhs[MULTIPATH_NUM];
+	struct netlink_nh_info nhs[MULTIPATH_NUM];
 	union g_addr *pref_src;
-} netlink_route_info_t;
+};
 
 /*
  * netlink_route_info_add_nh
@@ -192,13 +161,16 @@ typedef struct netlink_route_info_t_ {
  *
  * Returns true if a nexthop was added, false otherwise.
  */
-static int netlink_route_info_add_nh(netlink_route_info_t *ri,
+static int netlink_route_info_add_nh(struct netlink_route_info *ri,
 				     struct nexthop *nexthop,
 				     struct route_entry *re)
 {
-	netlink_nh_info_t nhi;
+	struct netlink_nh_info nhi;
 	union g_addr *src;
-	zebra_l3vni_t *zl3vni = NULL;
+	struct zebra_vrf *zvrf = NULL;
+	struct interface *ifp = NULL, *link_if = NULL;
+	struct zebra_if *zif = NULL;
+	vni_t vni = 0;
 
 	memset(&nhi, 0, sizeof(nhi));
 	src = NULL;
@@ -209,6 +181,7 @@ static int netlink_route_info_add_nh(netlink_route_info_t *ri,
 	nhi.recursive = nexthop->rparent ? 1 : 0;
 	nhi.type = nexthop->type;
 	nhi.if_index = nexthop->ifindex;
+	nhi.weight = nexthop->weight;
 
 	if (nexthop->type == NEXTHOP_TYPE_IPV4
 	    || nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX) {
@@ -219,7 +192,12 @@ static int netlink_route_info_add_nh(netlink_route_info_t *ri,
 
 	if (nexthop->type == NEXTHOP_TYPE_IPV6
 	    || nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX) {
-		nhi.gateway = &nexthop->gate;
+		/* Special handling for IPv4 route with IPv6 Link Local next hop
+		 */
+		if (ri->af == AF_INET)
+			nhi.gateway = &ipv4ll_gateway;
+		else
+			nhi.gateway = &nexthop->gate;
 	}
 
 	if (nexthop->type == NEXTHOP_TYPE_IFINDEX) {
@@ -230,15 +208,32 @@ static int netlink_route_info_add_nh(netlink_route_info_t *ri,
 	if (!nhi.gateway && nhi.if_index == 0)
 		return 0;
 
-	if (re && CHECK_FLAG(re->flags, ZEBRA_FLAG_EVPN_ROUTE)) {
+	if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_EVPN)) {
 		nhi.encap_info.encap_type = FPM_NH_ENCAP_VXLAN;
 
-		zl3vni = zl3vni_from_vrf(nexthop->vrf_id);
-		if (zl3vni && is_l3vni_oper_up(zl3vni)) {
-
-			/* Add VNI to VxLAN encap info */
-			nhi.encap_info.vxlan_encap.vni = zl3vni->vni;
+		/* Extract VNI id for the nexthop SVI interface */
+		zvrf = zebra_vrf_lookup_by_id(nexthop->vrf_id);
+		if (zvrf) {
+			ifp = if_lookup_by_index_per_ns(zvrf->zns,
+							nexthop->ifindex);
+			if (ifp) {
+				zif = (struct zebra_if *)ifp->info;
+				if (zif) {
+					if (IS_ZEBRA_IF_BRIDGE(ifp))
+						link_if = ifp;
+					else if (IS_ZEBRA_IF_VLAN(ifp))
+						link_if =
+						if_lookup_by_index_per_ns(
+							zvrf->zns,
+							zif->link_ifindex);
+					if (link_if)
+						vni = vni_id_from_svi(ifp,
+								      link_if);
+				}
+			}
 		}
+
+		nhi.encap_info.vxlan_encap.vni = vni;
 	}
 
 	/*
@@ -258,14 +253,7 @@ static int netlink_route_info_add_nh(netlink_route_info_t *ri,
  */
 static uint8_t netlink_proto_from_route_type(int type)
 {
-	switch (type) {
-	case ZEBRA_ROUTE_KERNEL:
-	case ZEBRA_ROUTE_CONNECT:
-		return RTPROT_KERNEL;
-
-	default:
-		return RTPROT_ZEBRA;
-	}
+	return zebra2proto(type);
 }
 
 /*
@@ -275,21 +263,24 @@ static uint8_t netlink_proto_from_route_type(int type)
  *
  * Returns true on success and false on failure.
  */
-static int netlink_route_info_fill(netlink_route_info_t *ri, int cmd,
+static int netlink_route_info_fill(struct netlink_route_info *ri, int cmd,
 				   rib_dest_t *dest, struct route_entry *re)
 {
 	struct nexthop *nexthop;
-	struct zebra_vrf *zvrf;
+	struct rib_table_info *table_info =
+		rib_table_info(rib_dest_table(dest));
+	struct zebra_vrf *zvrf = table_info->zvrf;
 
 	memset(ri, 0, sizeof(*ri));
 
 	ri->prefix = rib_dest_prefix(dest);
 	ri->af = rib_dest_af(dest);
 
+	if (zvrf && zvrf->zns)
+		ri->nlmsg_pid = zvrf->zns->netlink_dplane_out.snl.nl_pid;
+
 	ri->nlmsg_type = cmd;
-	zvrf = rib_dest_vrf(dest);
-	if (zvrf)
-		ri->rtm_table = zvrf->table_id;
+	ri->rtm_table = table_info->table_id;
 	ri->rtm_protocol = RTPROT_UNSPEC;
 
 	/*
@@ -338,10 +329,18 @@ static int netlink_route_info_fill(netlink_route_info_t *ri, int cmd,
 		}
 	}
 
-	/* If there is no useful nexthop then return. */
 	if (ri->num_nhs == 0) {
-		zfpm_debug("netlink_encode_route(): No useful nexthop.");
-		return 0;
+		switch (ri->rtm_type) {
+		case RTN_PROHIBIT:
+		case RTN_UNREACHABLE:
+		case RTN_BLACKHOLE:
+			break;
+		default:
+			/* If there is no useful nexthop then return. */
+			zfpm_debug(
+				"netlink_encode_route(): No useful nexthop.");
+			return 0;
+		}
 	}
 
 	return 1;
@@ -353,17 +352,18 @@ static int netlink_route_info_fill(netlink_route_info_t *ri, int cmd,
  * Returns the number of bytes written to the buffer. 0 or a negative
  * value indicates an error.
  */
-static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
-				     size_t in_buf_len)
+static int netlink_route_info_encode(struct netlink_route_info *ri,
+				     char *in_buf, size_t in_buf_len)
 {
 	size_t bytelen;
 	unsigned int nexthop_num = 0;
 	size_t buf_offset;
-	netlink_nh_info_t *nhi;
+	struct netlink_nh_info *nhi;
 	enum fpm_nh_encap_type_t encap;
-	struct rtattr *nest;
+	struct rtattr *nest, *inner_nest;
+	struct rtnexthop *rtnh;
 	struct vxlan_encap_info_t *vxlan;
-	int nest_len;
+	struct in6_addr ipv6;
 
 	struct {
 		struct nlmsghdr n;
@@ -386,11 +386,12 @@ static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
 
 	req->n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
 	req->n.nlmsg_flags = NLM_F_CREATE | NLM_F_REQUEST;
+	req->n.nlmsg_pid = ri->nlmsg_pid;
 	req->n.nlmsg_type = ri->nlmsg_type;
 	req->r.rtm_family = ri->af;
 
 	/*
-	 * rtm_table field is a uchar field which can accomodate table_id less
+	 * rtm_table field is a uchar field which can accommodate table_id less
 	 * than 256.
 	 * To support table id greater than 255, if the table_id is greater than
 	 * 255, set rtm_table to RT_TABLE_UNSPEC and add RTA_TABLE attribute
@@ -400,20 +401,21 @@ static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
 		req->r.rtm_table = ri->rtm_table;
 	else {
 		req->r.rtm_table = RT_TABLE_UNSPEC;
-		addattr32(&req->n, in_buf_len, RTA_TABLE, ri->rtm_table);
+		nl_attr_put32(&req->n, in_buf_len, RTA_TABLE, ri->rtm_table);
 	}
 
 	req->r.rtm_dst_len = ri->prefix->prefixlen;
 	req->r.rtm_protocol = ri->rtm_protocol;
 	req->r.rtm_scope = RT_SCOPE_UNIVERSE;
 
-	addattr_l(&req->n, in_buf_len, RTA_DST, &ri->prefix->u.prefix, bytelen);
+	nl_attr_put(&req->n, in_buf_len, RTA_DST, &ri->prefix->u.prefix,
+		    bytelen);
 
 	req->r.rtm_type = ri->rtm_type;
 
 	/* Metric. */
 	if (ri->metric)
-		addattr32(&req->n, in_buf_len, RTA_PRIORITY, *ri->metric);
+		nl_attr_put32(&req->n, in_buf_len, RTA_PRIORITY, *ri->metric);
 
 	if (ri->num_nhs == 0)
 		goto done;
@@ -422,12 +424,20 @@ static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
 		nhi = &ri->nhs[0];
 
 		if (nhi->gateway) {
-			addattr_l(&req->n, in_buf_len, RTA_GATEWAY,
-				  nhi->gateway, bytelen);
+			if (nhi->type == NEXTHOP_TYPE_IPV4_IFINDEX
+			    && ri->af == AF_INET6) {
+				ipv4_to_ipv4_mapped_ipv6(&ipv6,
+							 nhi->gateway->ipv4);
+				nl_attr_put(&req->n, in_buf_len, RTA_GATEWAY,
+					    &ipv6, bytelen);
+			} else
+				nl_attr_put(&req->n, in_buf_len, RTA_GATEWAY,
+					    nhi->gateway, bytelen);
 		}
 
 		if (nhi->if_index) {
-			addattr32(&req->n, in_buf_len, RTA_OIF, nhi->if_index);
+			nl_attr_put32(&req->n, in_buf_len, RTA_OIF,
+				      nhi->if_index);
 		}
 
 		encap = nhi->encap_info.encap_type;
@@ -436,12 +446,13 @@ static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
 		case FPM_NH_ENCAP_MAX:
 			break;
 		case FPM_NH_ENCAP_VXLAN:
-			addattr_l(&req->n, in_buf_len, RTA_ENCAP_TYPE, &encap,
-				  sizeof(uint16_t));
+			nl_attr_put16(&req->n, in_buf_len, RTA_ENCAP_TYPE,
+				      encap);
 			vxlan = &nhi->encap_info.vxlan_encap;
-			nest = addattr_nest(&req->n, in_buf_len, RTA_ENCAP);
-			addattr32(&req->n, in_buf_len, VXLAN_VNI, vxlan->vni);
-			addattr_nest_end(&req->n, nest);
+			nest = nl_attr_nest(&req->n, in_buf_len, RTA_ENCAP);
+			nl_attr_put32(&req->n, in_buf_len, VXLAN_VNI,
+				      vxlan->vni);
+			nl_attr_nest_end(&req->n, nest);
 			break;
 		}
 
@@ -451,64 +462,50 @@ static int netlink_route_info_encode(netlink_route_info_t *ri, char *in_buf,
 	/*
 	 * Multipath case.
 	 */
-	char buf[NL_PKT_BUF_SIZE];
-	struct rtattr *rta = (void *)buf;
-	struct rtnexthop *rtnh;
-
-	rta->rta_type = RTA_MULTIPATH;
-	rta->rta_len = RTA_LENGTH(0);
-	rtnh = RTA_DATA(rta);
+	nest = nl_attr_nest(&req->n, in_buf_len, RTA_MULTIPATH);
 
 	for (nexthop_num = 0; nexthop_num < ri->num_nhs; nexthop_num++) {
+		rtnh = nl_attr_rtnh(&req->n, in_buf_len);
 		nhi = &ri->nhs[nexthop_num];
 
-		rtnh->rtnh_len = sizeof(*rtnh);
-		rtnh->rtnh_flags = 0;
-		rtnh->rtnh_hops = 0;
-		rtnh->rtnh_ifindex = 0;
-		rta->rta_len += rtnh->rtnh_len;
-
-		if (nhi->gateway) {
-			rta_addattr_l(rta, sizeof(buf), RTA_GATEWAY,
-				      nhi->gateway, bytelen);
-			rtnh->rtnh_len += sizeof(struct rtattr) + bytelen;
-		}
+		if (nhi->gateway)
+			nl_attr_put(&req->n, in_buf_len, RTA_GATEWAY,
+				    nhi->gateway, bytelen);
 
 		if (nhi->if_index) {
 			rtnh->rtnh_ifindex = nhi->if_index;
 		}
 
+		rtnh->rtnh_hops = nhi->weight;
+
 		encap = nhi->encap_info.encap_type;
 		switch (encap) {
 		case FPM_NH_ENCAP_NONE:
 		case FPM_NH_ENCAP_MAX:
 			break;
 		case FPM_NH_ENCAP_VXLAN:
-			rta_addattr_l(rta, sizeof(buf), RTA_ENCAP_TYPE,
-				      &encap, sizeof(uint16_t));
-			rtnh->rtnh_len += sizeof(struct rtattr) +
-					  sizeof(uint16_t);
+			nl_attr_put16(&req->n, in_buf_len, RTA_ENCAP_TYPE,
+				      encap);
 			vxlan = &nhi->encap_info.vxlan_encap;
-			nest = rta_nest(rta, sizeof(buf), RTA_ENCAP);
-			rta_addattr_l(rta, sizeof(buf), VXLAN_VNI, &vxlan->vni,
-				      sizeof(uint32_t));
-			nest_len = rta_nest_end(rta, nest);
-			rtnh->rtnh_len += nest_len;
+			inner_nest =
+				nl_attr_nest(&req->n, in_buf_len, RTA_ENCAP);
+			nl_attr_put32(&req->n, in_buf_len, VXLAN_VNI,
+				      vxlan->vni);
+			nl_attr_nest_end(&req->n, inner_nest);
 			break;
 		}
 
-		rtnh = RTNH_NEXT(rtnh);
+		nl_attr_rtnh_end(&req->n, rtnh);
 	}
 
-	assert(rta->rta_len > RTA_LENGTH(0));
-	addattr_l(&req->n, in_buf_len, RTA_MULTIPATH, RTA_DATA(rta),
-		  RTA_PAYLOAD(rta));
+	nl_attr_nest_end(&req->n, nest);
+	assert(nest->rta_len > RTA_LENGTH(0));
 
 done:
 
 	if (ri->pref_src) {
-		addattr_l(&req->n, in_buf_len, RTA_PREFSRC, &ri->pref_src,
-			  bytelen);
+		nl_attr_put(&req->n, in_buf_len, RTA_PREFSRC, ri->pref_src,
+			    bytelen);
 	}
 
 	assert(req->n.nlmsg_len < in_buf_len);
@@ -520,22 +517,33 @@ done:
  *
  * Helper function to log the information in a route_info structure.
  */
-static void zfpm_log_route_info(netlink_route_info_t *ri, const char *label)
+static void zfpm_log_route_info(struct netlink_route_info *ri,
+				const char *label)
 {
-	netlink_nh_info_t *nhi;
+	struct netlink_nh_info *nhi;
 	unsigned int i;
+	char buf[PREFIX_STRLEN];
 
-	zfpm_debug("%s : %s %s/%d, Proto: %s, Metric: %u", label,
-		   nl_msg_type_to_str(ri->nlmsg_type),
-		   prefix_addr_to_a(ri->prefix), ri->prefix->prefixlen,
+	zfpm_debug("%s : %s %pFX, Proto: %s, Metric: %u", label,
+		   nl_msg_type_to_str(ri->nlmsg_type), ri->prefix,
 		   nl_rtproto_to_str(ri->rtm_protocol),
 		   ri->metric ? *ri->metric : 0);
 
 	for (i = 0; i < ri->num_nhs; i++) {
 		nhi = &ri->nhs[i];
+
+		if (nhi->gateway) {
+			if (ri->af == AF_INET)
+				inet_ntop(AF_INET, nhi->gateway, buf,
+					  sizeof(buf));
+			else
+				inet_ntop(AF_INET6, nhi->gateway, buf,
+					  sizeof(buf));
+		} else
+			strlcpy(buf, "none", sizeof(buf));
+
 		zfpm_debug("  Intf: %u, Gateway: %s, Recursive: %s, Type: %s, Encap type: %s",
-			   nhi->if_index, addr_to_a(ri->af, nhi->gateway),
-			   nhi->recursive ? "yes" : "no",
+			   nhi->if_index, buf, nhi->recursive ? "yes" : "no",
 			   nexthop_type_to_str(nhi->type),
 			   fpm_nh_encap_type_to_str(nhi->encap_info.encap_type)
 			   );
@@ -554,7 +562,7 @@ static void zfpm_log_route_info(netlink_route_info_t *ri, const char *label)
 int zfpm_netlink_encode_route(int cmd, rib_dest_t *dest, struct route_entry *re,
 			      char *in_buf, size_t in_buf_len)
 {
-	netlink_route_info_t ri_space, *ri;
+	struct netlink_route_info ri_space, *ri;
 
 	ri = &ri_space;
 
@@ -577,7 +585,6 @@ int zfpm_netlink_encode_route(int cmd, rib_dest_t *dest, struct route_entry *re,
 int zfpm_netlink_encode_mac(struct fpm_mac_info_t *mac, char *in_buf,
 			    size_t in_buf_len)
 {
-	char buf1[ETHER_ADDR_STRLEN];
 	size_t buf_offset;
 
 	struct macmsg {
@@ -613,18 +620,17 @@ int zfpm_netlink_encode_mac(struct fpm_mac_info_t *mac, char *in_buf,
 		req->ndm.ndm_flags |= NTF_EXT_LEARNED;
 
 	/* Add attributes */
-	addattr_l(&req->hdr, in_buf_len, NDA_LLADDR, &mac->macaddr, 6);
-	addattr_l(&req->hdr, in_buf_len, NDA_DST, &mac->r_vtep_ip, 4);
-	addattr32(&req->hdr, in_buf_len, NDA_MASTER, mac->svi_if);
-	addattr32(&req->hdr, in_buf_len, NDA_VNI, mac->vni);
+	nl_attr_put(&req->hdr, in_buf_len, NDA_LLADDR, &mac->macaddr, 6);
+	nl_attr_put(&req->hdr, in_buf_len, NDA_DST, &mac->r_vtep_ip, 4);
+	nl_attr_put32(&req->hdr, in_buf_len, NDA_MASTER, mac->svi_if);
+	nl_attr_put32(&req->hdr, in_buf_len, NDA_VNI, mac->vni);
 
 	assert(req->hdr.nlmsg_len < in_buf_len);
 
-	zfpm_debug("Tx %s family %s ifindex %u MAC %s DEST %s",
+	zfpm_debug("Tx %s family %s ifindex %u MAC %pEA DEST %pI4",
 		   nl_msg_type_to_str(req->hdr.nlmsg_type),
 		   nl_family_to_str(req->ndm.ndm_family), req->ndm.ndm_ifindex,
-		   prefix_mac2str(&mac->macaddr, buf1, sizeof(buf1)),
-		   inet_ntoa(mac->r_vtep_ip));
+		   &mac->macaddr, &mac->r_vtep_ip);
 
 	return req->hdr.nlmsg_len;
 }
