@@ -46,12 +46,13 @@
 #include "zebra/debug.h"
 #include "zebra/zapi_msg.h"
 
+DEFINE_MTYPE_STATIC(ZEBRA, ZEBRA_GR, "GR");
 
 /*
  * Forward declaration.
  */
 static struct zserv *zebra_gr_find_stale_client(struct zserv *client);
-static int32_t zebra_gr_route_stale_delete_timer_expiry(struct thread *thread);
+static void zebra_gr_route_stale_delete_timer_expiry(struct thread *thread);
 static int32_t zebra_gr_delete_stale_routes(struct client_gr_info *info);
 static void zebra_gr_process_client_stale_routes(struct zserv *client,
 						 vrf_id_t vrf_id);
@@ -111,14 +112,14 @@ static struct client_gr_info *zebra_gr_client_info_create(struct zserv *client)
 {
 	struct client_gr_info *info;
 
-	info = XCALLOC(MTYPE_TMP, sizeof(struct client_gr_info));
+	info = XCALLOC(MTYPE_ZEBRA_GR, sizeof(struct client_gr_info));
 
 	TAILQ_INSERT_TAIL(&(client->gr_info_queue), info, gr_info);
 	return info;
 }
 
 /*
- * A helper function to delte and destory client info.
+ * A helper function to delete and destroy client info.
  */
 static void zebra_gr_client_info_delte(struct zserv *client,
 				       struct client_gr_info *info)
@@ -127,16 +128,16 @@ static void zebra_gr_client_info_delte(struct zserv *client,
 
 	THREAD_OFF(info->t_stale_removal);
 
-	XFREE(MTYPE_TMP, info->current_prefix);
+	XFREE(MTYPE_ZEBRA_GR, info->current_prefix);
 
 	LOG_GR("%s: Instance info is being deleted for client %s", __func__,
 	       zebra_route_string(client->proto));
 
 	/* Delete all the stale routes. */
-	info->delete = true;
+	info->do_delete = true;
 	zebra_gr_delete_stale_routes(info);
 
-	XFREE(MTYPE_TMP, info);
+	XFREE(MTYPE_ZEBRA_GR, info);
 }
 
 /*
@@ -162,7 +163,7 @@ int32_t zebra_gr_client_disconnect(struct zserv *client)
 
 	client->restart_time = monotime(&tv);
 
-	/* For all the GR instance start the starle removal timer. */
+	/* For all the GR instance start the stale removal timer. */
 	TAILQ_FOREACH (info, &client->gr_info_queue, gr_info) {
 		if (ZEBRA_CLIENT_GR_ENABLED(info->capabilities)
 		    && (info->t_stale_removal == NULL)) {
@@ -222,8 +223,8 @@ static void zebra_gr_delete_stale_client(struct client_gr_info *info)
 	TAILQ_INIT(&(s_client->gr_info_queue));
 	listnode_delete(zrouter.stale_client_list, s_client);
 	if (info->stale_client)
-		XFREE(MTYPE_TMP, s_client);
-	XFREE(MTYPE_TMP, info);
+		zserv_client_delete(s_client);
+	XFREE(MTYPE_ZEBRA_GR, info);
 }
 
 /*
@@ -288,7 +289,7 @@ void zebra_gr_client_reconnect(struct zserv *client)
 	/* Delete the stale client */
 	listnode_delete(zrouter.stale_client_list, old_client);
 	/* Delete old client */
-	XFREE(MTYPE_TMP, old_client);
+	zserv_client_delete(old_client);
 }
 
 /*
@@ -298,7 +299,7 @@ void zebra_gr_client_reconnect(struct zserv *client)
 /*
  * Update the graceful restart information
  * for the client instance.
- * This function handles all the capabilties that are received.
+ * This function handles all the capabilities that are received.
  */
 static void zebra_client_update_info(struct zserv *client, struct zapi_cap *api)
 {
@@ -334,7 +335,7 @@ static void zebra_client_update_info(struct zserv *client, struct zapi_cap *api)
 		if (!info)
 			info = zebra_gr_client_info_create(client);
 
-		/* Udpate other parameters */
+		/* Update other parameters */
 		if (!info->gr_enable) {
 			client->gr_instance_count++;
 
@@ -444,7 +445,7 @@ void zread_client_capabilities(ZAPI_HANDLER_ARGS)
  * Delete all the stale routes that have not been refreshed
  * post restart.
  */
-static int32_t zebra_gr_route_stale_delete_timer_expiry(struct thread *thread)
+static void zebra_gr_route_stale_delete_timer_expiry(struct thread *thread)
 {
 	struct client_gr_info *info;
 	int32_t cnt = 0;
@@ -456,11 +457,11 @@ static int32_t zebra_gr_route_stale_delete_timer_expiry(struct thread *thread)
 
 	/* Set the flag to indicate all stale route deletion */
 	if (thread->u.val == 1)
-		info->delete = true;
+		info->do_delete = true;
 
 	cnt = zebra_gr_delete_stale_routes(info);
 
-	/* Retsart the timer */
+	/* Restart the timer */
 	if (cnt > 0) {
 		LOG_GR("%s: Client %s processed %d routes. Start timer again",
 		       __func__, zebra_route_string(client->proto), cnt);
@@ -471,14 +472,13 @@ static int32_t zebra_gr_route_stale_delete_timer_expiry(struct thread *thread)
 				 &info->t_stale_removal);
 	} else {
 		/* No routes to delete for the VRF */
-		LOG_GR("%s: Client %s all starle routes processed", __func__,
+		LOG_GR("%s: Client %s all stale routes processed", __func__,
 		       zebra_route_string(client->proto));
 
-		XFREE(MTYPE_TMP, info->current_prefix);
+		XFREE(MTYPE_ZEBRA_GR, info->current_prefix);
 		info->current_afi = 0;
 		zebra_gr_delete_stale_client(info);
 	}
-	return 0;
 }
 
 
@@ -490,19 +490,15 @@ static void zebra_gr_process_route_entry(struct zserv *client,
 					 struct route_node *rn,
 					 struct route_entry *re)
 {
-	char buf[PREFIX2STR_BUFFER];
-
 	if ((client == NULL) || (rn == NULL) || (re == NULL))
 		return;
 
 	/* If the route is not refreshed after restart, delete the entry */
 	if (re->uptime < client->restart_time) {
-		if (IS_ZEBRA_DEBUG_RIB) {
-			prefix2str(&rn->p, buf, sizeof(buf));
-			zlog_debug("%s: Client %s stale route %s is deleted",
+		if (IS_ZEBRA_DEBUG_RIB)
+			zlog_debug("%s: Client %s stale route %pFX is deleted",
 				   __func__, zebra_route_string(client->proto),
-				   buf);
-		}
+				   &rn->p);
 		rib_delnode(rn, re);
 	}
 }
@@ -581,10 +577,10 @@ static int32_t zebra_gr_delete_stale_route(struct client_gr_info *info,
 					 * Store the current prefix and afi
 					 */
 					if ((n >= ZEBRA_MAX_STALE_ROUTE_COUNT)
-					    && (info->delete == false)) {
+					    && (info->do_delete == false)) {
 						info->current_afi = afi;
 						info->current_prefix = XCALLOC(
-							MTYPE_TMP,
+							MTYPE_ZEBRA_GR,
 							sizeof(struct prefix));
 						prefix_copy(
 							info->current_prefix,
@@ -598,7 +594,7 @@ static int32_t zebra_gr_delete_stale_route(struct client_gr_info *info,
 		 * Reset the current prefix to indicate processing completion
 		 * of the current AFI
 		 */
-		XFREE(MTYPE_TMP, info->current_prefix);
+		XFREE(MTYPE_ZEBRA_GR, info->current_prefix);
 	}
 	return 0;
 }
@@ -653,26 +649,24 @@ static void zebra_gr_process_client_stale_routes(struct zserv *client,
 		return;
 
 	/* Check if route update completed for all AFI, SAFI */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi <= SAFI_MPLS_VPN; safi++) {
-			if (info->af_enabled[afi][safi]) {
-				if (!info->route_sync[afi][safi]) {
-					LOG_GR(
-					   "%s: Client %s route update not completed for AFI %d, SAFI %d",
-					   __func__, zebra_route_string(
-							    client->proto),
-					   afi, safi);
-					return;
-				}
+	FOREACH_AFI_SAFI_NSF (afi, safi) {
+		if (info->af_enabled[afi][safi]) {
+			if (!info->route_sync[afi][safi]) {
+				LOG_GR("%s: Client %s route update not completed for AFI %d, SAFI %d",
+				       __func__,
+				       zebra_route_string(client->proto), afi,
+				       safi);
+				return;
 			}
 		}
+	}
 
 	/*
 	 * Route update completed for all AFI, SAFI
 	 * Cancel the stale timer and process the routes
 	 */
 	if (info->t_stale_removal) {
-		LOG_GR("%s: Client %s cancled stale delete timer vrf %d",
+		LOG_GR("%s: Client %s canceled stale delete timer vrf %d",
 		       __func__, zebra_route_string(client->proto),
 		       info->vrf_id);
 		THREAD_OFF(info->t_stale_removal);
