@@ -23,6 +23,7 @@
 #include "vrf.h"
 #include "log.h"
 #include "lib_errors.h"
+#include "network.h"
 
 #include "zebra/rib.h"
 #include "zebra/zebra_dplane.h"
@@ -39,9 +40,6 @@
 #include "zebra/zebra_netns_id.h"
 #include "zebra/zebra_errors.h"
 
-/* default NS ID value used when VRF backend is not NETNS */
-#define NS_DEFAULT_INTERNAL 0
-
 /* in case NEWNSID not available, the NSID will be locally obtained
  */
 #define NS_BASE_NSID 0
@@ -76,7 +74,7 @@ static struct nlmsghdr *initiate_nlh(char *buf, unsigned int *seq, int type)
 	nlh->nlmsg_flags = NLM_F_REQUEST;
 	if (type == RTM_NEWNSID)
 		nlh->nlmsg_flags |= NLM_F_ACK;
-	nlh->nlmsg_seq = *seq = time(NULL);
+	nlh->nlmsg_seq = *seq = frr_sequence32_next();
 	return nlh;
 }
 
@@ -139,7 +137,6 @@ static ns_id_t extract_nsid(struct nlmsghdr *nlh, char *buf)
 	ns_id_t ns_id = NS_UNKNOWN;
 	int offset = NETLINK_ALIGN(sizeof(struct nlmsghdr))
 		     + NETLINK_ALIGN(sizeof(struct rtgenmsg));
-	int curr_length = offset;
 	void *tail = (void *)((char *)nlh + NETLINK_ALIGN(nlh->nlmsg_len));
 	struct nlattr *attr;
 
@@ -148,7 +145,6 @@ static ns_id_t extract_nsid(struct nlmsghdr *nlh, char *buf)
 	     && attr->nla_len >= sizeof(struct nlattr)
 	     && attr->nla_len <= NETLINK_NLATTR_LEN(tail, attr);
 	     attr += NETLINK_ALIGN(attr->nla_len)) {
-		curr_length += attr->nla_len;
 		if ((attr->nla_type & NLA_TYPE_MASK) == NETNSA_NSID) {
 			uint32_t *ptr = (uint32_t *)(attr);
 
@@ -159,27 +155,34 @@ static ns_id_t extract_nsid(struct nlmsghdr *nlh, char *buf)
 	return ns_id;
 }
 
-ns_id_t zebra_ns_id_get(const char *netnspath)
+/* fd_param = -1 is ignored.
+ * netnspath set to null is ignored.
+ * one of the 2 params is mandatory. netnspath is looked in priority
+ */
+ns_id_t zebra_ns_id_get(const char *netnspath, int fd_param)
 {
 	int ns_id = -1;
 	struct sockaddr_nl snl;
-	int fd, sock, ret;
+	int fd = -1, sock, ret;
 	unsigned int seq;
 	ns_id_t return_nsid = NS_UNKNOWN;
 
 	/* netns path check */
-	if (!netnspath)
+	if (!netnspath && fd_param == -1)
 		return NS_UNKNOWN;
-	fd = open(netnspath, O_RDONLY);
-	if (fd == -1)
-		return NS_UNKNOWN;
-
+	if (netnspath)  {
+		fd = open(netnspath, O_RDONLY);
+		if (fd == -1)
+			return NS_UNKNOWN;
+	} else if (fd_param != -1)
+		fd = fd_param;
 	/* netlink socket */
 	sock = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
 	if (sock < 0) {
 		flog_err_sys(EC_LIB_SOCKET, "netlink( %u) socket() error: %s",
 			     sock, safe_strerror(errno));
-		close(fd);
+		if (netnspath)
+			close(fd);
 		return NS_UNKNOWN;
 	}
 	memset(&snl, 0, sizeof(snl));
@@ -192,7 +195,8 @@ ns_id_t zebra_ns_id_get(const char *netnspath)
 			     "netlink( %u) socket() bind error: %s", sock,
 			     safe_strerror(errno));
 		close(sock);
-		close(fd);
+		if (netnspath)
+			close(fd);
 		return NS_UNKNOWN;
 	}
 
@@ -208,13 +212,14 @@ ns_id_t zebra_ns_id_get(const char *netnspath)
 	nlh->nlmsg_len += NETLINK_ALIGN(sizeof(struct rtgenmsg));
 	rt->rtgen_family = AF_UNSPEC;
 
-	addattr32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_FD, fd);
-	addattr32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_NSID, ns_id);
+	nl_attr_put32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_FD, fd);
+	nl_attr_put32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_NSID, ns_id);
 
 	ret = send_receive(sock, nlh, seq, buf);
 	if (ret < 0) {
 		close(sock);
-		close(fd);
+		if (netnspath)
+			close(fd);
 		return NS_UNKNOWN;
 	}
 	nlh = (struct nlmsghdr *)buf;
@@ -251,74 +256,72 @@ ns_id_t zebra_ns_id_get(const char *netnspath)
 		}
 	}
 
-	if (ret <= 0) {
-		if (errno != EEXIST && ret != 0) {
-			flog_err(
-				EC_LIB_SOCKET,
-				"netlink( %u) recvfrom() error 2 when reading: %s",
-				fd, safe_strerror(errno));
-			close(sock);
+	if (errno != EEXIST && ret != 0) {
+		flog_err(EC_LIB_SOCKET,
+			 "netlink( %u) recvfrom() error 2 when reading: %s", fd,
+			 safe_strerror(errno));
+		close(sock);
+		if (netnspath)
 			close(fd);
-			if (errno == ENOTSUP) {
-				zlog_debug("NEWNSID locally generated");
-				return zebra_ns_id_get_fallback(netnspath);
-			}
-			return NS_UNKNOWN;
+		if (errno == ENOTSUP) {
+			zlog_debug("NEWNSID locally generated");
+			return zebra_ns_id_get_fallback(netnspath);
 		}
-		/* message to send to netlink : GETNSID */
-		memset(buf, 0, NETLINK_SOCKET_BUFFER_SIZE);
-		nlh = initiate_nlh(buf, &seq, RTM_GETNSID);
-		rt = (struct rtgenmsg *)(buf + nlh->nlmsg_len);
-		nlh->nlmsg_len += NETLINK_ALIGN(sizeof(struct rtgenmsg));
-		rt->rtgen_family = AF_UNSPEC;
-
-		addattr32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_FD, fd);
-		addattr32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_NSID, ns_id);
-
-		ret = send_receive(sock, nlh, seq, buf);
-		if (ret < 0) {
-			close(sock);
-			close(fd);
-			return NS_UNKNOWN;
-		}
-		nlh = (struct nlmsghdr *)buf;
-		len = ret;
-		ret = 0;
-		do {
-			if (nlh->nlmsg_type >= NLMSG_MIN_TYPE) {
-				return_nsid = extract_nsid(nlh, buf);
-				if (return_nsid != NS_UNKNOWN)
-					break;
-			} else if (nlh->nlmsg_type == NLMSG_ERROR) {
-				struct nlmsgerr *err =
-					(struct nlmsgerr
-						 *)((char *)nlh
-						    + NETLINK_ALIGN(sizeof(
-							      struct
-							      nlmsghdr)));
-				if (err->error < 0)
-					errno = -err->error;
-				else
-					errno = err->error;
-				break;
-			}
-			len = len - NETLINK_ALIGN(nlh->nlmsg_len);
-			nlh = (struct nlmsghdr *)((char *)nlh
-						  + NETLINK_ALIGN(
-							    nlh->nlmsg_len));
-		} while (len != 0 && ret == 0);
+		return NS_UNKNOWN;
 	}
+	/* message to send to netlink : GETNSID */
+	memset(buf, 0, NETLINK_SOCKET_BUFFER_SIZE);
+	nlh = initiate_nlh(buf, &seq, RTM_GETNSID);
+	rt = (struct rtgenmsg *)(buf + nlh->nlmsg_len);
+	nlh->nlmsg_len += NETLINK_ALIGN(sizeof(struct rtgenmsg));
+	rt->rtgen_family = AF_UNSPEC;
 
-	close(fd);
+	nl_attr_put32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_FD, fd);
+	nl_attr_put32(nlh, NETLINK_SOCKET_BUFFER_SIZE, NETNSA_NSID, ns_id);
+
+	ret = send_receive(sock, nlh, seq, buf);
+	if (ret < 0) {
+		close(sock);
+		if (netnspath)
+			close(fd);
+		return NS_UNKNOWN;
+	}
+	nlh = (struct nlmsghdr *)buf;
+	len = ret;
+	ret = 0;
+	do {
+		if (nlh->nlmsg_type >= NLMSG_MIN_TYPE) {
+			return_nsid = extract_nsid(nlh, buf);
+			if (return_nsid != NS_UNKNOWN)
+				break;
+		} else if (nlh->nlmsg_type == NLMSG_ERROR) {
+			struct nlmsgerr *err =
+				(struct nlmsgerr *)((char *)nlh +
+						    NETLINK_ALIGN(sizeof(
+							    struct nlmsghdr)));
+			if (err->error < 0)
+				errno = -err->error;
+			else
+				errno = err->error;
+			break;
+		}
+		len = len - NETLINK_ALIGN(nlh->nlmsg_len);
+		nlh = (struct nlmsghdr *)((char *)nlh +
+					  NETLINK_ALIGN(nlh->nlmsg_len));
+	} while (len != 0 && ret == 0);
+
+	if (netnspath)
+		close(fd);
 	close(sock);
 	return return_nsid;
 }
 
 #else
-ns_id_t zebra_ns_id_get(const char *netnspath)
+ns_id_t zebra_ns_id_get(const char *netnspath, int fd __attribute__ ((unused)))
 {
 	return zebra_ns_id_get_fallback(netnspath);
 }
+
 #endif /* ! defined(HAVE_NETLINK) */
 
 #ifdef HAVE_NETNS
@@ -348,14 +351,14 @@ ns_id_t zebra_ns_id_get_default(void)
 	fd = open(NS_DEFAULT_NAME, O_RDONLY);
 
 	if (fd == -1)
-		return NS_DEFAULT_INTERNAL;
+		return NS_DEFAULT;
 	if (!vrf_is_backend_netns()) {
 		close(fd);
-		return NS_DEFAULT_INTERNAL;
+		return NS_DEFAULT;
 	}
 	close(fd);
-	return zebra_ns_id_get((char *)NS_DEFAULT_NAME);
+	return zebra_ns_id_get((char *)NS_DEFAULT_NAME, -1);
 #else  /* HAVE_NETNS */
-	return NS_DEFAULT_INTERNAL;
+	return NS_DEFAULT;
 #endif /* !HAVE_NETNS */
 }
