@@ -36,6 +36,27 @@
 extern "C" {
 #endif
 
+/* Memory type for PBR objects. */
+DECLARE_MTYPE(PBR_OBJ);
+
+struct zebra_pbr_action {
+	afi_t afi;
+
+	/* currently only one nexthop is supported */
+	union g_addr gate;
+
+	/* dest-interface */
+	ifindex_t ifindex;
+
+	/* dataplane info */
+	intptr_t dp_flow_ptr;
+
+	/* neigh */
+	struct zebra_neigh_ent *neigh;
+	/* zebra_pbr_rule is linked to neigh via neigh_listnode */
+	struct listnode neigh_listnode;
+};
+
 struct zebra_pbr_rule {
 	int sock;
 
@@ -43,6 +64,8 @@ struct zebra_pbr_rule {
 
 	char ifname[INTERFACE_NAMSIZ];
 
+	struct zebra_pbr_action action;
+
 	vrf_id_t vrf_id;
 };
 
@@ -54,6 +77,8 @@ struct zebra_pbr_rule {
 	(r->rule.filter.filter_bm & PBR_FILTER_SRC_PORT)
 #define IS_RULE_FILTERING_ON_DST_PORT(r) \
 	(r->rule.filter.filter_bm & PBR_FILTER_DST_PORT)
+#define IS_RULE_FILTERING_ON_DSFIELD(r) \
+	(r->rule.filter.filter_bm & PBR_FILTER_DSFIELD)
 #define IS_RULE_FILTERING_ON_FWMARK(r) \
 	(r->rule.filter.filter_bm & PBR_FILTER_FWMARK)
 
@@ -62,6 +87,17 @@ struct zebra_pbr_rule {
  *
  * This is a filter mapped on ipset entries
  */
+struct zebra_pbr_ipset_info {
+	/* type is encoded as uint32_t
+	 * but value is an enum ipset_type
+	 */
+	uint32_t type;
+
+	uint8_t family;
+
+	char ipset_name[ZEBRA_IPSET_NAME_SIZE];
+};
+
 struct zebra_pbr_ipset {
 	/*
 	 * Originating zclient sock fd, so we can know who to send
@@ -77,9 +113,13 @@ struct zebra_pbr_ipset {
 	 * but value is an enum ipset_type
 	 */
 	uint32_t type;
+
+	uint8_t family;
+
 	char ipset_name[ZEBRA_IPSET_NAME_SIZE];
 };
 
+
 /*
  * An IPSet Entry Filter
  *
@@ -148,13 +188,20 @@ struct zebra_pbr_iptable {
 	uint8_t protocol;
 
 	uint32_t nb_interface;
+	uint16_t flow_label;
+
+	uint8_t family;
 
 	struct list *interface_name_list;
 
+#define IPTABLE_INSTALL_QUEUED 1 << 1
+#define IPTABLE_UNINSTALL_QUEUED 1 << 2
+	uint8_t internal_flags;
 	char ipset_name[ZEBRA_IPSET_NAME_SIZE];
 };
 
 extern const struct message icmp_typecode_str[];
+extern const struct message icmpv6_typecode_str[];
 
 const char *zebra_pbr_ipset_type2str(uint32_t type);
 
@@ -168,26 +215,9 @@ void zebra_pbr_del_ipset_entry(struct zebra_pbr_ipset_entry *ipset);
 
 void zebra_pbr_add_iptable(struct zebra_pbr_iptable *iptable);
 void zebra_pbr_del_iptable(struct zebra_pbr_iptable *iptable);
-
-/*
- * Install specified rule for a specific interface.
- * It is possible that the user-defined sequence number and the one in the
- * forwarding plane may not coincide, hence the API requires a separate
- * rule priority - maps to preference/FRA_PRIORITY on Linux.
- */
-extern enum zebra_dplane_result kernel_add_pbr_rule(struct zebra_pbr_rule *rule);
-
-/*
- * Uninstall specified rule for a specific interface.
- */
-extern enum zebra_dplane_result kernel_del_pbr_rule(struct zebra_pbr_rule *rule);
-
-/*
- * Update specified rule for a specific interface.
- */
-extern enum zebra_dplane_result
-kernel_update_pbr_rule(struct zebra_pbr_rule *old_rule,
-		       struct zebra_pbr_rule *new_rule);
+void zebra_pbr_process_iptable(struct zebra_dplane_ctx *ctx);
+void zebra_pbr_process_ipset(struct zebra_dplane_ctx *ctx);
+void zebra_pbr_process_ipset_entry(struct zebra_dplane_ctx *ctx);
 
 /*
  * Get to know existing PBR rules in the kernel - typically called at startup.
@@ -197,8 +227,7 @@ extern void kernel_read_pbr_rules(struct zebra_ns *zns);
 /*
  * Handle success or failure of rule (un)install in the kernel.
  */
-extern void kernel_pbr_rule_add_del_status(struct zebra_pbr_rule *rule,
-					   enum zebra_dplane_status res);
+extern void zebra_pbr_dplane_result(struct zebra_dplane_ctx *ctx);
 
 /*
  * Handle success or failure of ipset kinds (un)install in the kernel.
@@ -210,9 +239,6 @@ extern void kernel_pbr_ipset_entry_add_del_status(
 				struct zebra_pbr_ipset_entry *ipset,
 				enum zebra_dplane_status res);
 
-extern void kernel_pbr_iptable_add_del_status(struct zebra_pbr_iptable *iptable,
-			      enum zebra_dplane_status res);
-
 /*
  * Handle rule delete notification from kernel.
  */
@@ -240,6 +266,8 @@ extern void zebra_pbr_iptable_free(void *arg);
 extern uint32_t zebra_pbr_iptable_hash_key(const void *arg);
 extern bool zebra_pbr_iptable_hash_equal(const void *arg1, const void *arg2);
 
+extern void zebra_pbr_config_write(struct vty *vty);
+extern void zebra_pbr_expand_action_update(bool enable);
 extern void zebra_pbr_init(void);
 extern void zebra_pbr_show_ipset_list(struct vty *vty, char *ipsetname);
 extern void zebra_pbr_show_iptable(struct vty *vty, char *iptable);
@@ -247,15 +275,18 @@ extern void zebra_pbr_iptable_update_interfacelist(struct stream *s,
 				   struct zebra_pbr_iptable *zpi);
 size_t zebra_pbr_tcpflags_snprintf(char *buffer, size_t len,
 				   uint16_t tcp_val);
+extern void zebra_pbr_show_rule(struct vty *vty);
+extern void zebra_pbr_show_rule_unit(struct zebra_pbr_rule *rule,
+				     struct vty *vty);
 
 DECLARE_HOOK(zebra_pbr_ipset_entry_get_stat,
 	     (struct zebra_pbr_ipset_entry *ipset, uint64_t *pkts,
 	      uint64_t *bytes),
-	     (ipset, pkts, bytes))
+	     (ipset, pkts, bytes));
 DECLARE_HOOK(zebra_pbr_iptable_get_stat,
 	     (struct zebra_pbr_iptable *iptable, uint64_t *pkts,
 	      uint64_t *bytes),
-	     (iptable, pkts, bytes))
+	     (iptable, pkts, bytes));
 DECLARE_HOOK(zebra_pbr_iptable_update,
 	     (int cmd, struct zebra_pbr_iptable *iptable), (cmd, iptable));
 
